@@ -51,6 +51,11 @@ public class VirtualFlowShim<T extends IndexedCell> extends VirtualFlow<T> {
         return super.getCellLength(index);
     }
 
+    @Override
+    public double getCellPosition(T cell) {
+        return super.getCellPosition(cell);
+    }
+
     @Override
     public void recreateCells() {
         super.recreateCells();
