@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -39,9 +39,11 @@ import com.sun.scenario.effect.compiler.model.Variable;
 import com.sun.scenario.effect.compiler.tree.FuncDef;
 import com.sun.scenario.effect.compiler.tree.ProgramUnit;
 import com.sun.scenario.effect.compiler.tree.TreeScanner;
-import org.antlr.stringtemplate.StringTemplate;
-import org.antlr.stringtemplate.StringTemplateGroup;
-import org.antlr.stringtemplate.language.DefaultTemplateLexer;
+import org.stringtemplate.v4.ST;
+import org.stringtemplate.v4.STGroup;
+import org.stringtemplate.v4.STGroupFile;
+
+import static java.nio.charset.StandardCharsets.UTF_8;
 
 /**
  */
@@ -274,40 +276,38 @@ public class MEBackend extends TreeScanner {
             interfaceDecl.append("implements "+interfaceName);
         }
 
-        Reader template = new InputStreamReader(getClass().getResourceAsStream("MEJavaGlue.stg"));
-        StringTemplateGroup group = new StringTemplateGroup(template, DefaultTemplateLexer.class);
-        StringTemplate jglue = group.getInstanceOf("glue");
-        jglue.setAttribute("effectName", effectName);
-        jglue.setAttribute("peerName", peerName);
-        jglue.setAttribute("genericsDecl", genericsDecl.toString());
-        jglue.setAttribute("interfaceDecl", interfaceDecl.toString());
-        jglue.setAttribute("usercode", usercode.toString());
-        jglue.setAttribute("samplers", samplers.toString());
-        jglue.setAttribute("srcRects", srcRects.toString());
-        jglue.setAttribute("constants", constants.toString());
-        jglue.setAttribute("params", jparams.toString());
-        jglue.setAttribute("paramDecls", jparamDecls.toString());
+        STGroup group = new STGroupFile(getClass().getResource("MEJavaGlue.stg"), UTF_8.displayName(), '$', '$');
+        ST jglue = group.getInstanceOf("glue");
+        jglue.add("effectName", effectName);
+        jglue.add("peerName", peerName);
+        jglue.add("genericsDecl", genericsDecl.toString());
+        jglue.add("interfaceDecl", interfaceDecl.toString());
+        jglue.add("usercode", usercode.toString());
+        jglue.add("samplers", samplers.toString());
+        jglue.add("srcRects", srcRects.toString());
+        jglue.add("constants", constants.toString());
+        jglue.add("params", jparams.toString());
+        jglue.add("paramDecls", jparamDecls.toString());
 
-        template = new InputStreamReader(getClass().getResourceAsStream("MENativeGlue.stg"));
-        group = new StringTemplateGroup(template, DefaultTemplateLexer.class);
-        StringTemplate cglue = group.getInstanceOf("glue");
-        cglue.setAttribute("peerName", peerName);
-        cglue.setAttribute("jniName", peerName.replace("_", "_1"));
-        cglue.setAttribute("paramDecls", cparamDecls.toString());
-        cglue.setAttribute("arrayGet", arrayGet.toString());
-        cglue.setAttribute("arrayRelease", arrayRelease.toString());
-        cglue.setAttribute("posDecls", posDecls.toString());
-        cglue.setAttribute("pixInitY", pixInitY.toString());
-        cglue.setAttribute("pixInitX", pixInitX.toString());
-        cglue.setAttribute("posIncrY", posIncrY.toString());
-        cglue.setAttribute("posInitY", posInitY.toString());
-        cglue.setAttribute("posIncrX", posIncrX.toString());
-        cglue.setAttribute("posInitX", posInitX.toString());
-        cglue.setAttribute("body", body);
+        group = new STGroupFile(getClass().getResource("MENativeGlue.stg"), UTF_8.displayName(), '$', '$');
+        ST cglue = group.getInstanceOf("glue");
+        cglue.add("peerName", peerName);
+        cglue.add("jniName", peerName.replace("_", "_1"));
+        cglue.add("paramDecls", cparamDecls.toString());
+        cglue.add("arrayGet", arrayGet.toString());
+        cglue.add("arrayRelease", arrayRelease.toString());
+        cglue.add("posDecls", posDecls.toString());
+        cglue.add("pixInitY", pixInitY.toString());
+        cglue.add("pixInitX", pixInitX.toString());
+        cglue.add("posIncrY", posIncrY.toString());
+        cglue.add("posInitY", posInitY.toString());
+        cglue.add("posIncrX", posIncrX.toString());
+        cglue.add("posInitX", posInitX.toString());
+        cglue.add("body", body);
 
         GenCode gen = new GenCode();
-        gen.javaCode = jglue.toString();
-        gen.nativeCode = cglue.toString();
+        gen.javaCode = jglue.render();
+        gen.nativeCode = cglue.render();
         return gen;
     }
 
