@@ -25,11 +25,11 @@
 
 package javafx.animation;
 
-import com.sun.scenario.animation.AbstractMasterTimer;
+import com.sun.scenario.animation.AbstractPrimaryTimer;
 
 public class SequentialTransitionShim {
 
-    public static SequentialTransition getSequentialTransition(AbstractMasterTimer timer) {
+    public static SequentialTransition getSequentialTransition(AbstractPrimaryTimer timer) {
         return new SequentialTransition(timer);
 
     }
