@@ -31,6 +31,7 @@ import javafx.geometry.HPos;
 import javafx.geometry.Insets;
 import javafx.geometry.Orientation;
 import javafx.geometry.VPos;
+import javafx.scene.Scene;
 import javafx.scene.image.Image;
 import test.javafx.scene.image.ImageForTesting;
 import javafx.scene.image.WritableImage;
@@ -39,6 +40,8 @@ import javafx.scene.shape.ClosePath;
 import javafx.scene.shape.LineTo;
 import javafx.scene.shape.MoveTo;
 import javafx.scene.shape.Path;
+import javafx.stage.Stage;
+import java.util.Random;
 import java.util.concurrent.atomic.AtomicBoolean;
 import com.sun.javafx.scene.DirtyBits;
 import com.sun.javafx.scene.NodeHelper;
@@ -1248,4 +1251,58 @@ public class RegionTest {
         NodeHelper.syncPeer(r);
         assertFalse(peer.isClean());
     }
+
+    // Test for JDK-8255415
+    @Test public void snappingASnappedValueGivesTheSameValueTest() {
+        Stage stage = new Stage();
+        Region region = new Region();
+        Scene scene = new Scene(region);
+        stage.setScene(scene);
+
+        double[] scales = new double[] {1.0, 1.25, 1.5, 1.75, 2.0, 1.374562997};
+
+        // test snapSizeX/snapSizeY methods
+
+        for (double scale : scales) {
+            stage.setRenderScaleX(scale);
+            for (int j = 0; j < 1000; j++) {
+                double value = new Random().nextDouble() * 100 - 50;
+                double snappedValue = region.snapSizeX(value);
+                double snapOfSnappedValue = region.snapSizeX(snappedValue);
+                assertEquals(snappedValue, snapOfSnappedValue, 1.0e-14);
+            }
+        }
+
+        for (double scale : scales) {
+            stage.setRenderScaleY(scale);
+            for (int j = 0; j < 1000; j++) {
+                double value = new Random().nextDouble() * 100 - 50;
+                double snappedValue = region.snapSizeY(value);
+                double snapOfSnappedValue = region.snapSizeY(snappedValue);
+                assertEquals(snappedValue, snapOfSnappedValue, 1.0e-14);
+            }
+        }
+
+        // test snapPortionX/snapPortionY methods
+
+        for (double scale : scales) {
+            stage.setRenderScaleX(scale);
+            for (int j = 0; j < 1000; j++) {
+                double value = new Random().nextDouble() * 100 - 50;
+                double snappedValue = RegionShim.snapPortionX(region, value);
+                double snapOfSnappedValue = RegionShim.snapPortionX(region, snappedValue);
+                assertEquals(snappedValue, snapOfSnappedValue, 1.0e-14);
+            }
+        }
+
+        for (double scale : scales) {
+            stage.setRenderScaleY(scale);
+            for (int j = 0; j < 1000; j++) {
+                double value = new Random().nextDouble() * 100 - 50;
+                double snappedValue = RegionShim.snapPortionY(region, value);
+                double snapOfSnappedValue = RegionShim.snapPortionY(region, snappedValue);
+                assertEquals(snappedValue, snapOfSnappedValue, 1.0e-14);
+            }
+        }
+    }
 }
