@@ -29,8 +29,10 @@
 #define GST_AUDIO_API
 #endif
 #else // GSTREAMER_LITE
-#ifndef GST_AUDIO_API
-#define GST_AUDIO_API GST_EXPORT
+#ifdef BUILDING_GST_AUDIO
+#define GST_AUDIO_API GST_API_EXPORT         /* from config.h */
+#else
+#define GST_AUDIO_API GST_API_IMPORT
 #endif
 #endif // GSTREAMER_LITE
 
