@@ -29,9 +29,19 @@
 #define GST_PBUTILS_API
 #endif
 #else // GSTREAMER_LITE
-#ifndef GST_PBUTILS_API
-#define GST_PBUTILS_API GST_EXPORT
+#ifdef BUILDING_GST_PBUTILS
+#define GST_PBUTILS_API GST_API_EXPORT         /* from config.h */
+#else
+#define GST_PBUTILS_API GST_API_IMPORT
 #endif
 #endif // GSTREAMER_LITE
 
+#ifndef GST_DISABLE_DEPRECATED
+#define GST_PBUTILS_DEPRECATED GST_PBUTILS_API
+#define GST_PBUTILS_DEPRECATED_FOR(f) GST_PBUTILS_API
+#else
+#define GST_PBUTILS_DEPRECATED G_DEPRECATED GST_PBUTILS_API
+#define GST_PBUTILS_DEPRECATED_FOR(f) G_DEPRECATED_FOR(f) GST_PBUTILS_API
+#endif
+
 #endif /* __GST_PBUTILS_PRELUDE_H__ */
