@@ -94,6 +94,14 @@ struct _GstPluginPrivate {
   GstStructure *cache_data;
 };
 
+/* Needed by GstMeta (to access meta seq) and GstBuffer (create/free/iterate) */
+typedef struct _GstMetaItem GstMetaItem;
+struct _GstMetaItem {
+  GstMetaItem *next;
+  guint64 seq_num;
+  GstMeta meta;
+};
+
 /* FIXME: could rename all priv_gst_* functions to __gst_* now */
 G_GNUC_INTERNAL  gboolean priv_gst_plugin_loading_have_whitelist (void);
 
@@ -136,6 +144,7 @@ G_GNUC_INTERNAL  void  _priv_gst_date_time_initialize (void);
 G_GNUC_INTERNAL  void  _priv_gst_allocator_cleanup (void);
 G_GNUC_INTERNAL  void  _priv_gst_caps_features_cleanup (void);
 G_GNUC_INTERNAL  void  _priv_gst_caps_cleanup (void);
+G_GNUC_INTERNAL  void  _priv_gst_debug_cleanup (void);
 
 /* called from gst_task_cleanup_all(). */
 G_GNUC_INTERNAL  void  _priv_gst_element_cleanup (void);
@@ -195,10 +204,10 @@ gchar *priv_gst_string_take_and_wrap (gchar * s);
 
 /* registry cache backends */
 G_GNUC_INTERNAL
-gboolean        priv_gst_registry_binary_read_cache (GstRegistry * registry, const char *location);
+gboolean    priv_gst_registry_binary_read_cache (GstRegistry * registry, const char *location);
 
 G_GNUC_INTERNAL
-gboolean        priv_gst_registry_binary_write_cache    (GstRegistry * registry, GList * plugins, const char *location);
+gboolean    priv_gst_registry_binary_write_cache  (GstRegistry * registry, GList * plugins, const char *location);
 
 
 G_GNUC_INTERNAL
@@ -361,10 +370,10 @@ struct _GstPlugin {
   /*< private >*/
   GstPluginDesc desc;
 
-  gchar *   filename;
-  gchar *   basename;       /* base name (non-dir part) of plugin path */
+  gchar * filename;
+  gchar * basename;       /* base name (non-dir part) of plugin path */
 
-  GModule * module;     /* contains the module if plugin is loaded */
+  GModule * module;   /* contains the module if plugin is loaded */
 
   off_t         file_size;
   time_t        file_mtime;
