@@ -30,7 +30,11 @@
 #endif
 #else // GSTREAMER_LITE
 #ifndef GST_BASE_API
-#define GST_BASE_API GST_EXPORT
+#ifdef BUILDING_GST_BASE
+#define GST_BASE_API GST_API_EXPORT        /* from config.h */
+#else
+#define GST_BASE_API GST_API_IMPORT
+#endif
 #endif
 #endif // GSTREAMER_LITE
 
