@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -103,7 +103,7 @@
     midSendAudioSpectrumEvent = (*env)->GetMethodID(env,
                                                     klass,
                                                     "sendAudioSpectrumEvent",
-                                                    "(DD)V");
+                                                    "(DDZ)V");
 
 }
 
