@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -87,7 +87,7 @@ static void append_log(NSMutableString *s, NSString *fmt, ...) {
 
 @implementation AVFMediaPlayer
 
-static void SpectrumCallbackProc(void *context, double duration);
+static void SpectrumCallbackProc(void *context, double duration, double timestamp);
 
 static CVReturn displayLinkCallback(CVDisplayLinkRef displayLink,
                                     const CVTimeStamp *inNow,
@@ -651,19 +651,19 @@ static CVReturn displayLinkCallback(CVDisplayLinkRef displayLink,
     eventHandler->SendNewFrameEvent(frame);
 }
 
-- (void) sendSpectrumEventDuration:(double)duration {
+- (void) sendSpectrumEventDuration:(double)duration timestamp:(double)timestamp {
     if (eventHandler) {
-        double timestamp = self.currentTime;
-        eventHandler->SendAudioSpectrumEvent(timestamp, duration);
+        // Always true for queryTimestamp to avoid hang. See JDK-8240694.
+        eventHandler->SendAudioSpectrumEvent(timestamp, duration, true);
     }
 }
 
 @end
 
-static void SpectrumCallbackProc(void *context, double duration) {
+static void SpectrumCallbackProc(void *context, double duration, double timestamp) {
     if (context) {
         AVFMediaPlayer *player = (__bridge AVFMediaPlayer*)context;
-        [player sendSpectrumEventDuration:duration];
+        [player sendSpectrumEventDuration:duration timestamp:timestamp];
     }
 }
 
