@@ -33,22 +33,14 @@ import static javafx.scene.web.WebEvent.RESIZED;
 import static javafx.scene.web.WebEvent.STATUS_CHANGED;
 import static javafx.scene.web.WebEvent.VISIBILITY_CHANGED;
 
-import com.sun.javafx.tk.Toolkit;
 import com.sun.webkit.UIClient;
 import com.sun.webkit.WebPage;
 import com.sun.webkit.graphics.WCImage;
 import com.sun.webkit.graphics.WCRectangle;
-import java.awt.AlphaComposite;
-import java.awt.Graphics2D;
-import java.awt.image.BufferedImage;
-import java.awt.image.DataBufferInt;
-import java.awt.image.SampleModel;
-import java.awt.image.SinglePixelPackedSampleModel;
 import java.io.File;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 import java.nio.ByteBuffer;
-import java.nio.IntBuffer;
 import java.security.AccessControlContext;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
@@ -58,10 +50,6 @@ import java.util.List;
 import java.util.Map;
 import javafx.event.EventHandler;
 import javafx.geometry.Rectangle2D;
-import javafx.scene.image.Image;
-import javafx.scene.image.PixelFormat;
-import javafx.scene.image.PixelReader;
-import javafx.scene.image.WritablePixelFormat;
 import javafx.scene.input.ClipboardContent;
 import javafx.scene.input.DataFormat;
 import javafx.scene.input.Dragboard;
@@ -81,6 +69,8 @@ public final class UIClientImpl implements UIClient {
     private final Accessor accessor;
     private FileChooser chooser;
     private static final Map<String, FileExtensionInfo> fileExtensionMap = new HashMap<>();
+    // for testing purposes only
+    private static String[] chooseFiles = null;
 
     private static class FileExtensionInfo {
         private String description;
@@ -232,7 +222,15 @@ public final class UIClientImpl implements UIClient {
         return false;
     }
 
+    // for testing purposes only
+    static void test_setChooseFiles(String[] files) {
+        chooseFiles = files;
+    }
+
     @Override public String[] chooseFile(String initialFileName, boolean multiple, String mimeFilters) {
+        if (chooseFiles != null) {
+            return chooseFiles;
+        }
         // get the toplevel window
         Window win = null;
         WebView view = accessor.getView();
@@ -342,7 +340,7 @@ public final class UIClientImpl implements UIClient {
                 //never happens
             }
         }
-        if (image != null) {
+        if (image != null && !image.isNull()) {
             ByteBuffer dragImageOffset = ByteBuffer.allocate(8);
             dragImageOffset.rewind();
             dragImageOffset.putInt(imageOffsetX);
@@ -364,26 +362,18 @@ public final class UIClientImpl implements UIClient {
             //QuantumClipboard.putContent have to be rewritten in Glass manner
             //with postponed data requests (DelayedCallback data object).
             if (isImageSource) {
-                Object platformImage = image.getWidth() > 0 && image.getHeight() > 0 ?
-                        image.getPlatformImage() : null;
                 String fileExtension = image.getFileExtension();
-                if (platformImage != null) {
-                    try {
-                        File temp = File.createTempFile("jfx", "." + fileExtension);
-                        temp.deleteOnExit();
-                        ImageIO.write(
-                            toBufferedImage(Toolkit.getImageAccessor().fromPlatformImage(
-                                Toolkit.getToolkit().loadPlatformImage(
-                                    platformImage
-                                )
-                            )),
-                            fileExtension,
-                            temp);
-                        content.put(DataFormat.FILES, Arrays.asList(temp));
-                    } catch (IOException | SecurityException e) {
-                        //That is ok. It was just an attempt.
-                        //e.printStackTrace();
-                    }
+                try {
+                    File temp = File.createTempFile("jfx", "." + fileExtension);
+                    temp.deleteOnExit();
+                    ImageIO.write(
+                        image.toBufferedImage(),
+                        fileExtension,
+                        temp);
+                    content.put(DataFormat.FILES, Arrays.asList(temp));
+                } catch (IOException | SecurityException e) {
+                    //That is ok. It was just an attempt.
+                    //e.printStackTrace();
                 }
             }
         }
@@ -403,143 +393,4 @@ public final class UIClientImpl implements UIClient {
         return accessor.getView() != null && content != null;
     }
 
-    private static int
-            getBestBufferedImageType(PixelFormat<?> fxFormat, BufferedImage bimg,
-                                     boolean isOpaque)
-    {
-        if (bimg != null) {
-            int bimgType = bimg.getType();
-            if (bimgType == BufferedImage.TYPE_INT_ARGB ||
-                bimgType == BufferedImage.TYPE_INT_ARGB_PRE ||
-                (isOpaque &&
-                     (bimgType == BufferedImage.TYPE_INT_BGR ||
-                      bimgType == BufferedImage.TYPE_INT_RGB)))
-            {
-                // We will allow the caller to give us a BufferedImage
-                // that has an alpha channel, but we might not otherwise
-                // construct one ourselves.
-                // We will also allow them to choose their own premultiply
-                // type which may not match the image.
-                // If left to our own devices we might choose a more specific
-                // format as indicated by the choices below.
-                return bimgType;
-            }
-        }
-        switch (fxFormat.getType()) {
-            default:
-            case BYTE_BGRA_PRE:
-            case INT_ARGB_PRE:
-                return BufferedImage.TYPE_INT_ARGB_PRE;
-            case BYTE_BGRA:
-            case INT_ARGB:
-                return BufferedImage.TYPE_INT_ARGB;
-            case BYTE_RGB:
-                return BufferedImage.TYPE_INT_RGB;
-            case BYTE_INDEXED:
-                return (fxFormat.isPremultiplied()
-                        ? BufferedImage.TYPE_INT_ARGB_PRE
-                        : BufferedImage.TYPE_INT_ARGB);
-        }
-    }
-
-    private static WritablePixelFormat<IntBuffer>
-        getAssociatedPixelFormat(BufferedImage bimg)
-    {
-        switch (bimg.getType()) {
-            // We lie here for xRGB, but we vetted that the src data was opaque
-            // so we can ignore the alpha.  We use ArgbPre instead of Argb
-            // just to get a loop that does not have divides in it if the
-            // PixelReader happens to not know the data is opaque.
-            case BufferedImage.TYPE_INT_RGB:
-            case BufferedImage.TYPE_INT_ARGB_PRE:
-                return PixelFormat.getIntArgbPreInstance();
-            case BufferedImage.TYPE_INT_ARGB:
-                return PixelFormat.getIntArgbInstance();
-            default:
-                // Should not happen...
-                throw new InternalError("Failed to validate BufferedImage type");
-        }
-    }
-
-    private static boolean checkFXImageOpaque(PixelReader pr, int iw, int ih) {
-        for (int x = 0; x < iw; x++) {
-            for (int y = 0; y < ih; y++) {
-                Color color = pr.getColor(x,y);
-                if (color.getOpacity() != 1.0) {
-                    return false;
-                }
-            }
-        }
-        return true;
-    }
-
-    private static BufferedImage fromFXImage(Image img, BufferedImage bimg) {
-        PixelReader pr = img.getPixelReader();
-        if (pr == null) {
-            return null;
-        }
-        int iw = (int) img.getWidth();
-        int ih = (int) img.getHeight();
-        PixelFormat<?> fxFormat = pr.getPixelFormat();
-        boolean srcPixelsAreOpaque = false;
-        switch (fxFormat.getType()) {
-            case INT_ARGB_PRE:
-            case INT_ARGB:
-            case BYTE_BGRA_PRE:
-            case BYTE_BGRA:
-                // Check fx image opacity only if
-                // supplied BufferedImage is without alpha channel
-                if (bimg != null &&
-                        (bimg.getType() == BufferedImage.TYPE_INT_BGR ||
-                         bimg.getType() == BufferedImage.TYPE_INT_RGB)) {
-                    srcPixelsAreOpaque = checkFXImageOpaque(pr, iw, ih);
-                }
-                break;
-            case BYTE_RGB:
-                srcPixelsAreOpaque = true;
-                break;
-        }
-        int prefBimgType = getBestBufferedImageType(pr.getPixelFormat(), bimg, srcPixelsAreOpaque);
-        if (bimg != null) {
-            int bw = bimg.getWidth();
-            int bh = bimg.getHeight();
-            if (bw < iw || bh < ih || bimg.getType() != prefBimgType) {
-                bimg = null;
-            } else if (iw < bw || ih < bh) {
-                Graphics2D g2d = bimg.createGraphics();
-                g2d.setComposite(AlphaComposite.Clear);
-                g2d.fillRect(0, 0, bw, bh);
-                g2d.dispose();
-            }
-        }
-        if (bimg == null) {
-            bimg = new BufferedImage(iw, ih, prefBimgType);
-        }
-        DataBufferInt db = (DataBufferInt)bimg.getRaster().getDataBuffer();
-        int data[] = db.getData();
-        int offset = bimg.getRaster().getDataBuffer().getOffset();
-        int scan =  0;
-        SampleModel sm = bimg.getRaster().getSampleModel();
-        if (sm instanceof SinglePixelPackedSampleModel) {
-            scan = ((SinglePixelPackedSampleModel)sm).getScanlineStride();
-        }
-
-        WritablePixelFormat<IntBuffer> pf = getAssociatedPixelFormat(bimg);
-        pr.getPixels(0, 0, iw, ih, pf, data, offset, scan);
-        return bimg;
-    }
-
-    // Method to implement the following via reflection:
-    //     SwingFXUtils.fromFXImage(img, null)
-    public static BufferedImage toBufferedImage(Image img) {
-        try {
-            return fromFXImage(img, null);
-        } catch (Exception ex) {
-            ex.printStackTrace(System.err);
-        }
-
-        // return null upon any exception
-        return null;
-    }
-
 }
