@@ -48,6 +48,7 @@ import com.sun.media.jfxmedia.track.Track;
 import com.sun.media.jfxmedia.track.VideoTrack;
 import com.sun.prism.Graphics;
 import com.sun.prism.Texture;
+import com.sun.prism.paint.Color;
 import com.sun.webkit.graphics.WCGraphicsContext;
 import com.sun.webkit.graphics.WCMediaPlayer;
 
@@ -355,7 +356,7 @@ final class WCMediaPlayerImpl extends WCMediaPlayer
             texture.unlock();
         } else {
             log.finest("  (Prism)renderImpl, texture is null, draw black rect");
-            gc.fillRect(x, y, w, h, 0xFF000000);
+            gc.fillRect(x, y, w, h, Color.BLACK);
         }
         log.finer("<<(Prism)renderImpl");
     }
