@@ -633,12 +633,16 @@ public final class WebPage {
                 return;
             }
             updateDirty(toPaint);
-
+            updateRendering();
         } finally {
             unlockPage();
         }
     }
 
+    public void updateRendering() {
+        twkUpdateRendering(getPage());
+    }
+
     public int getUpdateContentCycleID() {
         return updateContentCycleID;
     }
@@ -1155,6 +1159,12 @@ public final class WebPage {
         }
     }
 
+    // DRT support
+    public void forceRepaint() {
+        repaintAll();
+        updateContent(new WCRectangle(0, 0, width, height));
+    }
+
     public String getContentType(long frameID) {
         lockPage();
         try {
@@ -2597,6 +2607,7 @@ public final class WebPage {
     private native void twkSetBounds(long pPage, int x, int y, int w, int h);
     private native void twkPrePaint(long pPage);
     private native void twkUpdateContent(long pPage, WCRenderQueue rq, int x, int y, int w, int h);
+    private native void twkUpdateRendering(long pPage);
     private native void twkPostPaint(long pPage, WCRenderQueue rq,
                                      int x, int y, int w, int h);
 
