@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,7 +39,9 @@
 #include <wtf/Function.h>
 #include <wtf/Lock.h>
 #include <wtf/NumberOfCores.h>
+#include <wtf/PtrTag.h>
 #include <wtf/Threading.h>
+#include <wtf/text/StringCommon.h>
 
 // We don't have a NO_RETURN_DUE_TO_EXIT, nor should we. That's ridiculous.
 static bool hiddenTruthBecauseNoReturnIsStupid() { return true; }
@@ -108,6 +110,15 @@ template<typename T> T nextID(T id) { return static_cast<T>(id + 1); }
         CRASH();                                                        \
     } while (false)
 
+#define CHECK_NOT_EQ(_actual, _expected) do {                               \
+        if ((_actual) != (_expected))                                   \
+            break;                                                      \
+        crashLock.lock();                                               \
+        dataLog("FAILED while testing " #_actual ": expected not: ", _expected, ", actual: ", _actual, "\n"); \
+        WTFReportAssertionFailure(__FILE__, __LINE__, WTF_PRETTY_FUNCTION, "CHECK_NOT_EQ("#_actual ", " #_expected ")"); \
+        CRASH();                                                        \
+    } while (false)
+
 #if ENABLE(MASM_PROBE)
 bool isPC(MacroAssembler::RegisterID id)
 {
@@ -166,12 +177,32 @@ T compileAndRun(Generator&& generator, Arguments... arguments)
     return invoke<T>(compile(WTFMove(generator)), arguments...);
 }
 
+void emitFunctionPrologue(CCallHelpers& jit)
+{
+    jit.emitFunctionPrologue();
+#if CPU(ARM_THUMB2)
+    // MacroAssemblerARMv7 uses r6 as a temporary register, which is a
+    // callee-saved register, see 5.1.1 of the Procedure Call Standard for
+    // the ARM Architecture.
+    // http://infocenter.arm.com/help/topic/com.arm.doc.ihi0042f/IHI0042F_aapcs.pdf
+    jit.push(ARMRegisters::r6);
+#endif
+}
+
+void emitFunctionEpilogue(CCallHelpers& jit)
+{
+#if CPU(ARM_THUMB2)
+    jit.pop(ARMRegisters::r6);
+#endif
+    jit.emitFunctionEpilogue();
+}
+
 void testSimple()
 {
     CHECK_EQ(compileAndRun<int>([] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
         jit.move(CCallHelpers::TrustedImm32(42), GPRInfo::returnValueGPR);
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     }), 42);
 }
@@ -179,11 +210,11 @@ void testSimple()
 void testGetEffectiveAddress(size_t pointer, ptrdiff_t length, int32_t offset, CCallHelpers::Scale scale)
 {
     CHECK_EQ(compileAndRun<size_t>([=] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
         jit.move(CCallHelpers::TrustedImmPtr(bitwise_cast<void*>(pointer)), GPRInfo::regT0);
         jit.move(CCallHelpers::TrustedImmPtr(bitwise_cast<void*>(length)), GPRInfo::regT1);
         jit.getEffectiveAddress(CCallHelpers::BaseIndex(GPRInfo::regT0, GPRInfo::regT1, scale, offset), GPRInfo::returnValueGPR);
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     }), pointer + offset + (static_cast<size_t>(1) << static_cast<int>(scale)) * length);
 }
@@ -199,7 +230,7 @@ void testBranchTruncateDoubleToInt32(double val, int32_t expected)
     const bool isBigEndian = false;
 #endif
     CHECK_EQ(compileAndRun<int>([&] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
         jit.subPtr(CCallHelpers::TrustedImm32(stackAlignmentBytes()), MacroAssembler::stackPointerRegister);
         if (isBigEndian) {
             jit.store32(CCallHelpers::TrustedImm32(valAsUInt >> 32),
@@ -221,7 +252,7 @@ void testBranchTruncateDoubleToInt32(double val, int32_t expected)
 
         done.link(&jit);
         jit.addPtr(CCallHelpers::TrustedImm32(stackAlignmentBytes()), MacroAssembler::stackPointerRegister);
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     }), expected);
 }
@@ -282,25 +313,180 @@ static Vector<int32_t> int32Operands()
     };
 }
 
+#if CPU(X86_64)
+static Vector<int64_t> int64Operands()
+{
+    return Vector<int64_t> {
+        0,
+        1,
+        -1,
+        2,
+        -2,
+        42,
+        -42,
+        64,
+        std::numeric_limits<int32_t>::max(),
+        std::numeric_limits<int32_t>::min(),
+        std::numeric_limits<int64_t>::max(),
+        std::numeric_limits<int64_t>::min(),
+    };
+}
+#endif
+
+#if CPU(X86_64)
+void testBranchTestBit32RegReg()
+{
+    for (auto value : int32Operands()) {
+        auto test = compile([=] (CCallHelpers& jit) {
+            emitFunctionPrologue(jit);
+
+            auto branch = jit.branchTestBit32(MacroAssembler::NonZero, GPRInfo::argumentGPR0, GPRInfo::argumentGPR1);
+            jit.move(CCallHelpers::TrustedImm32(0), GPRInfo::returnValueGPR);
+            auto done = jit.jump();
+            branch.link(&jit);
+            jit.move(CCallHelpers::TrustedImm32(1), GPRInfo::returnValueGPR);
+            done.link(&jit);
+
+            emitFunctionEpilogue(jit);
+            jit.ret();
+        });
+
+        for (auto value2 : int32Operands())
+            CHECK_EQ(invoke<int>(test, value, value2), (value>>(value2%32))&1);
+    }
+}
+
+void testBranchTestBit32RegImm()
+{
+    for (auto value : int32Operands()) {
+        auto test = compile([=] (CCallHelpers& jit) {
+            emitFunctionPrologue(jit);
+
+            auto branch = jit.branchTestBit32(MacroAssembler::NonZero, GPRInfo::argumentGPR0, CCallHelpers::TrustedImm32(value));
+            jit.move(CCallHelpers::TrustedImm32(0), GPRInfo::returnValueGPR);
+            auto done = jit.jump();
+            branch.link(&jit);
+            jit.move(CCallHelpers::TrustedImm32(1), GPRInfo::returnValueGPR);
+            done.link(&jit);
+
+            emitFunctionEpilogue(jit);
+            jit.ret();
+        });
+
+        for (auto value2 : int32Operands())
+            CHECK_EQ(invoke<int>(test, value2), (value2>>(value%32))&1);
+    }
+}
+
+void testBranchTestBit32AddrImm()
+{
+    for (auto value : int32Operands()) {
+        auto test = compile([=] (CCallHelpers& jit) {
+            emitFunctionPrologue(jit);
+
+            auto branch = jit.branchTestBit32(MacroAssembler::NonZero, MacroAssembler::Address(GPRInfo::argumentGPR0, 0), CCallHelpers::TrustedImm32(value));
+            jit.move(CCallHelpers::TrustedImm32(0), GPRInfo::returnValueGPR);
+            auto done = jit.jump();
+            branch.link(&jit);
+            jit.move(CCallHelpers::TrustedImm32(1), GPRInfo::returnValueGPR);
+            done.link(&jit);
+
+            emitFunctionEpilogue(jit);
+            jit.ret();
+        });
+
+        for (auto value2 : int32Operands())
+            CHECK_EQ(invoke<int>(test, &value2), (value2>>(value%32))&1);
+    }
+}
+
+void testBranchTestBit64RegReg()
+{
+    for (auto value : int64Operands()) {
+        auto test = compile([=] (CCallHelpers& jit) {
+            emitFunctionPrologue(jit);
+
+            auto branch = jit.branchTestBit64(MacroAssembler::NonZero, GPRInfo::argumentGPR0, GPRInfo::argumentGPR1);
+            jit.move(CCallHelpers::TrustedImm64(0), GPRInfo::returnValueGPR);
+            auto done = jit.jump();
+            branch.link(&jit);
+            jit.move(CCallHelpers::TrustedImm64(1), GPRInfo::returnValueGPR);
+            done.link(&jit);
+
+            emitFunctionEpilogue(jit);
+            jit.ret();
+        });
+
+        for (auto value2 : int64Operands())
+            CHECK_EQ(invoke<long int>(test, value, value2), (value>>(value2%64))&1);
+    }
+}
+
+void testBranchTestBit64RegImm()
+{
+    for (auto value : int64Operands()) {
+        auto test = compile([=] (CCallHelpers& jit) {
+            emitFunctionPrologue(jit);
+
+            auto branch = jit.branchTestBit64(MacroAssembler::NonZero, GPRInfo::argumentGPR0, CCallHelpers::TrustedImm32(value));
+            jit.move(CCallHelpers::TrustedImm64(0), GPRInfo::returnValueGPR);
+            auto done = jit.jump();
+            branch.link(&jit);
+            jit.move(CCallHelpers::TrustedImm64(1), GPRInfo::returnValueGPR);
+            done.link(&jit);
+
+            emitFunctionEpilogue(jit);
+            jit.ret();
+        });
+
+        for (auto value2 : int64Operands())
+            CHECK_EQ(invoke<long int>(test, value2), (value2>>(value%64))&1);
+    }
+}
+
+void testBranchTestBit64AddrImm()
+{
+    for (auto value : int64Operands()) {
+        auto test = compile([=] (CCallHelpers& jit) {
+            emitFunctionPrologue(jit);
+
+            auto branch = jit.branchTestBit64(MacroAssembler::NonZero, MacroAssembler::Address(GPRInfo::argumentGPR0, 0), CCallHelpers::TrustedImm32(value));
+            jit.move(CCallHelpers::TrustedImm64(0), GPRInfo::returnValueGPR);
+            auto done = jit.jump();
+            branch.link(&jit);
+            jit.move(CCallHelpers::TrustedImm64(1), GPRInfo::returnValueGPR);
+            done.link(&jit);
+
+            emitFunctionEpilogue(jit);
+            jit.ret();
+        });
+
+        for (auto value2 : int64Operands())
+            CHECK_EQ(invoke<long int>(test, &value2), (value2>>(value%64))&1);
+    }
+}
+
+#endif
+
 void testCompareDouble(MacroAssembler::DoubleCondition condition)
 {
     double arg1 = 0;
     double arg2 = 0;
 
     auto compareDouble = compile([&, condition] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
 
         jit.loadDouble(CCallHelpers::TrustedImmPtr(&arg1), FPRInfo::fpRegT0);
         jit.loadDouble(CCallHelpers::TrustedImmPtr(&arg2), FPRInfo::fpRegT1);
         jit.move(CCallHelpers::TrustedImm32(-1), GPRInfo::returnValueGPR);
         jit.compareDouble(condition, FPRInfo::fpRegT0, FPRInfo::fpRegT1, GPRInfo::returnValueGPR);
 
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
 
     auto compareDoubleGeneric = compile([&, condition] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
 
         jit.loadDouble(CCallHelpers::TrustedImmPtr(&arg1), FPRInfo::fpRegT0);
         jit.loadDouble(CCallHelpers::TrustedImmPtr(&arg2), FPRInfo::fpRegT1);
@@ -309,7 +495,7 @@ void testCompareDouble(MacroAssembler::DoubleCondition condition)
         jit.move(CCallHelpers::TrustedImm32(0), GPRInfo::returnValueGPR);
         jump.link(&jit);
 
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
 
@@ -327,11 +513,11 @@ void testMul32WithImmediates()
 {
     for (auto immediate : int32Operands()) {
         auto mul = compile([=] (CCallHelpers& jit) {
-            jit.emitFunctionPrologue();
+            emitFunctionPrologue(jit);
 
             jit.mul32(CCallHelpers::TrustedImm32(immediate), GPRInfo::argumentGPR0, GPRInfo::returnValueGPR);
 
-            jit.emitFunctionEpilogue();
+            emitFunctionEpilogue(jit);
             jit.ret();
         });
 
@@ -340,6 +526,25 @@ void testMul32WithImmediates()
     }
 }
 
+#if CPU(ARM64)
+void testMul32SignExtend()
+{
+    for (auto value : int32Operands()) {
+        auto mul = compile([=] (CCallHelpers& jit) {
+            emitFunctionPrologue(jit);
+
+            jit.multiplySignExtend32(GPRInfo::argumentGPR0, GPRInfo::argumentGPR1, GPRInfo::returnValueGPR);
+
+            emitFunctionEpilogue(jit);
+            jit.ret();
+        });
+
+        for (auto value2 : int32Operands())
+            CHECK_EQ(invoke<long int>(mul, value, value2), ((long int) value) * ((long int) value2));
+    }
+}
+#endif
+
 #if CPU(X86) || CPU(X86_64) || CPU(ARM64)
 void testCompareFloat(MacroAssembler::DoubleCondition condition)
 {
@@ -347,19 +552,19 @@ void testCompareFloat(MacroAssembler::DoubleCondition condition)
     float arg2 = 0;
 
     auto compareFloat = compile([&, condition] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
 
         jit.loadFloat(CCallHelpers::TrustedImmPtr(&arg1), FPRInfo::fpRegT0);
         jit.loadFloat(CCallHelpers::TrustedImmPtr(&arg2), FPRInfo::fpRegT1);
         jit.move(CCallHelpers::TrustedImm32(-1), GPRInfo::returnValueGPR);
         jit.compareFloat(condition, FPRInfo::fpRegT0, FPRInfo::fpRegT1, GPRInfo::returnValueGPR);
 
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
 
     auto compareFloatGeneric = compile([&, condition] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
 
         jit.loadFloat(CCallHelpers::TrustedImmPtr(&arg1), FPRInfo::fpRegT0);
         jit.loadFloat(CCallHelpers::TrustedImmPtr(&arg2), FPRInfo::fpRegT1);
@@ -368,7 +573,7 @@ void testCompareFloat(MacroAssembler::DoubleCondition condition)
         jit.move(CCallHelpers::TrustedImm32(0), GPRInfo::returnValueGPR);
         jump.link(&jit);
 
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
 
@@ -388,7 +593,7 @@ void testProbeReadsArgumentRegisters()
 {
     bool probeWasCalled = false;
     compileAndRun<void>([&] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
 
         jit.pushPair(GPRInfo::argumentGPR0, GPRInfo::argumentGPR1);
         jit.pushPair(GPRInfo::argumentGPR2, GPRInfo::argumentGPR3);
@@ -424,7 +629,7 @@ void testProbeReadsArgumentRegisters()
         jit.popPair(GPRInfo::argumentGPR2, GPRInfo::argumentGPR3);
         jit.popPair(GPRInfo::argumentGPR0, GPRInfo::argumentGPR1);
 
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
     CHECK_EQ(probeWasCalled, true);
@@ -437,7 +642,7 @@ void testProbeWritesArgumentRegisters()
     // that our writes did take effect.
     unsigned probeCallCount = 0;
     compileAndRun<void>([&] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
 
         jit.pushPair(GPRInfo::argumentGPR0, GPRInfo::argumentGPR1);
         jit.pushPair(GPRInfo::argumentGPR2, GPRInfo::argumentGPR3);
@@ -486,7 +691,7 @@ void testProbeWritesArgumentRegisters()
         jit.popPair(GPRInfo::argumentGPR2, GPRInfo::argumentGPR3);
         jit.popPair(GPRInfo::argumentGPR0, GPRInfo::argumentGPR1);
 
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
     CHECK_EQ(probeCallCount, 2);
@@ -514,7 +719,7 @@ void testProbePreservesGPRS()
     CPUState originalState;
 
     compileAndRun<void>([&] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
 
         // Write expected values into the registers (except for sp, fp, and pc).
         jit.probe([&] (Probe::Context& context) {
@@ -588,7 +793,7 @@ void testProbePreservesGPRS()
                 CHECK_EQ(cpu.fpr<uint64_t>(id), originalState.fpr<uint64_t>(id));
         });
 
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
     CHECK_EQ(probeCallCount, 5);
@@ -616,7 +821,7 @@ void testProbeModifiesStackPointer(WTF::Function<void*(Probe::Context&)> compute
 #endif
 
     compileAndRun<void>([&] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
 
         // Preserve original stack pointer and modify the sp, and
         // write expected values into other registers (except for fp, and pc).
@@ -706,7 +911,7 @@ void testProbeModifiesStackPointer(WTF::Function<void*(Probe::Context&)> compute
             CHECK_EQ(cpu.sp(), originalSP);
         });
 
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
     CHECK_EQ(probeCallCount, 4);
@@ -756,12 +961,12 @@ void testProbeModifiesProgramCounter()
             continuationWasReached = true;
         });
 
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
 
     compileAndRun<void>([&] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
 
         // Write expected values into the registers.
         jit.probe([&] (Probe::Context& context) {
@@ -798,7 +1003,7 @@ void testProbeModifiesStackValues()
 #endif
 
     compileAndRun<void>([&] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
 
         // Write expected values into the registers.
         jit.probe([&] (Probe::Context& context) {
@@ -891,7 +1096,7 @@ void testProbeModifiesStackValues()
             cpu.sp() = originalSP;
         });
 
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
 
@@ -899,34 +1104,69 @@ void testProbeModifiesStackValues()
 }
 #endif // ENABLE(MASM_PROBE)
 
+void testOrImmMem()
+{
+    // FIXME: this does not test that the or does not touch beyond its width.
+    // I am not sure how to do such a test without a lot of complexity (running multiple threads, with a race on the high bits of the memory location).
+    uint64_t memoryLocation = 0x12341234;
+    auto or32 = compile([&] (CCallHelpers& jit) {
+        emitFunctionPrologue(jit);
+        jit.or32(CCallHelpers::TrustedImm32(42), CCallHelpers::AbsoluteAddress(&memoryLocation));
+        emitFunctionEpilogue(jit);
+        jit.ret();
+    });
+    invoke<void>(or32);
+    CHECK_EQ(memoryLocation, 0x12341234 | 42);
+
+    memoryLocation = 0x12341234;
+    auto or16 = compile([&] (CCallHelpers& jit) {
+        emitFunctionPrologue(jit);
+        jit.or16(CCallHelpers::TrustedImm32(42), CCallHelpers::AbsoluteAddress(&memoryLocation));
+        emitFunctionEpilogue(jit);
+        jit.ret();
+    });
+    invoke<void>(or16);
+    CHECK_EQ(memoryLocation, 0x12341234 | 42);
+
+    memoryLocation = 0x12341234;
+    auto or16InvalidLogicalImmInARM64 = compile([&] (CCallHelpers& jit) {
+        emitFunctionPrologue(jit);
+        jit.or16(CCallHelpers::TrustedImm32(0), CCallHelpers::AbsoluteAddress(&memoryLocation));
+        emitFunctionEpilogue(jit);
+        jit.ret();
+    });
+    invoke<void>(or16InvalidLogicalImmInARM64);
+    CHECK_EQ(memoryLocation, 0x12341234);
+}
+
 void testByteSwap()
 {
 #if CPU(X86_64) || CPU(ARM64)
     auto byteSwap16 = compile([] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
         jit.move(GPRInfo::argumentGPR0, GPRInfo::returnValueGPR);
         jit.byteSwap16(GPRInfo::returnValueGPR);
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
     CHECK_EQ(invoke<uint64_t>(byteSwap16, 0xaabbccddee001122), static_cast<uint64_t>(0x2211));
     CHECK_EQ(invoke<uint64_t>(byteSwap16, 0xaabbccddee00ffaa), static_cast<uint64_t>(0xaaff));
 
     auto byteSwap32 = compile([] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
         jit.move(GPRInfo::argumentGPR0, GPRInfo::returnValueGPR);
         jit.byteSwap32(GPRInfo::returnValueGPR);
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
     CHECK_EQ(invoke<uint64_t>(byteSwap32, 0xaabbccddee001122), static_cast<uint64_t>(0x221100ee));
     CHECK_EQ(invoke<uint64_t>(byteSwap32, 0xaabbccddee00ffaa), static_cast<uint64_t>(0xaaff00ee));
 
     auto byteSwap64 = compile([] (CCallHelpers& jit) {
-        jit.emitFunctionPrologue();
+        emitFunctionPrologue(jit);
         jit.move(GPRInfo::argumentGPR0, GPRInfo::returnValueGPR);
         jit.byteSwap64(GPRInfo::returnValueGPR);
-        jit.emitFunctionEpilogue();
+        emitFunctionEpilogue(jit);
         jit.ret();
     });
     CHECK_EQ(invoke<uint64_t>(byteSwap64, 0xaabbccddee001122), static_cast<uint64_t>(0x221100eeddccbbaa));
@@ -934,6 +1174,127 @@ void testByteSwap()
 #endif
 }
 
+void testMoveDoubleConditionally32()
+{
+#if CPU(X86_64) | CPU(ARM64)
+    double arg1 = 0;
+    double arg2 = 0;
+    const double zero = -0;
+
+    const double chosenDouble = 6.00000059604644775390625;
+    CHECK_EQ(static_cast<double>(static_cast<float>(chosenDouble)) == chosenDouble, false);
+
+    auto sel = compile([&] (CCallHelpers& jit) {
+        emitFunctionPrologue(jit);
+        jit.loadDouble(CCallHelpers::TrustedImmPtr(&zero), FPRInfo::returnValueFPR);
+        jit.loadDouble(CCallHelpers::TrustedImmPtr(&arg1), FPRInfo::fpRegT1);
+        jit.loadDouble(CCallHelpers::TrustedImmPtr(&arg2), FPRInfo::fpRegT2);
+
+        jit.move(MacroAssembler::TrustedImm32(-1), GPRInfo::regT0);
+        jit.moveDoubleConditionally32(MacroAssembler::Equal, GPRInfo::regT0, GPRInfo::regT0, FPRInfo::fpRegT1, FPRInfo::fpRegT2, FPRInfo::returnValueFPR);
+
+        emitFunctionEpilogue(jit);
+        jit.ret();
+    });
+
+    arg1 = chosenDouble;
+    arg2 = 43;
+    CHECK_EQ(invoke<double>(sel), chosenDouble);
+
+    arg1 = 43;
+    arg2 = chosenDouble;
+    CHECK_EQ(invoke<double>(sel), 43.0);
+
+#endif
+}
+
+void testMoveDoubleConditionally64()
+{
+#if CPU(X86_64) | CPU(ARM64)
+    double arg1 = 0;
+    double arg2 = 0;
+    const double zero = -0;
+
+    const double chosenDouble = 6.00000059604644775390625;
+    CHECK_EQ(static_cast<double>(static_cast<float>(chosenDouble)) == chosenDouble, false);
+
+    auto sel = compile([&] (CCallHelpers& jit) {
+        emitFunctionPrologue(jit);
+        jit.loadDouble(CCallHelpers::TrustedImmPtr(&zero), FPRInfo::returnValueFPR);
+        jit.loadDouble(CCallHelpers::TrustedImmPtr(&arg1), FPRInfo::fpRegT1);
+        jit.loadDouble(CCallHelpers::TrustedImmPtr(&arg2), FPRInfo::fpRegT2);
+
+        jit.move(MacroAssembler::TrustedImm64(-1), GPRInfo::regT0);
+        jit.moveDoubleConditionally64(MacroAssembler::Equal, GPRInfo::regT0, GPRInfo::regT0, FPRInfo::fpRegT1, FPRInfo::fpRegT2, FPRInfo::returnValueFPR);
+
+        emitFunctionEpilogue(jit);
+        jit.ret();
+    });
+
+    arg1 = chosenDouble;
+    arg2 = 43;
+    CHECK_EQ(invoke<double>(sel), chosenDouble);
+
+    arg1 = 43;
+    arg2 = chosenDouble;
+    CHECK_EQ(invoke<double>(sel), 43.0);
+
+#endif
+}
+
+static void testCagePreservesPACFailureBit()
+{
+#if GIGACAGE_ENABLED
+    // Placate ASan builds and any environments that disables the Gigacage.
+    if (!Gigacage::shouldBeEnabled())
+        return;
+
+    RELEASE_ASSERT(!Gigacage::isDisablingPrimitiveGigacageForbidden());
+    auto cage = compile([] (CCallHelpers& jit) {
+        emitFunctionPrologue(jit);
+        jit.cageConditionally(Gigacage::Primitive, GPRInfo::argumentGPR0, GPRInfo::argumentGPR1, GPRInfo::argumentGPR2);
+        jit.move(GPRInfo::argumentGPR0, GPRInfo::returnValueGPR);
+        emitFunctionEpilogue(jit);
+        jit.ret();
+    });
+
+    void* ptr = Gigacage::tryMalloc(Gigacage::Primitive, 1);
+    void* taggedPtr = tagArrayPtr(ptr, 1);
+    RELEASE_ASSERT(hasOneBitSet(Gigacage::size(Gigacage::Primitive) << 2));
+    void* notCagedPtr = reinterpret_cast<void*>(reinterpret_cast<uintptr_t>(ptr) + (Gigacage::size(Gigacage::Primitive) << 2));
+    CHECK_NOT_EQ(Gigacage::caged(Gigacage::Primitive, notCagedPtr), notCagedPtr);
+    void* taggedNotCagedPtr = tagArrayPtr(notCagedPtr, 1);
+
+    if (isARM64E()) {
+        // FIXME: This won't work if authentication failures trap but I don't know how to test for that right now.
+        CHECK_NOT_EQ(invoke<void*>(cage, taggedPtr, 2), ptr);
+        CHECK_EQ(invoke<void*>(cage, taggedNotCagedPtr, 1), untagArrayPtr(taggedPtr, 2));
+    } else
+        CHECK_EQ(invoke<void*>(cage, taggedPtr, 2), ptr);
+
+    CHECK_EQ(invoke<void*>(cage, taggedPtr, 1), ptr);
+
+    auto cageWithoutAuthentication = compile([] (CCallHelpers& jit) {
+        emitFunctionPrologue(jit);
+        jit.cageWithoutUntagging(Gigacage::Primitive, GPRInfo::argumentGPR0);
+        jit.move(GPRInfo::argumentGPR0, GPRInfo::returnValueGPR);
+        emitFunctionEpilogue(jit);
+        jit.ret();
+    });
+
+    CHECK_EQ(invoke<void*>(cageWithoutAuthentication, taggedPtr), taggedPtr);
+    if (isARM64E()) {
+        // FIXME: This won't work if authentication failures trap but I don't know how to test for that right now.
+        CHECK_NOT_EQ(invoke<void*>(cageWithoutAuthentication, taggedNotCagedPtr), taggedNotCagedPtr);
+        CHECK_NOT_EQ(untagArrayPtr(invoke<void*>(cageWithoutAuthentication, taggedNotCagedPtr), 1), notCagedPtr);
+        CHECK_NOT_EQ(invoke<void*>(cageWithoutAuthentication, taggedNotCagedPtr), taggedPtr);
+        CHECK_NOT_EQ(untagArrayPtr(invoke<void*>(cageWithoutAuthentication, taggedNotCagedPtr), 1), ptr);
+    }
+
+    Gigacage::free(Gigacage::Primitive, ptr);
+#endif
+}
+
 #define RUN(test) do {                          \
         if (!shouldRun(#test))                  \
             break;                              \
@@ -955,11 +1316,7 @@ void run(const char* filter)
     Deque<RefPtr<SharedTask<void()>>> tasks;
 
     auto shouldRun = [&] (const char* testName) -> bool {
-#if OS(UNIX)
-        return !filter || !!strcasestr(testName, filter);
-#else
-        return !filter || !!strstr(testName, filter);
-#endif
+        return !filter || WTF::findIgnoringASCIICaseWithoutLength(testName, filter) != WTF::notFound;
     };
 
     RUN(testSimple());
@@ -994,6 +1351,19 @@ void run(const char* filter)
     RUN(testCompareDouble(MacroAssembler::DoubleLessThanOrEqualOrUnordered));
     RUN(testMul32WithImmediates());
 
+#if CPU(X86_64)
+    RUN(testBranchTestBit32RegReg());
+    RUN(testBranchTestBit32RegImm());
+    RUN(testBranchTestBit32AddrImm());
+    RUN(testBranchTestBit64RegReg());
+    RUN(testBranchTestBit64RegImm());
+    RUN(testBranchTestBit64AddrImm());
+#endif
+
+#if CPU(ARM64)
+    RUN(testMul32SignExtend());
+#endif
+
 #if CPU(X86) || CPU(X86_64) || CPU(ARM64)
     RUN(testCompareFloat(MacroAssembler::DoubleEqual));
     RUN(testCompareFloat(MacroAssembler::DoubleNotEqual));
@@ -1020,6 +1390,12 @@ void run(const char* filter)
 #endif // ENABLE(MASM_PROBE)
 
     RUN(testByteSwap());
+    RUN(testMoveDoubleConditionally32());
+    RUN(testMoveDoubleConditionally64());
+
+    RUN(testCagePreservesPACFailureBit());
+
+    RUN(testOrImmMem());
 
     if (tasks.isEmpty())
         usage();
