@@ -28,11 +28,11 @@
 
 #if ENABLE(B3_JIT)
 
-#include "AirCode.h"
 #include "AirGenerationContext.h"
 #include "AirInstInlines.h"
 #include "B3StackmapGenerationParams.h"
 #include "B3ValueInlines.h"
+#include "CCallHelpers.h"
 
 namespace JSC { namespace B3 {
 
