@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,9 +28,7 @@
 
 #if ENABLE(B3_JIT)
 
-#include "B3BasicBlockInlines.h"
 #include "B3BreakCriticalEdges.h"
-#include "B3Dominators.h"
 #include "B3InsertionSetInlines.h"
 #include "B3PhaseScope.h"
 #include "B3ProcedureInlines.h"
@@ -49,7 +47,7 @@ namespace JSC { namespace B3 {
 namespace {
 
 namespace B3FixSSAInternal {
-static const bool verbose = false;
+static constexpr bool verbose = false;
 }
 
 void killDeadVariables(Procedure& proc)
