@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2015-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -40,7 +40,7 @@ namespace JSC { namespace B3 {
 namespace {
 
 namespace B3ReduceDoubleToFloatInternal {
-static const bool verbose = false;
+static constexpr bool verbose = false;
 }
 bool printRemainingConversions = false;
 
