@@ -29,6 +29,8 @@
 
 #if ENABLE(B3_JIT)
 
+#include "B3ValueInlines.h"
+
 namespace JSC { namespace B3 {
 
 WasmBoundsCheckValue::~WasmBoundsCheckValue()
@@ -36,7 +38,7 @@ WasmBoundsCheckValue::~WasmBoundsCheckValue()
 }
 
 WasmBoundsCheckValue::WasmBoundsCheckValue(Origin origin, GPRReg pinnedSize, Value* ptr, unsigned offset)
-    : Value(CheckedOpcode, WasmBoundsCheck, origin, ptr)
+    : Value(CheckedOpcode, WasmBoundsCheck, One, origin, ptr)
     , m_offset(offset)
     , m_boundsType(Type::Pinned)
 {
@@ -44,7 +46,7 @@ WasmBoundsCheckValue::WasmBoundsCheckValue(Origin origin, GPRReg pinnedSize, Val
 }
 
 WasmBoundsCheckValue::WasmBoundsCheckValue(Origin origin, Value* ptr, unsigned offset, size_t maximum)
-    : Value(CheckedOpcode, WasmBoundsCheck, origin, ptr)
+    : Value(CheckedOpcode, WasmBoundsCheck, One, origin, ptr)
     , m_offset(offset)
     , m_boundsType(Type::Maximum)
 {
@@ -55,11 +57,6 @@ WasmBoundsCheckValue::WasmBoundsCheckValue(Origin origin, Value* ptr, unsigned o
     m_bounds.maximum = maximum;
 }
 
-Value* WasmBoundsCheckValue::cloneImpl() const
-{
-    return new WasmBoundsCheckValue(*this);
-}
-
 void WasmBoundsCheckValue::dumpMeta(CommaPrinter& comma, PrintStream& out) const
 {
     switch (m_boundsType) {
