@@ -28,6 +28,7 @@
 
 #if ENABLE(B3_JIT)
 
+#include "AirArgInlines.h"
 #include "AirBlockInsertionSet.h"
 #include "AirCode.h"
 #include "AirHandleCalleeSaves.h"
@@ -44,6 +45,35 @@ GenerateAndAllocateRegisters::GenerateAndAllocateRegisters(Code& code)
     , m_map(code)
 { }
 
+ALWAYS_INLINE void GenerateAndAllocateRegisters::checkConsistency()
+{
+    // This isn't exactly the right option for this but adding a new one for just this seems silly.
+    if (Options::validateGraph() || Options::validateGraphAtEachPhase()) {
+        m_code.forEachTmp([&] (Tmp tmp) {
+            Reg reg = m_map[tmp].reg;
+            if (!reg)
+                return;
+
+            ASSERT(!m_availableRegs[tmp.bank()].contains(reg));
+            ASSERT(m_currentAllocation->at(reg) == tmp);
+        });
+
+        for (Reg reg : RegisterSet::allRegisters()) {
+            if (isDisallowedRegister(reg))
+                continue;
+
+            Tmp tmp = m_currentAllocation->at(reg);
+            if (!tmp) {
+                ASSERT(m_availableRegs[bankForReg(reg)].contains(reg));
+                continue;
+            }
+
+            ASSERT(!m_availableRegs[tmp.bank()].contains(reg));
+            ASSERT(m_map[tmp].reg == reg);
+        }
+    }
+}
+
 void GenerateAndAllocateRegisters::buildLiveRanges(UnifiedTmpLiveness& liveness)
 {
     m_liveRangeEnd = TmpMap<size_t>(m_code, 0);
@@ -54,6 +84,7 @@ void GenerateAndAllocateRegisters::buildLiveRanges(UnifiedTmpLiveness& liveness)
             if (!tmp.isReg())
                 m_liveRangeEnd[tmp] = m_globalInstIndex;
         }
+        ++m_globalInstIndex;
         for (Inst& inst : *block) {
             inst.forEachTmpFast([&] (Tmp tmp) {
                 if (!tmp.isReg())
@@ -65,6 +96,7 @@ void GenerateAndAllocateRegisters::buildLiveRanges(UnifiedTmpLiveness& liveness)
             if (!tmp.isReg())
                 m_liveRangeEnd[tmp] = m_globalInstIndex;
         }
+        ++m_globalInstIndex;
     }
 }
 
@@ -122,6 +154,18 @@ static ALWAYS_INLINE CCallHelpers::Address callFrameAddr(CCallHelpers& jit, intp
     return CCallHelpers::Address(reg);
 }
 
+ALWAYS_INLINE void GenerateAndAllocateRegisters::release(Tmp tmp, Reg reg)
+{
+    ASSERT(reg);
+    ASSERT(m_currentAllocation->at(reg) == tmp);
+    m_currentAllocation->at(reg) = Tmp();
+    ASSERT(!m_availableRegs[tmp.bank()].contains(reg));
+    m_availableRegs[tmp.bank()].set(reg);
+    ASSERT(m_map[tmp].reg == reg);
+    m_map[tmp].reg = Reg();
+}
+
+
 ALWAYS_INLINE void GenerateAndAllocateRegisters::flush(Tmp tmp, Reg reg)
 {
     ASSERT(tmp);
@@ -136,10 +180,8 @@ ALWAYS_INLINE void GenerateAndAllocateRegisters::spill(Tmp tmp, Reg reg)
 {
     ASSERT(reg);
     ASSERT(m_map[tmp].reg == reg);
-    m_availableRegs[tmp.bank()].set(reg);
-    m_currentAllocation->at(reg) = Tmp();
     flush(tmp, reg);
-    m_map[tmp].reg = Reg();
+    release(tmp, reg);
 }
 
 ALWAYS_INLINE void GenerateAndAllocateRegisters::alloc(Tmp tmp, Reg reg, bool isDef)
@@ -179,10 +221,7 @@ ALWAYS_INLINE void GenerateAndAllocateRegisters::freeDeadTmpsIfNeeded()
         if (m_liveRangeEnd[tmp] >= m_globalInstIndex)
             continue;
 
-        Reg reg = Reg::fromIndex(i);
-        m_map[tmp].reg = Reg();
-        m_availableRegs[tmp.bank()].set(reg);
-        m_currentAllocation->at(i) = Tmp();
+        release(tmp, Reg::fromIndex(i));
     }
 }
 
@@ -243,21 +282,74 @@ void GenerateAndAllocateRegisters::prepareForGeneration()
     handleCalleeSaves(m_code, RegisterSet::calleeSaveRegisters());
     allocateEscapedStackSlots(m_code);
 
-    // Each Tmp gets its own stack slot.
-    auto createStackSlot = [&] (const Tmp& tmp) {
-        TmpData data;
-        data.spillSlot = m_code.addStackSlot(8, StackSlotKind::Spill);
-        data.reg = Reg();
-        m_map[tmp] = data;
-#if !ASSERT_DISABLED
-        m_allTmps[tmp.bank()].append(tmp);
-#endif
-    };
+    insertBlocksForFlushAfterTerminalPatchpoints();
 
+#if ASSERT_ENABLED
     m_code.forEachTmp([&] (Tmp tmp) {
         ASSERT(!tmp.isReg());
-        createStackSlot(tmp);
+        m_allTmps[tmp.bank()].append(tmp);
     });
+#endif
+
+    m_liveness = makeUnique<UnifiedTmpLiveness>(m_code);
+
+    {
+        buildLiveRanges(*m_liveness);
+
+        Vector<StackSlot*, 16> freeSlots;
+        Vector<StackSlot*, 4> toFree;
+        m_globalInstIndex = 0;
+        for (BasicBlock* block : m_code) {
+            auto assignStackSlotToTmp = [&] (Tmp tmp) {
+                if (tmp.isReg())
+                    return;
+
+                TmpData& data = m_map[tmp];
+                if (data.spillSlot) {
+                    if (m_liveRangeEnd[tmp] == m_globalInstIndex)
+                        toFree.append(data.spillSlot);
+                    return;
+                }
+
+                if (freeSlots.size())
+                    data.spillSlot = freeSlots.takeLast();
+                else
+                    data.spillSlot = m_code.addStackSlot(8, StackSlotKind::Spill);
+                data.reg = Reg();
+            };
+
+            auto flushToFreeList = [&] {
+                for (auto* stackSlot : toFree)
+                    freeSlots.append(stackSlot);
+                toFree.clear();
+            };
+
+            for (Tmp tmp : m_liveness->liveAtHead(block))
+                assignStackSlotToTmp(tmp);
+            flushToFreeList();
+
+            ++m_globalInstIndex;
+
+            for (Inst& inst : *block) {
+                Vector<Tmp, 4> seenTmps;
+                inst.forEachTmpFast([&] (Tmp tmp) {
+                    if (seenTmps.contains(tmp))
+                        return;
+                    seenTmps.append(tmp);
+                    assignStackSlotToTmp(tmp);
+                });
+
+                flushToFreeList();
+                ++m_globalInstIndex;
+            }
+
+            for (Tmp tmp : m_liveness->liveAtTail(block))
+                assignStackSlotToTmp(tmp);
+            flushToFreeList();
+
+            ++m_globalInstIndex;
+        }
+    }
 
     m_allowedRegisters = RegisterSet();
 
@@ -266,7 +358,9 @@ void GenerateAndAllocateRegisters::prepareForGeneration()
 
         for (Reg reg : m_registers[bank]) {
             m_allowedRegisters.set(reg);
-            createStackSlot(Tmp(reg));
+            TmpData& data = m_map[Tmp(reg)];
+            data.spillSlot = m_code.addStackSlot(8, StackSlotKind::Spill);
+            data.reg = Reg();
         }
     });
 
@@ -286,11 +380,37 @@ void GenerateAndAllocateRegisters::prepareForGeneration()
 
     lowerStackArgs(m_code);
 
+#if ASSERT_ENABLED
     // Verify none of these passes add any tmps.
-#if !ASSERT_DISABLED
     forEachBank([&] (Bank bank) {
-        ASSERT(m_allTmps[bank].size() - m_registers[bank].size() == m_code.numTmps(bank));
+        ASSERT(m_allTmps[bank].size() == m_code.numTmps(bank));
     });
+
+    {
+        // Verify that lowerStackArgs didn't change Tmp liveness at the boundaries for the Tmps and Registers we model.
+        UnifiedTmpLiveness liveness(m_code);
+        for (BasicBlock* block : m_code) {
+            auto assertLivenessAreEqual = [&] (auto a, auto b) {
+                HashSet<Tmp> livenessA;
+                HashSet<Tmp> livenessB;
+                for (Tmp tmp : a) {
+                    if (tmp.isReg() && isDisallowedRegister(tmp.reg()))
+                        continue;
+                    livenessA.add(tmp);
+                }
+                for (Tmp tmp : b) {
+                    if (tmp.isReg() && isDisallowedRegister(tmp.reg()))
+                        continue;
+                    livenessB.add(tmp);
+                }
+
+                ASSERT(livenessA == livenessB);
+            };
+
+            assertLivenessAreEqual(m_liveness->liveAtHead(block), liveness.liveAtHead(block));
+            assertLivenessAreEqual(m_liveness->liveAtTail(block), liveness.liveAtTail(block));
+        }
+    }
 #endif
 }
 
@@ -300,12 +420,9 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
 
     TimingScope timingScope("Air::generateAndAllocateRegisters");
 
-    insertBlocksForFlushAfterTerminalPatchpoints();
-
     DisallowMacroScratchRegisterUsage disallowScratch(*m_jit);
 
-    UnifiedTmpLiveness liveness(m_code);
-    buildLiveRanges(liveness);
+    buildLiveRanges(*m_liveness);
 
     IndexMap<BasicBlock*, IndexMap<Reg, Tmp>> currentAllocationMap(m_code.size());
     {
@@ -318,7 +435,7 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
 
         for (unsigned i = m_code.numEntrypoints(); i--;) {
             BasicBlock* entrypoint = m_code.entrypoint(i).block();
-            for (Tmp tmp : liveness.liveAtHead(entrypoint)) {
+            for (Tmp tmp : m_liveness->liveAtHead(entrypoint)) {
                 if (tmp.isReg())
                     currentAllocationMap[entrypoint][tmp.reg()] = tmp;
             }
@@ -380,7 +497,7 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
         }
 
         forEachBank([&] (Bank bank) {
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
             // By default, everything is spilled at block boundaries. We do this after we process each block
             // so we don't have to walk all Tmps, since #Tmps >> #Available regs. Instead, we walk the register file at
             // each block boundary and clear entries in this map.
@@ -406,8 +523,12 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
             m_availableRegs[tmp.bank()].clear(reg);
         }
 
+        ++m_globalInstIndex;
+
         bool isReplayingSameInst = false;
         for (size_t instIndex = 0; instIndex < block->size(); ++instIndex) {
+            checkConsistency();
+
             if (instIndex && !isReplayingSameInst)
                 startLabel = m_jit->labelIgnoringWatchpoints();
 
@@ -420,6 +541,46 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
             m_namedUsedRegs = RegisterSet();
             m_namedDefdRegs = RegisterSet();
 
+            bool needsToGenerate = ([&] () -> bool {
+                // FIXME: We should consider trying to figure out if we can also elide Mov32s
+                if (!(inst.kind.opcode == Move || inst.kind.opcode == MoveDouble))
+                    return true;
+
+                ASSERT(inst.args.size() >= 2);
+                Arg source = inst.args[0];
+                Arg dest = inst.args[1];
+                if (!source.isTmp() || !dest.isTmp())
+                    return true;
+
+                // FIXME: We don't track where the last use of a reg is globally so we don't know where we can elide them.
+                ASSERT(source.isReg() || m_liveRangeEnd[source.tmp()] >= m_globalInstIndex);
+                if (source.isReg() || m_liveRangeEnd[source.tmp()] != m_globalInstIndex)
+                    return true;
+
+                // If we are doing a self move at the end of the temps liveness we can trivially elide the move.
+                if (source == dest)
+                    return false;
+
+                Reg sourceReg = m_map[source.tmp()].reg;
+                // If the value is not already materialized into a register we may still move it into one so let the normal generation code run.
+                if (!sourceReg)
+                    return true;
+
+                ASSERT(m_currentAllocation->at(sourceReg) == source.tmp());
+
+                if (dest.isReg() && dest.reg() != sourceReg)
+                    return true;
+
+                if (Reg oldReg = m_map[dest.tmp()].reg)
+                    release(dest.tmp(), oldReg);
+
+                m_map[dest.tmp()].reg = sourceReg;
+                m_currentAllocation->at(sourceReg) = dest.tmp();
+                m_map[source.tmp()].reg = Reg();
+                return false;
+            })();
+            checkConsistency();
+
             inst.forEachArg([&] (Arg& arg, Arg::Role role, Bank, Width) {
                 if (!arg.isTmp())
                     return;
@@ -481,7 +642,7 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
             allocNamed(m_namedUsedRegs, false); // Must come before the defd registers since we may use and def the same register.
             allocNamed(m_namedDefdRegs, true);
 
-            {
+            if (needsToGenerate) {
                 auto tryAllocate = [&] {
                     Vector<Tmp*, 8> usesToAlloc;
                     Vector<Tmp*, 8> defsToAlloc;
@@ -594,7 +755,7 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
                         everySuccessorGetsOurRegisterState = false;
                 }
                 if (!everySuccessorGetsOurRegisterState) {
-                    for (Tmp tmp : liveness.liveAtTail(block)) {
+                    for (Tmp tmp : m_liveness->liveAtTail(block)) {
                         if (tmp.isReg() && isDisallowedRegister(tmp.reg()))
                             continue;
                         if (Reg reg = m_map[tmp].reg)
@@ -604,7 +765,9 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
             }
 
             if (!inst.isTerminal()) {
-                CCallHelpers::Jump jump = inst.generate(*m_jit, context);
+                CCallHelpers::Jump jump;
+                if (needsToGenerate)
+                    jump = inst.generate(*m_jit, context);
                 ASSERT_UNUSED(jump, !jump.isSet());
 
                 for (Reg reg : clobberedRegisters) {
@@ -615,7 +778,7 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
                     m_map[tmp].reg = Reg();
                 }
             } else {
-                bool needsToGenerate = true;
+                ASSERT(needsToGenerate);
                 if (inst.kind.opcode == Jump && block->successorBlock(0) == m_code.findNextBlock(block))
                     needsToGenerate = false;
 
@@ -624,12 +787,7 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
 
                     // We currently don't represent the full epilogue in Air, so we need to
                     // have this override.
-                    if (m_code.frameSize()) {
-                        m_jit->emitRestore(m_code.calleeSaveRegisterAtOffsetList());
-                        m_jit->emitFunctionEpilogue();
-                    } else
-                        m_jit->emitFunctionEpilogueWithEmptyFrame();
-                    m_jit->ret();
+                    m_code.emitEpilogue(*m_jit);
                 }
 
                 if (needsToGenerate) {
@@ -671,6 +829,8 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
             if (Tmp tmp = currentAllocation[i])
                 m_map[tmp].reg = Reg();
         }
+
+        ++m_globalInstIndex;
     }
 
     for (auto& entry : m_blocksAfterTerminalPatchForSpilling) {
