@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2015-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -162,7 +162,7 @@ protected:
 
     bool hasBeenSimplified(IndexType tmpIndex)
     {
-        if (!ASSERT_DISABLED) {
+        if (ASSERT_ENABLED) {
             if (!!m_coalescedTmps[tmpIndex])
                 ASSERT(getAlias(tmpIndex) != tmpIndex);
         }
@@ -192,32 +192,45 @@ protected:
         const auto& adjacentsOfU = m_adjacencyList[u];
         const auto& adjacentsOfV = m_adjacencyList[v];
 
-        if (adjacentsOfU.size() + adjacentsOfV.size() < registerCount()) {
+        Vector<IndexType, MacroAssembler::numGPRs + MacroAssembler::numFPRs> highOrderAdjacents;
+        RELEASE_ASSERT(registerCount() <= MacroAssembler::numGPRs + MacroAssembler::numFPRs);
+        unsigned numCandidates = adjacentsOfU.size() + adjacentsOfV.size();
+        if (numCandidates < registerCount()) {
             // Shortcut: if the total number of adjacents is less than the number of register, the condition is always met.
             return true;
         }
 
-        HashSet<IndexType> highOrderAdjacents;
-
         for (IndexType adjacentTmpIndex : adjacentsOfU) {
             ASSERT(adjacentTmpIndex != v);
             ASSERT(adjacentTmpIndex != u);
+            numCandidates--;
             if (!hasBeenSimplified(adjacentTmpIndex) && m_degrees[adjacentTmpIndex] >= registerCount()) {
-                auto addResult = highOrderAdjacents.add(adjacentTmpIndex);
-                if (addResult.isNewEntry && highOrderAdjacents.size() >= registerCount())
+                ASSERT(std::find(highOrderAdjacents.begin(), highOrderAdjacents.end(), adjacentTmpIndex) == highOrderAdjacents.end());
+                highOrderAdjacents.uncheckedAppend(adjacentTmpIndex);
+                if (highOrderAdjacents.size() >= registerCount())
                     return false;
-            }
+            } else if (highOrderAdjacents.size() + numCandidates < registerCount())
+                return true;
         }
+        ASSERT(numCandidates == adjacentsOfV.size());
+
+        auto iteratorEndHighOrderAdjacentsOfU = highOrderAdjacents.end();
         for (IndexType adjacentTmpIndex : adjacentsOfV) {
             ASSERT(adjacentTmpIndex != u);
             ASSERT(adjacentTmpIndex != v);
-            if (!hasBeenSimplified(adjacentTmpIndex) && m_degrees[adjacentTmpIndex] >= registerCount()) {
-                auto addResult = highOrderAdjacents.add(adjacentTmpIndex);
-                if (addResult.isNewEntry && highOrderAdjacents.size() >= registerCount())
+            numCandidates--;
+            if (!hasBeenSimplified(adjacentTmpIndex)
+                && m_degrees[adjacentTmpIndex] >= registerCount()
+                && std::find(highOrderAdjacents.begin(), iteratorEndHighOrderAdjacentsOfU, adjacentTmpIndex) == iteratorEndHighOrderAdjacentsOfU) {
+                ASSERT(std::find(iteratorEndHighOrderAdjacentsOfU, highOrderAdjacents.end(), adjacentTmpIndex) == highOrderAdjacents.end());
+                highOrderAdjacents.uncheckedAppend(adjacentTmpIndex);
+                if (highOrderAdjacents.size() >= registerCount())
                     return false;
-            }
+            } else if (highOrderAdjacents.size() + numCandidates < registerCount())
+                return true;
         }
 
+        ASSERT(!numCandidates);
         ASSERT(highOrderAdjacents.size() < registerCount());
         return true;
     }
@@ -324,7 +337,7 @@ protected:
     void assignColors()
     {
         ASSERT(m_simplifyWorklist.isEmpty());
-        ASSERT(!m_spillWorklist.bitCount());
+        ASSERT(m_spillWorklist.isEmpty());
 
         // Reclaim as much memory as possible.
         m_interferenceEdges.clear();
@@ -503,7 +516,7 @@ protected:
     struct InterferenceEdgeHash {
         static unsigned hash(const InterferenceEdge& key) { return key.hash(); }
         static bool equal(const InterferenceEdge& a, const InterferenceEdge& b) { return a == b; }
-        static const bool safeToCompareToEmptyOrDeleted = true;
+        static constexpr bool safeToCompareToEmptyOrDeleted = true;
     };
     typedef SimpleClassHashTraits<InterferenceEdge> InterferenceEdgeHashTraits;
 
@@ -635,7 +648,7 @@ public:
         // added to the select stack, it's not on either list, but only on the select stack.
         // Once on the select stack, logically, it's no longer in the interference graph.
         auto assertInvariants = [&] () {
-            if (ASSERT_DISABLED)
+            if (!ASSERT_ENABLED)
                 return;
             if (!shouldValidateIRAtEachPhase())
                 return;
@@ -669,16 +682,16 @@ public:
                 assertInvariants();
             }
 
-            if (m_spillWorklist.bitCount()) {
+            if (!m_spillWorklist.isEmpty()) {
                 selectSpill();
                 changed = true;
                 ASSERT(m_simplifyWorklist.size() == 1);
             }
         } while (changed);
 
-        if (!ASSERT_DISABLED) {
+        if (ASSERT_ENABLED) {
             ASSERT(!m_simplifyWorklist.size());
-            ASSERT(!m_spillWorklist.bitCount());
+            ASSERT(m_spillWorklist.isEmpty());
             IndexType firstNonRegIndex = m_lastPrecoloredRegisterIndex + 1;
             for (IndexType i = firstNonRegIndex; i < m_degrees.size(); ++i)
                 ASSERT(hasBeenSimplified(i));
@@ -714,7 +727,7 @@ protected:
             // No coalescing will remove the interference.
             moveIndex = UINT_MAX;
 
-            if (!ASSERT_DISABLED) {
+            if (ASSERT_ENABLED) {
                 if (isPrecolored(v))
                     ASSERT(isPrecolored(u));
             }
@@ -897,7 +910,7 @@ protected:
             if (traceDebug)
                 dataLogLn("Moving tmp ", tmpIndex, " from spill list to simplify list because it's degree is now less than k");
 
-            if (!ASSERT_DISABLED)
+            if (ASSERT_ENABLED)
                 ASSERT(m_unspillableTmps.contains(tmpIndex) || m_spillWorklist.contains(tmpIndex));
             m_spillWorklist.quickClear(tmpIndex);
 
@@ -992,14 +1005,14 @@ public:
                 coalesce();
             else if (!m_freezeWorklist.isEmpty())
                 freeze();
-            else if (m_spillWorklist.bitCount())
+            else if (!m_spillWorklist.isEmpty())
                 selectSpill();
 
             if (traceDebug) {
                 dataLog("After Graph simplification iteration\n");
                 dumpWorkLists(WTF::dataFile());
             }
-        } while (!m_simplifyWorklist.isEmpty() || !m_worklistMoves.isEmpty() || !m_freezeWorklist.isEmpty() || m_spillWorklist.bitCount());
+        } while (!m_simplifyWorklist.isEmpty() || !m_worklistMoves.isEmpty() || !m_freezeWorklist.isEmpty() || !m_spillWorklist.isEmpty());
 
         assignColors();
     }
@@ -1021,7 +1034,7 @@ protected:
     }
 
     // Low-degree vertex can always be colored: just pick any of the color taken by any
-    // other adjacent verices.
+    // other adjacent vertices.
     // The "Simplify" phase takes a low-degree out of the interference graph to simplify it.
     void simplify()
     {
@@ -1789,13 +1802,9 @@ public:
         padInterference(m_code);
 
         allocateOnBank<GP>();
-        m_numIterations = 0;
         allocateOnBank<FP>();
 
         fixSpillsAfterTerminals(m_code);
-
-        if (reportStats)
-            dataLog("Num iterations = ", m_numIterations, "\n");
     }
 
 private:
@@ -1809,11 +1818,14 @@ private:
         // Tmp into an unspillable Tmp.
         // https://bugs.webkit.org/show_bug.cgi?id=152699
 
-        while (true) {
-            ++m_numIterations;
+        unsigned numIterations = 0;
+        bool done = false;
+
+        while (!done) {
+            ++numIterations;
 
             if (traceDebug)
-                dataLog("Code at iteration ", m_numIterations, ":\n", m_code);
+                dataLog("Code at iteration ", numIterations, ":\n", m_code);
 
             // FIXME: One way to optimize this code is to remove the recomputation inside the fixpoint.
             // We need to recompute because spilling adds tmps, but we could just update tmpWidth when we
@@ -1822,7 +1834,7 @@ private:
             // spill code we emit. Since we currently recompute TmpWidth after spilling, the newly
             // created Tmps may get narrower use/def widths. On the other hand, the spiller already
             // selects which move instruction to use based on the original Tmp's widths, so it may not
-            // matter than a subsequent iteration sees a coservative width for the new Tmps. Also, the
+            // matter than a subsequent iteration sees a conservative width for the new Tmps. Also, the
             // recomputation may not actually be a performance problem; it's likely that a better way to
             // improve performance of TmpWidth is to replace its HashMap with something else. It's
             // possible that most of the TmpWidth overhead is from queries of TmpWidth rather than the
@@ -1835,7 +1847,7 @@ private:
                 if (!allocator.requiresSpilling()) {
                     this->assignRegistersToTmp<bank>(allocator);
                     if (traceDebug)
-                        dataLog("Successfull allocation at iteration ", m_numIterations, ":\n", m_code);
+                        dataLog("Successfull allocation at iteration ", numIterations, ":\n", m_code);
 
                     return true;
                 }
@@ -1844,7 +1856,6 @@ private:
                 return false;
             };
 
-            bool done;
             if (useIRC()) {
                 ColoringAllocator<bank, IRC> allocator(m_code, m_tmpWidth, m_useCounts, unspillableTmps);
                 done = doAllocation(allocator);
@@ -1852,9 +1863,8 @@ private:
                 ColoringAllocator<bank, Briggs> allocator(m_code, m_tmpWidth, m_useCounts, unspillableTmps);
                 done = doAllocation(allocator);
             }
-            if (done)
-                return;
         }
+        dataLogLnIf(reportStats, "Num iterations = ", numIterations, " for bank: ", bank);
     }
 
     template<Bank bank>
@@ -2187,7 +2197,6 @@ private:
     Code& m_code;
     TmpWidth m_tmpWidth;
     UseCounts<Tmp>& m_useCounts;
-    unsigned m_numIterations { 0 };
 };
 
 } // anonymous namespace
