@@ -29,7 +29,8 @@
 
 #include "AirArg.h"
 #include "AirKind.h"
-#include "CCallHelpers.h"
+#include "MacroAssembler.h"
+#include <wtf/ScopedLambda.h>
 
 namespace JSC {
 
@@ -191,7 +192,7 @@ struct Inst {
     // Generate some code for this instruction. This is, like, literally our backend. If this is the
     // terminal, it returns the jump that needs to be linked for the "then" case, with the "else"
     // case being fall-through. This function is auto-generated by opcode_generator.rb.
-    CCallHelpers::Jump generate(CCallHelpers&, GenerationContext&);
+    MacroAssembler::Jump generate(CCallHelpers&, GenerationContext&);
 
     // If source arguments benefits from being aliased to a destination argument,
     // this return the index of the destination argument.
