@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -48,7 +48,7 @@ namespace JSC { namespace B3 { namespace Air {
 namespace {
 
 namespace AirLowerAfterRegAllocInternal {
-static const bool verbose = false;
+static constexpr bool verbose = false;
 }
 
 } // anonymous namespace
