@@ -28,13 +28,10 @@
 
 #if ENABLE(B3_JIT)
 
-#include "AirArgInlines.h"
 #include "AirCode.h"
 #include "AirInsertionSet.h"
 #include "AirInstInlines.h"
 #include "AirPhaseScope.h"
-#include "StackAlignment.h"
-#include <wtf/ListDump.h>
 
 namespace JSC { namespace B3 { namespace Air {
 
