@@ -34,13 +34,13 @@
 
 #include "JSCInlines.h"
 
-using namespace JSC;
-
 namespace Deprecated {
 
-ScriptObject::ScriptObject(ExecState* scriptState, JSObject* object)
-    : ScriptValue(scriptState->vm(), object)
-    , m_scriptState(scriptState)
+using namespace JSC;
+
+ScriptObject::ScriptObject(JSGlobalObject* globalObject, JSObject* object)
+    : ScriptValue(globalObject->vm(), object)
+    , m_globalObject(globalObject)
 {
 }
 
