@@ -49,14 +49,6 @@ function toLength(target)
     return +(length > 0 ? (length < @MAX_SAFE_INTEGER ? length : @MAX_SAFE_INTEGER) : 0);
 }
 
-@globalPrivate
-function isDictionary(object)
-{
-    "use strict";
-
-    return object == null || typeof object === "object";
-}
-
 @globalPrivate
 @getter
 @overriddenName="get [Symbol.species]"
@@ -76,7 +68,7 @@ function speciesConstructor(obj, defaultConstructor)
         return defaultConstructor;
     if (!@isObject(constructor))
         @throwTypeError("|this|.constructor is not an Object or undefined");
-    constructor = constructor.@speciesSymbol;
+    constructor = constructor.@@species;
     if (@isUndefinedOrNull(constructor))
         return defaultConstructor;
     if (@isConstructor(constructor))
@@ -95,14 +87,14 @@ function copyDataProperties(target, source, excludedSet)
     if (@isUndefinedOrNull(source))
         return target;
 
-    let from = @toObject(source);
-    let keys = @Reflect.@ownKeys(from); 
-    let keysLength = keys.length;
-    for (let i = 0; i < keysLength; i++) {
-        let nextKey = keys[i];
+    var from = @toObject(source);
+    var keys = @ownKeys(from);
+    var keysLength = keys.length;
+    for (var i = 0; i < keysLength; i++) {
+        var nextKey = keys[i];
         if (!excludedSet.@has(nextKey)) {
             if (@propertyIsEnumerable(from, nextKey)) {
-                let propValue = from[nextKey];
+                var propValue = from[nextKey];
                 @defineEnumerableWritableConfigurableDataProperty(target, nextKey, propValue);
             }
         }
@@ -122,13 +114,13 @@ function copyDataPropertiesNoExclusions(target, source)
     if (@isUndefinedOrNull(source))
         return target;
 
-    let from = @toObject(source);
-    let keys = @Reflect.@ownKeys(from); 
-    let keysLength = keys.length;
-    for (let i = 0; i < keysLength; i++) {
-        let nextKey = keys[i];
+    var from = @toObject(source);
+    var keys = @ownKeys(from);
+    var keysLength = keys.length;
+    for (var i = 0; i < keysLength; i++) {
+        var nextKey = keys[i];
         if (@propertyIsEnumerable(from, nextKey)) {
-            let propValue = from[nextKey];
+            var propValue = from[nextKey];
             @defineEnumerableWritableConfigurableDataProperty(target, nextKey, propValue);
         }
     }
