@@ -25,7 +25,7 @@
 
 @constructor
 @globalPrivate
-function createMapIterator(iteratedObject, kind)
+function MapIterator(iteratedObject, kind)
 {
     "use strict";
 
@@ -42,7 +42,7 @@ function values()
     if (!@isMap(this))
         @throwTypeError("Map.prototype.values requires that |this| be Map");
 
-    return new @createMapIterator(this, @iterationKindValue);
+    return new @MapIterator(this, @iterationKindValue);
 }
 
 function keys()
@@ -52,7 +52,7 @@ function keys()
     if (!@isMap(this))
         @throwTypeError("Map.prototype.keys requires that |this| be Map");
 
-    return new @createMapIterator(this, @iterationKindKey);
+    return new @MapIterator(this, @iterationKindKey);
 }
 
 function entries()
@@ -62,7 +62,7 @@ function entries()
     if (!@isMap(this))
         @throwTypeError("Map.prototype.entries requires that |this| be Map");
 
-    return new @createMapIterator(this, @iterationKindKeyValue);
+    return new @MapIterator(this, @iterationKindEntries);
 }
 
 function forEach(callback /*, thisArg */)
