@@ -25,24 +25,24 @@
 
 #pragma once
 
+#include "CodeBlock.h"
+#include "PackedCellPtr.h"
 #include "Watchpoint.h"
 
 namespace JSC {
 
-class CodeBlock;
-
-class CodeBlockJettisoningWatchpoint : public Watchpoint {
+class CodeBlockJettisoningWatchpoint final : public Watchpoint {
 public:
     CodeBlockJettisoningWatchpoint(CodeBlock* codeBlock)
-        : m_codeBlock(codeBlock)
+        : Watchpoint(Watchpoint::Type::CodeBlockJettisoning)
+        , m_codeBlock(codeBlock)
     {
     }
 
-protected:
-    void fireInternal(VM&, const FireDetail&) override;
+    void fireInternal(VM&, const FireDetail&);
 
 private:
-    CodeBlock* m_codeBlock;
+    JSC_WATCHPOINT_FIELD(PackedCellPtr<CodeBlock>, m_codeBlock);
 };
 
 } // namespace JSC
