@@ -35,7 +35,7 @@ namespace JSC { namespace DFG {
 
 void FrequentExitSite::dump(PrintStream& out) const
 {
-    out.print("bc#", m_bytecodeOffset, ": ", m_kind, "/", m_jitType, "/", m_inlineKind);
+    out.print(m_bytecodeIndex, ": ", m_kind, "/", m_jitType, "/", m_inlineKind);
 }
 
 ExitProfile::ExitProfile() { }
@@ -50,15 +50,14 @@ bool ExitProfile::add(CodeBlock* owner, const FrequentExitSite& site)
 
     CODEBLOCK_LOG_EVENT(owner, "frequentExit", (site));
 
-    if (Options::verboseExitProfile())
-        dataLog(pointerDump(owner), ": Adding exit site: ", site, "\n");
+    dataLogLnIf(Options::verboseExitProfile(), pointerDump(owner), ": Adding exit site: ", site);
 
     ExitProfile& profile = owner->unlinkedCodeBlock()->exitProfile();
 
     // If we've never seen any frequent exits then create the list and put this site
     // into it.
     if (!profile.m_frequentExitSites) {
-        profile.m_frequentExitSites = std::make_unique<Vector<FrequentExitSite>>();
+        profile.m_frequentExitSites = makeUnique<Vector<FrequentExitSite>>();
         profile.m_frequentExitSites->append(site);
         return true;
     }
@@ -75,7 +74,7 @@ bool ExitProfile::add(CodeBlock* owner, const FrequentExitSite& site)
     return true;
 }
 
-Vector<FrequentExitSite> ExitProfile::exitSitesFor(unsigned bytecodeIndex)
+Vector<FrequentExitSite> ExitProfile::exitSitesFor(BytecodeIndex bytecodeIndex)
 {
     Vector<FrequentExitSite> result;
 
@@ -83,7 +82,7 @@ Vector<FrequentExitSite> ExitProfile::exitSitesFor(unsigned bytecodeIndex)
         return result;
 
     for (unsigned i = 0; i < m_frequentExitSites->size(); ++i) {
-        if (m_frequentExitSites->at(i).bytecodeOffset() == bytecodeIndex)
+        if (m_frequentExitSites->at(i).bytecodeIndex() == bytecodeIndex)
             result.append(m_frequentExitSites->at(i));
     }
 
