@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011, 2015 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -35,7 +35,7 @@ namespace JSC {
 // Values may be unboxed primitives (int32, double, or cell), or boxed as a JSValue.
 // For boxed values, we may know the type of boxing that has taken place.
 // (May also need bool, array, object, string types!)
-enum DataFormat {
+enum DataFormat : uint8_t {
     DataFormatNone = 0,
     DataFormatInt32 = 1,
     DataFormatInt52 = 2, // Int52's are left-shifted by 12 by default.
@@ -44,11 +44,13 @@ enum DataFormat {
     DataFormatBoolean = 5,
     DataFormatCell = 6,
     DataFormatStorage = 7,
-    DataFormatJS = 8,
+    DataFormatBigInt32 = 8, // FIXME: https://bugs.webkit.org/show_bug.cgi?id=210957 Actually support BigInt32 DataFormat.
+    DataFormatJS = 16,
     DataFormatJSInt32 = DataFormatJS | DataFormatInt32,
     DataFormatJSDouble = DataFormatJS | DataFormatDouble,
     DataFormatJSCell = DataFormatJS | DataFormatCell,
     DataFormatJSBoolean = DataFormatJS | DataFormatBoolean,
+    DataFormatJSBigInt32 = DataFormatJS | DataFormatBigInt32,
 
     // Marker deliminating ordinary data formats and OSR-only data formats.
     DataFormatOSRMarker = 32,
@@ -76,6 +78,8 @@ inline const char* dataFormatToString(DataFormat dataFormat)
         return "Boolean";
     case DataFormatStorage:
         return "Storage";
+    case DataFormatBigInt32:
+        return "BigInt32";
     case DataFormatJS:
         return "JS";
     case DataFormatJSInt32:
@@ -86,6 +90,8 @@ inline const char* dataFormatToString(DataFormat dataFormat)
         return "JSCell";
     case DataFormatJSBoolean:
         return "JSBoolean";
+    case DataFormatJSBigInt32:
+        return "JSBigInt32";
     case DataFormatDead:
         return "Dead";
     default:
