@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2015-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -54,6 +54,9 @@ void PropertyCondition::dumpInContext(PrintStream& out, DumpContext* context) co
     case Equivalence:
         out.print(m_header.type(), " of ", m_header.pointer(), " with ", inContext(requiredValue(), context));
         return;
+    case CustomFunctionEquivalence:
+        out.print(m_header.type(), " of ", m_header.pointer());
+        return;
     case HasPrototype:
         out.print(m_header.type(), " with prototype ", inContext(JSValue(prototype()), context));
         return;
@@ -86,6 +89,7 @@ bool PropertyCondition::isStillValidAssumingImpurePropertyWatchpoint(
     case Absence:
     case AbsenceOfSetEffect:
     case Equivalence:
+    case CustomFunctionEquivalence:
         if (!structure->propertyAccessesAreCacheable()) {
             if (PropertyConditionInternal::verbose)
                 dataLog("Invalid because property accesses are not cacheable.\n");
@@ -248,7 +252,13 @@ bool PropertyCondition::isStillValidAssumingImpurePropertyWatchpoint(
         }
 
         return true;
-    } }
+    }
+    case CustomFunctionEquivalence: {
+        if (structure->staticPropertiesReified())
+            return false;
+        return !!structure->findPropertyHashEntry(uid());
+    }
+    }
 
     RELEASE_ASSERT_NOT_REACHED();
     return false;
@@ -263,6 +273,7 @@ bool PropertyCondition::validityRequiresImpurePropertyWatchpoint(Structure* stru
     case Presence:
     case Absence:
     case Equivalence:
+    case CustomFunctionEquivalence:
         return structure->needImpurePropertyWatchpoint();
     case AbsenceOfSetEffect:
     case HasPrototype:
@@ -288,6 +299,7 @@ bool PropertyCondition::isStillValid(Structure* structure, JSObject* base) const
         break;
     case Presence:
     case Equivalence:
+    case CustomFunctionEquivalence:
         if (structure->typeInfo().getOwnPropertySlotIsImpure())
             return false;
         break;
@@ -320,8 +332,7 @@ bool PropertyCondition::isWatchableWhenValid(
             set = structure->propertyReplacementWatchpointSet(offset);
             break;
         case EnsureWatchability:
-            set = structure->ensurePropertyReplacementWatchpointSet(
-                *structure->vm(), offset);
+            set = structure->ensurePropertyReplacementWatchpointSet(structure->vm(), offset);
             break;
         }
 
@@ -331,6 +342,21 @@ bool PropertyCondition::isWatchableWhenValid(
         break;
     }
 
+    case CustomFunctionEquivalence: {
+        // We just use the structure transition watchpoint for this. A structure S starts
+        // off with a property P in the static property hash table. If S transitions to
+        // S', either P remains in the static property table or not. If not, then we
+        // are no longer valid. So the above check of transitionWatchpointSetHasBeenInvalidated
+        // is sufficient.
+        //
+        // We could make this smarter in the future, since we sometimes reify static properties.
+        // We could make this adapt to looking at the object's storage for such reified custom
+        // functions, but we don't do that right now. We just allow this property condition to
+        // invalidate and create an Equivalence watchpoint for the materialized property sometime
+        // in the future.
+        break;
+    }
+
     default:
         break;
     }
@@ -352,20 +378,6 @@ bool PropertyCondition::isWatchable(
         && isWatchableWhenValid(structure, effort);
 }
 
-bool PropertyCondition::isStillLive() const
-{
-    if (hasPrototype() && prototype() && !Heap::isMarked(prototype()))
-        return false;
-
-    if (hasRequiredValue()
-        && requiredValue()
-        && requiredValue().isCell()
-        && !Heap::isMarked(requiredValue().asCell()))
-        return false;
-
-    return true;
-}
-
 void PropertyCondition::validateReferences(const TrackedReferences& tracked) const
 {
     if (hasPrototype())
@@ -418,6 +430,9 @@ void printInternal(PrintStream& out, JSC::PropertyCondition::Kind condition)
     case JSC::PropertyCondition::Equivalence:
         out.print("Equivalence");
         return;
+    case JSC::PropertyCondition::CustomFunctionEquivalence:
+        out.print("CustomFunctionEquivalence");
+        return;
     case JSC::PropertyCondition::HasPrototype:
         out.print("HasPrototype");
         return;
