@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, 2014-2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2008-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,6 +26,7 @@
 #include "config.h"
 #include "StructureStubInfo.h"
 
+#include "CacheableIdentifierInlines.h"
 #include "JSObject.h"
 #include "JSCInlines.h"
 #include "PolymorphicAccess.h"
@@ -36,13 +37,12 @@ namespace JSC {
 #if ENABLE(JIT)
 
 namespace StructureStubInfoInternal {
-static const bool verbose = false;
+static constexpr bool verbose = false;
 }
 
 StructureStubInfo::StructureStubInfo(AccessType accessType)
-    : callSiteIndex(UINT_MAX)
-    , accessType(accessType)
-    , cacheType(CacheType::Unset)
+    : accessType(accessType)
+    , m_cacheType(CacheType::Unset)
     , countdown(1) // For a totally clear stub, we'll patch it after the first execution.
     , repatchCount(0)
     , numberOfCoolDowns(0)
@@ -52,6 +52,10 @@ StructureStubInfo::StructureStubInfo(AccessType accessType)
     , everConsidered(false)
     , prototypeIsKnownObject(false)
     , sawNonCell(false)
+    , hasConstantIdentifier(true)
+    , propertyIsString(false)
+    , propertyIsInt32(false)
+    , propertyIsSymbol(false)
 {
 }
 
@@ -59,46 +63,49 @@ StructureStubInfo::~StructureStubInfo()
 {
 }
 
-void StructureStubInfo::initGetByIdSelf(CodeBlock* codeBlock, Structure* baseObjectStructure, PropertyOffset offset)
+void StructureStubInfo::initGetByIdSelf(CodeBlock* codeBlock, Structure* baseObjectStructure, PropertyOffset offset, CacheableIdentifier identifier)
 {
-    cacheType = CacheType::GetByIdSelf;
+    ASSERT(hasConstantIdentifier);
+    setCacheType(CacheType::GetByIdSelf);
+    m_getByIdSelfIdentifier = identifier;
+    codeBlock->vm().heap.writeBarrier(codeBlock);
 
     u.byIdSelf.baseObjectStructure.set(
-        *codeBlock->vm(), codeBlock, baseObjectStructure);
+        codeBlock->vm(), codeBlock, baseObjectStructure);
     u.byIdSelf.offset = offset;
 }
 
 void StructureStubInfo::initArrayLength()
 {
-    cacheType = CacheType::ArrayLength;
+    setCacheType(CacheType::ArrayLength);
 }
 
 void StructureStubInfo::initStringLength()
 {
-    cacheType = CacheType::StringLength;
+    setCacheType(CacheType::StringLength);
 }
 
 void StructureStubInfo::initPutByIdReplace(CodeBlock* codeBlock, Structure* baseObjectStructure, PropertyOffset offset)
 {
-    cacheType = CacheType::PutByIdReplace;
+    setCacheType(CacheType::PutByIdReplace);
 
     u.byIdSelf.baseObjectStructure.set(
-        *codeBlock->vm(), codeBlock, baseObjectStructure);
+        codeBlock->vm(), codeBlock, baseObjectStructure);
     u.byIdSelf.offset = offset;
 }
 
 void StructureStubInfo::initInByIdSelf(CodeBlock* codeBlock, Structure* baseObjectStructure, PropertyOffset offset)
 {
-    cacheType = CacheType::InByIdSelf;
+    setCacheType(CacheType::InByIdSelf);
 
     u.byIdSelf.baseObjectStructure.set(
-        *codeBlock->vm(), codeBlock, baseObjectStructure);
+        codeBlock->vm(), codeBlock, baseObjectStructure);
     u.byIdSelf.offset = offset;
 }
 
 void StructureStubInfo::deref()
 {
-    switch (cacheType) {
+    switch (m_cacheType) {
     case CacheType::Stub:
         delete u.stub;
         return;
@@ -116,7 +123,7 @@ void StructureStubInfo::deref()
 
 void StructureStubInfo::aboutToDie()
 {
-    switch (cacheType) {
+    switch (m_cacheType) {
     case CacheType::Stub:
         u.stub->aboutToDie();
         return;
@@ -133,95 +140,101 @@ void StructureStubInfo::aboutToDie()
 }
 
 AccessGenerationResult StructureStubInfo::addAccessCase(
-    const GCSafeConcurrentJSLocker& locker, CodeBlock* codeBlock, const Identifier& ident, std::unique_ptr<AccessCase> accessCase)
+    const GCSafeConcurrentJSLocker& locker, CodeBlock* codeBlock, CacheableIdentifier ident, std::unique_ptr<AccessCase> accessCase)
 {
-    VM& vm = *codeBlock->vm();
-
-    if (StructureStubInfoInternal::verbose)
-        dataLog("Adding access case: ", accessCase, "\n");
-
-    if (!accessCase)
-        return AccessGenerationResult::GaveUp;
-
-    AccessGenerationResult result;
-
-    if (cacheType == CacheType::Stub) {
-        result = u.stub->addCase(locker, vm, codeBlock, *this, ident, WTFMove(accessCase));
+    checkConsistency();
 
+    VM& vm = codeBlock->vm();
+    ASSERT(vm.heap.isDeferred());
+    AccessGenerationResult result = ([&] () -> AccessGenerationResult {
         if (StructureStubInfoInternal::verbose)
-            dataLog("Had stub, result: ", result, "\n");
+            dataLog("Adding access case: ", accessCase, "\n");
 
-        if (result.shouldResetStubAndFireWatchpoints())
-            return result;
+        if (!accessCase)
+            return AccessGenerationResult::GaveUp;
 
-        if (!result.buffered()) {
-            bufferedStructures.clear();
-            return result;
+        AccessGenerationResult result;
+
+        if (m_cacheType == CacheType::Stub) {
+            result = u.stub->addCase(locker, vm, codeBlock, *this, WTFMove(accessCase));
+
+            if (StructureStubInfoInternal::verbose)
+                dataLog("Had stub, result: ", result, "\n");
+
+            if (result.shouldResetStubAndFireWatchpoints())
+                return result;
+
+            if (!result.buffered()) {
+                bufferedStructures.clear();
+                return result;
+            }
+        } else {
+            std::unique_ptr<PolymorphicAccess> access = makeUnique<PolymorphicAccess>();
+
+            Vector<std::unique_ptr<AccessCase>, 2> accessCases;
+
+            std::unique_ptr<AccessCase> previousCase = AccessCase::fromStructureStubInfo(vm, codeBlock, ident, *this);
+            if (previousCase)
+                accessCases.append(WTFMove(previousCase));
+
+            accessCases.append(WTFMove(accessCase));
+
+            result = access->addCases(locker, vm, codeBlock, *this, WTFMove(accessCases));
+
+            if (StructureStubInfoInternal::verbose)
+                dataLog("Created stub, result: ", result, "\n");
+
+            if (result.shouldResetStubAndFireWatchpoints())
+                return result;
+
+            if (!result.buffered()) {
+                bufferedStructures.clear();
+                return result;
+            }
+
+            setCacheType(CacheType::Stub);
+            u.stub = access.release();
         }
-    } else {
-        std::unique_ptr<PolymorphicAccess> access = std::make_unique<PolymorphicAccess>();
 
-        Vector<std::unique_ptr<AccessCase>, 2> accessCases;
-
-        std::unique_ptr<AccessCase> previousCase =
-            AccessCase::fromStructureStubInfo(vm, codeBlock, *this);
-        if (previousCase)
-            accessCases.append(WTFMove(previousCase));
-
-        accessCases.append(WTFMove(accessCase));
-
-        result = access->addCases(locker, vm, codeBlock, *this, ident, WTFMove(accessCases));
-
-        if (StructureStubInfoInternal::verbose)
-            dataLog("Created stub, result: ", result, "\n");
-
-        if (result.shouldResetStubAndFireWatchpoints())
-            return result;
+        ASSERT(m_cacheType == CacheType::Stub);
+        RELEASE_ASSERT(!result.generatedSomeCode());
 
+        // If we didn't buffer any cases then bail. If this made no changes then we'll just try again
+        // subject to cool-down.
         if (!result.buffered()) {
+            if (StructureStubInfoInternal::verbose)
+                dataLog("Didn't buffer anything, bailing.\n");
             bufferedStructures.clear();
             return result;
         }
 
-        cacheType = CacheType::Stub;
-        u.stub = access.release();
-    }
+        // The buffering countdown tells us if we should be repatching now.
+        if (bufferingCountdown) {
+            if (StructureStubInfoInternal::verbose)
+                dataLog("Countdown is too high: ", bufferingCountdown, ".\n");
+            return result;
+        }
 
-    RELEASE_ASSERT(!result.generatedSomeCode());
-
-    // If we didn't buffer any cases then bail. If this made no changes then we'll just try again
-    // subject to cool-down.
-    if (!result.buffered()) {
-        if (StructureStubInfoInternal::verbose)
-            dataLog("Didn't buffer anything, bailing.\n");
+        // Forget the buffered structures so that all future attempts to cache get fully handled by the
+        // PolymorphicAccess.
         bufferedStructures.clear();
-        return result;
-    }
 
-    // The buffering countdown tells us if we should be repatching now.
-    if (bufferingCountdown) {
+        result = u.stub->regenerate(locker, vm, codeBlock, *this);
+
         if (StructureStubInfoInternal::verbose)
-            dataLog("Countdown is too high: ", bufferingCountdown, ".\n");
+            dataLog("Regeneration result: ", result, "\n");
+
+        RELEASE_ASSERT(!result.buffered());
+
+        if (!result.generatedSomeCode())
+            return result;
+
+        // If we generated some code then we don't want to attempt to repatch in the future until we
+        // gather enough cases.
+        bufferingCountdown = Options::repatchBufferingCountdown();
         return result;
-    }
-
-    // Forget the buffered structures so that all future attempts to cache get fully handled by the
-    // PolymorphicAccess.
-    bufferedStructures.clear();
-
-    result = u.stub->regenerate(locker, vm, codeBlock, *this, ident);
-
-    if (StructureStubInfoInternal::verbose)
-        dataLog("Regeneration result: ", result, "\n");
-
-    RELEASE_ASSERT(!result.buffered());
-
-    if (!result.generatedSomeCode())
-        return result;
-
-    // If we generated some code then we don't want to attempt to repatch in the future until we
-    // gather enough cases.
-    bufferingCountdown = Options::repatchBufferingCountdown();
+    })();
+    vm.heap.writeBarrier(codeBlock);
     return result;
 }
 
@@ -229,7 +242,7 @@ void StructureStubInfo::reset(CodeBlock* codeBlock)
 {
     bufferedStructures.clear();
 
-    if (cacheType == CacheType::Unset)
+    if (m_cacheType == CacheType::Unset)
         return;
 
     if (Options::verboseOSR()) {
@@ -239,17 +252,20 @@ void StructureStubInfo::reset(CodeBlock* codeBlock)
     }
 
     switch (accessType) {
-    case AccessType::TryGet:
-        resetGetByID(codeBlock, *this, GetByIDKind::Try);
+    case AccessType::TryGetById:
+        resetGetBy(codeBlock, *this, GetByKind::Try);
         break;
-    case AccessType::Get:
-        resetGetByID(codeBlock, *this, GetByIDKind::Normal);
+    case AccessType::GetById:
+        resetGetBy(codeBlock, *this, GetByKind::Normal);
         break;
-    case AccessType::GetWithThis:
-        resetGetByID(codeBlock, *this, GetByIDKind::WithThis);
+    case AccessType::GetByIdWithThis:
+        resetGetBy(codeBlock, *this, GetByKind::WithThis);
         break;
-    case AccessType::GetDirect:
-        resetGetByID(codeBlock, *this, GetByIDKind::Direct);
+    case AccessType::GetByIdDirect:
+        resetGetBy(codeBlock, *this, GetByKind::Direct);
+        break;
+    case AccessType::GetByVal:
+        resetGetBy(codeBlock, *this, GetByKind::NormalByVal);
         break;
     case AccessType::Put:
         resetPutByID(codeBlock, *this);
@@ -263,23 +279,45 @@ void StructureStubInfo::reset(CodeBlock* codeBlock)
     }
 
     deref();
-    cacheType = CacheType::Unset;
+    setCacheType(CacheType::Unset);
+}
+
+void StructureStubInfo::visitAggregate(SlotVisitor& visitor)
+{
+    switch (m_cacheType) {
+    case CacheType::Unset:
+    case CacheType::ArrayLength:
+    case CacheType::StringLength:
+    case CacheType::PutByIdReplace:
+    case CacheType::InByIdSelf:
+        return;
+    case CacheType::GetByIdSelf:
+        m_getByIdSelfIdentifier.visitAggregate(visitor);
+        return;
+    case CacheType::Stub:
+        u.stub->visitAggregate(visitor);
+        return;
+    }
+
+    RELEASE_ASSERT_NOT_REACHED();
+    return;
 }
 
 void StructureStubInfo::visitWeakReferences(CodeBlock* codeBlock)
 {
-    VM& vm = *codeBlock->vm();
+    VM& vm = codeBlock->vm();
 
-    bufferedStructures.genericFilter(
-        [&] (Structure* structure) -> bool {
-            return Heap::isMarked(structure);
+    bufferedStructures.removeIf(
+        [&] (auto& pair) -> bool {
+            Structure* structure = pair.first;
+            return !vm.heap.isMarked(structure);
         });
 
-    switch (cacheType) {
+    switch (m_cacheType) {
     case CacheType::GetByIdSelf:
     case CacheType::PutByIdReplace:
     case CacheType::InByIdSelf:
-        if (Heap::isMarked(u.byIdSelf.baseObjectStructure.get()))
+        if (vm.heap.isMarked(u.byIdSelf.baseObjectStructure.get()))
             return;
         break;
     case CacheType::Stub:
@@ -296,7 +334,7 @@ void StructureStubInfo::visitWeakReferences(CodeBlock* codeBlock)
 
 bool StructureStubInfo::propagateTransitions(SlotVisitor& visitor)
 {
-    switch (cacheType) {
+    switch (m_cacheType) {
     case CacheType::Unset:
     case CacheType::ArrayLength:
     case CacheType::StringLength:
@@ -313,15 +351,15 @@ bool StructureStubInfo::propagateTransitions(SlotVisitor& visitor)
     return true;
 }
 
-StubInfoSummary StructureStubInfo::summary() const
+StubInfoSummary StructureStubInfo::summary(VM& vm) const
 {
     StubInfoSummary takesSlowPath = StubInfoSummary::TakesSlowPath;
     StubInfoSummary simple = StubInfoSummary::Simple;
-    if (cacheType == CacheType::Stub) {
+    if (m_cacheType == CacheType::Stub) {
         PolymorphicAccess* list = u.stub;
         for (unsigned i = 0; i < list->size(); ++i) {
             const AccessCase& access = list->at(i);
-            if (access.doesCalls()) {
+            if (access.doesCalls(vm)) {
                 takesSlowPath = StubInfoSummary::TakesSlowPathAndMakesCalls;
                 simple = StubInfoSummary::MakesCalls;
                 break;
@@ -338,21 +376,39 @@ StubInfoSummary StructureStubInfo::summary() const
     return simple;
 }
 
-StubInfoSummary StructureStubInfo::summary(const StructureStubInfo* stubInfo)
+StubInfoSummary StructureStubInfo::summary(VM& vm, const StructureStubInfo* stubInfo)
 {
     if (!stubInfo)
         return StubInfoSummary::NoInformation;
 
-    return stubInfo->summary();
+    return stubInfo->summary(vm);
 }
 
 bool StructureStubInfo::containsPC(void* pc) const
 {
-    if (cacheType != CacheType::Stub)
+    if (m_cacheType != CacheType::Stub)
         return false;
     return u.stub->containsPC(pc);
 }
 
+void StructureStubInfo::setCacheType(CacheType newCacheType)
+{
+    if (m_cacheType == CacheType::GetByIdSelf)
+        m_getByIdSelfIdentifier = nullptr;
+    m_cacheType = newCacheType;
+}
+
+#if ASSERT_ENABLED
+void StructureStubInfo::checkConsistency()
+{
+    if (thisValueIsInThisGPR()) {
+        // We currently use a union for both "thisGPR" and "propertyGPR". If this were
+        // not the case, we'd need to take one of them out of the union.
+        RELEASE_ASSERT(hasConstantIdentifier);
+    }
+}
+#endif // ASSERT_ENABLED
+
 #endif // ENABLE(JIT)
 
 } // namespace JSC
