@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,6 +30,7 @@
 #include "ArrayProfile.h"
 #include "DFGAbstractValueClobberEpoch.h"
 #include "DFGFiltrationResult.h"
+#include "DFGFlushFormat.h"
 #include "DFGFrozenValue.h"
 #include "DFGNodeFlags.h"
 #include "DFGStructureAbstractValue.h"
@@ -47,6 +48,7 @@ namespace DFG {
 
 class Graph;
 struct Node;
+class VariableAccessData;
 
 struct AbstractValue {
     AbstractValue()
@@ -188,11 +190,24 @@ struct AbstractValue {
             && !m_value;
     }
 
+    bool isBytecodeTop() const
+    {
+        return (m_type | SpecBytecodeTop) == m_type
+            && m_structure.isTop()
+            && m_arrayModes == ALL_ARRAY_MODES
+            && !m_value;
+    }
+
     bool valueIsTop() const
     {
         return !m_value && m_type;
     }
 
+    bool isInt52Any() const
+    {
+        return !(m_type & ~SpecInt52Any);
+    }
+
     JSValue value() const
     {
         return m_value;
@@ -268,7 +283,7 @@ struct AbstractValue {
         if (other.isClear())
             return false;
 
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
         AbstractValue oldMe = *this;
 #endif
         bool result = false;
@@ -289,7 +304,7 @@ struct AbstractValue {
         return result;
     }
 
-    bool mergeOSREntryValue(Graph&, JSValue);
+    bool mergeOSREntryValue(Graph&, JSValue, VariableAccessData*, Node*);
 
     void merge(SpeculatedType type)
     {
@@ -321,7 +336,7 @@ struct AbstractValue {
     // with SpecCell.
     FiltrationResult filter(Graph&, const RegisteredStructureSet&, SpeculatedType admittedTypes = SpecNone);
 
-    FiltrationResult filterArrayModes(ArrayModes);
+    FiltrationResult filterArrayModes(ArrayModes, SpeculatedType admittedTypes = SpecNone);
 
     ALWAYS_INLINE FiltrationResult filter(SpeculatedType type)
     {
@@ -369,20 +384,35 @@ struct AbstractValue {
 
     bool contains(RegisteredStructure) const;
 
-    bool validate(JSValue value) const
+    bool validateOSREntryValue(JSValue value, FlushFormat format) const
     {
-        if (isHeapTop())
+        if (isBytecodeTop())
             return true;
 
-        if (!!m_value && m_value != value)
-            return false;
+        if (format == FlushedInt52) {
+            if (!isInt52Any())
+                return false;
 
-        if (mergeSpeculations(m_type, speculationFromValue(value)) != m_type)
-            return false;
+            if (!validateTypeAcceptingBoxedInt52(value))
+                return false;
 
-        if (value.isEmpty()) {
-            ASSERT(m_type & SpecEmpty);
-            return true;
+            if (!!m_value) {
+                ASSERT(m_value.isAnyInt());
+                ASSERT(value.isAnyInt());
+                if (jsDoubleNumber(m_value.asAnyInt()) != jsDoubleNumber(value.asAnyInt()))
+                    return false;
+            }
+        } else {
+            if (!!m_value && m_value != value)
+                return false;
+
+            if (mergeSpeculations(m_type, speculationFromValue(value)) != m_type)
+                return false;
+
+            if (value.isEmpty()) {
+                ASSERT(m_type & SpecEmpty);
+                return true;
+            }
         }
 
         if (!!value && value.isCell()) {
@@ -401,12 +431,12 @@ struct AbstractValue {
             || !arrayModesAreClearOrTop(m_arrayModes);
     }
 
-#if ASSERT_DISABLED
+#if ASSERT_ENABLED
+    JS_EXPORT_PRIVATE void checkConsistency() const;
+    void assertIsRegistered(Graph&) const;
+#else
     void checkConsistency() const { }
     void assertIsRegistered(Graph&) const { }
-#else
-    void checkConsistency() const;
-    void assertIsRegistered(Graph&) const;
 #endif
 
     ResultType resultType() const;
@@ -490,26 +520,19 @@ private:
             m_arrayModes |= to;
     }
 
-    bool validateType(JSValue value) const
+    bool validateTypeAcceptingBoxedInt52(JSValue value) const
     {
-        if (isHeapTop())
+        if (isBytecodeTop())
             return true;
 
-        // Constant folding always represents Int52's in a double (i.e. AnyIntAsDouble).
-        // So speculationFromValue(value) for an Int52 value will return AnyIntAsDouble,
-        // and that's fine - the type validates just fine.
-        SpeculatedType type = m_type;
-        if (type & SpecInt52Only)
-            type |= SpecAnyIntAsDouble;
-
-        if (mergeSpeculations(type, speculationFromValue(value)) != type)
-            return false;
-
-        if (value.isEmpty()) {
-            ASSERT(m_type & SpecEmpty);
-            return true;
+        if (m_type & SpecInt52Any) {
+            if (mergeSpeculations(m_type, int52AwareSpeculationFromValue(value)) == m_type)
+                return true;
         }
 
+        if (mergeSpeculations(m_type, speculationFromValue(value)) != m_type)
+            return false;
+
         return true;
     }
 
@@ -530,7 +553,7 @@ private:
     void filterArrayModesByType();
 
 #if USE(JSVALUE64) && !defined(NDEBUG)
-    void ensureCanInitializeWithZeros();
+    JS_EXPORT_PRIVATE void ensureCanInitializeWithZeros();
 #endif
 
     bool shouldBeClear() const;
@@ -544,12 +567,12 @@ private:
 namespace WTF {
 template <>
 struct VectorTraits<JSC::DFG::AbstractValue> : VectorTraitsBase<false, JSC::DFG::AbstractValue> {
-    static const bool canInitializeWithMemset = true;
+    static constexpr bool canInitializeWithMemset = true;
 };
 
 template <>
 struct HashTraits<JSC::DFG::AbstractValue> : GenericHashTraits<JSC::DFG::AbstractValue> {
-    static const bool emptyValueIsZero = true;
+    static constexpr bool emptyValueIsZero = true;
 };
 };
 #endif // USE(JSVALUE64)
