@@ -31,13 +31,15 @@
 
 namespace JSC { namespace DFG {
 
-class AdaptiveInferredPropertyValueWatchpoint : public AdaptiveInferredPropertyValueWatchpointBase {
+class AdaptiveInferredPropertyValueWatchpoint final : public AdaptiveInferredPropertyValueWatchpointBase {
 public:
     typedef AdaptiveInferredPropertyValueWatchpointBase Base;
     AdaptiveInferredPropertyValueWatchpoint(const ObjectPropertyCondition&, CodeBlock*);
 
 private:
-    void handleFire(VM&, const FireDetail&) override;
+    bool isValid() const final;
+
+    void handleFire(VM&, const FireDetail&) final;
 
     CodeBlock* m_codeBlock;
 };
