@@ -137,12 +137,15 @@ public:
         makeHeapTopForNode(edge.node());
     }
 
+    unsigned size() const { return m_block->valuesAtTail.size(); }
     unsigned numberOfArguments() const { return m_block->valuesAtTail.numberOfArguments(); }
     unsigned numberOfLocals() const { return m_block->valuesAtTail.numberOfLocals(); }
-    AbstractValue& operand(int operand) { return m_block->valuesAtTail.operand(operand); }
-    AbstractValue& operand(VirtualRegister operand) { return m_block->valuesAtTail.operand(operand); }
+    unsigned numberOfTmps() const { return m_block->valuesAtTail.numberOfTmps(); }
+    AbstractValue& atIndex(size_t index) { return m_block->valuesAtTail.at(index); }
+    AbstractValue& operand(Operand operand) { return m_block->valuesAtTail.operand(operand); }
     AbstractValue& local(size_t index) { return m_block->valuesAtTail.local(index); }
     AbstractValue& argument(size_t index) { return m_block->valuesAtTail.argument(index); }
+    AbstractValue& tmp(size_t index) { return m_block->valuesAtTail.tmp(index); }
 
     void clobberStructures()
     {
@@ -165,7 +168,7 @@ public:
     void setStructureClobberState(StructureClobberState state) { RELEASE_ASSERT(state == m_block->cfaStructureClobberStateAtTail); }
     void setIsValid(bool isValid) { m_block->cfaDidFinish = isValid; }
     void setBranchDirection(BranchDirection) { }
-    void setFoundConstants(bool) { }
+    void setShouldTryConstantFolding(bool) { }
 
     void trustEdgeProofs() { m_trustEdgeProofs = true; }
     void dontTrustEdgeProofs() { m_trustEdgeProofs = false; }
