@@ -98,9 +98,12 @@ void CommonData::removeDisposableCallSiteIndex(DisposableCallSiteIndex callSite)
 void CommonData::shrinkToFit()
 {
     codeOrigins.shrinkToFit();
+    dfgIdentifiers.shrinkToFit();
     weakReferences.shrinkToFit();
+    weakStructureReferences.shrinkToFit();
     transitions.shrinkToFit();
     catchEntrypoints.shrinkToFit();
+    jumpReplacements.shrinkToFit();
 }
 
 static Lock pcCodeBlockMapLock;
@@ -212,7 +215,7 @@ void CommonData::finalizeCatchEntrypoints()
     std::sort(catchEntrypoints.begin(), catchEntrypoints.end(),
         [] (const CatchEntrypointData& a, const CatchEntrypointData& b) { return a.bytecodeIndex < b.bytecodeIndex; });
 
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
     for (unsigned i = 0; i + 1 < catchEntrypoints.size(); ++i)
         ASSERT(catchEntrypoints[i].bytecodeIndex <= catchEntrypoints[i + 1].bytecodeIndex);
 #endif
