@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2015 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,9 +28,10 @@
 #if ENABLE(DFG_JIT)
 
 #include "DFGCommonData.h"
-#include "InferredValue.h"
+#include "FunctionExecutable.h"
 #include "JSArrayBufferView.h"
 #include "ObjectPropertyCondition.h"
+#include "SymbolTable.h"
 #include "Watchpoint.h"
 #include <wtf/CommaPrinter.h>
 #include <wtf/HashSet.h>
@@ -38,12 +39,18 @@
 namespace JSC { namespace DFG {
 
 class Graph;
+struct Prefix;
 
 template<typename T>
 struct SetPointerAdaptor {
     static void add(CodeBlock* codeBlock, T set, CommonData& common)
     {
-        return set->add(common.watchpoints.add(codeBlock));
+        CodeBlockJettisoningWatchpoint* watchpoint = nullptr;
+        {
+            ConcurrentJSLocker locker(codeBlock->m_lock);
+            watchpoint = common.watchpoints.add(codeBlock);
+        }
+        return set->add(WTFMove(watchpoint));
     }
     static bool hasBeenInvalidated(T set)
     {
@@ -55,15 +62,27 @@ struct SetPointerAdaptor {
     }
 };
 
-struct InferredValueAdaptor {
-    static void add(CodeBlock*, InferredValue*, CommonData&);
-    static bool hasBeenInvalidated(InferredValue* inferredValue)
+struct SymbolTableAdaptor {
+    static void add(CodeBlock*, SymbolTable*, CommonData&);
+    static bool hasBeenInvalidated(SymbolTable* symbolTable)
     {
-        return inferredValue->hasBeenInvalidated();
+        return symbolTable->singleton().hasBeenInvalidated();
     }
-    static void dumpInContext(PrintStream& out, InferredValue* inferredValue, DumpContext*)
+    static void dumpInContext(PrintStream& out, SymbolTable* symbolTable, DumpContext*)
     {
-        out.print(RawPointer(inferredValue));
+        out.print(RawPointer(symbolTable));
+    }
+};
+
+struct FunctionExecutableAdaptor {
+    static void add(CodeBlock*, FunctionExecutable*, CommonData&);
+    static bool hasBeenInvalidated(FunctionExecutable* executable)
+    {
+        return executable->singleton().hasBeenInvalidated();
+    }
+    static void dumpInContext(PrintStream& out, FunctionExecutable* executable, DumpContext*)
+    {
+        out.print(RawPointer(executable));
     }
 };
 
@@ -94,7 +113,7 @@ struct AdaptiveStructureWatchpointAdaptor {
 
 template<typename WatchpointSetType, typename Adaptor = SetPointerAdaptor<WatchpointSetType>>
 class GenericDesiredWatchpoints {
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
     typedef HashMap<WatchpointSetType, bool> StateMap;
 #endif
 public:
@@ -154,7 +173,8 @@ public:
 
     void addLazily(WatchpointSet*);
     void addLazily(InlineWatchpointSet&);
-    void addLazily(InferredValue*);
+    void addLazily(SymbolTable*);
+    void addLazily(FunctionExecutable*);
     void addLazily(JSArrayBufferView*);
 
     // It's recommended that you don't call this directly. Use Graph::watchCondition(), which does
@@ -175,9 +195,13 @@ public:
     {
         return m_inlineSets.isWatched(&set);
     }
-    bool isWatched(InferredValue* inferredValue)
+    bool isWatched(SymbolTable* symbolTable)
     {
-        return m_inferredValues.isWatched(inferredValue);
+        return m_symbolTables.isWatched(symbolTable);
+    }
+    bool isWatched(FunctionExecutable* executable)
+    {
+        return m_functionExecutables.isWatched(executable);
     }
     bool isWatched(JSArrayBufferView* view)
     {
@@ -188,12 +212,12 @@ public:
         return m_adaptiveStructureSets.isWatched(key);
     }
     void dumpInContext(PrintStream&, DumpContext*) const;
-    void dump(PrintStream&) const;
 
 private:
     GenericDesiredWatchpoints<WatchpointSet*> m_sets;
     GenericDesiredWatchpoints<InlineWatchpointSet*> m_inlineSets;
-    GenericDesiredWatchpoints<InferredValue*, InferredValueAdaptor> m_inferredValues;
+    GenericDesiredWatchpoints<SymbolTable*, SymbolTableAdaptor> m_symbolTables;
+    GenericDesiredWatchpoints<FunctionExecutable*, FunctionExecutableAdaptor> m_functionExecutables;
     GenericDesiredWatchpoints<JSArrayBufferView*, ArrayBufferViewWatchpointAdaptor> m_bufferViews;
     GenericDesiredWatchpoints<ObjectPropertyCondition, AdaptiveStructureWatchpointAdaptor> m_adaptiveStructureSets;
 };
