@@ -168,6 +168,10 @@ void printInternal(PrintStream& out, LocationKind kind)
         out.print("RegExpObjectLastIndexLoc");
         return;
 
+    case DateFieldLoc:
+        out.print("DateFieldLoc");
+        return;
+
     case MapBucketLoc:
         out.print("MapBucketLoc");
         return;
@@ -192,6 +196,10 @@ void printInternal(PrintStream& out, LocationKind kind)
         out.print("WeakMapGetLoc");
         return;
 
+    case InternalFieldObjectLoc:
+        out.print("InternalFieldObjectLoc");
+        return;
+
     case DOMStateLoc:
         out.print("DOMStateLoc");
         return;
