@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -55,6 +55,7 @@ enum LocationKind {
     IndexedPropertyStorageLoc,
     InvalidationPointLoc,
     IsFunctionLoc,
+    IsConstructorLoc,
     IsObjectOrNullLoc,
     NamedPropertyLoc,
     RegExpObjectLastIndexLoc,
@@ -64,12 +65,14 @@ enum LocationKind {
     PrototypeLoc,
     StackLoc,
     StackPayloadLoc,
+    DateFieldLoc,
     MapBucketLoc,
     MapBucketHeadLoc,
     MapBucketValueLoc,
     MapBucketKeyLoc,
     MapBucketNextLoc,
     WeakMapGetLoc,
+    InternalFieldObjectLoc,
     DOMStateLoc,
 };
 
@@ -148,7 +151,7 @@ private:
 struct HeapLocationHash {
     static unsigned hash(const HeapLocation& key) { return key.hash(); }
     static bool equal(const HeapLocation& a, const HeapLocation& b) { return a == b; }
-    static const bool safeToCompareToEmptyOrDeleted = true;
+    static constexpr bool safeToCompareToEmptyOrDeleted = true;
 };
 
 LocationKind indexedPropertyLocForResultType(NodeFlags);
@@ -183,13 +186,11 @@ namespace WTF {
 void printInternal(PrintStream&, JSC::DFG::LocationKind);
 
 template<typename T> struct DefaultHash;
-template<> struct DefaultHash<JSC::DFG::HeapLocation> {
-    typedef JSC::DFG::HeapLocationHash Hash;
-};
+template<> struct DefaultHash<JSC::DFG::HeapLocation> : JSC::DFG::HeapLocationHash { };
 
 template<typename T> struct HashTraits;
 template<> struct HashTraits<JSC::DFG::HeapLocation> : SimpleClassHashTraits<JSC::DFG::HeapLocation> {
-    static const bool emptyValueIsZero = false;
+    static constexpr bool emptyValueIsZero = false;
 };
 
 } // namespace WTF
