@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,16 +31,12 @@
 #include "DFGGraph.h"
 #include "DFGInsertionSet.h"
 #include "DFGPhase.h"
-#include "DFGPredictionPropagationPhase.h"
-#include "DFGVariableAccessDataDump.h"
-#include "JSCInlines.h"
-#include <wtf/HashMethod.h>
-#include <wtf/StdUnorderedMap.h>
+#include "JSCJSValueInlines.h"
 
 namespace JSC { namespace DFG {
 
 namespace DFGIntegerCheckCombiningPhaseInternal {
-static const bool verbose = false;
+static constexpr bool verbose = false;
 }
 
 class IntegerCheckCombiningPhase : public Phase {
@@ -61,7 +57,7 @@ public:
             RangeKey result;
             result.m_kind = Addition;
             result.m_source = edge.sanitized();
-            result.m_key = 0;
+            result.m_key = nullptr;
             return result;
         }
 
@@ -145,8 +141,8 @@ public:
         }
 
         int32_t m_minBound { 0 };
-        CodeOrigin m_minOrigin;
         int32_t m_maxBound { 0 };
+        CodeOrigin m_minOrigin;
         CodeOrigin m_maxOrigin;
         unsigned m_count { 0 }; // If this is zero then the bounds won't necessarily make sense.
         bool m_hoisted { false };
