@@ -30,11 +30,10 @@
 
 #include "DFGBasicBlockInlines.h"
 #include "DFGBlockInsertionSet.h"
-#include "DFGCFG.h"
 #include "DFGGraph.h"
 #include "DFGLoopPreHeaderCreationPhase.h"
 #include "DFGPhase.h"
-#include "JSCInlines.h"
+#include "JSCJSValueInlines.h"
 
 namespace JSC { namespace DFG {
 
@@ -50,16 +49,16 @@ public:
         RELEASE_ASSERT(m_graph.m_plan.mode() == FTLForOSREntryMode);
         RELEASE_ASSERT(m_graph.m_form == ThreadedCPS);
 
-        unsigned bytecodeIndex = m_graph.m_plan.osrEntryBytecodeIndex();
+        BytecodeIndex bytecodeIndex = m_graph.m_plan.osrEntryBytecodeIndex();
         RELEASE_ASSERT(bytecodeIndex);
-        RELEASE_ASSERT(bytecodeIndex != UINT_MAX);
+        RELEASE_ASSERT(bytecodeIndex.offset());
 
         // Needed by createPreHeader().
         m_graph.ensureCPSDominators();
 
         CodeBlock* baseline = m_graph.m_profiledBlock;
 
-        BasicBlock* target = 0;
+        BasicBlock* target = nullptr;
         for (unsigned blockIndex = m_graph.numBlocks(); blockIndex--;) {
             BasicBlock* block = m_graph.block(blockIndex);
             if (!block)
@@ -92,7 +91,7 @@ public:
         BasicBlock* newRoot = insertionSet.insert(0, 1);
 
         // We'd really like to use an unset origin, but ThreadedCPS won't allow that.
-        NodeOrigin origin = NodeOrigin(CodeOrigin(0), CodeOrigin(0), false);
+        NodeOrigin origin = NodeOrigin(CodeOrigin(BytecodeIndex(0)), CodeOrigin(BytecodeIndex(0)), false);
 
         Vector<Node*> locals(baseline->numCalleeLocals());
         for (int local = 0; local < baseline->numCalleeLocals(); ++local) {
@@ -102,10 +101,10 @@ public:
             VariableAccessData* variable = previousHead->variableAccessData();
             locals[local] = newRoot->appendNode(
                 m_graph, variable->prediction(), ExtractOSREntryLocal, origin,
-                OpInfo(variable->local().offset()));
+                OpInfo(variable->operand().virtualRegister()));
 
             newRoot->appendNode(
-                m_graph, SpecNone, MovHint, origin, OpInfo(variable->local().offset()),
+                m_graph, SpecNone, MovHint, origin, OpInfo(variable->operand().virtualRegister()),
                 Edge(locals[local]));
         }
 
@@ -117,11 +116,11 @@ public:
         for (int argument = 0; argument < baseline->numParameters(); ++argument) {
             Node* oldNode = target->variablesAtHead.argument(argument);
             if (!oldNode) {
-                // Just for sanity, always have a SetArgument even if it's not needed.
+                // Just for sanity, always have a SetArgumentDefinitely even if it's not needed.
                 oldNode = newArguments[argument];
             }
             Node* node = newRoot->appendNode(
-                m_graph, SpecNone, SetArgument, origin,
+                m_graph, SpecNone, SetArgumentDefinitely, origin,
                 OpInfo(oldNode->variableAccessData()));
             newArguments[argument] = node;
         }
