@@ -53,7 +53,7 @@ public:
                 return;
             }
 
-            callIfAppropriate(m_read, VirtualRegister(heap.payload().value32()));
+            callIfAppropriate(m_read, heap.operand());
             return;
         }
 
@@ -68,7 +68,7 @@ public:
         // We expect stack writes to already be precisely characterized by DFG::clobberize().
         if (heap.kind() == Stack) {
             RELEASE_ASSERT(!heap.payload().isTop());
-            callIfAppropriate(m_unconditionalWrite, VirtualRegister(heap.payload().value32()));
+            callIfAppropriate(m_unconditionalWrite, heap.operand());
             return;
         }
 
@@ -87,12 +87,12 @@ public:
 
         RELEASE_ASSERT(location.heap().kind() == Stack);
 
-        m_def(VirtualRegister(location.heap().payload().value32()), node);
+        m_def(location.heap().operand(), node);
     }
 
 private:
     template<typename Functor>
-    void callIfAppropriate(const Functor& functor, VirtualRegister operand)
+    void callIfAppropriate(const Functor& functor, Operand operand)
     {
         if (operand.isLocal() && static_cast<unsigned>(operand.toLocal()) >= m_graph.block(0)->variablesAtHead.numberOfLocals())
             return;
@@ -109,15 +109,15 @@ private:
             if (!inlineCallFrame) {
                 // Read the outermost arguments and argument count.
                 for (unsigned i = numberOfArgumentsToSkip; i < static_cast<unsigned>(m_graph.m_codeBlock->numParameters()); i++)
-                    m_read(virtualRegisterForArgument(i));
-                m_read(VirtualRegister(CallFrameSlot::argumentCount));
+                    m_read(virtualRegisterForArgumentIncludingThis(i));
+                m_read(VirtualRegister(CallFrameSlot::argumentCountIncludingThis));
                 return;
             }
 
             for (unsigned i = numberOfArgumentsToSkip; i < inlineCallFrame->argumentsWithFixup.size(); i++)
-                m_read(VirtualRegister(inlineCallFrame->stackOffset + virtualRegisterForArgument(i).offset()));
+                m_read(VirtualRegister(inlineCallFrame->stackOffset + virtualRegisterForArgumentIncludingThis(i).offset()));
             if (inlineCallFrame->isVarargs())
-                m_read(VirtualRegister(inlineCallFrame->stackOffset + CallFrameSlot::argumentCount));
+                m_read(VirtualRegister(inlineCallFrame->stackOffset + CallFrameSlot::argumentCountIncludingThis));
         };
 
         auto readSpread = [&] (Node* spread) {
@@ -130,7 +130,7 @@ private:
                 // This reads from a constant buffer.
                 return;
             }
-            InlineCallFrame* inlineCallFrame = spread->child1()->origin.semantic.inlineCallFrame;
+            InlineCallFrame* inlineCallFrame = spread->child1()->origin.semantic.inlineCallFrame();
             unsigned numberOfArgumentsToSkip = spread->child1()->numberOfArgumentsToSkip();
             readFrame(inlineCallFrame, numberOfArgumentsToSkip);
         };
@@ -158,6 +158,7 @@ private:
         case CreateDirectArguments:
         case CreateScopedArguments:
         case CreateClonedArguments:
+        case CreateArgumentsButterfly:
         case PhantomDirectArguments:
         case PhantomClonedArguments:
         case GetRestLength:
@@ -192,9 +193,9 @@ private:
             } else {
                 InlineCallFrame* inlineCallFrame;
                 if (m_node->hasArgumentsChild() && m_node->argumentsChild())
-                    inlineCallFrame = m_node->argumentsChild()->origin.semantic.inlineCallFrame;
+                    inlineCallFrame = m_node->argumentsChild()->origin.semantic.inlineCallFrame();
                 else
-                    inlineCallFrame = m_node->origin.semantic.inlineCallFrame;
+                    inlineCallFrame = m_node->origin.semantic.inlineCallFrame();
 
                 unsigned numberOfArgumentsToSkip = 0;
                 if (m_node->op() == GetMyArgumentByVal || m_node->op() == GetMyArgumentByValOutOfBounds) {
@@ -220,19 +221,19 @@ private:
         }
 
         case GetArgument: {
-            InlineCallFrame* inlineCallFrame = m_node->origin.semantic.inlineCallFrame;
+            InlineCallFrame* inlineCallFrame = m_node->origin.semantic.inlineCallFrame();
             unsigned indexIncludingThis = m_node->argumentIndex();
             if (!inlineCallFrame) {
                 if (indexIncludingThis < static_cast<unsigned>(m_graph.m_codeBlock->numParameters()))
-                    m_read(virtualRegisterForArgument(indexIncludingThis));
-                m_read(VirtualRegister(CallFrameSlot::argumentCount));
+                    m_read(virtualRegisterForArgumentIncludingThis(indexIncludingThis));
+                m_read(VirtualRegister(CallFrameSlot::argumentCountIncludingThis));
                 break;
             }
 
             ASSERT_WITH_MESSAGE(inlineCallFrame->isVarargs(), "GetArgument is only used for InlineCallFrame if the call frame is varargs.");
             if (indexIncludingThis < inlineCallFrame->argumentsWithFixup.size())
-                m_read(VirtualRegister(inlineCallFrame->stackOffset + virtualRegisterForArgument(indexIncludingThis).offset()));
-            m_read(VirtualRegister(inlineCallFrame->stackOffset + CallFrameSlot::argumentCount));
+                m_read(VirtualRegister(inlineCallFrame->stackOffset + virtualRegisterForArgumentIncludingThis(indexIncludingThis).offset()));
+            m_read(VirtualRegister(inlineCallFrame->stackOffset + CallFrameSlot::argumentCountIncludingThis));
             break;
         }
 
@@ -240,7 +241,7 @@ private:
             // All of the outermost arguments, except this, are read in sloppy mode.
             if (!m_graph.m_codeBlock->isStrictMode()) {
                 for (unsigned i = m_graph.m_codeBlock->numParameters(); i--;)
-                    m_read(virtualRegisterForArgument(i));
+                    m_read(virtualRegisterForArgumentIncludingThis(i));
             }
 
             // The stack header is read.
@@ -248,15 +249,15 @@ private:
                 m_read(VirtualRegister(i));
 
             // Read all of the inline arguments and call frame headers that we didn't already capture.
-            for (InlineCallFrame* inlineCallFrame = m_node->origin.semantic.inlineCallFrame; inlineCallFrame; inlineCallFrame = inlineCallFrame->getCallerInlineFrameSkippingTailCalls()) {
+            for (InlineCallFrame* inlineCallFrame = m_node->origin.semantic.inlineCallFrame(); inlineCallFrame; inlineCallFrame = inlineCallFrame->getCallerInlineFrameSkippingTailCalls()) {
                 if (!inlineCallFrame->isStrictMode()) {
                     for (unsigned i = inlineCallFrame->argumentsWithFixup.size(); i--;)
-                        m_read(VirtualRegister(inlineCallFrame->stackOffset + virtualRegisterForArgument(i).offset()));
+                        m_read(VirtualRegister(inlineCallFrame->stackOffset + virtualRegisterForArgumentIncludingThis(i).offset()));
                 }
                 if (inlineCallFrame->isClosureCall)
                     m_read(VirtualRegister(inlineCallFrame->stackOffset + CallFrameSlot::callee));
                 if (inlineCallFrame->isVarargs())
-                    m_read(VirtualRegister(inlineCallFrame->stackOffset + CallFrameSlot::argumentCount));
+                    m_read(VirtualRegister(inlineCallFrame->stackOffset + CallFrameSlot::argumentCountIncludingThis));
             }
             break;
         } }
