@@ -32,8 +32,7 @@
 #include "DFGGraph.h"
 #include "DFGInsertionSet.h"
 #include "DFGPhase.h"
-#include "DFGVariableAccessDataDump.h"
-#include "JSCInlines.h"
+#include "JSCJSValueInlines.h"
 #include <wtf/HashMap.h>
 
 namespace JSC { namespace DFG {
@@ -50,7 +49,7 @@ struct CheckData {
     bool m_arrayModeHoistingOkay;
 
     CheckData()
-        : m_structure(0)
+        : m_structure(nullptr)
         , m_arrayModeIsValid(false)
         , m_arrayModeHoistingOkay(false)
     {
@@ -64,7 +63,7 @@ struct CheckData {
     }
 
     CheckData(ArrayMode arrayMode)
-        : m_structure(0)
+        : m_structure(nullptr)
         , m_arrayMode(arrayMode)
         , m_arrayModeIsValid(true)
         , m_arrayModeHoistingOkay(true)
@@ -122,10 +121,10 @@ public:
                 // cases where we need to append, we first carefully extract everything we need
                 // from the node, before doing any appending.
                 switch (node->op()) {
-                case SetArgument: {
+                case SetArgumentDefinitely: {
                     // Insert a GetLocal and a CheckStructure immediately following this
-                    // SetArgument, if the variable was a candidate for structure hoisting.
-                    // If the basic block previously only had the SetArgument as its
+                    // SetArgumentDefinitely, if the variable was a candidate for structure hoisting.
+                    // If the basic block previously only had the SetArgumentDefinitely as its
                     // variable-at-tail, then replace it with this GetLocal.
                     VariableAccessData* variable = node->variableAccessData();
                     HashMap<VariableAccessData*, CheckData>::iterator iter = m_map.find(variable);
@@ -143,33 +142,42 @@ public:
                     Node* getLocal = insertionSet.insertNode(
                         indexInBlock + 1, variable->prediction(), GetLocal, origin,
                         OpInfo(variable), Edge(node));
+
+                    auto needsEmptyCheck = [](Node* node) -> bool {
+                        if (!(SpecCellCheck & SpecEmpty))
+                            return false;
+                        VirtualRegister local = node->variableAccessData()->operand().virtualRegister();
+                        auto* inlineCallFrame = node->origin.semantic.inlineCallFrame();
+                        if ((local - (inlineCallFrame ? inlineCallFrame->stackOffset : 0)) == virtualRegisterForArgumentIncludingThis(0)) {
+                            // |this| can be the TDZ value. The call entrypoint won't have |this| as TDZ,
+                            // but a catch or a loop OSR entry may have |this| be TDZ.
+                            return true;
+                        }
+                        return false;
+                    };
+
                     if (iter->value.m_structure) {
                         auto checkOp = CheckStructure;
-                        if (SpecCellCheck & SpecEmpty) {
-                            VirtualRegister local = node->variableAccessData()->local();
-                            auto* inlineCallFrame = node->origin.semantic.inlineCallFrame;
-                            if ((local - (inlineCallFrame ? inlineCallFrame->stackOffset : 0)) == virtualRegisterForArgument(0)) {
-                                // |this| can be the TDZ value. The call entrypoint won't have |this| as TDZ,
-                                // but a catch or a loop OSR entry may have |this| be TDZ.
-                                checkOp = CheckStructureOrEmpty;
-                            }
-                        }
-
+                        if (needsEmptyCheck(node))
+                            checkOp = CheckStructureOrEmpty;
                         insertionSet.insertNode(
                             indexInBlock + 1, SpecNone, checkOp, origin,
                             OpInfo(m_graph.addStructureSet(iter->value.m_structure)),
                             Edge(getLocal, CellUse));
                     } else if (iter->value.m_arrayModeIsValid) {
                         ASSERT(iter->value.m_arrayModeHoistingOkay);
+                        auto checkOp = CheckArray;
+                        if (needsEmptyCheck(node))
+                            checkOp = CheckArrayOrEmpty;
                         insertionSet.insertNode(
-                            indexInBlock + 1, SpecNone, CheckArray, origin,
+                            indexInBlock + 1, SpecNone, checkOp, origin,
                             OpInfo(iter->value.m_arrayMode.asWord()),
                             Edge(getLocal, CellUse));
                     } else
                         RELEASE_ASSERT_NOT_REACHED();
 
-                    if (block->variablesAtTail.operand(variable->local()) == node)
-                        block->variablesAtTail.operand(variable->local()) = getLocal;
+                    if (block->variablesAtTail.operand(variable->operand()) == node)
+                        block->variablesAtTail.operand(variable->operand()) = getLocal;
 
                     m_graph.substituteGetLocal(*block, indexInBlock, variable, getLocal);
 
@@ -188,12 +196,12 @@ public:
                     NodeOrigin origin = node->origin;
                     Edge child1 = node->child1();
 
+                    // Note: On 64-bit platforms, cell checks allow the empty value to flow through.
+                    // This means that this structure/array check may see the empty value as input. We need
+                    // to emit a node that explicitly handles the empty value. Most of the time, CheckStructureOrEmpty/CheckArrayOrEmpty
+                    // will be folded to CheckStructure/CheckArray because AI proves that the incoming value is
+                    // definitely not empty.
                     if (iter->value.m_structure) {
-                        // Note: On 64-bit platforms, cell checks allow the empty value to flow through.
-                        // This means that this structure check may see the empty value as input. We need
-                        // to emit a node that explicitly handles the empty value. Most of the time, CheckStructureOrEmpty
-                        // will be folded to CheckStructure because AI proves that the incoming value is
-                        // definitely not empty.
                         insertionSet.insertNode(
                             indexForChecks, SpecNone, (SpecCellCheck & SpecEmpty) ? CheckStructureOrEmpty : CheckStructure,
                             originForChecks.withSemantic(origin.semantic),
@@ -202,7 +210,7 @@ public:
                     } else if (iter->value.m_arrayModeIsValid) {
                         ASSERT(iter->value.m_arrayModeHoistingOkay);
                         insertionSet.insertNode(
-                            indexForChecks, SpecNone, CheckArray,
+                            indexForChecks, SpecNone, (SpecCellCheck & SpecEmpty) ? CheckArrayOrEmpty : CheckArray,
                             originForChecks.withSemantic(origin.semantic),
                             OpInfo(iter->value.m_arrayMode.asWord()),
                             Edge(child1.node(), CellUse));
@@ -271,12 +279,14 @@ private:
                 case PutByValAlias:
                 case GetArrayLength:
                 case CheckArray:
+                case CheckNeutered:
                 case GetIndexedPropertyStorage:
                 case GetTypedArrayByteOffset:
                 case Phantom:
                 case MovHint:
                 case MultiGetByOffset:
                 case MultiPutByOffset:
+                case MultiDeleteByOffset:
                     // Don't count these uses.
                     break;
 
@@ -336,6 +346,7 @@ private:
                 }
 
                 case CheckStructure:
+                case CheckNeutered:
                 case GetByOffset:
                 case PutByOffset:
                 case PutStructure:
@@ -351,6 +362,7 @@ private:
                 case MovHint:
                 case MultiGetByOffset:
                 case MultiPutByOffset:
+                case MultiDeleteByOffset:
                     // Don't count these uses.
                     break;
 
@@ -446,7 +458,7 @@ private:
                 continue;
             const Operands<Optional<JSValue>>& mustHandleValues = m_graph.m_plan.mustHandleValues();
             for (size_t i = 0; i < mustHandleValues.size(); ++i) {
-                int operand = mustHandleValues.operandForIndex(i);
+                Operand operand = mustHandleValues.operandForIndex(i);
                 Node* node = block->variablesAtHead.operand(operand);
                 if (!node)
                     continue;
@@ -490,7 +502,7 @@ private:
             return;
         if (result.iterator->value.m_structure == structure.get())
             return;
-        result.iterator->value.m_structure = 0;
+        result.iterator->value.m_structure = nullptr;
     }
 
     void noticeStructureCheck(VariableAccessData* variable, RegisteredStructureSet set)
@@ -581,7 +593,7 @@ struct StructureTypeCheck {
 
     static void disableHoisting(CheckData& checkData)
     {
-        checkData.m_structure = 0;
+        checkData.m_structure = nullptr;
     }
 
     static bool isContravenedByValue(CheckData& checkData, JSValue value)
