@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,8 +28,6 @@
 
 #if ENABLE(DFG_JIT)
 
-#include "JSCInlines.h"
-
 namespace WTF {
 
 using namespace JSC::DFG;
@@ -97,12 +95,18 @@ void printInternal(PrintStream& out, UseKind useKind)
     case RegExpObjectUse:
         out.print("RegExpObject");
         return;
+    case PromiseObjectUse:
+        out.print("PromiseObject");
+        return;
     case ProxyObjectUse:
         out.print("ProxyObject");
         return;
     case DerivedArrayUse:
         out.print("DerivedArray");
         return;
+    case DateObjectUse:
+        out.print("DateObject");
+        return;
     case MapObjectUse:
         out.print("MapObject");
         return;
@@ -139,8 +143,14 @@ void printInternal(PrintStream& out, UseKind useKind)
     case SymbolUse:
         out.print("Symbol");
         return;
-    case BigIntUse:
-        out.print("BigInt");
+    case AnyBigIntUse:
+        out.print("AnyBigInt");
+        return;
+    case HeapBigIntUse:
+        out.print("HeapBigInt");
+        return;
+    case BigInt32Use:
+        out.print("BigInt32");
         return;
     case StringObjectUse:
         out.print("StringObject");
@@ -157,6 +167,9 @@ void printInternal(PrintStream& out, UseKind useKind)
     case NotCellUse:
         out.print("NotCell");
         return;
+    case NotCellNorBigIntUse:
+        out.print("NotCellNorBigInt");
+        return;
     case KnownOtherUse:
         out.print("KnownOther");
         return;
