@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -55,6 +55,7 @@ enum UseKind {
     ArrayUse,
     FunctionUse,
     FinalObjectUse,
+    PromiseObjectUse,
     RegExpObjectUse,
     ProxyObjectUse,
     DerivedArrayUse,
@@ -65,7 +66,10 @@ enum UseKind {
     KnownStringUse,
     KnownPrimitiveUse, // This bizarre type arises for op_strcat, which has a bytecode guarantee that it will only see primitives (i.e. not objects).
     SymbolUse,
-    BigIntUse,
+    AnyBigIntUse,
+    HeapBigIntUse,
+    BigInt32Use,
+    DateObjectUse,
     MapObjectUse,
     SetObjectUse,
     WeakMapObjectUse,
@@ -76,6 +80,7 @@ enum UseKind {
     NotStringVarUse,
     NotSymbolUse,
     NotCellUse,
+    NotCellNorBigIntUse,
     KnownOtherUse,
     OtherUse,
     MiscUse,
@@ -102,7 +107,7 @@ inline SpeculatedType typeFilterFor(UseKind useKind)
     case KnownInt32Use:
         return SpecInt32Only;
     case Int52RepUse:
-        return SpecAnyInt;
+        return SpecInt52Any;
     case AnyIntUse:
         return SpecInt32Only | SpecAnyIntAsDouble;
     case NumberUse:
@@ -150,8 +155,16 @@ inline SpeculatedType typeFilterFor(UseKind useKind)
         return SpecHeapTop & ~SpecObject;
     case SymbolUse:
         return SpecSymbol;
-    case BigIntUse:
+    case AnyBigIntUse:
         return SpecBigInt;
+    case HeapBigIntUse:
+        return SpecHeapBigInt;
+    case BigInt32Use:
+        return SpecBigInt32;
+    case PromiseObjectUse:
+        return SpecPromiseObject;
+    case DateObjectUse:
+        return SpecDateObject;
     case MapObjectUse:
         return SpecMapObject;
     case SetObjectUse:
@@ -172,6 +185,8 @@ inline SpeculatedType typeFilterFor(UseKind useKind)
         return ~SpecSymbol;
     case NotCellUse:
         return ~SpecCellCheck;
+    case NotCellNorBigIntUse:
+        return ~SpecCellCheck & ~SpecBigInt;
     case KnownOtherUse:
     case OtherUse:
         return SpecOther;
@@ -206,24 +221,6 @@ inline bool mayHaveTypeCheck(UseKind kind)
     return !shouldNotHaveTypeCheck(kind);
 }
 
-inline bool isNumerical(UseKind kind)
-{
-    switch (kind) {
-    case Int32Use:
-    case KnownInt32Use:
-    case NumberUse:
-    case RealNumberUse:
-    case Int52RepUse:
-    case DoubleRepUse:
-    case DoubleRepRealUse:
-    case AnyIntUse:
-    case DoubleRepAnyIntUse:
-        return true;
-    default:
-        return false;
-    }
-}
-
 inline bool isDouble(UseKind kind)
 {
     switch (kind) {
@@ -248,15 +245,17 @@ inline bool isCell(UseKind kind)
     case FunctionUse:
     case FinalObjectUse:
     case RegExpObjectUse:
+    case PromiseObjectUse:
     case ProxyObjectUse:
     case DerivedArrayUse:
     case StringIdentUse:
     case StringUse:
     case KnownStringUse:
     case SymbolUse:
-    case BigIntUse:
+    case HeapBigIntUse:
     case StringObjectUse:
     case StringOrStringObjectUse:
+    case DateObjectUse:
     case MapObjectUse:
     case SetObjectUse:
     case WeakMapObjectUse:
@@ -305,6 +304,7 @@ inline bool checkMayCrashIfInputIsEmpty(UseKind kind)
     case OtherUse:
     case MiscUse:
     case NotCellUse:
+    case NotCellNorBigIntUse:
         return false;
     default:
         return true;
