@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,34 +26,34 @@
 #include "config.h"
 #include "DFGWorklist.h"
 
-#include "CodeBlock.h"
 #include "DFGSafepoint.h"
 #include "DeferGC.h"
-#include "JSCInlines.h"
+#include "JSCellInlines.h"
 #include "ReleaseHeapAccessScope.h"
 #include <mutex>
+#include <wtf/CompilationThread.h>
 
 namespace JSC { namespace DFG {
 
 #if ENABLE(DFG_JIT)
 
-class Worklist::ThreadBody : public AutomaticThread {
+class Worklist::ThreadBody final : public AutomaticThread {
 public:
     ThreadBody(const AbstractLocker& locker, Worklist& worklist, ThreadData& data, Box<Lock> lock, Ref<AutomaticThreadCondition>&& condition, int relativePriority)
-        : AutomaticThread(locker, lock, WTFMove(condition))
+        : AutomaticThread(locker, lock, WTFMove(condition), ThreadType::Compiler)
         , m_worklist(worklist)
         , m_data(data)
         , m_relativePriority(relativePriority)
     {
     }
 
-    const char* name() const override
+    const char* name() const final
     {
         return m_worklist.m_threadName.data();
     }
 
-protected:
-    PollResult poll(const AbstractLocker& locker) override
+private:
+    PollResult poll(const AbstractLocker& locker) final
     {
         if (m_worklist.m_queue.isEmpty())
             return PollResult::Wait;
@@ -73,7 +73,7 @@ protected:
 
     class WorkScope;
     friend class WorkScope;
-    class WorkScope {
+    class WorkScope final {
     public:
         WorkScope(ThreadBody& thread)
             : m_thread(thread)
@@ -93,7 +93,7 @@ protected:
         ThreadBody& m_thread;
     };
 
-    WorkResult work() override
+    WorkResult work() final
     {
         WorkScope workScope(*this);
 
@@ -105,12 +105,11 @@ protected:
             m_plan->notifyCompiling();
         }
 
-        if (Options::verboseCompilationQueue())
-            dataLog(m_worklist, ": Compiling ", m_plan->key(), " asynchronously\n");
+        dataLogLnIf(Options::verboseCompilationQueue(), m_worklist, ": Compiling ", m_plan->key(), " asynchronously");
 
         // There's no way for the GC to be safepointing since we own rightToRun.
         if (m_plan->vm()->heap.worldIsStopped()) {
-            dataLog("Heap is stoped but here we are! (1)\n");
+            dataLog("Heap is stopped but here we are! (1)\n");
             RELEASE_ASSERT_NOT_REACHED();
         }
         m_plan->compileInThread(&m_data);
@@ -133,32 +132,29 @@ protected:
                 dataLog(": Compiled ", m_plan->key(), " asynchronously\n");
             }
 
-            m_worklist.m_readyPlans.append(m_plan);
-
             RELEASE_ASSERT(!m_plan->vm()->heap.worldIsStopped());
+            m_worklist.m_readyPlans.append(WTFMove(m_plan));
             m_worklist.m_planCompiled.notifyAll();
         }
 
         return WorkResult::Continue;
     }
 
-    void threadDidStart() override
+    void threadDidStart() final
     {
-        if (Options::verboseCompilationQueue())
-            dataLog(m_worklist, ": Thread started\n");
+        dataLogLnIf(Options::verboseCompilationQueue(), m_worklist, ": Thread started");
 
         if (m_relativePriority)
             Thread::current().changePriority(m_relativePriority);
 
-        m_compilationScope = std::make_unique<CompilationScope>();
+        m_compilationScope = makeUnique<CompilationScope>();
     }
 
-    void threadIsStopping(const AbstractLocker&) override
+    void threadIsStopping(const AbstractLocker&) final
     {
         // We're holding the Worklist::m_lock, so we should be careful not to deadlock.
 
-        if (Options::verboseCompilationQueue())
-            dataLog(m_worklist, ": Thread will stop\n");
+        dataLogLnIf(Options::verboseCompilationQueue(), m_worklist, ": Thread will stop");
 
         ASSERT(!m_plan);
 
@@ -166,7 +162,6 @@ protected:
         m_plan = nullptr;
     }
 
-private:
     Worklist& m_worklist;
     ThreadData& m_data;
     int m_relativePriority;
@@ -185,9 +180,8 @@ static CString createWorklistName(CString&& tierName)
 
 Worklist::Worklist(CString&& tierName)
     : m_threadName(createWorklistName(WTFMove(tierName)))
-    , m_lock(Box<Lock>::create())
     , m_planEnqueued(AutomaticThreadCondition::create())
-    , m_numberOfActiveThreads(0)
+    , m_lock(Box<Lock>::create())
 {
 }
 
@@ -215,7 +209,7 @@ void Worklist::finishCreation(unsigned numberOfThreads, int relativePriority)
 
 void Worklist::createNewThread(const AbstractLocker& locker, int relativePriority)
 {
-    std::unique_ptr<ThreadData> data = std::make_unique<ThreadData>(this);
+    std::unique_ptr<ThreadData> data = makeUnique<ThreadData>(this);
     data->m_thread = adoptRef(new ThreadBody(locker, *this, *data, m_lock, m_planEnqueued.copyRef(), relativePriority));
     m_threads.append(WTFMove(data));
 }
@@ -338,8 +332,7 @@ Worklist::State Worklist::completeAllReadyPlansForVM(VM& vm, CompilationKey requ
         RefPtr<Plan> plan = myReadyPlans.takeLast();
         CompilationKey currentKey = plan->key();
 
-        if (Options::verboseCompilationQueue())
-            dataLog(*this, ": Completing ", currentKey, "\n");
+        dataLogLnIf(Options::verboseCompilationQueue(), *this, ": Completing ", currentKey);
 
         RELEASE_ASSERT(plan->stage() == Plan::Ready);
 
@@ -381,7 +374,7 @@ void Worklist::resumeAllThreads()
 
 void Worklist::visitWeakReferences(SlotVisitor& visitor)
 {
-    VM* vm = visitor.heap()->vm();
+    VM* vm = &visitor.heap()->vm();
     {
         LockHolder locker(*m_lock);
         for (PlanMap::iterator iter = m_plans.begin(); iter != m_plans.end(); ++iter) {
@@ -640,7 +633,7 @@ Worklist* existingWorklistForIndexOrNull(unsigned index)
         return existingGlobalFTLWorklistOrNull();
     default:
         RELEASE_ASSERT_NOT_REACHED();
-        return 0;
+        return nullptr;
     }
 }
 
