@@ -28,11 +28,12 @@
 #if ENABLE(FTL_JIT)
 
 #include "CallFrame.h"
+#include "CodeLocation.h"
 #include "MacroAssemblerCodeRef.h"
 
 namespace JSC { namespace FTL {
 
-// Points to a function of prototype: EncodedJSValue (*)(ExecState*).
+// Points to a function of prototype: EncodedJSValue (*)(JSGlobalObject*).
 using GeneratedFunction = CodeLocationLabel<JSEntryPtrTag>;
 
 } } // namespace JSC::FTL
