@@ -28,17 +28,9 @@
 
 #if ENABLE(FTL_JIT)
 
-#include "AirGenerationContext.h"
 #include "B3StackmapGenerationParams.h"
-#include "B3StackmapValue.h"
-#include "CodeBlock.h"
-#include "DFGBasicBlock.h"
-#include "DFGNode.h"
-#include "FTLExitArgument.h"
 #include "FTLJITCode.h"
-#include "FTLLocation.h"
 #include "FTLState.h"
-#include "JSCInlines.h"
 
 namespace JSC { namespace FTL {
 
@@ -47,10 +39,10 @@ using namespace DFG;
 
 OSRExitDescriptor::OSRExitDescriptor(
     DataFormat profileDataFormat, MethodOfGettingAValueProfile valueProfile,
-    unsigned numberOfArguments, unsigned numberOfLocals)
+    unsigned numberOfArguments, unsigned numberOfLocals, unsigned numberOfTmps)
     : m_profileDataFormat(profileDataFormat)
     , m_valueProfile(valueProfile)
-    , m_values(numberOfArguments, numberOfLocals)
+    , m_values(numberOfArguments, numberOfLocals, numberOfTmps)
 {
 }
 
@@ -65,20 +57,20 @@ void OSRExitDescriptor::validateReferences(const TrackedReferences& trackedRefer
 
 Ref<OSRExitHandle> OSRExitDescriptor::emitOSRExit(
     State& state, ExitKind exitKind, const NodeOrigin& nodeOrigin, CCallHelpers& jit,
-    const StackmapGenerationParams& params, unsigned offset)
+    const StackmapGenerationParams& params, uint32_t dfgNodeIndex, unsigned offset)
 {
     Ref<OSRExitHandle> handle =
-        prepareOSRExitHandle(state, exitKind, nodeOrigin, params, offset);
+        prepareOSRExitHandle(state, exitKind, nodeOrigin, params, dfgNodeIndex, offset);
     handle->emitExitThunk(state, jit);
     return handle;
 }
 
 Ref<OSRExitHandle> OSRExitDescriptor::emitOSRExitLater(
     State& state, ExitKind exitKind, const NodeOrigin& nodeOrigin,
-    const StackmapGenerationParams& params, unsigned offset)
+    const StackmapGenerationParams& params, uint32_t dfgNodeIndex, unsigned offset)
 {
     RefPtr<OSRExitHandle> handle =
-        prepareOSRExitHandle(state, exitKind, nodeOrigin, params, offset);
+        prepareOSRExitHandle(state, exitKind, nodeOrigin, params, dfgNodeIndex, offset);
     params.addLatePath(
         [handle, &state] (CCallHelpers& jit) {
             handle->emitExitThunk(state, jit);
@@ -88,11 +80,11 @@ Ref<OSRExitHandle> OSRExitDescriptor::emitOSRExitLater(
 
 Ref<OSRExitHandle> OSRExitDescriptor::prepareOSRExitHandle(
     State& state, ExitKind exitKind, const NodeOrigin& nodeOrigin,
-    const StackmapGenerationParams& params, unsigned offset)
+    const StackmapGenerationParams& params, uint32_t dfgNodeIndex, unsigned offset)
 {
     unsigned index = state.jitCode->osrExit.size();
     OSRExit& exit = state.jitCode->osrExit.alloc(
-        this, exitKind, nodeOrigin.forExit, nodeOrigin.semantic, nodeOrigin.wasHoisted);
+        this, exitKind, nodeOrigin.forExit, nodeOrigin.semantic, nodeOrigin.wasHoisted, dfgNodeIndex);
     Ref<OSRExitHandle> handle = adoptRef(*new OSRExitHandle(index, exit));
     for (unsigned i = offset; i < params.size(); ++i)
         exit.m_valueReps.append(params[i]);
@@ -102,8 +94,8 @@ Ref<OSRExitHandle> OSRExitDescriptor::prepareOSRExitHandle(
 
 OSRExit::OSRExit(
     OSRExitDescriptor* descriptor, ExitKind exitKind, CodeOrigin codeOrigin,
-    CodeOrigin codeOriginForExitProfile, bool wasHoisted)
-    : OSRExitBase(exitKind, codeOrigin, codeOriginForExitProfile, wasHoisted)
+    CodeOrigin codeOriginForExitProfile, bool wasHoisted, uint32_t dfgNodeIndex)
+    : OSRExitBase(exitKind, codeOrigin, codeOriginForExitProfile, wasHoisted, dfgNodeIndex)
     , m_descriptor(descriptor)
 {
 }
@@ -117,5 +109,3 @@ CodeLocationJump<JSInternalPtrTag> OSRExit::codeLocationForRepatch(CodeBlock* ft
 } } // namespace JSC::FTL
 
 #endif // ENABLE(FTL_JIT)
-
-
