@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,12 +28,10 @@
 
 #if ENABLE(FTL_JIT)
 
-#include "CodeBlockWithJITType.h"
 #include "FTLForOSREntryJITCode.h"
 #include "FTLJITCode.h"
 #include "FTLJITFinalizer.h"
 #include "FTLPatchpointExceptionHandle.h"
-#include <stdio.h>
 
 namespace JSC { namespace FTL {
 
@@ -60,14 +58,14 @@ State::State(Graph& graph)
         break;
     }
 
-    graph.m_plan.setFinalizer(std::make_unique<JITFinalizer>(graph.m_plan));
+    graph.m_plan.setFinalizer(makeUnique<JITFinalizer>(graph.m_plan));
     finalizer = static_cast<JITFinalizer*>(graph.m_plan.finalizer());
 
-    proc = std::make_unique<Procedure>();
+    proc = makeUnique<Procedure>();
 
     proc->setOriginPrinter(
         [] (PrintStream& out, B3::Origin origin) {
-            out.print("DFG:", bitwise_cast<Node*>(origin.data()));
+            out.print(bitwise_cast<Node*>(origin.data()));
         });
 
     proc->setFrontendData(&graph);
