@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -53,6 +53,9 @@ struct FreeCell {
         return descramble(scrambledNext, secret);
     }
 
+    static ptrdiff_t offsetOfScrambledNext() { return OBJECT_OFFSETOF(FreeCell, scrambledNext); }
+
+    uint64_t preservedBitsForCrashAnalysis;
     uintptr_t scrambledNext;
 };
 
@@ -63,8 +66,8 @@ public:
 
     void clear();
 
-    void initializeList(FreeCell* head, uintptr_t secret, unsigned bytes);
-    void initializeBump(char* payloadEnd, unsigned remaining);
+    JS_EXPORT_PRIVATE void initializeList(FreeCell* head, uintptr_t secret, unsigned bytes);
+    JS_EXPORT_PRIVATE void initializeBump(char* payloadEnd, unsigned remaining);
 
     bool allocationWillFail() const { return !head() && !m_remaining; }
     bool allocationWillSucceed() const { return !allocationWillFail(); }
@@ -86,7 +89,7 @@ public:
     static ptrdiff_t offsetOfOriginalSize() { return OBJECT_OFFSETOF(FreeList, m_originalSize); }
     static ptrdiff_t offsetOfCellSize() { return OBJECT_OFFSETOF(FreeList, m_cellSize); }
 
-    void dump(PrintStream&) const;
+    JS_EXPORT_PRIVATE void dump(PrintStream&) const;
 
     unsigned cellSize() const { return m_cellSize; }
 
