@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,17 +26,16 @@
 #include "config.h"
 #include "IncrementalSweeper.h"
 
-#include "Heap.h"
-#include "JSObject.h"
-#include "JSString.h"
+#include "DeferGC.h"
+#include "HeapInlines.h"
 #include "MarkedBlock.h"
-#include "JSCInlines.h"
+#include "VM.h"
 
 namespace JSC {
 
-static const Seconds sweepTimeSlice = 10_ms;
-static const double sweepTimeTotal = .10;
-static const double sweepTimeMultiplier = 1.0 / sweepTimeTotal;
+static constexpr Seconds sweepTimeSlice = 10_ms;
+static constexpr double sweepTimeTotal = .10;
+static constexpr double sweepTimeMultiplier = 1.0 / sweepTimeTotal;
 
 void IncrementalSweeper::scheduleTimer()
 {
