@@ -25,21 +25,28 @@
 
 #include "config.h"
 #include "IsoAlignedMemoryAllocator.h"
+#include "MarkedBlock.h"
 
 namespace JSC {
 
-IsoAlignedMemoryAllocator::IsoAlignedMemoryAllocator()
+IsoAlignedMemoryAllocator::IsoAlignedMemoryAllocator(CString name)
+#if ENABLE(MALLOC_HEAP_BREAKDOWN)
+    : m_debugHeap(name.data())
+#endif
 {
+    UNUSED_PARAM(name);
 }
 
 IsoAlignedMemoryAllocator::~IsoAlignedMemoryAllocator()
 {
+#if !ENABLE(MALLOC_HEAP_BREAKDOWN)
     for (unsigned i = 0; i < m_blocks.size(); ++i) {
         void* block = m_blocks[i];
         if (!m_committed[i])
             WTF::fastCommitAlignedMemory(block, MarkedBlock::blockSize);
         fastAlignedFree(block);
     }
+#endif
 }
 
 void* IsoAlignedMemoryAllocator::tryAllocateAlignedMemory(size_t alignment, size_t size)
@@ -49,6 +56,9 @@ void* IsoAlignedMemoryAllocator::tryAllocateAlignedMemory(size_t alignment, size
     RELEASE_ASSERT(alignment == MarkedBlock::blockSize);
     RELEASE_ASSERT(size == MarkedBlock::blockSize);
 
+#if ENABLE(MALLOC_HEAP_BREAKDOWN)
+    return m_debugHeap.memalign(MarkedBlock::blockSize, MarkedBlock::blockSize, true);
+#else
     auto locker = holdLock(m_lock);
 
     m_firstUncommitted = m_committed.findBit(m_firstUncommitted, false);
@@ -69,10 +79,14 @@ void* IsoAlignedMemoryAllocator::tryAllocateAlignedMemory(size_t alignment, size
         m_committed.resize(m_blocks.capacity());
     m_committed[index] = true;
     return result;
+#endif
 }
 
 void IsoAlignedMemoryAllocator::freeAlignedMemory(void* basePtr)
 {
+#if ENABLE(MALLOC_HEAP_BREAKDOWN)
+    m_debugHeap.free(basePtr);
+#else
     auto locker = holdLock(m_lock);
 
     auto iter = m_blockIndices.find(basePtr);
@@ -81,6 +95,7 @@ void IsoAlignedMemoryAllocator::freeAlignedMemory(void* basePtr)
     m_committed[index] = false;
     m_firstUncommitted = std::min(index, m_firstUncommitted);
     WTF::fastDecommitAlignedMemory(basePtr, MarkedBlock::blockSize);
+#endif
 }
 
 void IsoAlignedMemoryAllocator::dump(PrintStream& out) const
@@ -88,5 +103,29 @@ void IsoAlignedMemoryAllocator::dump(PrintStream& out) const
     out.print("Iso(", RawPointer(this), ")");
 }
 
+void* IsoAlignedMemoryAllocator::tryAllocateMemory(size_t size)
+{
+#if ENABLE(MALLOC_HEAP_BREAKDOWN)
+    return m_debugHeap.malloc(size);
+#else
+    return FastMalloc::tryMalloc(size);
+#endif
+}
+
+void IsoAlignedMemoryAllocator::freeMemory(void* pointer)
+{
+#if ENABLE(MALLOC_HEAP_BREAKDOWN)
+    m_debugHeap.free(pointer);
+#else
+    FastMalloc::free(pointer);
+#endif
+}
+
+void* IsoAlignedMemoryAllocator::tryReallocateMemory(void*, size_t)
+{
+    // In IsoSubspace-managed PreciseAllocation, we must not perform realloc.
+    RELEASE_ASSERT_NOT_REACHED();
+}
+
 } // namespace JSC
 
