@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,23 +27,23 @@
 #include "MarkedBlock.h"
 
 #include "AlignedMemoryAllocator.h"
-#include "BlockDirectoryInlines.h"
 #include "FreeListInlines.h"
-#include "JSCast.h"
-#include "JSDestructibleObject.h"
-#include "JSCInlines.h"
+#include "JSCJSValueInlines.h"
 #include "MarkedBlockInlines.h"
-#include "SuperSampler.h"
 #include "SweepingScope.h"
 #include <wtf/CommaPrinter.h>
 
 namespace JSC {
+namespace MarkedBlockInternal {
+static constexpr bool verbose = false;
+}
 
-const size_t MarkedBlock::blockSize;
-
-static const bool computeBalance = false;
+static constexpr bool computeBalance = false;
 static size_t balance;
 
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(MarkedBlock);
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(MarkedBlockHandle);
+
 MarkedBlock::Handle* MarkedBlock::tryCreate(Heap& heap, AlignedMemoryAllocator* alignedMemoryAllocator)
 {
     if (computeBalance) {
@@ -61,11 +61,9 @@ MarkedBlock::Handle* MarkedBlock::tryCreate(Heap& heap, AlignedMemoryAllocator*
 
 MarkedBlock::Handle::Handle(Heap& heap, AlignedMemoryAllocator* alignedMemoryAllocator, void* blockSpace)
     : m_alignedMemoryAllocator(alignedMemoryAllocator)
-    , m_weakSet(heap.vm(), CellContainer())
+    , m_weakSet(heap.vm())
 {
-    m_block = new (NotNull, blockSpace) MarkedBlock(*heap.vm(), *this);
-
-    m_weakSet.setContainer(*m_block);
+    m_block = new (NotNull, blockSpace) MarkedBlock(heap.vm(), *this);
 
     heap.didAllocateBlock(blockSize);
 }
@@ -87,7 +85,7 @@ MarkedBlock::Handle::~Handle()
 MarkedBlock::MarkedBlock(VM& vm, Handle& handle)
 {
     new (&footer()) Footer(vm, handle);
-    if (false)
+    if (MarkedBlockInternal::verbose)
         dataLog(RawPointer(this), ": Allocated.\n");
 }
 
@@ -114,22 +112,16 @@ void MarkedBlock::Handle::unsweepWithNoNewlyAllocated()
     m_isFreeListed = false;
 }
 
-void MarkedBlock::Handle::setIsFreeListed()
-{
-    m_directory->setIsEmpty(NoLockingNecessary, this, false);
-    m_isFreeListed = true;
-}
-
 void MarkedBlock::Handle::stopAllocating(const FreeList& freeList)
 {
     auto locker = holdLock(blockFooter().m_lock);
 
-    if (false)
+    if (MarkedBlockInternal::verbose)
         dataLog(RawPointer(this), ": MarkedBlock::Handle::stopAllocating!\n");
     ASSERT(!directory()->isAllocated(NoLockingNecessary, this));
 
     if (!isFreeListed()) {
-        if (false)
+        if (MarkedBlockInternal::verbose)
             dataLog("There ain't no newly allocated.\n");
         // This means that we either didn't use this block at all for allocation since last GC,
         // or someone had already done stopAllocating() before.
@@ -137,7 +129,7 @@ void MarkedBlock::Handle::stopAllocating(const FreeList& freeList)
         return;
     }
 
-    if (false)
+    if (MarkedBlockInternal::verbose)
         dataLog("Free list: ", freeList, "\n");
 
     // Roll back to a coherent state for Heap introspection. Cells newly
@@ -148,17 +140,17 @@ void MarkedBlock::Handle::stopAllocating(const FreeList& freeList)
     blockFooter().m_newlyAllocatedVersion = heap()->objectSpace().newlyAllocatedVersion();
 
     forEachCell(
-        [&] (HeapCell* cell, HeapCell::Kind) -> IterationStatus {
+        [&] (size_t, HeapCell* cell, HeapCell::Kind) -> IterationStatus {
             block().setNewlyAllocated(cell);
             return IterationStatus::Continue;
         });
 
     freeList.forEach(
         [&] (HeapCell* cell) {
-            if (false)
+            if (MarkedBlockInternal::verbose)
                 dataLog("Free cell: ", RawPointer(cell), "\n");
             if (m_attributes.destruction == NeedsDestruction)
-                cell->zap();
+                cell->zap(HeapCell::StopAllocating);
             block().clearNewlyAllocated(cell);
         });
 
@@ -183,13 +175,13 @@ void MarkedBlock::Handle::resumeAllocating(FreeList& freeList)
     {
         auto locker = holdLock(blockFooter().m_lock);
 
-        if (false)
+        if (MarkedBlockInternal::verbose)
             dataLog(RawPointer(this), ": MarkedBlock::Handle::resumeAllocating!\n");
         ASSERT(!directory()->isAllocated(NoLockingNecessary, this));
         ASSERT(!isFreeListed());
 
         if (!block().hasAnyNewlyAllocated()) {
-            if (false)
+            if (MarkedBlockInternal::verbose)
                 dataLog("There ain't no newly allocated.\n");
             // This means we had already exhausted the block when we stopped allocation.
             freeList.clear();
@@ -202,18 +194,9 @@ void MarkedBlock::Handle::resumeAllocating(FreeList& freeList)
     sweep(&freeList);
 }
 
-void MarkedBlock::Handle::zap(const FreeList& freeList)
-{
-    freeList.forEach(
-        [&] (HeapCell* cell) {
-            if (m_attributes.destruction == NeedsDestruction)
-                cell->zap();
-        });
-}
-
 void MarkedBlock::aboutToMarkSlow(HeapVersion markingVersion)
 {
-    ASSERT(vm()->heap.objectSpace().isMarking());
+    ASSERT(vm().heap.objectSpace().isMarking());
     auto locker = holdLock(footer().m_lock);
 
     if (!areMarksStale(markingVersion))
@@ -223,7 +206,7 @@ void MarkedBlock::aboutToMarkSlow(HeapVersion markingVersion)
 
     if (handle().directory()->isAllocated(holdLock(directory->bitvectorLock()), &handle())
         || !marksConveyLivenessDuringMarking(markingVersion)) {
-        if (false)
+        if (MarkedBlockInternal::verbose)
             dataLog(RawPointer(this), ": Clearing marks without doing anything else.\n");
         // We already know that the block is full and is already recognized as such, or that the
         // block did not survive the previous GC. So, we can clear mark bits the old fashioned
@@ -233,7 +216,7 @@ void MarkedBlock::aboutToMarkSlow(HeapVersion markingVersion)
         // we created a newlyAllocated.
         footer().m_marks.clearAll();
     } else {
-        if (false)
+        if (MarkedBlockInternal::verbose)
             dataLog(RawPointer(this), ": Doing things.\n");
         HeapVersion newlyAllocatedVersion = space()->newlyAllocatedVersion();
         if (footer().m_newlyAllocatedVersion == newlyAllocatedVersion) {
@@ -278,16 +261,16 @@ void MarkedBlock::resetMarks()
     footer().m_markingVersion = MarkedSpace::nullVersion;
 }
 
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
 void MarkedBlock::assertMarksNotStale()
 {
-    ASSERT(footer().m_markingVersion == vm()->heap.objectSpace().markingVersion());
+    ASSERT(footer().m_markingVersion == vm().heap.objectSpace().markingVersion());
 }
-#endif // !ASSERT_DISABLED
+#endif // ASSERT_ENABLED
 
 bool MarkedBlock::areMarksStale()
 {
-    return areMarksStale(vm()->heap.objectSpace().markingVersion());
+    return areMarksStale(vm().heap.objectSpace().markingVersion());
 }
 
 bool MarkedBlock::Handle::areMarksStale()
@@ -297,13 +280,13 @@ bool MarkedBlock::Handle::areMarksStale()
 
 bool MarkedBlock::isMarked(const void* p)
 {
-    return isMarked(vm()->heap.objectSpace().markingVersion(), p);
+    return isMarked(vm().heap.objectSpace().markingVersion(), p);
 }
 
 void MarkedBlock::Handle::didConsumeFreeList()
 {
     auto locker = holdLock(blockFooter().m_lock);
-    if (false)
+    if (MarkedBlockInternal::verbose)
         dataLog(RawPointer(this), ": MarkedBlock::Handle::didConsumeFreeList!\n");
     ASSERT(isFreeListed());
     m_isFreeListed = false;
@@ -334,9 +317,9 @@ void MarkedBlock::Handle::removeFromDirectory()
     m_directory->removeBlock(this);
 }
 
-void MarkedBlock::Handle::didAddToDirectory(BlockDirectory* directory, size_t index)
+void MarkedBlock::Handle::didAddToDirectory(BlockDirectory* directory, unsigned index)
 {
-    ASSERT(m_index == std::numeric_limits<size_t>::max());
+    ASSERT(m_index == std::numeric_limits<unsigned>::max());
     ASSERT(!m_directory);
 
     RELEASE_ASSERT(directory->subspace()->alignedMemoryAllocator() == m_alignedMemoryAllocator);
@@ -366,29 +349,29 @@ void MarkedBlock::Handle::didAddToDirectory(BlockDirectory* directory, size_t in
 
 void MarkedBlock::Handle::didRemoveFromDirectory()
 {
-    ASSERT(m_index != std::numeric_limits<size_t>::max());
+    ASSERT(m_index != std::numeric_limits<unsigned>::max());
     ASSERT(m_directory);
 
-    m_index = std::numeric_limits<size_t>::max();
+    m_index = std::numeric_limits<unsigned>::max();
     m_directory = nullptr;
     blockFooter().m_subspace = nullptr;
 }
 
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
 void MarkedBlock::assertValidCell(VM& vm, HeapCell* cell) const
 {
-    RELEASE_ASSERT(&vm == this->vm());
+    RELEASE_ASSERT(&vm == &this->vm());
     RELEASE_ASSERT(const_cast<MarkedBlock*>(this)->handle().cellAlign(cell) == cell);
 }
-#endif
+#endif // ASSERT_ENABLED
 
 void MarkedBlock::Handle::dumpState(PrintStream& out)
 {
     CommaPrinter comma;
     directory()->forEachBitVectorWithName(
         holdLock(directory()->bitvectorLock()),
-        [&] (FastBitVector& bitvector, const char* name) {
-            out.print(comma, name, ":", bitvector[index()] ? "YES" : "no");
+        [&](auto vectorRef, const char* name) {
+            out.print(comma, name, ":", vectorRef[index()] ? "YES" : "no");
         });
 }
 
