@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,7 +34,7 @@ class Heap;
 class StopIfNecessaryTimer : public JSRunLoopTimer {
 public:
     using Base = JSRunLoopTimer;
-    explicit StopIfNecessaryTimer(VM*);
+    explicit StopIfNecessaryTimer(VM&);
 
     void doWork(VM&) override;
 
