@@ -34,17 +34,17 @@
 namespace JSC {
 
 namespace BinarySwitchInternal {
-static const bool verbose = false;
+static constexpr bool verbose = false;
 }
 
 static unsigned globalCounter; // We use a different seed every time we are invoked.
 
 BinarySwitch::BinarySwitch(GPRReg value, const Vector<int64_t>& cases, Type type)
-    : m_value(value)
+    : m_type(type)
+    , m_value(value)
     , m_weakRandom(globalCounter++)
     , m_index(0)
     , m_caseIndex(UINT_MAX)
-    , m_type(type)
 {
     if (cases.isEmpty())
         return;
@@ -60,7 +60,7 @@ BinarySwitch::BinarySwitch(GPRReg value, const Vector<int64_t>& cases, Type type
     if (BinarySwitchInternal::verbose)
         dataLog("Sorted cases: ", listDump(m_cases), "\n");
 
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
     for (unsigned i = 1; i < m_cases.size(); ++i)
         ASSERT(m_cases[i - 1] < m_cases[i], i, m_cases.size(), m_cases[i].value, m_cases[i].index);
 #endif
