@@ -42,11 +42,11 @@ extern "C" EncodedJSValue HOST_CALL_RETURN_VALUE_OPTION getHostCallReturnValue()
 #if COMPILER(GCC_COMPATIBLE)
 
 // This is a public declaration only to convince CLANG not to elide it.
-extern "C" EncodedJSValue HOST_CALL_RETURN_VALUE_OPTION getHostCallReturnValueWithExecState(ExecState*) REFERENCED_FROM_ASM WTF_INTERNAL;
+extern "C" EncodedJSValue HOST_CALL_RETURN_VALUE_OPTION getHostCallReturnValueWithExecState(CallFrame*) REFERENCED_FROM_ASM WTF_INTERNAL;
 
 inline void initializeHostCallReturnValue()
 {
-    getHostCallReturnValueWithExecState(0);
+    getHostCallReturnValueWithExecState(nullptr);
 }
 
 #else // COMPILER(GCC_COMPATIBLE)
