@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,9 +27,9 @@
 
 namespace JSC {
 
-class ExecState;
+class CallFrame;
 class VM;
 
-void genericUnwind(VM*, ExecState*);
+void genericUnwind(VM&, CallFrame*);
 
 } // namespace JSC
