@@ -27,6 +27,7 @@
 
 #if ENABLE(JIT)
 
+#include "BytecodeIndex.h"
 #include <wtf/AutomaticThread.h>
 #include <wtf/FastMalloc.h>
 #include <wtf/HashSet.h>
@@ -53,9 +54,8 @@ public:
     bool completeAllForVM(VM&); // Return true if any JIT work happened.
     void poll(VM&);
 
-    void compileLater(CodeBlock*, unsigned loopOSREntryBytecodeOffset = 0);
-
-    void compileNow(CodeBlock*, unsigned loopOSREntryBytecodeOffset = 0);
+    void compileLater(CodeBlock*, BytecodeIndex loopOSREntryBytecodeIndex = BytecodeIndex(0));
+    void compileNow(CodeBlock*, BytecodeIndex loopOSREntryBytecodeIndex = BytecodeIndex(0));
 
     static JITWorklist& ensureGlobalWorklist();
     static JITWorklist* existingGlobalWorklistOrNull();
