@@ -194,10 +194,10 @@ private:
         return !!(m_set[i >> 3] & (1 << (i & 7)));
     }
 
-    static const unsigned totalNumberOfRegisters =
+    static constexpr unsigned totalNumberOfRegisters =
         GPRInfo::numberOfRegisters + FPRInfo::numberOfRegisters;
 
-    static const unsigned numberOfBytesInTempRegisterSet =
+    static constexpr unsigned numberOfBytesInTempRegisterSet =
         (totalNumberOfRegisters + 7) >> 3;
 
     uint8_t m_set[numberOfBytesInTempRegisterSet];
