@@ -25,6 +25,8 @@
 
 #pragma once
 
+#include "GPRInfo.h"
+#include "Instruction.h"
 #include "JSCJSValue.h"
 #include "MacroAssemblerCodeRef.h"
 #include "Opcode.h"
@@ -32,7 +34,6 @@
 namespace JSC {
 
 class VM;
-struct Instruction;
 
 #if ENABLE(C_LOOP)
 typedef OpcodeID LLIntCode;
@@ -42,8 +43,9 @@ typedef void (*LLIntCode)();
 
 namespace LLInt {
 
-extern "C" JS_EXPORT_PRIVATE Opcode g_opcodeMap[numOpcodeIDs];
-extern "C" JS_EXPORT_PRIVATE Opcode g_opcodeMapWide[numOpcodeIDs];
+extern "C" JS_EXPORT_PRIVATE Opcode g_opcodeMap[numOpcodeIDs + numWasmOpcodeIDs];
+extern "C" JS_EXPORT_PRIVATE Opcode g_opcodeMapWide16[numOpcodeIDs + numWasmOpcodeIDs];
+extern "C" JS_EXPORT_PRIVATE Opcode g_opcodeMapWide32[numOpcodeIDs + numWasmOpcodeIDs];
 
 class Data {
 
@@ -52,16 +54,21 @@ public:
 
 private:
     static uint8_t s_exceptionInstructions[maxOpcodeLength + 1];
+    static uint8_t s_wasmExceptionInstructions[maxOpcodeLength + 1];
 
     friend void initialize();
 
     friend Instruction* exceptionInstructions();
+    friend Instruction* wasmExceptionInstructions();
     friend Opcode* opcodeMap();
-    friend Opcode* opcodeMapWide();
+    friend Opcode* opcodeMapWide16();
+    friend Opcode* opcodeMapWide32();
     friend Opcode getOpcode(OpcodeID);
-    friend Opcode getOpcodeWide(OpcodeID);
+    friend Opcode getOpcodeWide16(OpcodeID);
+    friend Opcode getOpcodeWide32(OpcodeID);
     template<PtrTag tag> friend MacroAssemblerCodePtr<tag> getCodePtr(OpcodeID);
-    template<PtrTag tag> friend MacroAssemblerCodePtr<tag> getWideCodePtr(OpcodeID);
+    template<PtrTag tag> friend MacroAssemblerCodePtr<tag> getWide16CodePtr(OpcodeID);
+    template<PtrTag tag> friend MacroAssemblerCodePtr<tag> getWide32CodePtr(OpcodeID);
     template<PtrTag tag> friend MacroAssemblerCodeRef<tag> getCodeRef(OpcodeID);
 };
 
@@ -72,14 +79,24 @@ inline Instruction* exceptionInstructions()
     return reinterpret_cast<Instruction*>(Data::s_exceptionInstructions);
 }
 
+inline Instruction* wasmExceptionInstructions()
+{
+    return bitwise_cast<Instruction*>(Data::s_wasmExceptionInstructions);
+}
+
 inline Opcode* opcodeMap()
 {
     return g_opcodeMap;
 }
 
-inline Opcode* opcodeMapWide()
+inline Opcode* opcodeMapWide16()
 {
-    return g_opcodeMapWide;
+    return g_opcodeMapWide16;
+}
+
+inline Opcode* opcodeMapWide32()
+{
+    return g_opcodeMapWide32;
 }
 
 inline Opcode getOpcode(OpcodeID id)
@@ -91,12 +108,21 @@ inline Opcode getOpcode(OpcodeID id)
 #endif
 }
 
-inline Opcode getOpcodeWide(OpcodeID id)
+inline Opcode getOpcodeWide16(OpcodeID id)
 {
 #if ENABLE(COMPUTED_GOTO_OPCODES)
-    return g_opcodeMapWide[id];
+    return g_opcodeMapWide16[id];
 #else
-    return static_cast<Opcode>(id - numOpcodeIDs);
+    return static_cast<Opcode>(id + numOpcodeIDs);
+#endif
+}
+
+inline Opcode getOpcodeWide32(OpcodeID id)
+{
+#if ENABLE(COMPUTED_GOTO_OPCODES)
+    return g_opcodeMapWide32[id];
+#else
+    return static_cast<Opcode>(id + numOpcodeIDs*2);
 #endif
 }
 
@@ -109,13 +135,31 @@ ALWAYS_INLINE MacroAssemblerCodePtr<tag> getCodePtr(OpcodeID opcodeID)
 }
 
 template<PtrTag tag>
-ALWAYS_INLINE MacroAssemblerCodePtr<tag> getWideCodePtr(OpcodeID opcodeID)
+ALWAYS_INLINE MacroAssemblerCodePtr<tag> getWide16CodePtr(OpcodeID opcodeID)
 {
-    void* address = reinterpret_cast<void*>(getOpcodeWide(opcodeID));
+    void* address = reinterpret_cast<void*>(getOpcodeWide16(opcodeID));
     address = retagCodePtr<BytecodePtrTag, tag>(address);
     return MacroAssemblerCodePtr<tag>::createFromExecutableAddress(address);
 }
 
+template<PtrTag tag>
+ALWAYS_INLINE MacroAssemblerCodePtr<tag> getWide32CodePtr(OpcodeID opcodeID)
+{
+    void* address = reinterpret_cast<void*>(getOpcodeWide32(opcodeID));
+    address = retagCodePtr<BytecodePtrTag, tag>(address);
+    return MacroAssemblerCodePtr<tag>::createFromExecutableAddress(address);
+}
+
+template<PtrTag tag>
+ALWAYS_INLINE MacroAssemblerCodePtr<tag> getCodePtr(const Instruction& instruction)
+{
+    if (instruction.isWide16())
+        return getWide16CodePtr<tag>(instruction.opcodeID());
+    if (instruction.isWide32())
+        return getWide32CodePtr<tag>(instruction.opcodeID());
+    return getCodePtr<tag>(instruction.opcodeID());
+}
+
 template<PtrTag tag>
 ALWAYS_INLINE MacroAssemblerCodeRef<tag> getCodeRef(OpcodeID opcodeID)
 {
@@ -140,9 +184,14 @@ ALWAYS_INLINE void* getCodePtr(OpcodeID id)
     return reinterpret_cast<void*>(getOpcode(id));
 }
 
-ALWAYS_INLINE void* getWideCodePtr(OpcodeID id)
+ALWAYS_INLINE void* getWide16CodePtr(OpcodeID id)
 {
-    return reinterpret_cast<void*>(getOpcodeWide(id));
+    return reinterpret_cast<void*>(getOpcodeWide16(id));
+}
+
+ALWAYS_INLINE void* getWide32CodePtr(OpcodeID id)
+{
+    return reinterpret_cast<void*>(getOpcodeWide32(id));
 }
 #endif
 
@@ -151,4 +200,24 @@ ALWAYS_INLINE void* getCodePtr(JSC::EncodedJSValue glueHelper())
     return bitwise_cast<void*>(glueHelper);
 }
 
+#if ENABLE(JIT)
+struct Registers {
+    static constexpr GPRReg pcGPR = GPRInfo::regT4;
+
+#if CPU(X86_64) && !OS(WINDOWS)
+    static constexpr GPRReg metadataTableGPR = GPRInfo::regCS1;
+    static constexpr GPRReg pbGPR = GPRInfo::regCS2;
+#elif CPU(X86_64) && OS(WINDOWS)
+    static constexpr GPRReg metadataTableGPR = GPRInfo::regCS3;
+    static constexpr GPRReg pbGPR = GPRInfo::regCS4;
+#elif CPU(ARM64)
+    static constexpr GPRReg metadataTableGPR = GPRInfo::regCS6;
+    static constexpr GPRReg pbGPR = GPRInfo::regCS7;
+#elif CPU(MIPS) || CPU(ARM_THUMB2)
+    static constexpr GPRReg metadataTableGPR = GPRInfo::regCS0;
+    static constexpr GPRReg pbGPR = GPRInfo::regCS1;
+#endif
+};
+#endif
+
 } } // namespace JSC::LLInt
