@@ -31,7 +31,7 @@ require "config"
 #
 # node.children -> Returns an array of immediate children.
 #
-# node.descendents -> Returns an array of all strict descendants (children
+# node.descendants -> Returns an array of all strict descendants (children
 #     and children of children, transitively).
 #
 # node.flatten -> Returns an array containing the strict descendants and
@@ -727,26 +727,31 @@ end
 class Variable < NoChildren
     attr_reader :name
     
-    def initialize(codeOrigin, name)
+    def initialize(codeOrigin, name, originalName = nil)
         super(codeOrigin)
         @name = name
+        @originalName = originalName
     end
     
     @@mapping = {}
     
-    def self.forName(codeOrigin, name)
+    def self.forName(codeOrigin, name, originalName = nil)
         unless @@mapping[name]
-            @@mapping[name] = Variable.new(codeOrigin, name)
+            @@mapping[name] = Variable.new(codeOrigin, name, originalName)
         end
         @@mapping[name]
     end
+
+    def originalName
+        @originalName || name
+    end
     
     def dump
-        name
+        originalName
     end
     
     def inspect
-        "<variable #{name} at #{codeOriginString}>"
+        "<variable #{originalName} at #{codeOriginString}>"
     end
 end
 
@@ -806,12 +811,16 @@ class BaseIndex < Node
         @base = base
         @index = index
         @scale = scale
-        raise unless [1, 2, 4, 8].member? @scale
         @offset = offset
     end
-    
+
+    def scaleValue
+        raise unless [1, 2, 4, 8].member? scale.value
+        scale.value
+    end
+
     def scaleShift
-        case scale
+        case scaleValue
         when 1
             0
         when 2
@@ -821,7 +830,7 @@ class BaseIndex < Node
         when 8
             3
         else
-            raise "Bad scale at #{codeOriginString}"
+            raise "Bad scale: #{scale.value} at #{codeOriginString}"
         end
     end
     
@@ -834,11 +843,11 @@ class BaseIndex < Node
     end
     
     def mapChildren
-        BaseIndex.new(codeOrigin, (yield @base), (yield @index), @scale, (yield @offset))
+        BaseIndex.new(codeOrigin, (yield @base), (yield @index), (yield @scale), (yield @offset))
     end
     
     def dump
-        "#{offset.dump}[#{base.dump}, #{index.dump}, #{scale}]"
+        "#{offset.dump}[#{base.dump}, #{index.dump}, #{scale.value}]"
     end
     
     def address?
@@ -908,7 +917,11 @@ class Instruction < Node
         @operands = operands
         @annotation = annotation
     end
-    
+
+    def cloneWithNewOperands(newOperands)
+        Instruction.new(self.codeOrigin, self.opcode, newOperands, self.annotation)
+    end
+
     def children
         operands
     end
@@ -929,6 +942,7 @@ class Instruction < Node
             $asm.putGlobalAnnotation
         when "emit"
             $asm.puts "#{operands[0].dump}"
+        when "tagReturnAddress", "untagReturnAddress", "removeCodePtrTag", "untagArrayPtr"
         else
             raise "Unhandled opcode #{opcode} at #{codeOriginString}"
         end
@@ -960,7 +974,7 @@ class Error < NoChildren
 end
 
 class ConstExpr < NoChildren
-    attr_reader :variable, :value
+    attr_reader :value
 
     def initialize(codeOrigin, value)
         super(codeOrigin)
@@ -1015,11 +1029,10 @@ $labelMapping = {}
 $referencedExternLabels = Array.new
 
 class Label < NoChildren
-    attr_reader :name
-    
-    def initialize(codeOrigin, name)
+    def initialize(codeOrigin, name, definedInFile = false)
         super(codeOrigin)
         @name = name
+        @definedInFile = definedInFile
         @extern = true
         @global = false
     end
@@ -1028,7 +1041,7 @@ class Label < NoChildren
         if $labelMapping[name]
             raise "Label name collision: #{name}" unless $labelMapping[name].is_a? Label
         else
-            $labelMapping[name] = Label.new(codeOrigin, name)
+            $labelMapping[name] = Label.new(codeOrigin, name, definedInFile)
         end
         if definedInFile
             $labelMapping[name].clearExtern()
@@ -1075,6 +1088,10 @@ class Label < NoChildren
         @global
     end
 
+    def name
+        @name
+    end
+
     def dump
         "#{name}:"
     end
@@ -1101,6 +1118,9 @@ class LocalLabel < NoChildren
     
     def self.unique(comment)
         newName = "_#{comment}"
+        if $emitWinAsm and newName.length > 90
+            newName = newName[0...45] + "___" + newName[-45..-1]
+        end
         if $labelMapping[newName]
             while $labelMapping[newName = "_#{@@uniqueNameCounter}_#{comment}"]
                 @@uniqueNameCounter += 1
@@ -1420,7 +1440,9 @@ class IfThenElse < Node
     end
     
     def mapChildren
-        IfThenElse.new(codeOrigin, (yield @predicate), (yield @thenCase), (yield @elseCase))
+        ifThenElse = IfThenElse.new(codeOrigin, (yield @predicate), (yield @thenCase))
+        ifThenElse.elseCase = yield @elseCase
+        ifThenElse
     end
     
     def dump
@@ -1454,13 +1476,18 @@ end
 class MacroCall < Node
     attr_reader :name, :operands, :annotation
     
-    def initialize(codeOrigin, name, operands, annotation)
+    def initialize(codeOrigin, name, operands, annotation, originalName = nil)
         super(codeOrigin)
         @name = name
         @operands = operands
         raise unless @operands
         @operands.each{|v| raise unless v}
         @annotation = annotation
+        @originalName = originalName
+    end
+
+    def originalName
+        @originalName || name
     end
     
     def children
@@ -1468,11 +1495,11 @@ class MacroCall < Node
     end
     
     def mapChildren(&proc)
-        MacroCall.new(codeOrigin, @name, @operands.map(&proc), @annotation)
+        MacroCall.new(codeOrigin, @name, @operands.map(&proc), @annotation, @originalName)
     end
     
     def dump
-        "\t#{name}(" + operands.collect{|v| v.dump}.join(", ") + ")"
+        "\t#{originalName}(" + operands.collect{|v| v.dump}.join(", ") + ")"
     end
 end
 
