@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,10 +27,7 @@
 #include "ProfilerBytecodeSequence.h"
 
 #include "CodeBlock.h"
-#include "InterpreterInlines.h"
 #include "JSCInlines.h"
-#include "JSGlobalObject.h"
-#include "Operands.h"
 #include <wtf/StringPrintStream.h>
 
 namespace JSC { namespace Profiler {
@@ -76,22 +73,22 @@ const Bytecode& BytecodeSequence::forBytecodeIndex(unsigned bytecodeIndex) const
     return at(indexForBytecodeIndex(bytecodeIndex));
 }
 
-void BytecodeSequence::addSequenceProperties(ExecState* exec, JSObject* result) const
+void BytecodeSequence::addSequenceProperties(JSGlobalObject* globalObject, JSObject* result) const
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
-    JSArray* header = constructEmptyArray(exec, 0);
+    JSArray* header = constructEmptyArray(globalObject, nullptr);
     RETURN_IF_EXCEPTION(scope, void());
     for (unsigned i = 0; i < m_header.size(); ++i) {
-        header->putDirectIndex(exec, i, jsString(exec, String::fromUTF8(m_header[i])));
+        header->putDirectIndex(globalObject, i, jsString(vm, String::fromUTF8(m_header[i])));
         RETURN_IF_EXCEPTION(scope, void());
     }
     result->putDirect(vm, vm.propertyNames->header, header);
 
-    JSArray* sequence = constructEmptyArray(exec, 0);
+    JSArray* sequence = constructEmptyArray(globalObject, nullptr);
     RETURN_IF_EXCEPTION(scope, void());
     for (unsigned i = 0; i < m_sequence.size(); ++i) {
-        sequence->putDirectIndex(exec, i, m_sequence[i].toJS(exec));
+        sequence->putDirectIndex(globalObject, i, m_sequence[i].toJS(globalObject));
         RETURN_IF_EXCEPTION(scope, void());
     }
     result->putDirect(vm, vm.propertyNames->bytecode, sequence);
