@@ -133,6 +133,17 @@ public:
 
     void overflowCheckNotNeeded() { clearNeedsOverflowCheck(); }
 
+    template<typename Functor>
+    void fill(size_t count, const Functor& func)
+    {
+        ASSERT(!m_size);
+        ensureCapacity(count);
+        if (Base::hasOverflowed())
+            return;
+        m_size = count;
+        func(reinterpret_cast<JSValue*>(&slotFor(0)));
+    }
+
 private:
     void expandCapacity();
     void expandCapacity(int newCapacity);
@@ -154,15 +165,15 @@ private:
         return &slotFor(0);
     }
 
-#if ASSERT_DISABLED
-    void setNeedsOverflowCheck() { }
-    void clearNeedsOverflowCheck() { }
-#else
+#if ASSERT_ENABLED
     void setNeedsOverflowCheck() { m_needsOverflowCheck = true; }
     void clearNeedsOverflowCheck() { m_needsOverflowCheck = false; }
 
     bool m_needsOverflowCheck { false };
-#endif
+#else
+    void setNeedsOverflowCheck() { }
+    void clearNeedsOverflowCheck() { }
+#endif // ASSERT_ENABLED
     int m_size;
     int m_capacity;
     EncodedJSValue m_inlineBuffer[inlineCapacity];
@@ -181,9 +192,9 @@ public:
     {
     }
 
-    ArgList(ExecState* exec)
-        : m_args(reinterpret_cast<JSValue*>(&exec[CallFrame::argumentOffset(0)]))
-        , m_argCount(exec->argumentCount())
+    ArgList(CallFrame* callFrame)
+        : m_args(reinterpret_cast<JSValue*>(&callFrame[CallFrame::argumentOffset(0)]))
+        , m_argCount(callFrame->argumentCount())
     {
     }
 
