@@ -27,10 +27,12 @@
 
 #include "ArrayBufferSharingMode.h"
 #include "GCIncomingRefCounted.h"
+#include "Watchpoint.h"
 #include "Weak.h"
 #include <wtf/CagedPtr.h>
 #include <wtf/CheckedArithmetic.h>
-#include <wtf/Function.h>
+#include <wtf/PackedRefPtr.h>
+#include <wtf/SharedTask.h>
 #include <wtf/StdLibExtras.h>
 #include <wtf/ThreadSafeRefCounted.h>
 #include <wtf/text/WTFString.h>
@@ -44,18 +46,21 @@ class ArrayBuffer;
 class ArrayBufferView;
 class JSArrayBuffer;
 
-typedef Function<void(void*)> ArrayBufferDestructorFunction;
+using ArrayBufferDestructorFunction = RefPtr<SharedTask<void(void*)>>;
+using PackedArrayBufferDestructorFunction = PackedRefPtr<SharedTask<void(void*)>>;
 
 class SharedArrayBufferContents : public ThreadSafeRefCounted<SharedArrayBufferContents> {
 public:
-    SharedArrayBufferContents(void* data, ArrayBufferDestructorFunction&&);
+    SharedArrayBufferContents(void* data, unsigned size, ArrayBufferDestructorFunction&&);
     ~SharedArrayBufferContents();
 
-    void* data() const { return m_data.getMayBeNull(); }
+    void* data() const { return m_data.getMayBeNull(m_sizeInBytes); }
 
 private:
-    CagedPtr<Gigacage::Primitive, void> m_data;
-    ArrayBufferDestructorFunction m_destructor;
+    using DataType = CagedPtr<Gigacage::Primitive, void, tagCagedPtr>;
+    DataType m_data;
+    PackedArrayBufferDestructorFunction m_destructor;
+    unsigned m_sizeInBytes;
 };
 
 class ArrayBufferContents {
@@ -73,7 +78,7 @@ public:
 
     explicit operator bool() { return !!m_data; }
 
-    void* data() const { return m_data.getMayBeNull(); }
+    void* data() const { return m_data.getMayBeNull(sizeInBytes()); }
     unsigned sizeInBytes() const { return m_sizeInBytes; }
 
     bool isShared() const { return m_shared; }
@@ -96,9 +101,10 @@ private:
     void copyTo(ArrayBufferContents&);
     void shareWith(ArrayBufferContents&);
 
-    ArrayBufferDestructorFunction m_destructor;
-    RefPtr<SharedArrayBufferContents> m_shared;
-    CagedPtr<Gigacage::Primitive, void> m_data;
+    using DataType = CagedPtr<Gigacage::Primitive, void, tagCagedPtr>;
+    DataType m_data;
+    PackedArrayBufferDestructorFunction m_destructor;
+    PackedRefPtr<SharedArrayBufferContents> m_shared;
     unsigned m_sizeInBytes;
 };
 
@@ -129,8 +135,8 @@ public:
 
     inline size_t gcSizeEstimateInBytes() const;
 
-    JS_EXPORT_PRIVATE Ref<ArrayBuffer> slice(double begin, double end) const;
-    JS_EXPORT_PRIVATE Ref<ArrayBuffer> slice(double begin) const;
+    JS_EXPORT_PRIVATE RefPtr<ArrayBuffer> slice(double begin, double end) const;
+    JS_EXPORT_PRIVATE RefPtr<ArrayBuffer> slice(double begin) const;
 
     inline void pin();
     inline void unpin();
@@ -145,46 +151,50 @@ public:
 
     void neuter(VM&);
     bool isNeutered() { return !m_contents.m_data; }
+    InlineWatchpointSet& neuteringWatchpointSet() { return m_neuteringWatchpointSet; }
 
     static ptrdiff_t offsetOfData() { return OBJECT_OFFSETOF(ArrayBuffer, m_contents) + OBJECT_OFFSETOF(ArrayBufferContents, m_data); }
 
     ~ArrayBuffer() { }
 
+    JS_EXPORT_PRIVATE static Ref<SharedTask<void(void*)>> primitiveGigacageDestructor();
+
 private:
     static Ref<ArrayBuffer> create(unsigned numElements, unsigned elementByteSize, ArrayBufferContents::InitializationPolicy);
     static Ref<ArrayBuffer> createInternal(ArrayBufferContents&&, const void*, unsigned);
     static RefPtr<ArrayBuffer> tryCreate(unsigned numElements, unsigned elementByteSize, ArrayBufferContents::InitializationPolicy);
     ArrayBuffer(ArrayBufferContents&&);
-    Ref<ArrayBuffer> sliceImpl(unsigned begin, unsigned end) const;
+    RefPtr<ArrayBuffer> sliceImpl(unsigned begin, unsigned end) const;
     inline unsigned clampIndex(double index) const;
     static inline unsigned clampValue(double x, unsigned left, unsigned right);
 
-    void notifyIncommingReferencesOfTransfer(VM&);
+    void notifyNeutering(VM&);
 
     ArrayBufferContents m_contents;
+    InlineWatchpointSet m_neuteringWatchpointSet { IsWatched };
+public:
+    Weak<JSArrayBuffer> m_wrapper;
+private:
     Checked<unsigned> m_pinCount;
     bool m_isWasmMemory;
     // m_locked == true means that some API user fetched m_contents directly from a TypedArray object,
     // the buffer is backed by a WebAssembly.Memory, or is a SharedArrayBuffer.
     bool m_locked;
-
-public:
-    Weak<JSArrayBuffer> m_wrapper;
 };
 
 void* ArrayBuffer::data()
 {
-    return m_contents.m_data.getMayBeNull();
+    return m_contents.data();
 }
 
 const void* ArrayBuffer::data() const
 {
-    return m_contents.m_data.getMayBeNull();
+    return m_contents.data();
 }
 
 unsigned ArrayBuffer::byteLength() const
 {
-    return m_contents.m_sizeInBytes;
+    return m_contents.sizeInBytes();
 }
 
 bool ArrayBuffer::isShared() const
