@@ -51,7 +51,8 @@ protected:
 private:
     BooleanConstructor(VM&, Structure*);
 };
+STATIC_ASSERT_ISO_SUBSPACE_SHARABLE(BooleanConstructor, InternalFunction);
 
-JSObject* constructBooleanFromImmediateBoolean(ExecState*, JSGlobalObject*, JSValue);
+JSObject* constructBooleanFromImmediateBoolean(JSGlobalObject*, JSValue);
 
 } // namespace JSC
