@@ -21,17 +21,16 @@
 #include "config.h"
 #include "BooleanObject.h"
 
-#include "JSScope.h"
 #include "JSCInlines.h"
 
 namespace JSC {
 
 STATIC_ASSERT_IS_TRIVIALLY_DESTRUCTIBLE(BooleanObject);
 
-const ClassInfo BooleanObject::s_info = { "Boolean", &JSWrapperObject::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(BooleanObject) };
+const ClassInfo BooleanObject::s_info = { "Boolean", &Base::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(BooleanObject) };
 
 BooleanObject::BooleanObject(VM& vm, Structure* structure)
-    : JSWrapperObject(vm, structure)
+    : Base(vm, structure)
 {
 }
 
@@ -41,4 +40,9 @@ void BooleanObject::finishCreation(VM& vm)
     ASSERT(inherits(vm, info()));
 }
 
+String BooleanObject::toStringName(const JSObject*, JSGlobalObject*)
+{
+    return "Boolean"_s;
+}
+
 } // namespace JSC
