@@ -25,19 +25,12 @@
 
 #include "config.h"
 
-#include "BatchedTransitionOptimizer.h"
-#include "CodeBlock.h"
-#include "Debugger.h"
 #include "EvalCodeBlock.h"
 #include "FunctionCodeBlock.h"
-#include "JIT.h"
-#include "JSCInlines.h"
-#include "LLIntEntrypoint.h"
+#include "JSCellInlines.h"
 #include "ModuleProgramCodeBlock.h"
-#include "Parser.h"
+#include "NativeExecutable.h"
 #include "ProgramCodeBlock.h"
-#include "TypeProfiler.h"
-#include "VMInlines.h"
 #include <wtf/CommaPrinter.h>
 
 namespace JSC {
