@@ -23,10 +23,9 @@
 #include "config.h"
 #include "InternalFunction.h"
 
-#include "FunctionPrototype.h"
-#include "JSGlobalObject.h"
-#include "JSString.h"
+#include "JSBoundFunction.h"
 #include "JSCInlines.h"
+#include "ProxyObject.h"
 
 namespace JSC {
 
@@ -35,27 +34,28 @@ STATIC_ASSERT_IS_TRIVIALLY_DESTRUCTIBLE(InternalFunction);
 const ClassInfo InternalFunction::s_info = { "Function", &Base::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(InternalFunction) };
 
 InternalFunction::InternalFunction(VM& vm, Structure* structure, NativeFunction functionForCall, NativeFunction functionForConstruct)
-    : JSDestructibleObject(vm, structure)
+    : Base(vm, structure)
     , m_functionForCall(functionForCall)
     , m_functionForConstruct(functionForConstruct ? functionForConstruct : callHostFunctionAsConstructor)
+    , m_globalObject(vm, this, structure->globalObject())
 {
-    // exec->vm() wants callees to not be large allocations.
-    RELEASE_ASSERT(!isLargeAllocation());
     ASSERT_WITH_MESSAGE(m_functionForCall, "[[Call]] must be implemented");
     ASSERT(m_functionForConstruct);
 }
 
-void InternalFunction::finishCreation(VM& vm, const String& name, NameVisibility nameVisibility)
+void InternalFunction::finishCreation(VM& vm, const String& name, NameAdditionMode nameAdditionMode)
 {
     Base::finishCreation(vm);
     ASSERT(jsDynamicCast<InternalFunction*>(vm, this));
     ASSERT(methodTable(vm)->getCallData == InternalFunction::info()->methodTable.getCallData);
     ASSERT(methodTable(vm)->getConstructData == InternalFunction::info()->methodTable.getConstructData);
-    ASSERT(type() == InternalFunctionType);
-    JSString* nameString = jsString(&vm, name);
+    ASSERT(type() == InternalFunctionType || type() == NullSetterFunctionType);
+    JSString* nameString = jsString(vm, name);
     m_originalName.set(vm, this, nameString);
-    if (nameVisibility == NameVisibility::Visible)
+    if (nameAdditionMode == NameAdditionMode::WithStructureTransition)
         putDirect(vm, vm.propertyNames->name, nameString, PropertyAttribute::ReadOnly | PropertyAttribute::DontEnum);
+    else
+        putDirectWithoutTransition(vm, vm.propertyNames->name, nameString, PropertyAttribute::ReadOnly | PropertyAttribute::DontEnum);
 }
 
 void InternalFunction::visitChildren(JSCell* cell, SlotVisitor& visitor)
@@ -84,21 +84,26 @@ const String InternalFunction::displayName(VM& vm)
     return String();
 }
 
-CallType InternalFunction::getCallData(JSCell* cell, CallData& callData)
+CallData InternalFunction::getCallData(JSCell* cell)
 {
     auto* function = jsCast<InternalFunction*>(cell);
     ASSERT(function->m_functionForCall);
+
+    CallData callData;
+    callData.type = CallData::Type::Native;
     callData.native.function = function->m_functionForCall;
-    return CallType::Host;
+    return callData;
 }
 
-ConstructType InternalFunction::getConstructData(JSCell* cell, ConstructData& constructData)
+CallData InternalFunction::getConstructData(JSCell* cell)
 {
+    CallData constructData;
     auto* function = jsCast<InternalFunction*>(cell);
-    if (function->m_functionForConstruct == callHostFunctionAsConstructor)
-        return ConstructType::None;
-    constructData.native.function = function->m_functionForConstruct;
-    return ConstructType::Host;
+    if (function->m_functionForConstruct != callHostFunctionAsConstructor) {
+        constructData.type = CallData::Type::Native;
+        constructData.native.function = function->m_functionForConstruct;
+    }
+    return constructData;
 }
 
 const String InternalFunction::calculatedDisplayName(VM& vm)
@@ -111,41 +116,67 @@ const String InternalFunction::calculatedDisplayName(VM& vm)
     return name();
 }
 
-Structure* InternalFunction::createSubclassStructureSlow(ExecState* exec, JSValue newTarget, Structure* baseClass)
+Structure* InternalFunction::createSubclassStructure(JSGlobalObject* globalObject, JSObject* newTarget, Structure* baseClass)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
-    ASSERT(!newTarget || newTarget.isConstructor(vm));
-    ASSERT(newTarget && newTarget != exec->jsCallee());
 
     ASSERT(baseClass->hasMonoProto());
 
     // newTarget may be an InternalFunction if we were called from Reflect.construct.
     JSFunction* targetFunction = jsDynamicCast<JSFunction*>(vm, newTarget);
-    JSGlobalObject* lexicalGlobalObject = exec->lexicalGlobalObject();
+    JSGlobalObject* baseGlobalObject = baseClass->globalObject();
 
     if (LIKELY(targetFunction)) {
-        Structure* structure = targetFunction->rareData(vm)->internalFunctionAllocationStructure();
-        if (LIKELY(structure && structure->classInfo() == baseClass->classInfo()))
+        FunctionRareData* rareData = targetFunction->ensureRareData(vm);
+        Structure* structure = rareData->internalFunctionAllocationStructure();
+        if (LIKELY(structure && structure->classInfo() == baseClass->classInfo() && structure->globalObject() == baseGlobalObject))
             return structure;
 
         // Note, Reflect.construct might cause the profile to churn but we don't care.
-        JSValue prototypeValue = newTarget.get(exec, vm.propertyNames->prototype);
+        JSValue prototypeValue = targetFunction->get(globalObject, vm.propertyNames->prototype);
         RETURN_IF_EXCEPTION(scope, nullptr);
         if (JSObject* prototype = jsDynamicCast<JSObject*>(vm, prototypeValue))
-            return targetFunction->rareData(vm)->createInternalFunctionAllocationStructureFromBase(vm, lexicalGlobalObject, prototype, baseClass);
+            return rareData->createInternalFunctionAllocationStructureFromBase(vm, baseGlobalObject, prototype, baseClass);
     } else {
-        JSValue prototypeValue = newTarget.get(exec, vm.propertyNames->prototype);
+        JSValue prototypeValue = newTarget->get(globalObject, vm.propertyNames->prototype);
         RETURN_IF_EXCEPTION(scope, nullptr);
         if (JSObject* prototype = jsDynamicCast<JSObject*>(vm, prototypeValue)) {
             // This only happens if someone Reflect.constructs our builtin constructor with another builtin constructor as the new.target.
             // Thus, we don't care about the cost of looking up the structure from our hash table every time.
-            return vm.structureCache.emptyStructureForPrototypeFromBaseStructure(lexicalGlobalObject, prototype, baseClass);
+            return vm.structureCache.emptyStructureForPrototypeFromBaseStructure(baseGlobalObject, prototype, baseClass);
         }
     }
 
     return baseClass;
 }
 
+// https://tc39.es/ecma262/#sec-getfunctionrealm
+JSGlobalObject* getFunctionRealm(VM& vm, JSObject* object)
+{
+    ASSERT(object->isCallable(vm));
+
+    while (true) {
+        if (object->inherits<JSBoundFunction>(vm)) {
+            object = jsCast<JSBoundFunction*>(object)->targetFunction();
+            continue;
+        }
+
+        if (object->type() == ProxyObjectType) {
+            auto* proxy = jsCast<ProxyObject*>(object);
+            // Per step 4.a, a TypeError should be thrown for revoked Proxy, yet we skip it since:
+            // a) It is barely observable anyway: "prototype" lookup in createSubclassStructure() will throw for revoked Proxy.
+            // b) Throwing getFunctionRealm() will restrict calling it inline as an argument of createSubclassStructure().
+            // c) There is ongoing discussion on removing it: https://github.com/tc39/ecma262/issues/1798.
+            if (!proxy->isRevoked()) {
+                object = proxy->target();
+                continue;
+            }
+        }
+
+        return object->globalObject(vm);
+    }
+}
+
 
 } // namespace JSC
