@@ -27,10 +27,10 @@
 
 namespace JSC {
 
-enum IterationKind : uint32_t {
-    IterateKey,
-    IterateValue,
-    IterateKeyValue,
+enum class IterationKind : uint32_t {
+    Keys,
+    Values,
+    Entries,
 };
 
 
