@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2017 Caio Lima <ticaiolima@gmail.com>
- * Copyright (C) 2017-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -59,14 +59,19 @@
 
 namespace JSC {
 
-const ClassInfo JSBigInt::s_info =
-    { "JSBigInt", nullptr, nullptr, nullptr, CREATE_METHOD_TABLE(JSBigInt) };
+const ClassInfo JSBigInt::s_info = { "BigInt", nullptr, nullptr, nullptr, CREATE_METHOD_TABLE(JSBigInt) };
 
-JSBigInt::JSBigInt(VM& vm, Structure* structure, unsigned length)
+JSBigInt::JSBigInt(VM& vm, Structure* structure, Digit* data, unsigned length)
     : Base(vm, structure)
     , m_length(length)
+    , m_data(data, length)
 { }
 
+void JSBigInt::destroy(JSCell* thisCell)
+{
+    static_cast<JSBigInt*>(thisCell)->~JSBigInt();
+}
+
 void JSBigInt::initialize(InitializationType initType)
 {
     if (initType == InitializationType::WithZero)
@@ -84,19 +89,13 @@ JSBigInt* JSBigInt::createZero(VM& vm)
     return zeroBigInt;
 }
 
-inline size_t JSBigInt::allocationSize(unsigned length)
+JSBigInt* JSBigInt::tryCreateWithLength(JSGlobalObject* globalObject, unsigned length)
 {
-    size_t sizeWithPadding = WTF::roundUpToMultipleOf<sizeof(size_t)>(sizeof(JSBigInt));
-    return sizeWithPadding + length * sizeof(Digit);
-}
-
-JSBigInt* JSBigInt::tryCreateWithLength(ExecState* exec, unsigned length)
-{
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (UNLIKELY(length > maxLength)) {
-        throwOutOfMemoryError(exec, scope);
+        throwOutOfMemoryError(globalObject, scope);
         return nullptr;
     }
 
@@ -108,7 +107,8 @@ JSBigInt* JSBigInt::tryCreateWithLength(ExecState* exec, unsigned length)
 JSBigInt* JSBigInt::createWithLengthUnchecked(VM& vm, unsigned length)
 {
     ASSERT(length <= maxLength);
-    JSBigInt* bigInt = new (NotNull, allocateCell<JSBigInt>(vm.heap, allocationSize(length))) JSBigInt(vm, vm.bigIntStructure.get(), length);
+    void* data = Gigacage::malloc(Gigacage::Primitive, length * sizeof(Digit));
+    JSBigInt* bigInt = new (NotNull, allocateCell<JSBigInt>(vm.heap)) JSBigInt(vm, vm.bigIntStructure.get(), reinterpret_cast<Digit*>(data), length);
     bigInt->finishCreation(vm);
     return bigInt;
 }
@@ -183,7 +183,7 @@ JSBigInt* JSBigInt::createFrom(VM& vm, bool value)
     return bigInt;
 }
 
-JSValue JSBigInt::toPrimitive(ExecState*, PreferredPrimitiveType) const
+JSValue JSBigInt::toPrimitive(JSGlobalObject*, PreferredPrimitiveType) const
 {
     return const_cast<JSBigInt*>(this);
 }
@@ -201,34 +201,45 @@ Optional<uint8_t> JSBigInt::singleDigitValueForString()
     return { };
 }
 
-JSBigInt* JSBigInt::parseInt(ExecState* exec, StringView s, ErrorParseMode parserMode)
+JSBigInt* JSBigInt::parseInt(JSGlobalObject* globalObject, StringView s, ErrorParseMode parserMode)
 {
     if (s.is8Bit())
-        return parseInt(exec, s.characters8(), s.length(), parserMode);
-    return parseInt(exec, s.characters16(), s.length(), parserMode);
+        return parseInt(globalObject, s.characters8(), s.length(), parserMode);
+    return parseInt(globalObject, s.characters16(), s.length(), parserMode);
 }
 
-JSBigInt* JSBigInt::parseInt(ExecState* exec, VM& vm, StringView s, uint8_t radix, ErrorParseMode parserMode, ParseIntSign sign)
+JSBigInt* JSBigInt::parseInt(JSGlobalObject* globalObject, VM& vm, StringView s, uint8_t radix, ErrorParseMode parserMode, ParseIntSign sign)
 {
     if (s.is8Bit())
-        return parseInt(exec, vm, s.characters8(), s.length(), 0, radix, parserMode, sign, ParseIntMode::DisallowEmptyString);
-    return parseInt(exec, vm, s.characters16(), s.length(), 0, radix, parserMode, sign, ParseIntMode::DisallowEmptyString);
+        return parseInt(globalObject, vm, s.characters8(), s.length(), 0, radix, parserMode, sign, ParseIntMode::DisallowEmptyString);
+    return parseInt(globalObject, vm, s.characters16(), s.length(), 0, radix, parserMode, sign, ParseIntMode::DisallowEmptyString);
 }
 
-JSBigInt* JSBigInt::stringToBigInt(ExecState* exec, StringView s)
+JSBigInt* JSBigInt::stringToBigInt(JSGlobalObject* globalObject, StringView s)
 {
-    return parseInt(exec, s, ErrorParseMode::IgnoreExceptions);
+    return parseInt(globalObject, s, ErrorParseMode::IgnoreExceptions);
 }
 
-String JSBigInt::toString(ExecState* exec, unsigned radix)
+String JSBigInt::toString(JSGlobalObject* globalObject, unsigned radix)
 {
     if (this->isZero())
-        return exec->vm().smallStrings.singleCharacterStringRep('0');
+        return globalObject->vm().smallStrings.singleCharacterStringRep('0');
 
     if (hasOneBitSet(radix))
-        return toStringBasePowerOfTwo(exec, this, radix);
+        return toStringBasePowerOfTwo(globalObject->vm(), globalObject, this, radix);
 
-    return toStringGeneric(exec, this, radix);
+    return toStringGeneric(globalObject->vm(), globalObject, this, radix);
+}
+
+String JSBigInt::tryGetString(VM& vm, JSBigInt* bigInt, unsigned radix)
+{
+    if (bigInt->isZero())
+        return vm.smallStrings.singleCharacterStringRep('0');
+
+    if (hasOneBitSet(radix))
+        return toStringBasePowerOfTwo(vm, nullptr, bigInt, radix);
+
+    return toStringGeneric(vm, nullptr, bigInt, radix);
 }
 
 // Multiplies {this} with {factor} and adds {summand} to the result.
@@ -237,9 +248,98 @@ void JSBigInt::inplaceMultiplyAdd(Digit factor, Digit summand)
     internalMultiplyAdd(this, factor, summand, length(), this);
 }
 
-JSBigInt* JSBigInt::multiply(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::exponentiate(JSGlobalObject* globalObject, JSBigInt* base, JSBigInt* exponent)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
+    auto scope = DECLARE_THROW_SCOPE(vm);
+
+    if (exponent->sign()) {
+        throwRangeError(globalObject, scope, "Negative exponent is not allowed"_s);
+        return nullptr;
+    }
+
+    // 2. If base is 0n and exponent is 0n, return 1n.
+    if (exponent->isZero())
+        return JSBigInt::createFrom(vm, 1);
+
+    // 3. Return a BigInt representing the mathematical value of base raised
+    //    to the power exponent.
+    if (base->isZero())
+        return base;
+
+    if (base->length() == 1 && base->digit(0) == 1) {
+        // (-1) ** even_number == 1.
+        if (base->sign() && !(exponent->digit(0) & 1))
+            return JSBigInt::unaryMinus(vm, base);
+
+        // (-1) ** odd_number == -1; 1 ** anything == 1.
+        return base;
+    }
+
+    // For all bases >= 2, very large exponents would lead to unrepresentable
+    // results.
+    static_assert(maxLengthBits < std::numeric_limits<Digit>::max(), "maxLengthBits needs to be less than digit::max()");
+    if (exponent->length() > 1) {
+        throwRangeError(globalObject, scope, "BigInt generated from this operation is too big"_s);
+        return nullptr;
+    }
+
+    Digit expValue = exponent->digit(0);
+    if (expValue == 1)
+        return base;
+    if (expValue >= maxLengthBits) {
+        throwRangeError(globalObject, scope, "BigInt generated from this operation is too big"_s);
+        return nullptr;
+    }
+
+    static_assert(maxLengthBits <= maxInt, "maxLengthBits needs to be <= maxInt");
+    int n = static_cast<int>(expValue);
+    if (base->length() == 1 && base->digit(0) == 2) {
+        // Fast path for 2^n.
+        int neededDigits = 1 + (n / digitBits);
+        JSBigInt* result = JSBigInt::tryCreateWithLength(globalObject, neededDigits);
+        RETURN_IF_EXCEPTION(scope, nullptr);
+
+        result->initialize(InitializationType::WithZero);
+        // All bits are zero. Now set the n-th bit.
+        Digit msd = static_cast<Digit>(1) << (n % digitBits);
+        result->setDigit(neededDigits - 1, msd);
+        // Result is negative for odd powers of -2n.
+        if (base->sign())
+            result->setSign(static_cast<bool>(n & 1));
+
+        return result;
+    }
+
+    JSBigInt* result = nullptr;
+    JSBigInt* runningSquare = base;
+
+    // This implicitly sets the result's sign correctly.
+    if (n & 1)
+        result = base;
+
+    n >>= 1;
+    for (; n; n >>= 1) {
+        JSBigInt* maybeResult = JSBigInt::multiply(globalObject, runningSquare, runningSquare);
+        RETURN_IF_EXCEPTION(scope, nullptr);
+        runningSquare = maybeResult;
+        if (n & 1) {
+            if (!result)
+                result = runningSquare;
+            else {
+                maybeResult = JSBigInt::multiply(globalObject, result, runningSquare);
+                RETURN_IF_EXCEPTION(scope, nullptr);
+                result = maybeResult;
+            }
+        }
+    }
+
+    return result;
+}
+
+JSBigInt* JSBigInt::multiply(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
+{
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (x->isZero())
@@ -248,7 +348,7 @@ JSBigInt* JSBigInt::multiply(ExecState* exec, JSBigInt* x, JSBigInt* y)
         return y;
 
     unsigned resultLength = x->length() + y->length();
-    JSBigInt* result = JSBigInt::tryCreateWithLength(exec, resultLength);
+    JSBigInt* result = JSBigInt::tryCreateWithLength(globalObject, resultLength);
     RETURN_IF_EXCEPTION(scope, nullptr);
     result->initialize(InitializationType::WithZero);
 
@@ -259,14 +359,14 @@ JSBigInt* JSBigInt::multiply(ExecState* exec, JSBigInt* x, JSBigInt* y)
     return result->rightTrim(vm);
 }
 
-JSBigInt* JSBigInt::divide(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::divide(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
 {
     // 1. If y is 0n, throw a RangeError exception.
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (y->isZero()) {
-        throwRangeError(exec, scope, "0 is an invalid divisor value."_s);
+        throwRangeError(globalObject, scope, "0 is an invalid divisor value."_s);
         return nullptr;
     }
 
@@ -286,7 +386,7 @@ JSBigInt* JSBigInt::divide(ExecState* exec, JSBigInt* x, JSBigInt* y)
         Digit remainder;
         absoluteDivWithDigitDivisor(vm, x, divisor, &quotient, remainder);
     } else {
-        absoluteDivWithBigIntDivisor(exec, x, y, &quotient, nullptr);
+        absoluteDivWithBigIntDivisor(globalObject, x, y, &quotient, nullptr);
         RETURN_IF_EXCEPTION(scope, nullptr);
     }
 
@@ -314,14 +414,14 @@ JSBigInt* JSBigInt::unaryMinus(VM& vm, JSBigInt* x)
     return result;
 }
 
-JSBigInt* JSBigInt::remainder(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::remainder(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
 {
     // 1. If y is 0n, throw a RangeError exception.
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (y->isZero()) {
-        throwRangeError(exec, scope, "0 is an invalid divisor value."_s);
+        throwRangeError(globalObject, scope, "0 is an invalid divisor value."_s);
         return nullptr;
     }
 
@@ -344,7 +444,7 @@ JSBigInt* JSBigInt::remainder(ExecState* exec, JSBigInt* x, JSBigInt* y)
         remainder = createWithLengthUnchecked(vm, 1);
         remainder->setDigit(0, remainderDigit);
     } else {
-        absoluteDivWithBigIntDivisor(exec, x, y, nullptr, &remainder);
+        absoluteDivWithBigIntDivisor(globalObject, x, y, nullptr, &remainder);
         RETURN_IF_EXCEPTION(scope, nullptr);
     }
 
@@ -352,15 +452,29 @@ JSBigInt* JSBigInt::remainder(ExecState* exec, JSBigInt* x, JSBigInt* y)
     return remainder->rightTrim(vm);
 }
 
-JSBigInt* JSBigInt::add(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::inc(JSGlobalObject* globalObject, JSBigInt* x)
 {
-    VM& vm = exec->vm();
+    // FIXME: we can probably do something a fair bit more efficient here
+    VM& vm = globalObject->vm();
+    return add(globalObject, x, vm.bigIntConstantOne.get());
+}
+
+JSBigInt* JSBigInt::dec(JSGlobalObject* globalObject, JSBigInt* x)
+{
+    // FIXME: we can probably do something a fair bit more efficient here
+    VM& vm = globalObject->vm();
+    return sub(globalObject, x, vm.bigIntConstantOne.get());
+}
+
+JSBigInt* JSBigInt::add(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
+{
+    VM& vm = globalObject->vm();
     bool xSign = x->sign();
 
     // x + y == x + y
     // -x + -y == -(x + y)
     if (xSign == y->sign())
-        return absoluteAdd(exec, x, y, xSign);
+        return absoluteAdd(globalObject, x, y, xSign);
 
     // x + -y == x - y == -(y - x)
     // -x + y == y - x == -(x - y)
@@ -371,14 +485,14 @@ JSBigInt* JSBigInt::add(ExecState* exec, JSBigInt* x, JSBigInt* y)
     return absoluteSub(vm, y, x, !xSign);
 }
 
-JSBigInt* JSBigInt::sub(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::sub(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     bool xSign = x->sign();
     if (xSign != y->sign()) {
         // x - (-y) == x + y
         // (-x) - y == -(x + y)
-        return absoluteAdd(exec, x, y, xSign);
+        return absoluteAdd(globalObject, x, y, xSign);
     }
     // x - y == -(y - x)
     // (-x) - (-y) == y - x == -(x - y)
@@ -389,9 +503,9 @@ JSBigInt* JSBigInt::sub(ExecState* exec, JSBigInt* x, JSBigInt* y)
     return absoluteSub(vm, y, x, !xSign);
 }
 
-JSBigInt* JSBigInt::bitwiseAnd(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::bitwiseAnd(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (!x->sign() && !y->sign()) {
@@ -403,14 +517,14 @@ JSBigInt* JSBigInt::bitwiseAnd(ExecState* exec, JSBigInt* x, JSBigInt* y)
         int resultLength = std::max(x->length(), y->length()) + 1;
         // (-x) & (-y) == ~(x-1) & ~(y-1) == ~((x-1) | (y-1))
         // == -(((x-1) | (y-1)) + 1)
-        JSBigInt* result = absoluteSubOne(exec, x, resultLength);
+        JSBigInt* result = absoluteSubOne(globalObject, x, resultLength);
         RETURN_IF_EXCEPTION(scope, nullptr);
 
-        JSBigInt* y1 = absoluteSubOne(exec, y, y->length());
+        JSBigInt* y1 = absoluteSubOne(globalObject, y, y->length());
         RETURN_IF_EXCEPTION(scope, nullptr);
         result = absoluteOr(vm, result, y1);
         scope.release();
-        return absoluteAddOne(exec, result, SignOption::Signed);
+        return absoluteAddOne(globalObject, result, SignOption::Signed);
     }
 
     ASSERT(x->sign() != y->sign());
@@ -419,14 +533,14 @@ JSBigInt* JSBigInt::bitwiseAnd(ExecState* exec, JSBigInt* x, JSBigInt* y)
         std::swap(x, y);
 
     // x & (-y) == x & ~(y-1) == x & ~(y-1)
-    JSBigInt* y1 = absoluteSubOne(exec, y, y->length());
+    JSBigInt* y1 = absoluteSubOne(globalObject, y, y->length());
     RETURN_IF_EXCEPTION(scope, nullptr);
     return absoluteAndNot(vm, x, y1);
 }
 
-JSBigInt* JSBigInt::bitwiseOr(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::bitwiseOr(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     unsigned resultLength = std::max(x->length(), y->length());
@@ -439,15 +553,15 @@ JSBigInt* JSBigInt::bitwiseOr(ExecState* exec, JSBigInt* x, JSBigInt* y)
     if (x->sign() && y->sign()) {
         // (-x) | (-y) == ~(x-1) | ~(y-1) == ~((x-1) & (y-1))
         // == -(((x-1) & (y-1)) + 1)
-        JSBigInt* result = absoluteSubOne(exec, x, resultLength);
+        JSBigInt* result = absoluteSubOne(globalObject, x, resultLength);
         RETURN_IF_EXCEPTION(scope, nullptr);
-        JSBigInt* y1 = absoluteSubOne(exec, y, y->length());
+        JSBigInt* y1 = absoluteSubOne(globalObject, y, y->length());
         RETURN_IF_EXCEPTION(scope, nullptr);
         result = absoluteAnd(vm, result, y1);
         RETURN_IF_EXCEPTION(scope, nullptr);
 
         scope.release();
-        return absoluteAddOne(exec, result, SignOption::Signed);
+        return absoluteAddOne(globalObject, result, SignOption::Signed);
     }
 
     ASSERT(x->sign() != y->sign());
@@ -457,17 +571,17 @@ JSBigInt* JSBigInt::bitwiseOr(ExecState* exec, JSBigInt* x, JSBigInt* y)
         std::swap(x, y);
 
     // x | (-y) == x | ~(y-1) == ~((y-1) &~ x) == -(((y-1) &~ x) + 1)
-    JSBigInt* result = absoluteSubOne(exec, y, resultLength);
+    JSBigInt* result = absoluteSubOne(globalObject, y, resultLength);
     RETURN_IF_EXCEPTION(scope, nullptr);
     result = absoluteAndNot(vm, result, x);
 
     scope.release();
-    return absoluteAddOne(exec, result, SignOption::Signed);
+    return absoluteAddOne(globalObject, result, SignOption::Signed);
 }
 
-JSBigInt* JSBigInt::bitwiseXor(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::bitwiseXor(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (!x->sign() && !y->sign()) {
@@ -479,9 +593,9 @@ JSBigInt* JSBigInt::bitwiseXor(ExecState* exec, JSBigInt* x, JSBigInt* y)
         int resultLength = std::max(x->length(), y->length());
 
         // (-x) ^ (-y) == ~(x-1) ^ ~(y-1) == (x-1) ^ (y-1)
-        JSBigInt* result = absoluteSubOne(exec, x, resultLength);
+        JSBigInt* result = absoluteSubOne(globalObject, x, resultLength);
         RETURN_IF_EXCEPTION(scope, nullptr);
-        JSBigInt* y1 = absoluteSubOne(exec, y, y->length());
+        JSBigInt* y1 = absoluteSubOne(globalObject, y, y->length());
         RETURN_IF_EXCEPTION(scope, nullptr);
 
         scope.release();
@@ -495,34 +609,44 @@ JSBigInt* JSBigInt::bitwiseXor(ExecState* exec, JSBigInt* x, JSBigInt* y)
         std::swap(x, y);
 
     // x ^ (-y) == x ^ ~(y-1) == ~(x ^ (y-1)) == -((x ^ (y-1)) + 1)
-    JSBigInt* result = absoluteSubOne(exec, y, resultLength);
+    JSBigInt* result = absoluteSubOne(globalObject, y, resultLength);
     RETURN_IF_EXCEPTION(scope, nullptr);
 
     result = absoluteXor(vm, result, x);
     scope.release();
-    return absoluteAddOne(exec, result, SignOption::Signed);
+    return absoluteAddOne(globalObject, result, SignOption::Signed);
 }
 
-JSBigInt* JSBigInt::leftShift(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::leftShift(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
 {
     if (y->isZero() || x->isZero())
         return x;
 
     if (y->sign())
-        return rightShiftByAbsolute(exec, x, y);
+        return rightShiftByAbsolute(globalObject, x, y);
 
-    return leftShiftByAbsolute(exec, x, y);
+    return leftShiftByAbsolute(globalObject, x, y);
 }
 
-JSBigInt* JSBigInt::signedRightShift(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::signedRightShift(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
 {
     if (y->isZero() || x->isZero())
         return x;
 
     if (y->sign())
-        return leftShiftByAbsolute(exec, x, y);
+        return leftShiftByAbsolute(globalObject, x, y);
 
-    return rightShiftByAbsolute(exec, x, y);
+    return rightShiftByAbsolute(globalObject, x, y);
+}
+
+JSBigInt* JSBigInt::bitwiseNot(JSGlobalObject* globalObject, JSBigInt* x)
+{
+    if (x->sign()) {
+        // ~(-x) == ~(~(x-1)) == x-1
+        return absoluteSubOne(globalObject, x, x->length());
+    }
+    // ~x == -x-1 == -(x+1)
+    return absoluteAddOne(globalObject, x, SignOption::Signed);
 }
 
 #if USE(JSVALUE32_64)
@@ -637,11 +761,7 @@ inline JSBigInt::Digit JSBigInt::digitDiv(Digit high, Digit low, Digit divisor,
 #else
     static constexpr Digit halfDigitBase = 1ull << halfDigitBits;
     // Adapted from Warren, Hacker's Delight, p. 152.
-#if USE(JSVALUE64)
-    unsigned s = clz64(divisor);
-#else
-    unsigned s = clz32(divisor);
-#endif
+    unsigned s = clz(divisor);
     // If {s} is digitBits here, it causes an undefined behavior.
     // But {s} is never digitBits since {divisor} is never zero here.
     ASSERT(s != digitBits);
@@ -822,12 +942,12 @@ inline JSBigInt::ComparisonResult JSBigInt::absoluteCompare(JSBigInt* x, JSBigIn
     return x->digit(i) > y->digit(i) ? ComparisonResult::GreaterThan : ComparisonResult::LessThan;
 }
 
-JSBigInt* JSBigInt::absoluteAdd(ExecState* exec, JSBigInt* x, JSBigInt* y, bool resultSign)
+JSBigInt* JSBigInt::absoluteAdd(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y, bool resultSign)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
 
     if (x->length() < y->length())
-        return absoluteAdd(exec, y, x, resultSign);
+        return absoluteAdd(globalObject, y, x, resultSign);
 
     if (x->isZero()) {
         ASSERT(y->isZero());
@@ -837,7 +957,7 @@ JSBigInt* JSBigInt::absoluteAdd(ExecState* exec, JSBigInt* x, JSBigInt* y, bool
     if (y->isZero())
         return resultSign == x->sign() ? x : unaryMinus(vm, x);
 
-    JSBigInt* result = JSBigInt::tryCreateWithLength(exec, x->length() + 1);
+    JSBigInt* result = JSBigInt::tryCreateWithLength(globalObject, x->length() + 1);
     if (!result)
         return nullptr;
     Digit carry = 0;
@@ -944,11 +1064,11 @@ void JSBigInt::absoluteDivWithDigitDivisor(VM& vm, JSBigInt* x, Digit divisor, J
 // Both {quotient} and {remainder} are optional, for callers that are only
 // interested in one of them.
 // See Knuth, Volume 2, section 4.3.1, Algorithm D.
-void JSBigInt::absoluteDivWithBigIntDivisor(ExecState* exec, JSBigInt* dividend, JSBigInt* divisor, JSBigInt** quotient, JSBigInt** remainder)
+void JSBigInt::absoluteDivWithBigIntDivisor(JSGlobalObject* globalObject, JSBigInt* dividend, JSBigInt* divisor, JSBigInt** quotient, JSBigInt** remainder)
 {
     ASSERT(divisor->length() >= 2);
     ASSERT(dividend->length() >= divisor->length());
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     // The unusual variable names inside this function are consistent with
@@ -961,11 +1081,11 @@ void JSBigInt::absoluteDivWithBigIntDivisor(ExecState* exec, JSBigInt* dividend,
     // The quotient to be computed.
     JSBigInt* q = nullptr;
     if (quotient != nullptr)
-        q = createWithLengthUnchecked(exec->vm(), m + 1);
+        q = createWithLengthUnchecked(globalObject->vm(), m + 1);
 
     // In each iteration, {qhatv} holds {divisor} * {current quotient digit}.
     // "v" is the book's name for {divisor}, "qhat" the current quotient digit.
-    JSBigInt* qhatv = tryCreateWithLength(exec, n + 1);
+    JSBigInt* qhatv = tryCreateWithLength(globalObject, n + 1);
     RETURN_IF_EXCEPTION(scope, void());
 
     // D1.
@@ -974,16 +1094,16 @@ void JSBigInt::absoluteDivWithBigIntDivisor(ExecState* exec, JSBigInt* dividend,
     // overflowing (they take a two digits wide input, and return a one digit
     // result).
     Digit lastDigit = divisor->digit(n - 1);
-    unsigned shift = sizeof(lastDigit) == 8 ? clz64(lastDigit) : clz32(lastDigit);
+    unsigned shift = clz(lastDigit);
 
     if (shift > 0) {
-        divisor = absoluteLeftShiftAlwaysCopy(exec, divisor, shift, LeftShiftMode::SameSizeResult);
+        divisor = absoluteLeftShiftAlwaysCopy(globalObject, divisor, shift, LeftShiftMode::SameSizeResult);
         RETURN_IF_EXCEPTION(scope, void());
     }
 
     // Holds the (continuously updated) remaining part of the dividend, which
     // eventually becomes the remainder.
-    JSBigInt* u = absoluteLeftShiftAlwaysCopy(exec, dividend, shift, LeftShiftMode::AlwaysAddOneDigit);
+    JSBigInt* u = absoluteLeftShiftAlwaysCopy(globalObject, dividend, shift, LeftShiftMode::AlwaysAddOneDigit);
     RETURN_IF_EXCEPTION(scope, void());
 
     // D2.
@@ -1049,7 +1169,7 @@ void JSBigInt::absoluteDivWithBigIntDivisor(ExecState* exec, JSBigInt* dividend,
     }
 }
 
-// Returns whether (factor1 * factor2) > (high << kDigitBits) + low.
+// Returns whether (factor1 * factor2) > (high << digitBits) + low.
 inline bool JSBigInt::productGreaterThan(Digit factor1, Digit factor2, Digit high, Digit low)
 {
     Digit resultHigh;
@@ -1112,14 +1232,14 @@ void JSBigInt::inplaceRightShift(unsigned shift)
 }
 
 // Always copies the input, even when {shift} == 0.
-JSBigInt* JSBigInt::absoluteLeftShiftAlwaysCopy(ExecState* exec, JSBigInt* x, unsigned shift, LeftShiftMode mode)
+JSBigInt* JSBigInt::absoluteLeftShiftAlwaysCopy(JSGlobalObject* globalObject, JSBigInt* x, unsigned shift, LeftShiftMode mode)
 {
     ASSERT(shift < digitBits);
     ASSERT(!x->isZero());
 
     unsigned n = x->length();
     unsigned resultLength = mode == LeftShiftMode::AlwaysAddOneDigit ? n + 1 : n;
-    JSBigInt* result = tryCreateWithLength(exec, resultLength);
+    JSBigInt* result = tryCreateWithLength(globalObject, resultLength);
     if (!result)
         return nullptr;
 
@@ -1226,7 +1346,7 @@ JSBigInt* JSBigInt::absoluteXor(VM& vm, JSBigInt* x, JSBigInt* y)
     return absoluteBitwiseOp(vm, x, y, ExtraDigitsHandling::Copy, SymmetricOp::Symmetric, digitOperation);
 }
 
-JSBigInt* JSBigInt::absoluteAddOne(ExecState* exec, JSBigInt* x, SignOption signOption)
+JSBigInt* JSBigInt::absoluteAddOne(JSGlobalObject* globalObject, JSBigInt* x, SignOption signOption)
 {
     unsigned inputLength = x->length();
     // The addition will overflow into a new digit if all existing digits are
@@ -1240,7 +1360,7 @@ JSBigInt* JSBigInt::absoluteAddOne(ExecState* exec, JSBigInt* x, SignOption sign
     }
 
     unsigned resultLength = inputLength + willOverflow;
-    JSBigInt* result = tryCreateWithLength(exec, resultLength);
+    JSBigInt* result = tryCreateWithLength(globalObject, resultLength);
     if (!result)
         return nullptr;
 
@@ -1256,17 +1376,17 @@ JSBigInt* JSBigInt::absoluteAddOne(ExecState* exec, JSBigInt* x, SignOption sign
         ASSERT(!carry);
 
     result->setSign(signOption == SignOption::Signed);
-    return result->rightTrim(exec->vm());
+    return result->rightTrim(globalObject->vm());
 }
 
-JSBigInt* JSBigInt::absoluteSubOne(ExecState* exec, JSBigInt* x, unsigned resultLength)
+JSBigInt* JSBigInt::absoluteSubOne(JSGlobalObject* globalObject, JSBigInt* x, unsigned resultLength)
 {
     ASSERT(!x->isZero());
     ASSERT(resultLength >= x->length());
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
-    JSBigInt* result = tryCreateWithLength(exec, resultLength);
+    JSBigInt* result = tryCreateWithLength(globalObject, resultLength);
     RETURN_IF_EXCEPTION(scope, nullptr);
 
     unsigned length = x->length();
@@ -1283,14 +1403,14 @@ JSBigInt* JSBigInt::absoluteSubOne(ExecState* exec, JSBigInt* x, unsigned result
     return result->rightTrim(vm);
 }
 
-JSBigInt* JSBigInt::leftShiftByAbsolute(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::leftShiftByAbsolute(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     auto optionalShift = toShiftAmount(y);
     if (!optionalShift) {
-        throwRangeError(exec, scope, "BigInt generated from this operation is too big"_s);
+        throwRangeError(globalObject, scope, "BigInt generated from this operation is too big"_s);
         return nullptr;
     }
 
@@ -1301,11 +1421,11 @@ JSBigInt* JSBigInt::leftShiftByAbsolute(ExecState* exec, JSBigInt* x, JSBigInt*
     bool grow = bitsShift && (x->digit(length - 1) >> (digitBits - bitsShift));
     int resultLength = length + digitShift + grow;
     if (static_cast<unsigned>(resultLength) > maxLength) {
-        throwRangeError(exec, scope, "BigInt generated from this operation is too big"_s);
+        throwRangeError(globalObject, scope, "BigInt generated from this operation is too big"_s);
         return nullptr;
     }
 
-    JSBigInt* result = tryCreateWithLength(exec, resultLength);
+    JSBigInt* result = tryCreateWithLength(globalObject, resultLength);
     RETURN_IF_EXCEPTION(scope, nullptr);
     if (!bitsShift) {
         unsigned i = 0;
@@ -1335,9 +1455,9 @@ JSBigInt* JSBigInt::leftShiftByAbsolute(ExecState* exec, JSBigInt* x, JSBigInt*
     return result->rightTrim(vm);
 }
 
-JSBigInt* JSBigInt::rightShiftByAbsolute(ExecState* exec, JSBigInt* x, JSBigInt* y)
+JSBigInt* JSBigInt::rightShiftByAbsolute(JSGlobalObject* globalObject, JSBigInt* x, JSBigInt* y)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     unsigned length = x->length();
     bool sign = x->sign();
     auto optionalShift = toShiftAmount(y);
@@ -1401,7 +1521,7 @@ JSBigInt* JSBigInt::rightShiftByAbsolute(ExecState* exec, JSBigInt* x, JSBigInt*
             // Since the result is negative, rounding down means adding one to
             // its absolute value. This cannot overflow.
             result = result->rightTrim(vm);
-            return absoluteAddOne(exec, result, SignOption::Signed);
+            return absoluteAddOne(globalObject, result, SignOption::Signed);
         }
     }
 
@@ -1435,11 +1555,7 @@ static constexpr size_t bitsPerCharTableMultiplier = 1u << bitsPerCharTableShift
 // Divide bit length of the BigInt by bits representable per character.
 uint64_t JSBigInt::calculateMaximumCharactersRequired(unsigned length, unsigned radix, Digit lastDigit, bool sign)
 {
-    unsigned leadingZeros;
-    if (sizeof(lastDigit) == 8)
-        leadingZeros = clz64(lastDigit);
-    else
-        leadingZeros = clz32(lastDigit);
+    unsigned leadingZeros = clz(lastDigit);
 
     size_t bitLength = length * digitBits - leadingZeros;
 
@@ -1463,33 +1579,30 @@ uint64_t JSBigInt::calculateMaximumCharactersRequired(unsigned length, unsigned
     return maximumCharactersRequired;
 }
 
-String JSBigInt::toStringBasePowerOfTwo(ExecState* exec, JSBigInt* x, unsigned radix)
+String JSBigInt::toStringBasePowerOfTwo(VM& vm, JSGlobalObject* globalObject, JSBigInt* x, unsigned radix)
 {
     ASSERT(hasOneBitSet(radix));
     ASSERT(radix >= 2 && radix <= 32);
     ASSERT(!x->isZero());
-    VM& vm = exec->vm();
 
     const unsigned length = x->length();
     const bool sign = x->sign();
-    const unsigned bitsPerChar = ctz32(radix);
+    const unsigned bitsPerChar = ctz(radix);
     const unsigned charMask = radix - 1;
     // Compute the length of the resulting string: divide the bit length of the
     // BigInt by the number of bits representable per character (rounding up).
     const Digit msd = x->digit(length - 1);
 
-#if USE(JSVALUE64)
-    const unsigned msdLeadingZeros = clz64(msd);
-#else
-    const unsigned msdLeadingZeros = clz32(msd);
-#endif
+    const unsigned msdLeadingZeros = clz(msd);
 
     const size_t bitLength = length * digitBits - msdLeadingZeros;
     const size_t charsRequired = (bitLength + bitsPerChar - 1) / bitsPerChar + sign;
 
     if (charsRequired > JSString::MaxLength) {
-        auto scope = DECLARE_THROW_SCOPE(vm);
-        throwOutOfMemoryError(exec, scope);
+        if (globalObject) {
+            auto scope = DECLARE_THROW_SCOPE(vm);
+            throwOutOfMemoryError(globalObject, scope);
+        }
         return String();
     }
 
@@ -1528,14 +1641,12 @@ String JSBigInt::toStringBasePowerOfTwo(ExecState* exec, JSBigInt* x, unsigned r
     return StringImpl::adopt(WTFMove(resultString));
 }
 
-String JSBigInt::toStringGeneric(ExecState* exec, JSBigInt* x, unsigned radix)
+String JSBigInt::toStringGeneric(VM& vm, JSGlobalObject* globalObject, JSBigInt* x, unsigned radix)
 {
     // FIXME: [JSC] Revisit usage of Vector into JSBigInt::toString
     // https://bugs.webkit.org/show_bug.cgi?id=18067
     Vector<LChar> resultString;
 
-    VM& vm = exec->vm();
-
     ASSERT(radix >= 2 && radix <= 36);
     ASSERT(!x->isZero());
 
@@ -1546,8 +1657,10 @@ String JSBigInt::toStringGeneric(ExecState* exec, JSBigInt* x, unsigned radix)
     uint64_t maximumCharactersRequired = calculateMaximumCharactersRequired(length, radix, x->digit(length - 1), sign);
 
     if (maximumCharactersRequired > JSString::MaxLength) {
-        auto scope = DECLARE_THROW_SCOPE(vm);
-        throwOutOfMemoryError(exec, scope);
+        if (globalObject) {
+            auto scope = DECLARE_THROW_SCOPE(vm);
+            throwOutOfMemoryError(globalObject, scope);
+        }
         return String();
     }
 
@@ -1639,7 +1752,7 @@ JSBigInt* JSBigInt::rightTrim(VM& vm)
     return trimmedBigInt;
 }
 
-JSBigInt* JSBigInt::allocateFor(ExecState* exec, VM& vm, unsigned radix, unsigned charcount)
+JSBigInt* JSBigInt::allocateFor(JSGlobalObject* globalObject, VM& vm, unsigned radix, unsigned charcount)
 {
     ASSERT(2 <= radix && radix <= 36);
 
@@ -1661,9 +1774,9 @@ JSBigInt* JSBigInt::allocateFor(ExecState* exec, VM& vm, unsigned radix, unsigne
         }
     }
 
-    if (exec) {
+    if (globalObject) {
         auto scope = DECLARE_THROW_SCOPE(vm);
-        throwOutOfMemoryError(exec, scope);
+        throwOutOfMemoryError(globalObject, scope);
     }
     return nullptr;
 }
@@ -1673,25 +1786,25 @@ size_t JSBigInt::estimatedSize(JSCell* cell, VM& vm)
     return Base::estimatedSize(cell, vm) + jsCast<JSBigInt*>(cell)->m_length * sizeof(Digit);
 }
 
-double JSBigInt::toNumber(ExecState* exec) const
+double JSBigInt::toNumber(JSGlobalObject* globalObject) const
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
-    throwTypeError(exec, scope, "Conversion from 'BigInt' to 'number' is not allowed."_s);
+    throwTypeError(globalObject, scope, "Conversion from 'BigInt' to 'number' is not allowed."_s);
     return 0.0;
 }
 
-bool JSBigInt::getPrimitiveNumber(ExecState* exec, double& number, JSValue& result) const
+bool JSBigInt::getPrimitiveNumber(JSGlobalObject* globalObject, double& number, JSValue& result) const
 {
     result = this;
-    number = toNumber(exec);
+    number = toNumber(globalObject);
     return true;
 }
 
 template <typename CharType>
-JSBigInt* JSBigInt::parseInt(ExecState* exec, CharType*  data, unsigned length, ErrorParseMode errorParseMode)
+JSBigInt* JSBigInt::parseInt(JSGlobalObject* globalObject, CharType*  data, unsigned length, ErrorParseMode errorParseMode)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
 
     unsigned p = 0;
     while (p < length && isStrWhiteSpace(data[p]))
@@ -1700,13 +1813,13 @@ JSBigInt* JSBigInt::parseInt(ExecState* exec, CharType*  data, unsigned length,
     // Check Radix from frist characters
     if (static_cast<unsigned>(p) + 1 < static_cast<unsigned>(length) && data[p] == '0') {
         if (isASCIIAlphaCaselessEqual(data[p + 1], 'b'))
-            return parseInt(exec, vm, data, length, p + 2, 2, errorParseMode, ParseIntSign::Unsigned, ParseIntMode::DisallowEmptyString);
+            return parseInt(globalObject, vm, data, length, p + 2, 2, errorParseMode, ParseIntSign::Unsigned, ParseIntMode::DisallowEmptyString);
 
         if (isASCIIAlphaCaselessEqual(data[p + 1], 'x'))
-            return parseInt(exec, vm, data, length, p + 2, 16, errorParseMode, ParseIntSign::Unsigned, ParseIntMode::DisallowEmptyString);
+            return parseInt(globalObject, vm, data, length, p + 2, 16, errorParseMode, ParseIntSign::Unsigned, ParseIntMode::DisallowEmptyString);
 
         if (isASCIIAlphaCaselessEqual(data[p + 1], 'o'))
-            return parseInt(exec, vm, data, length, p + 2, 8, errorParseMode, ParseIntSign::Unsigned, ParseIntMode::DisallowEmptyString);
+            return parseInt(globalObject, vm, data, length, p + 2, 8, errorParseMode, ParseIntSign::Unsigned, ParseIntMode::DisallowEmptyString);
     }
 
     ParseIntSign sign = ParseIntSign::Unsigned;
@@ -1719,7 +1832,7 @@ JSBigInt* JSBigInt::parseInt(ExecState* exec, CharType*  data, unsigned length,
         }
     }
 
-    JSBigInt* result = parseInt(exec, vm, data, length, p, 10, errorParseMode, sign);
+    JSBigInt* result = parseInt(globalObject, vm, data, length, p, 10, errorParseMode, sign);
 
     if (result && !result->isZero())
         result->setSign(sign == ParseIntSign::Signed);
@@ -1728,7 +1841,7 @@ JSBigInt* JSBigInt::parseInt(ExecState* exec, CharType*  data, unsigned length,
 }
 
 template <typename CharType>
-JSBigInt* JSBigInt::parseInt(ExecState* exec, VM& vm, CharType* data, unsigned length, unsigned startIndex, unsigned radix, ErrorParseMode errorParseMode, ParseIntSign sign, ParseIntMode parseMode)
+JSBigInt* JSBigInt::parseInt(JSGlobalObject* globalObject, VM& vm, CharType* data, unsigned length, unsigned startIndex, unsigned radix, ErrorParseMode errorParseMode, ParseIntSign sign, ParseIntMode parseMode)
 {
     ASSERT(length >= 0);
     unsigned p = startIndex;
@@ -1736,9 +1849,9 @@ JSBigInt* JSBigInt::parseInt(ExecState* exec, VM& vm, CharType* data, unsigned l
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (parseMode != ParseIntMode::AllowEmptyString && startIndex == length) {
-        ASSERT(exec);
+        ASSERT(globalObject);
         if (errorParseMode == ErrorParseMode::ThrowExceptions)
-            throwVMError(exec, scope, createSyntaxError(exec, "Failed to parse String to BigInt"));
+            throwVMError(globalObject, scope, createSyntaxError(globalObject, "Failed to parse String to BigInt"));
         return nullptr;
     }
 
@@ -1760,8 +1873,11 @@ JSBigInt* JSBigInt::parseInt(ExecState* exec, VM& vm, CharType* data, unsigned l
     unsigned limita = 'a' + (radix - 10);
     unsigned limitA = 'A' + (radix - 10);
 
-    JSBigInt* result = allocateFor(exec, vm, radix, length - p);
+    JSBigInt* result = allocateFor(globalObject, vm, radix, length - p);
     RETURN_IF_EXCEPTION(scope, nullptr);
+    // result can still be null if we don't have access to global object, as allocateFor cannot throw an exception in that case.
+    if (!result)
+        return nullptr;
 
     result->initialize(InitializationType::WithZero);
 
@@ -1783,9 +1899,9 @@ JSBigInt* JSBigInt::parseInt(ExecState* exec, VM& vm, CharType* data, unsigned l
     if (p == length)
         return result->rightTrim(vm);
 
-    ASSERT(exec);
+    ASSERT(globalObject);
     if (errorParseMode == ErrorParseMode::ThrowExceptions)
-        throwVMError(exec, scope, createSyntaxError(exec, "Failed to parse String to BigInt"));
+        throwVMError(globalObject, scope, createSyntaxError(globalObject, "Failed to parse String to BigInt"));
 
     return nullptr;
 }
@@ -1802,9 +1918,9 @@ inline void JSBigInt::setDigit(unsigned n, Digit value)
     dataStorage()[n] = value;
 }
 
-JSObject* JSBigInt::toObject(ExecState* exec, JSGlobalObject* globalObject) const
+JSObject* JSBigInt::toObject(JSGlobalObject* globalObject) const
 {
-    return BigIntObject::create(exec->vm(), globalObject, const_cast<JSBigInt*>(this));
+    return BigIntObject::create(globalObject->vm(), globalObject, const_cast<JSBigInt*>(this));
 }
 
 bool JSBigInt::equalsToNumber(JSValue numValue)
@@ -1866,7 +1982,7 @@ JSBigInt::ComparisonResult JSBigInt::compareToDouble(JSBigInt* x, double y)
 
     int xLength = x->length();
     Digit xMSD = x->digit(xLength - 1);
-    int msdLeadingZeros = sizeof(xMSD) == 8  ? clz64(xMSD) : clz32(xMSD);
+    int msdLeadingZeros = clz(xMSD);
 
     int xBitLength = xLength * digitBits - msdLeadingZeros;
     int yBitLength = exponent + 1;
