@@ -22,11 +22,12 @@
 
 #include <mutex>
 #include <wtf/Assertions.h>
+#include <wtf/ForbidHeapAllocation.h>
 #include <wtf/Lock.h>
 #include <wtf/Noncopyable.h>
 #include <wtf/RefPtr.h>
 #include <wtf/Threading.h>
-#include <wtf/text/AtomicStringTable.h>
+#include <wtf/text/AtomStringTable.h>
 
 namespace JSC {
 
@@ -49,8 +50,10 @@ namespace JSC {
 // DropAllLocks object takes care to release the JSLock only if your
 // thread acquired it to begin with.
 
-class ExecState;
+class CallFrame;
 class VM;
+class JSGlobalObject;
+class JSLock;
 
 // This class is used to protect the initialization of the legacy single
 // shared VM.
@@ -67,12 +70,11 @@ class JSLockHolder {
 public:
     JS_EXPORT_PRIVATE JSLockHolder(VM*);
     JS_EXPORT_PRIVATE JSLockHolder(VM&);
-    JS_EXPORT_PRIVATE JSLockHolder(ExecState*);
+    JS_EXPORT_PRIVATE JSLockHolder(JSGlobalObject*);
 
     JS_EXPORT_PRIVATE ~JSLockHolder();
-private:
-    void init();
 
+private:
     RefPtr<VM> m_vm;
 };
 
@@ -85,8 +87,8 @@ public:
     JS_EXPORT_PRIVATE void lock();
     JS_EXPORT_PRIVATE void unlock();
 
-    static void lock(ExecState*);
-    static void unlock(ExecState*);
+    static void lock(JSGlobalObject*);
+    static void unlock(JSGlobalObject*);
     static void lock(VM&);
     static void unlock(VM&);
 
@@ -105,7 +107,7 @@ public:
     class DropAllLocks {
         WTF_MAKE_NONCOPYABLE(DropAllLocks);
     public:
-        JS_EXPORT_PRIVATE DropAllLocks(ExecState*);
+        JS_EXPORT_PRIVATE DropAllLocks(JSGlobalObject*);
         JS_EXPORT_PRIVATE DropAllLocks(VM*);
         JS_EXPORT_PRIVATE DropAllLocks(VM&);
         JS_EXPORT_PRIVATE ~DropAllLocks();
@@ -119,6 +121,13 @@ public:
         unsigned m_dropDepth;
     };
 
+    void makeWebThreadAware()
+    {
+        m_isWebThreadAware = true;
+    }
+
+    bool isWebThreadAware() const { return m_isWebThreadAware; }
+
 private:
     void lock(intptr_t lockCount);
     void unlock(intptr_t unlockCount);
@@ -130,6 +139,7 @@ private:
     void grabAllLocks(DropAllLocks*, unsigned lockCount);
 
     Lock m_lock;
+    bool m_isWebThreadAware { false };
     // We cannot make m_ownerThread an optional (instead of pairing it with an explicit
     // m_hasOwnerThread) because currentThreadIsHoldingLock() may be called from a
     // different thread, and an optional is vulnerable to races.
@@ -140,7 +150,7 @@ private:
     unsigned m_lockDropDepth;
     bool m_shouldReleaseHeapAccess;
     VM* m_vm;
-    AtomicStringTable* m_entryAtomicStringTable;
+    AtomStringTable* m_entryAtomStringTable;
 };
 
 } // namespace
