@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,67 +27,25 @@
 #include "JSRunLoopTimer.h"
 
 #include "IncrementalSweeper.h"
-#include "JSCInlines.h"
-#include "JSObject.h"
-#include "JSString.h"
-
-#include <wtf/MainThread.h>
+#include "VM.h"
+#include <mutex>
 #include <wtf/NoTailCalls.h>
-#include <wtf/Threading.h>
 
 #if USE(GLIB_EVENT_LOOP)
 #include <glib.h>
 #include <wtf/glib/RunLoopSourcePriority.h>
 #endif
 
-#include <mutex>
-
 namespace JSC {
 
-const Seconds JSRunLoopTimer::s_decade { 60 * 60 * 24 * 365 * 10 };
-
 static inline JSRunLoopTimer::Manager::EpochTime epochTime(Seconds delay)
 {
-#if USE(CF)
-    return Seconds { CFAbsoluteTimeGetCurrent() + delay.value() };
-#else
     return MonotonicTime::now().secondsSinceEpoch() + delay;
-#endif
 }
 
-#if USE(CF)
-void JSRunLoopTimer::Manager::timerDidFireCallback(CFRunLoopTimerRef, void* contextPtr)
-{
-    static_cast<JSRunLoopTimer::Manager*>(contextPtr)->timerDidFire();
-}
-
-void JSRunLoopTimer::Manager::PerVMData::setRunLoop(Manager* manager, CFRunLoopRef newRunLoop)
-{
-    if (runLoop) {
-        CFRunLoopRemoveTimer(runLoop.get(), timer.get(), kCFRunLoopCommonModes);
-        CFRunLoopTimerInvalidate(timer.get());
-        runLoop.clear();
-        timer.clear();
-    }
-
-    if (newRunLoop) {
-        runLoop = newRunLoop;
-        memset(&context, 0, sizeof(CFRunLoopTimerContext));
-        RELEASE_ASSERT(manager);
-        context.info = manager;
-        timer = adoptCF(CFRunLoopTimerCreate(kCFAllocatorDefault, CFAbsoluteTimeGetCurrent() + s_decade.seconds(), CFAbsoluteTimeGetCurrent() + s_decade.seconds(), 0, 0, JSRunLoopTimer::Manager::timerDidFireCallback, &context));
-        CFRunLoopAddTimer(runLoop.get(), timer.get(), kCFRunLoopCommonModes);
-
-        EpochTime scheduleTime = epochTime(s_decade);
-        for (auto& pair : timers)
-            scheduleTime = std::min(pair.second, scheduleTime);
-        CFRunLoopTimerSetNextFireDate(timer.get(), scheduleTime.value());
-    }
-}
-#else
-JSRunLoopTimer::Manager::PerVMData::PerVMData(Manager& manager)
-    : runLoop(&RunLoop::current())
-    , timer(std::make_unique<RunLoop::Timer<Manager>>(*runLoop, &manager, &JSRunLoopTimer::Manager::timerDidFireCallback))
+JSRunLoopTimer::Manager::PerVMData::PerVMData(Manager& manager, RunLoop& runLoop)
+    : runLoop(runLoop)
+    , timer(makeUnique<RunLoop::Timer<Manager>>(runLoop, &manager, &JSRunLoopTimer::Manager::timerDidFireCallback))
 {
 #if USE(GLIB_EVENT_LOOP)
     timer->setPriority(RunLoopSourcePriority::JavascriptTimer);
@@ -99,13 +57,14 @@ void JSRunLoopTimer::Manager::timerDidFireCallback()
 {
     timerDidFire();
 }
-#endif
 
 JSRunLoopTimer::Manager::PerVMData::~PerVMData()
 {
-#if USE(CF)
-    setRunLoop(nullptr, nullptr);
-#endif
+    // Because RunLoop::Timer is not reference counted, we need to deallocate it
+    // on the same thread on which it fires; otherwise, we might deallocate it
+    // while it's firing.
+    runLoop->dispatch([timer = WTFMove(timer)] {
+    });
 }
 
 void JSRunLoopTimer::Manager::timerDidFire()
@@ -114,21 +73,12 @@ void JSRunLoopTimer::Manager::timerDidFire()
 
     {
         auto locker = holdLock(m_lock);
-#if USE(CF)
-        CFRunLoopRef currentRunLoop = CFRunLoopGetCurrent();
-#else
         RunLoop* currentRunLoop = &RunLoop::current();
-#endif
         EpochTime nowEpochTime = epochTime(0_s);
         for (auto& entry : m_mapping) {
-            PerVMData& data = entry.value;
-#if USE(CF)
-            if (data.runLoop.get() != currentRunLoop)
+            PerVMData& data = *entry.value;
+            if (data.runLoop.ptr() != currentRunLoop)
                 continue;
-#else
-            if (data.runLoop != currentRunLoop)
-                continue;
-#endif
 
             EpochTime scheduleTime = epochTime(s_decade);
             for (size_t i = 0; i < data.timers.size(); ++i) {
@@ -148,11 +98,7 @@ void JSRunLoopTimer::Manager::timerDidFire()
                 timersToFire.append(WTFMove(pair.first));
             }
 
-#if USE(CF)
-            CFRunLoopTimerSetNextFireDate(data.timer.get(), scheduleTime.value());
-#else
             data.timer->startOneShot(std::max(0_s, scheduleTime - MonotonicTime::now().secondsSinceEpoch()));
-#endif
         }
     }
 
@@ -172,10 +118,7 @@ JSRunLoopTimer::Manager& JSRunLoopTimer::Manager::shared()
 
 void JSRunLoopTimer::Manager::registerVM(VM& vm)
 {
-    PerVMData data { *this };
-#if USE(CF)
-    data.setRunLoop(this, vm.runLoop());
-#endif
+    auto data = makeUnique<PerVMData>(*this, vm.runLoop());
 
     auto locker = holdLock(m_lock);
     auto addResult = m_mapping.add({ vm.apiLock() }, WTFMove(data));
@@ -199,7 +142,7 @@ void JSRunLoopTimer::Manager::scheduleTimer(JSRunLoopTimer& timer, Seconds delay
     auto iter = m_mapping.find(timer.m_apiLock);
     RELEASE_ASSERT(iter != m_mapping.end()); // We don't allow calling this after the VM dies.
 
-    PerVMData& data = iter->value;
+    PerVMData& data = *iter->value;
     EpochTime scheduleTime = fireEpochTime;
     bool found = false;
     for (auto& entry : data.timers) {
@@ -213,11 +156,7 @@ void JSRunLoopTimer::Manager::scheduleTimer(JSRunLoopTimer& timer, Seconds delay
     if (!found)
         data.timers.append({ timer, fireEpochTime });
 
-#if USE(CF)
-    CFRunLoopTimerSetNextFireDate(data.timer.get(), scheduleTime.value());
-#else
     data.timer->startOneShot(std::max(0_s, scheduleTime - MonotonicTime::now().secondsSinceEpoch()));
-#endif
 }
 
 void JSRunLoopTimer::Manager::cancelTimer(JSRunLoopTimer& timer)
@@ -229,7 +168,7 @@ void JSRunLoopTimer::Manager::cancelTimer(JSRunLoopTimer& timer)
         return;
     }
 
-    PerVMData& data = iter->value;
+    PerVMData& data = *iter->value;
     EpochTime scheduleTime = epochTime(s_decade);
     for (unsigned i = 0; i < data.timers.size(); ++i) {
         {
@@ -248,11 +187,7 @@ void JSRunLoopTimer::Manager::cancelTimer(JSRunLoopTimer& timer)
         scheduleTime = std::min(scheduleTime, data.timers[i].second);
     }
 
-#if USE(CF)
-    CFRunLoopTimerSetNextFireDate(data.timer.get(), scheduleTime.value());
-#else
     data.timer->startOneShot(std::max(0_s, scheduleTime - MonotonicTime::now().secondsSinceEpoch()));
-#endif
 }
 
 Optional<Seconds> JSRunLoopTimer::Manager::timeUntilFire(JSRunLoopTimer& timer)
@@ -261,7 +196,7 @@ Optional<Seconds> JSRunLoopTimer::Manager::timeUntilFire(JSRunLoopTimer& timer)
     auto iter = m_mapping.find(timer.m_apiLock);
     RELEASE_ASSERT(iter != m_mapping.end()); // We only allow this to be called with a live VM.
 
-    PerVMData& data = iter->value;
+    PerVMData& data = *iter->value;
     for (auto& entry : data.timers) {
         if (entry.first.ptr() == &timer) {
             EpochTime nowEpochTime = epochTime(0_s);
@@ -272,18 +207,6 @@ Optional<Seconds> JSRunLoopTimer::Manager::timeUntilFire(JSRunLoopTimer& timer)
     return WTF::nullopt;
 }
 
-#if USE(CF)
-void JSRunLoopTimer::Manager::didChangeRunLoop(VM& vm, CFRunLoopRef newRunLoop)
-{
-    auto locker = holdLock(m_lock);
-    auto iter = m_mapping.find({ vm.apiLock() });
-    RELEASE_ASSERT(iter != m_mapping.end());
-
-    PerVMData& data = iter->value;
-    data.setRunLoop(this, newRunLoop);
-}
-#endif
-
 void JSRunLoopTimer::timerDidFire()
 {
     NO_TAIL_CALLS();
@@ -297,7 +220,7 @@ void JSRunLoopTimer::timerDidFire()
         }
     }
 
-    std::lock_guard<JSLock> lock(m_apiLock.get());
+    auto locker = holdLock(m_apiLock.get());
     RefPtr<VM> vm = m_apiLock->vm();
     if (!vm) {
         // The VM has been destroyed, so we should just give up.
@@ -307,8 +230,8 @@ void JSRunLoopTimer::timerDidFire()
     doWork(*vm);
 }
 
-JSRunLoopTimer::JSRunLoopTimer(VM* vm)
-    : m_apiLock(vm->apiLock())
+JSRunLoopTimer::JSRunLoopTimer(VM& vm)
+    : m_apiLock(vm.apiLock())
 {
 }
 
