@@ -47,17 +47,18 @@ class LLIntOffsetsExtractor;
 // JSSegmentedVariableObject has its own GC tracing functionality, since it knows the
 // exact dimensions of the variables array at all times.
 
-// Except for JSGlobalObject, subclasses of this don't call the destructor and leak memory.
-
 class JSSegmentedVariableObject : public JSSymbolTableObject {
     friend class JIT;
     friend class LLIntOffsetsExtractor;
-
 public:
     using Base = JSSymbolTableObject;
 
     DECLARE_INFO;
 
+    static constexpr bool needsDestruction = true;
+    template<typename CellType, SubspaceAccess>
+    static void subspaceFor(VM&) { RELEASE_ASSERT_NOT_REACHED(); }
+
     bool isValidScopeOffset(ScopeOffset offset)
     {
         return !!offset && offset.offset() < m_variables.size();
@@ -76,7 +77,7 @@ public:
 
     WriteBarrier<Unknown>* assertVariableIsInThisObject(WriteBarrier<Unknown>* variablePointer)
     {
-        if (!ASSERT_DISABLED)
+        if (ASSERT_ENABLED)
             findVariableIndex(variablePointer);
         return variablePointer;
     }
@@ -86,17 +87,7 @@ public:
     JS_EXPORT_PRIVATE ScopeOffset addVariables(unsigned numberOfVariablesToAdd, JSValue);
 
     JS_EXPORT_PRIVATE static void visitChildren(JSCell*, SlotVisitor&);
-    JS_EXPORT_PRIVATE static void heapSnapshot(JSCell*, HeapSnapshotBuilder&);
-
-    static void destroy(JSCell*);
-
-    template<typename, SubspaceAccess>
-    static CompleteSubspace* subspaceFor(VM& vm)
-    {
-        return &vm.segmentedVariableObjectSpace;
-    }
-
-    const ClassInfo* classInfo() const { return m_classInfo; }
+    JS_EXPORT_PRIVATE static void analyzeHeap(JSCell*, HeapAnalyzer&);
 
 protected:
     JSSegmentedVariableObject(VM&, Structure*, JSScope*);
@@ -107,9 +98,9 @@ protected:
 
 private:
     SegmentedVector<WriteBarrier<Unknown>, 16> m_variables;
-    const ClassInfo* m_classInfo;
-    ConcurrentJSLock m_lock;
+#ifndef NDEBUG
     bool m_alreadyDestroyed { false }; // We use these assertions to check that we aren't doing ancient hacks that result in this being destroyed more than once.
+#endif
 };
 
 } // namespace JSC
