@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 1999-2002 Harri Porten (porten@kde.org)
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
- *  Copyright (C) 2004-2017 Apple Inc. All rights reserved.
+ *  Copyright (C) 2004-2019 Apple Inc. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -23,13 +23,12 @@
 #include "config.h"
 #include "JSString.h"
 
-#include "JSGlobalObject.h"
 #include "JSGlobalObjectFunctions.h"
-#include "JSObject.h"
-#include "JSCInlines.h"
+#include "JSGlobalObjectInlines.h"
+#include "JSObjectInlines.h"
 #include "StringObject.h"
-#include "StringPrototype.h"
 #include "StrongInlines.h"
+#include "StructureInlines.h"
 
 namespace JSC {
 
@@ -64,36 +63,35 @@ void JSRopeString::RopeBuilder<RecordOverflow>::expand()
     m_strings.append(string);
 }
 
-void JSString::destroy(JSCell* cell)
-{
-    static_cast<JSString*>(cell)->JSString::~JSString();
-}
-
 void JSString::dumpToStream(const JSCell* cell, PrintStream& out)
 {
-    VM& vm = *cell->vm();
+    VM& vm = cell->vm();
     const JSString* thisObject = jsCast<const JSString*>(cell);
     out.printf("<%p, %s, [%u], ", thisObject, thisObject->className(vm), thisObject->length());
     uintptr_t pointer = thisObject->m_fiber;
-    if (pointer & isRopeInPointer)
-        out.printf("[rope]");
-    else {
-        if (WTF::StringImpl* ourImpl = bitwise_cast<StringImpl*>(pointer)) {
-        if (ourImpl->is8Bit())
-            out.printf("[8 %p]", ourImpl->characters8());
+    if (pointer & isRopeInPointer) {
+        if (pointer & JSRopeString::isSubstringInPointer)
+            out.printf("[substring]");
         else
-            out.printf("[16 %p]", ourImpl->characters16());
-    }
+            out.printf("[rope]");
+    } else {
+        if (WTF::StringImpl* ourImpl = bitwise_cast<StringImpl*>(pointer)) {
+            if (ourImpl->is8Bit())
+                out.printf("[8 %p]", ourImpl->characters8());
+            else
+                out.printf("[16 %p]", ourImpl->characters16());
+        }
     }
     out.printf(">");
 }
 
-bool JSString::equalSlowCase(ExecState* exec, JSString* other) const
+bool JSString::equalSlowCase(JSGlobalObject* globalObject, JSString* other) const
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
-    String str1 = value(exec);
-    String str2 = other->value(exec);
+    String str1 = value(globalObject);
+    RETURN_IF_EXCEPTION(scope, false);
+    String str2 = other->value(globalObject);
     RETURN_IF_EXCEPTION(scope, false);
     return WTF::equal(*str1.impl(), *str2.impl());
 }
@@ -110,14 +108,15 @@ size_t JSString::estimatedSize(JSCell* cell, VM& vm)
 void JSString::visitChildren(JSCell* cell, SlotVisitor& visitor)
 {
     JSString* thisObject = asString(cell);
+    ASSERT_GC_OBJECT_INHERITS(thisObject, info());
     Base::visitChildren(thisObject, visitor);
 
     uintptr_t pointer = thisObject->m_fiber;
     if (pointer & isRopeInPointer) {
-        if ((pointer & JSRopeString::stringMask) == JSRopeString::substringSentinel()) {
+        if (pointer & JSRopeString::isSubstringInPointer) {
             visitor.appendUnbarriered(static_cast<JSRopeString*>(thisObject)->fiber1());
-        return;
-    }
+            return;
+        }
         for (unsigned index = 0; index < JSRopeString::s_maxInternalRopeLength; ++index) {
             JSString* fiber = nullptr;
             switch (index) {
@@ -144,7 +143,7 @@ void JSString::visitChildren(JSCell* cell, SlotVisitor& visitor)
         visitor.reportExtraMemoryVisited(impl->costDuringGC());
 }
 
-static const unsigned maxLengthForOnStackResolve = 2048;
+static constexpr unsigned maxLengthForOnStackResolve = 2048;
 
 void JSRopeString::resolveRopeInternal8(LChar* buffer) const
 {
@@ -153,26 +152,7 @@ void JSRopeString::resolveRopeInternal8(LChar* buffer) const
         return;
     }
 
-    resolveRopeInternal8NoSubstring(buffer);
-}
-
-void JSRopeString::resolveRopeInternal8NoSubstring(LChar* buffer) const
-{
-    for (size_t i = 0; i < s_maxInternalRopeLength && fiber(i); ++i) {
-        if (fiber(i)->isRope()) {
-            resolveRopeSlowCase8(buffer);
-            return;
-        }
-    }
-
-    LChar* position = buffer;
-    for (size_t i = 0; i < s_maxInternalRopeLength && fiber(i); ++i) {
-        const StringImpl& fiberString = *fiber(i)->valueInternal().impl();
-        unsigned length = fiberString.length();
-        StringImpl::copyCharacters(position, fiberString.characters8(), length);
-        position += length;
-    }
-    ASSERT((buffer + length()) == position);
+    resolveRopeInternalNoSubstring(buffer);
 }
 
 void JSRopeString::resolveRopeInternal16(UChar* buffer) const
@@ -183,10 +163,11 @@ void JSRopeString::resolveRopeInternal16(UChar* buffer) const
         return;
     }
 
-    resolveRopeInternal16NoSubstring(buffer);
+    resolveRopeInternalNoSubstring(buffer);
 }
 
-void JSRopeString::resolveRopeInternal16NoSubstring(UChar* buffer) const
+template<typename CharacterType>
+void JSRopeString::resolveRopeInternalNoSubstring(CharacterType* buffer) const
 {
     for (size_t i = 0; i < s_maxInternalRopeLength && fiber(i); ++i) {
         if (fiber(i)->isRope()) {
@@ -195,7 +176,7 @@ void JSRopeString::resolveRopeInternal16NoSubstring(UChar* buffer) const
         }
     }
 
-    UChar* position = buffer;
+    CharacterType* position = buffer;
     for (size_t i = 0; i < s_maxInternalRopeLength && fiber(i); ++i) {
         const StringImpl& fiberString = *fiber(i)->valueInternal().impl();
         unsigned length = fiberString.length();
@@ -208,26 +189,26 @@ void JSRopeString::resolveRopeInternal16NoSubstring(UChar* buffer) const
     ASSERT((buffer + length()) == position);
 }
 
-AtomicString JSRopeString::resolveRopeToAtomicString(ExecState* exec) const
+AtomString JSRopeString::resolveRopeToAtomString(JSGlobalObject* globalObject) const
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (length() > maxLengthForOnStackResolve) {
         scope.release();
-        return resolveRopeWithFunction(exec, [&] (Ref<StringImpl>&& newImpl) {
-            return AtomicStringImpl::add(newImpl.ptr());
+        return resolveRopeWithFunction(globalObject, [&] (Ref<StringImpl>&& newImpl) {
+            return AtomStringImpl::add(newImpl.ptr());
         });
     }
 
     if (is8Bit()) {
         LChar buffer[maxLengthForOnStackResolve];
         resolveRopeInternal8(buffer);
-        convertToNonRope(AtomicStringImpl::add(buffer, length()));
+        convertToNonRope(AtomStringImpl::add(buffer, length()));
     } else {
         UChar buffer[maxLengthForOnStackResolve];
         resolveRopeInternal16(buffer);
-        convertToNonRope(AtomicStringImpl::add(buffer, length()));
+        convertToNonRope(AtomStringImpl::add(buffer, length()));
     }
 
     // If we resolved a string that didn't previously exist, notify the heap that we've grown.
@@ -248,36 +229,36 @@ inline void JSRopeString::convertToNonRope(String&& string) const
     ASSERT(!JSString::isRope());
 }
 
-RefPtr<AtomicStringImpl> JSRopeString::resolveRopeToExistingAtomicString(ExecState* exec) const
+RefPtr<AtomStringImpl> JSRopeString::resolveRopeToExistingAtomString(JSGlobalObject* globalObject) const
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (length() > maxLengthForOnStackResolve) {
-        RefPtr<AtomicStringImpl> existingAtomicString;
-        resolveRopeWithFunction(exec, [&] (Ref<StringImpl>&& newImpl) -> Ref<StringImpl> {
-            existingAtomicString = AtomicStringImpl::lookUp(newImpl.ptr());
-            if (existingAtomicString)
-                return makeRef(*existingAtomicString);
+        RefPtr<AtomStringImpl> existingAtomString;
+        resolveRopeWithFunction(globalObject, [&] (Ref<StringImpl>&& newImpl) -> Ref<StringImpl> {
+            existingAtomString = AtomStringImpl::lookUp(newImpl.ptr());
+            if (existingAtomString)
+                return makeRef(*existingAtomString);
             return WTFMove(newImpl);
         });
         RETURN_IF_EXCEPTION(scope, nullptr);
-            return existingAtomicString;
-        }
+        return existingAtomString;
+    }
 
     if (is8Bit()) {
         LChar buffer[maxLengthForOnStackResolve];
         resolveRopeInternal8(buffer);
-        if (RefPtr<AtomicStringImpl> existingAtomicString = AtomicStringImpl::lookUp(buffer, length())) {
-            convertToNonRope(*existingAtomicString);
-            return existingAtomicString;
+        if (RefPtr<AtomStringImpl> existingAtomString = AtomStringImpl::lookUp(buffer, length())) {
+            convertToNonRope(*existingAtomString);
+            return existingAtomString;
         }
     } else {
         UChar buffer[maxLengthForOnStackResolve];
         resolveRopeInternal16(buffer);
-        if (RefPtr<AtomicStringImpl> existingAtomicString = AtomicStringImpl::lookUp(buffer, length())) {
-            convertToNonRope(*existingAtomicString);
-            return existingAtomicString;
+        if (RefPtr<AtomStringImpl> existingAtomString = AtomStringImpl::lookUp(buffer, length())) {
+            convertToNonRope(*existingAtomString);
+            return existingAtomString;
         }
     }
 
@@ -285,11 +266,11 @@ RefPtr<AtomicStringImpl> JSRopeString::resolveRopeToExistingAtomicString(ExecSta
 }
 
 template<typename Function>
-const String& JSRopeString::resolveRopeWithFunction(ExecState* nullOrExecForOOM, Function&& function) const
+const String& JSRopeString::resolveRopeWithFunction(JSGlobalObject* nullOrGlobalObjectForOOM, Function&& function) const
 {
     ASSERT(isRope());
 
-    VM& vm = *this->vm();
+    VM& vm = this->vm();
     if (isSubstring()) {
         ASSERT(!substringBase()->isRope());
         auto newImpl = substringBase()->valueInternal().substringSharingImpl(substringOffset(), length());
@@ -301,12 +282,12 @@ const String& JSRopeString::resolveRopeWithFunction(ExecState* nullOrExecForOOM,
         LChar* buffer;
         auto newImpl = StringImpl::tryCreateUninitialized(length(), buffer);
         if (!newImpl) {
-            outOfMemory(nullOrExecForOOM);
+            outOfMemory(nullOrGlobalObjectForOOM);
             return nullString();
         }
         vm.heap.reportExtraMemoryAllocated(newImpl->cost());
 
-        resolveRopeInternal8NoSubstring(buffer);
+        resolveRopeInternalNoSubstring(buffer);
         convertToNonRope(function(newImpl.releaseNonNull()));
         return valueInternal();
     }
@@ -314,19 +295,19 @@ const String& JSRopeString::resolveRopeWithFunction(ExecState* nullOrExecForOOM,
     UChar* buffer;
     auto newImpl = StringImpl::tryCreateUninitialized(length(), buffer);
     if (!newImpl) {
-        outOfMemory(nullOrExecForOOM);
+        outOfMemory(nullOrGlobalObjectForOOM);
         return nullString();
     }
     vm.heap.reportExtraMemoryAllocated(newImpl->cost());
 
-    resolveRopeInternal16NoSubstring(buffer);
+    resolveRopeInternalNoSubstring(buffer);
     convertToNonRope(function(newImpl.releaseNonNull()));
     return valueInternal();
 }
 
-const String& JSRopeString::resolveRope(ExecState* nullOrExecForOOM) const
+const String& JSRopeString::resolveRope(JSGlobalObject* nullOrGlobalObjectForOOM) const
 {
-    return resolveRopeWithFunction(nullOrExecForOOM, [] (Ref<StringImpl>&& newImpl) {
+    return resolveRopeWithFunction(nullOrGlobalObjectForOOM, [] (Ref<StringImpl>&& newImpl) {
         return WTFMove(newImpl);
     });
 }
@@ -341,45 +322,10 @@ const String& JSRopeString::resolveRope(ExecState* nullOrExecForOOM) const
 // Vector before performing any concatenation, but by working backwards we likely
 // only fill the queue with the number of substrings at any given level in a
 // rope-of-ropes.)
-void JSRopeString::resolveRopeSlowCase8(LChar* buffer) const
+template<typename CharacterType>
+void JSRopeString::resolveRopeSlowCase(CharacterType* buffer) const
 {
-    LChar* position = buffer + length(); // We will be working backwards over the rope.
-    Vector<JSString*, 32, UnsafeVectorOverflow> workQueue; // Putting strings into a Vector is only OK because there are no GC points in this method.
-
-    for (size_t i = 0; i < s_maxInternalRopeLength && fiber(i); ++i)
-        workQueue.append(fiber(i));
-
-    while (!workQueue.isEmpty()) {
-        JSString* currentFiber = workQueue.last();
-        workQueue.removeLast();
-
-        const LChar* characters;
-
-        if (currentFiber->isRope()) {
-            JSRopeString* currentFiberAsRope = static_cast<JSRopeString*>(currentFiber);
-            if (!currentFiberAsRope->isSubstring()) {
-                for (size_t i = 0; i < s_maxInternalRopeLength && currentFiberAsRope->fiber(i); ++i)
-                    workQueue.append(currentFiberAsRope->fiber(i));
-                continue;
-            }
-            ASSERT(!currentFiberAsRope->substringBase()->isRope());
-            characters =
-                currentFiberAsRope->substringBase()->valueInternal().characters8() +
-                currentFiberAsRope->substringOffset();
-        } else
-            characters = currentFiber->valueInternal().characters8();
-
-        unsigned length = currentFiber->length();
-        position -= length;
-        StringImpl::copyCharacters(position, characters, length);
-    }
-
-    ASSERT(buffer == position);
-}
-
-void JSRopeString::resolveRopeSlowCase(UChar* buffer) const
-{
-    UChar* position = buffer + length(); // We will be working backwards over the rope.
+    CharacterType* position = buffer + length(); // We will be working backwards over the rope.
     Vector<JSString*, 32, UnsafeVectorOverflow> workQueue; // These strings are kept alive by the parent rope, so using a Vector is OK.
 
     for (size_t i = 0; i < s_maxInternalRopeLength && fiber(i); ++i)
@@ -421,37 +367,37 @@ void JSRopeString::resolveRopeSlowCase(UChar* buffer) const
     ASSERT(buffer == position);
 }
 
-void JSRopeString::outOfMemory(ExecState* nullOrExecForOOM) const
+void JSRopeString::outOfMemory(JSGlobalObject* nullOrGlobalObjectForOOM) const
 {
     ASSERT(isRope());
-    if (nullOrExecForOOM) {
-        VM& vm = nullOrExecForOOM->vm();
+    if (nullOrGlobalObjectForOOM) {
+        VM& vm = nullOrGlobalObjectForOOM->vm();
         auto scope = DECLARE_THROW_SCOPE(vm);
-        throwOutOfMemoryError(nullOrExecForOOM, scope);
+        throwOutOfMemoryError(nullOrGlobalObjectForOOM, scope);
     }
 }
 
-JSValue JSString::toPrimitive(ExecState*, PreferredPrimitiveType) const
+JSValue JSString::toPrimitive(JSGlobalObject*, PreferredPrimitiveType) const
 {
     return const_cast<JSString*>(this);
 }
 
-bool JSString::getPrimitiveNumber(ExecState* exec, double& number, JSValue& result) const
+bool JSString::getPrimitiveNumber(JSGlobalObject* globalObject, double& number, JSValue& result) const
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
-    StringView view = unsafeView(exec);
+    StringView view = unsafeView(globalObject);
     RETURN_IF_EXCEPTION(scope, false);
     result = this;
     number = jsToNumber(view);
     return false;
 }
 
-double JSString::toNumber(ExecState* exec) const
+double JSString::toNumber(JSGlobalObject* globalObject) const
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
-    StringView view = unsafeView(exec);
+    StringView view = unsafeView(globalObject);
     RETURN_IF_EXCEPTION(scope, 0);
     return jsToNumber(view);
 }
@@ -463,21 +409,21 @@ inline StringObject* StringObject::create(VM& vm, JSGlobalObject* globalObject,
     return object;
 }
 
-JSObject* JSString::toObject(ExecState* exec, JSGlobalObject* globalObject) const
+JSObject* JSString::toObject(JSGlobalObject* globalObject) const
 {
-    return StringObject::create(exec->vm(), globalObject, const_cast<JSString*>(this));
+    return StringObject::create(globalObject->vm(), globalObject, const_cast<JSString*>(this));
 }
 
-JSValue JSString::toThis(JSCell* cell, ExecState* exec, ECMAMode ecmaMode)
+JSValue JSString::toThis(JSCell* cell, JSGlobalObject* globalObject, ECMAMode ecmaMode)
 {
-    if (ecmaMode == StrictMode)
+    if (ecmaMode.isStrict())
         return cell;
-    return StringObject::create(exec->vm(), exec->lexicalGlobalObject(), asString(cell));
+    return StringObject::create(globalObject->vm(), globalObject, asString(cell));
 }
 
-bool JSString::getStringPropertyDescriptor(ExecState* exec, PropertyName propertyName, PropertyDescriptor& descriptor)
+bool JSString::getStringPropertyDescriptor(JSGlobalObject* globalObject, PropertyName propertyName, PropertyDescriptor& descriptor)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     if (propertyName == vm.propertyNames->length) {
         descriptor.setDescriptor(jsNumber(length()), PropertyAttribute::DontEnum | PropertyAttribute::DontDelete | PropertyAttribute::ReadOnly);
         return true;
@@ -485,7 +431,7 @@ bool JSString::getStringPropertyDescriptor(ExecState* exec, PropertyName propert
 
     Optional<uint32_t> index = parseIndex(propertyName);
     if (index && index.value() < length()) {
-        descriptor.setDescriptor(getIndex(exec, index.value()), PropertyAttribute::DontDelete | PropertyAttribute::ReadOnly);
+        descriptor.setDescriptor(getIndex(globalObject, index.value()), PropertyAttribute::DontDelete | PropertyAttribute::ReadOnly);
         return true;
     }
 
@@ -497,7 +443,7 @@ JSString* jsStringWithCacheSlowCase(VM& vm, StringImpl& stringImpl)
     if (JSString* string = vm.stringCache.get(&stringImpl))
         return string;
 
-    JSString* string = jsString(&vm, String(stringImpl));
+    JSString* string = jsString(vm, String(stringImpl));
     vm.lastCachedString.set(vm, string);
     return string;
 }
