@@ -32,7 +32,7 @@ namespace JSC {
 
 const ClassInfo JSWeakSet::s_info = { "WeakSet", &Base::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(JSWeakSet) };
 
-String JSWeakSet::toStringName(const JSC::JSObject*, ExecState*)
+String JSWeakSet::toStringName(const JSC::JSObject*, JSGlobalObject*)
 {
     return "Object"_s;
 }
