@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,12 +27,9 @@
 #include "MapPrototype.h"
 
 #include "BuiltinNames.h"
-#include "Error.h"
-#include "ExceptionHelpers.h"
-#include "IteratorOperations.h"
 #include "JSCInlines.h"
 #include "JSMap.h"
-#include "Lookup.h"
+#include "JSMapIterator.h"
 
 #include "MapPrototype.lut.h"
 
@@ -43,105 +40,134 @@ const ClassInfo MapPrototype::s_info = { "Map", &Base::s_info, &mapPrototypeTabl
 /* Source for MapPrototype.lut.h
 @begin mapPrototypeTable
   forEach   JSBuiltin  DontEnum|Function 0
-  values    JSBuiltin  DontEnum|Function 0
-  keys      JSBuiltin  DontEnum|Function 0
 @end
 */
 
-static EncodedJSValue JSC_HOST_CALL mapProtoFuncClear(ExecState*);
-static EncodedJSValue JSC_HOST_CALL mapProtoFuncDelete(ExecState*);
-static EncodedJSValue JSC_HOST_CALL mapProtoFuncGet(ExecState*);
-static EncodedJSValue JSC_HOST_CALL mapProtoFuncHas(ExecState*);
-static EncodedJSValue JSC_HOST_CALL mapProtoFuncSet(ExecState*);
+static EncodedJSValue JSC_HOST_CALL mapProtoFuncClear(JSGlobalObject*, CallFrame*);
+static EncodedJSValue JSC_HOST_CALL mapProtoFuncDelete(JSGlobalObject*, CallFrame*);
+static EncodedJSValue JSC_HOST_CALL mapProtoFuncGet(JSGlobalObject*, CallFrame*);
+static EncodedJSValue JSC_HOST_CALL mapProtoFuncHas(JSGlobalObject*, CallFrame*);
+static EncodedJSValue JSC_HOST_CALL mapProtoFuncSet(JSGlobalObject*, CallFrame*);
+static EncodedJSValue JSC_HOST_CALL mapProtoFuncValues(JSGlobalObject*, CallFrame*);
+static EncodedJSValue JSC_HOST_CALL mapProtoFuncKeys(JSGlobalObject*, CallFrame*);
+static EncodedJSValue JSC_HOST_CALL mapProtoFuncEntries(JSGlobalObject*, CallFrame*);
 
-static EncodedJSValue JSC_HOST_CALL mapProtoFuncSize(ExecState*);
+static EncodedJSValue JSC_HOST_CALL mapProtoFuncSize(JSGlobalObject*, CallFrame*);
 
 void MapPrototype::finishCreation(VM& vm, JSGlobalObject* globalObject)
 {
     Base::finishCreation(vm);
     ASSERT(inherits(vm, info()));
-    didBecomePrototype();
 
     JSC_NATIVE_FUNCTION_WITHOUT_TRANSITION(vm.propertyNames->clear, mapProtoFuncClear, static_cast<unsigned>(PropertyAttribute::DontEnum), 0);
     JSC_NATIVE_FUNCTION_WITHOUT_TRANSITION(vm.propertyNames->deleteKeyword, mapProtoFuncDelete, static_cast<unsigned>(PropertyAttribute::DontEnum), 1);
     JSC_NATIVE_INTRINSIC_FUNCTION_WITHOUT_TRANSITION(vm.propertyNames->get, mapProtoFuncGet, static_cast<unsigned>(PropertyAttribute::DontEnum), 1, JSMapGetIntrinsic);
     JSC_NATIVE_INTRINSIC_FUNCTION_WITHOUT_TRANSITION(vm.propertyNames->has, mapProtoFuncHas, static_cast<unsigned>(PropertyAttribute::DontEnum), 1, JSMapHasIntrinsic);
     JSC_NATIVE_INTRINSIC_FUNCTION_WITHOUT_TRANSITION(vm.propertyNames->set, mapProtoFuncSet, static_cast<unsigned>(PropertyAttribute::DontEnum), 2, JSMapSetIntrinsic);
+    JSC_NATIVE_INTRINSIC_FUNCTION_WITHOUT_TRANSITION(vm.propertyNames->builtinNames().keysPublicName(), mapProtoFuncKeys, static_cast<unsigned>(PropertyAttribute::DontEnum), 0, JSMapKeysIntrinsic);
+    JSC_NATIVE_INTRINSIC_FUNCTION_WITHOUT_TRANSITION(vm.propertyNames->builtinNames().valuesPublicName(), mapProtoFuncValues, static_cast<unsigned>(PropertyAttribute::DontEnum), 0, JSMapValuesIntrinsic);
 
     JSC_NATIVE_INTRINSIC_FUNCTION_WITHOUT_TRANSITION(vm.propertyNames->builtinNames().getPrivateName(), mapProtoFuncGet, static_cast<unsigned>(PropertyAttribute::DontEnum), 1, JSMapGetIntrinsic);
     JSC_NATIVE_INTRINSIC_FUNCTION_WITHOUT_TRANSITION(vm.propertyNames->builtinNames().setPrivateName(), mapProtoFuncSet, static_cast<unsigned>(PropertyAttribute::DontEnum), 2, JSMapSetIntrinsic);
 
-    JSFunction* entries = JSFunction::create(vm, mapPrototypeEntriesCodeGenerator(vm), globalObject);
+    JSFunction* entries = JSFunction::create(vm, globalObject, 0, vm.propertyNames->builtinNames().entriesPublicName().string(), mapProtoFuncEntries, JSMapEntriesIntrinsic);
     putDirectWithoutTransition(vm, vm.propertyNames->builtinNames().entriesPublicName(), entries, static_cast<unsigned>(PropertyAttribute::DontEnum));
     putDirectWithoutTransition(vm, vm.propertyNames->iteratorSymbol, entries, static_cast<unsigned>(PropertyAttribute::DontEnum));
-    putDirectWithoutTransition(vm, vm.propertyNames->toStringTagSymbol, jsString(&vm, "Map"), PropertyAttribute::DontEnum | PropertyAttribute::ReadOnly);
+    JSC_TO_STRING_TAG_WITHOUT_TRANSITION();
 
-    JSC_NATIVE_GETTER(vm.propertyNames->size, mapProtoFuncSize, PropertyAttribute::DontEnum | PropertyAttribute::Accessor);
+    JSC_NATIVE_GETTER_WITHOUT_TRANSITION(vm.propertyNames->size, mapProtoFuncSize, PropertyAttribute::DontEnum | PropertyAttribute::Accessor);
+
+    globalObject->installMapPrototypeWatchpoint(this);
 }
 
-ALWAYS_INLINE static JSMap* getMap(CallFrame* callFrame, JSValue thisValue)
+ALWAYS_INLINE static JSMap* getMap(JSGlobalObject* globalObject, JSValue thisValue)
 {
-    VM& vm = callFrame->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (UNLIKELY(!thisValue.isCell())) {
-        throwVMError(callFrame, scope, createNotAnObjectError(callFrame, thisValue));
+        throwVMError(globalObject, scope, createNotAnObjectError(globalObject, thisValue));
         return nullptr;
     }
 
     auto* map = jsDynamicCast<JSMap*>(vm, thisValue.asCell());
     if (LIKELY(map))
         return map;
-    throwTypeError(callFrame, scope, "Map operation called on non-Map object"_s);
+    throwTypeError(globalObject, scope, "Map operation called on non-Map object"_s);
     return nullptr;
 }
 
-EncodedJSValue JSC_HOST_CALL mapProtoFuncClear(CallFrame* callFrame)
+EncodedJSValue JSC_HOST_CALL mapProtoFuncClear(JSGlobalObject* globalObject, CallFrame* callFrame)
 {
-    JSMap* map = getMap(callFrame, callFrame->thisValue());
+    JSMap* map = getMap(globalObject, callFrame->thisValue());
     if (!map)
         return JSValue::encode(jsUndefined());
-    map->clear(callFrame);
+    map->clear(globalObject);
     return JSValue::encode(jsUndefined());
 }
 
-EncodedJSValue JSC_HOST_CALL mapProtoFuncDelete(CallFrame* callFrame)
+EncodedJSValue JSC_HOST_CALL mapProtoFuncDelete(JSGlobalObject* globalObject, CallFrame* callFrame)
 {
-    JSMap* map = getMap(callFrame, callFrame->thisValue());
+    JSMap* map = getMap(globalObject, callFrame->thisValue());
     if (!map)
         return JSValue::encode(jsUndefined());
-    return JSValue::encode(jsBoolean(map->remove(callFrame, callFrame->argument(0))));
+    return JSValue::encode(jsBoolean(map->remove(globalObject, callFrame->argument(0))));
 }
 
-EncodedJSValue JSC_HOST_CALL mapProtoFuncGet(CallFrame* callFrame)
+EncodedJSValue JSC_HOST_CALL mapProtoFuncGet(JSGlobalObject* globalObject, CallFrame* callFrame)
 {
-    JSMap* map = getMap(callFrame, callFrame->thisValue());
+    JSMap* map = getMap(globalObject, callFrame->thisValue());
     if (!map)
         return JSValue::encode(jsUndefined());
-    return JSValue::encode(map->get(callFrame, callFrame->argument(0)));
+    return JSValue::encode(map->get(globalObject, callFrame->argument(0)));
 }
 
-EncodedJSValue JSC_HOST_CALL mapProtoFuncHas(CallFrame* callFrame)
+EncodedJSValue JSC_HOST_CALL mapProtoFuncHas(JSGlobalObject* globalObject, CallFrame* callFrame)
 {
-    JSMap* map = getMap(callFrame, callFrame->thisValue());
+    JSMap* map = getMap(globalObject, callFrame->thisValue());
     if (!map)
         return JSValue::encode(jsUndefined());
-    return JSValue::encode(jsBoolean(map->has(callFrame, callFrame->argument(0))));
+    return JSValue::encode(jsBoolean(map->has(globalObject, callFrame->argument(0))));
 }
 
-EncodedJSValue JSC_HOST_CALL mapProtoFuncSet(CallFrame* callFrame)
+EncodedJSValue JSC_HOST_CALL mapProtoFuncSet(JSGlobalObject* globalObject, CallFrame* callFrame)
 {
     JSValue thisValue = callFrame->thisValue();
-    JSMap* map = getMap(callFrame, thisValue);
+    JSMap* map = getMap(globalObject, thisValue);
     if (!map)
         return JSValue::encode(jsUndefined());
-    map->set(callFrame, callFrame->argument(0), callFrame->argument(1));
+    map->set(globalObject, callFrame->argument(0), callFrame->argument(1));
     return JSValue::encode(thisValue);
 }
 
-EncodedJSValue JSC_HOST_CALL mapProtoFuncSize(CallFrame* callFrame)
+inline JSValue createMapIteratorObject(JSGlobalObject* globalObject, CallFrame* callFrame, IterationKind kind)
 {
-    JSMap* map = getMap(callFrame, callFrame->thisValue());
+    VM& vm = globalObject->vm();
+    JSValue thisValue = callFrame->thisValue();
+    JSMap* map = getMap(globalObject, thisValue);
+    if (!map)
+        return jsUndefined();
+    return JSMapIterator::create(vm, globalObject->mapIteratorStructure(), map, kind);
+}
+
+EncodedJSValue JSC_HOST_CALL mapProtoFuncValues(JSGlobalObject* globalObject, CallFrame* callFrame)
+{
+    return JSValue::encode(createMapIteratorObject(globalObject, callFrame, IterationKind::Values));
+}
+
+EncodedJSValue JSC_HOST_CALL mapProtoFuncKeys(JSGlobalObject* globalObject, CallFrame* callFrame)
+{
+    return JSValue::encode(createMapIteratorObject(globalObject, callFrame, IterationKind::Keys));
+}
+
+EncodedJSValue JSC_HOST_CALL mapProtoFuncEntries(JSGlobalObject* globalObject, CallFrame* callFrame)
+{
+    return JSValue::encode(createMapIteratorObject(globalObject, callFrame, IterationKind::Entries));
+}
+
+EncodedJSValue JSC_HOST_CALL mapProtoFuncSize(JSGlobalObject* globalObject, CallFrame* callFrame)
+{
+    JSMap* map = getMap(globalObject, callFrame->thisValue());
     if (!map)
         return JSValue::encode(jsUndefined());
     return JSValue::encode(jsNumber(map->size()));
