@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,7 +29,7 @@
 
 namespace JSC {
 
-static const unsigned minimumReservedZoneSize = 16 * KB;
+static constexpr unsigned minimumReservedZoneSize = 16 * KB;
 
 } // namespace JSC
 
