@@ -24,12 +24,23 @@
 
 namespace JSC {
 
-class NativeErrorPrototype final : public ErrorPrototype {
+class NativeErrorPrototype final : public ErrorPrototypeBase {
 private:
     NativeErrorPrototype(VM&, Structure*);
 
 public:
-    typedef ErrorPrototype Base;
+    using Base = ErrorPrototypeBase;
+    template<typename CellType, SubspaceAccess>
+    static IsoSubspace* subspaceFor(VM& vm)
+    {
+        STATIC_ASSERT_ISO_SUBSPACE_SHARABLE(NativeErrorPrototype, Base);
+        return &vm.plainObjectSpace;
+    }
+
+    static Structure* createStructure(VM& vm, JSGlobalObject* globalObject, JSValue prototype)
+    {
+        return Structure::create(vm, globalObject, prototype, TypeInfo(ObjectType, StructureFlags), info());
+    }
 
     static NativeErrorPrototype* create(VM& vm, Structure* structure, const String& name)
     {
