@@ -34,7 +34,7 @@ class NativeExecutable final : public ExecutableBase {
     friend class LLIntOffsetsExtractor;
 public:
     typedef ExecutableBase Base;
-    static const unsigned StructureFlags = Base::StructureFlags | StructureIsImmortal;
+    static constexpr unsigned StructureFlags = Base::StructureFlags | StructureIsImmortal;
 
     static NativeExecutable* create(VM&, Ref<JITCode>&& callThunk, TaggedNativeFunction, Ref<JITCode>&& constructThunk, TaggedNativeFunction constructor, const String& name);
 
@@ -48,8 +48,8 @@ public:
 
     CodeBlockHash hashFor(CodeSpecializationKind) const;
 
-    TaggedNativeFunction function() { return m_function; }
-    TaggedNativeFunction constructor() { return m_constructor; }
+    TaggedNativeFunction function() const { return m_function; }
+    TaggedNativeFunction constructor() const { return m_constructor; }
 
     TaggedNativeFunction nativeFunctionFor(CodeSpecializationKind kind)
     {
@@ -76,11 +76,9 @@ public:
     const DOMJIT::Signature* signatureFor(CodeSpecializationKind) const;
     Intrinsic intrinsic() const;
 
-protected:
-    void finishCreation(VM&, Ref<JITCode>&& callThunk, Ref<JITCode>&& constructThunk, const String& name);
-
 private:
     NativeExecutable(VM&, TaggedNativeFunction, TaggedNativeFunction constructor);
+    void finishCreation(VM&, Ref<JITCode>&& callThunk, Ref<JITCode>&& constructThunk, const String& name);
 
     TaggedNativeFunction m_function;
     TaggedNativeFunction m_constructor;
