@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2018 Apple Inc. All rights reserved.
+ *  Copyright (C) 2005-2020 Apple Inc. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -21,6 +21,8 @@
 #pragma once
 
 #include "DOMAnnotation.h"
+#include "DisallowVMEntry.h"
+#include "GetVM.h"
 #include "JSCJSValue.h"
 #include "PropertyName.h"
 #include "PropertyOffset.h"
@@ -29,7 +31,6 @@
 #include <wtf/ForbidHeapAllocation.h>
 
 namespace JSC {
-class ExecState;
 class GetterSetter;
 class JSObject;
 class JSModuleEnvironment;
@@ -112,28 +113,23 @@ public:
         ModuleNamespace, // ModuleNamespaceObject's environment access.
     };
 
-    explicit PropertySlot(const JSValue thisValue, InternalMethodType internalMethodType)
-        : m_offset(invalidOffset)
-        , m_thisValue(thisValue)
-        , m_slotBase(nullptr)
-        , m_watchpointSet(nullptr)
-        , m_cacheability(CachingAllowed)
-        , m_propertyType(TypeUnset)
+    explicit PropertySlot(const JSValue thisValue, InternalMethodType internalMethodType, VM* vmForInquiry = nullptr)
+        : m_thisValue(thisValue)
         , m_internalMethodType(internalMethodType)
-        , m_additionalDataType(AdditionalDataType::None)
-        , m_isTaintedByOpaqueObject(false)
     {
+        if (isVMInquiry())
+            disallowVMEntry.emplace(*vmForInquiry);
     }
 
     // FIXME: Remove this slotBase / receiver behavior difference in custom values and custom accessors.
     // https://bugs.webkit.org/show_bug.cgi?id=158014
-    typedef EncodedJSValue (*GetValueFunc)(ExecState*, EncodedJSValue thisValue, PropertyName);
+    typedef EncodedJSValue (*GetValueFunc)(JSGlobalObject*, EncodedJSValue thisValue, PropertyName);
 
-    JSValue getValue(ExecState*, PropertyName) const;
-    JSValue getValue(ExecState*, unsigned propertyName) const;
+    JSValue getValue(JSGlobalObject*, PropertyName) const;
+    JSValue getValue(JSGlobalObject*, uint64_t propertyName) const;
     JSValue getPureResult() const;
 
-    bool isCacheable() const { return m_cacheability == CachingAllowed && m_offset != invalidOffset; }
+    bool isCacheable() const { return isUnset() || m_cacheability == CachingAllowed; }
     bool isUnset() const { return m_propertyType == TypeUnset; }
     bool isValue() const { return m_propertyType == TypeValue; }
     bool isAccessor() const { return m_propertyType == TypeGetter; }
@@ -146,6 +142,7 @@ public:
     bool isTaintedByOpaqueObject() const { return m_isTaintedByOpaqueObject; }
 
     InternalMethodType internalMethodType() const { return m_internalMethodType; }
+    bool isVMInquiry() const { return m_internalMethodType == InternalMethodType::VMInquiry; }
 
     void disableCaching()
     {
@@ -217,7 +214,8 @@ public:
         ASSERT(slotBase);
         m_slotBase = slotBase;
         m_propertyType = TypeValue;
-        m_offset = invalidOffset;
+
+        ASSERT(m_cacheability == CachingDisallowed);
     }
 
     void setValue(JSObject* slotBase, unsigned attributes, JSValue value, PropertyOffset offset)
@@ -232,6 +230,8 @@ public:
         m_slotBase = slotBase;
         m_propertyType = TypeValue;
         m_offset = offset;
+
+        m_cacheability = CachingAllowed;
     }
 
     void setValue(JSString*, unsigned attributes, JSValue value)
@@ -242,9 +242,10 @@ public:
         m_data.value = JSValue::encode(value);
         m_attributes = attributes;
 
-        m_slotBase = 0;
+        m_slotBase = nullptr;
         m_propertyType = TypeValue;
-        m_offset = invalidOffset;
+
+        ASSERT(m_cacheability == CachingDisallowed);
     }
 
     void setValueModuleNamespace(JSObject* slotBase, unsigned attributes, JSValue value, JSModuleEnvironment* environment, ScopeOffset scopeOffset)
@@ -267,7 +268,7 @@ public:
         ASSERT(slotBase);
         m_slotBase = slotBase;
         m_propertyType = TypeCustom;
-        m_offset = invalidOffset;
+        ASSERT(m_cacheability == CachingDisallowed);
     }
 
     void setCustom(JSObject* slotBase, unsigned attributes, GetValueFunc getValue, DOMAttributeAnnotation domAttribute)
@@ -289,7 +290,8 @@ public:
         ASSERT(slotBase);
         m_slotBase = slotBase;
         m_propertyType = TypeCustom;
-        m_offset = !invalidOffset;
+
+        m_cacheability = CachingAllowed;
     }
 
     void setCacheableCustom(JSObject* slotBase, unsigned attributes, GetValueFunc getValue, DOMAttributeAnnotation domAttribute)
@@ -313,7 +315,8 @@ public:
         ASSERT(slotBase);
         m_slotBase = slotBase;
         m_propertyType = TypeCustomAccessor;
-        m_offset = invalidOffset;
+
+        ASSERT(m_cacheability == CachingDisallowed);
     }
 
     void setGetterSlot(JSObject* slotBase, unsigned attributes, GetterSetter* getterSetter)
@@ -327,7 +330,8 @@ public:
         ASSERT(slotBase);
         m_slotBase = slotBase;
         m_propertyType = TypeGetter;
-        m_offset = invalidOffset;
+
+        ASSERT(m_cacheability == CachingDisallowed);
     }
 
     void setCacheableGetterSlot(JSObject* slotBase, unsigned attributes, GetterSetter* getterSetter, PropertyOffset offset)
@@ -342,6 +346,8 @@ public:
         m_slotBase = slotBase;
         m_propertyType = TypeGetter;
         m_offset = offset;
+
+        m_cacheability = CachingAllowed;
     }
 
     JSValue thisValue() const
@@ -359,9 +365,8 @@ public:
         m_data.value = JSValue::encode(jsUndefined());
         m_attributes = PropertyAttribute::ReadOnly | PropertyAttribute::DontDelete | PropertyAttribute::DontEnum;
 
-        m_slotBase = 0;
+        m_slotBase = nullptr;
         m_propertyType = TypeValue;
-        m_offset = invalidOffset;
     }
 
     void setWatchpointSet(WatchpointSet& set)
@@ -370,9 +375,9 @@ public:
     }
 
 private:
-    JS_EXPORT_PRIVATE JSValue functionGetter(ExecState*) const;
-    JS_EXPORT_PRIVATE JSValue customGetter(ExecState*, PropertyName) const;
-    JS_EXPORT_PRIVATE JSValue customAccessorGetter(ExecState*, PropertyName) const;
+    JS_EXPORT_PRIVATE JSValue functionGetter(JSGlobalObject*) const;
+    JS_EXPORT_PRIVATE JSValue customGetter(JSGlobalObject*, PropertyName) const;
+    JS_EXPORT_PRIVATE JSValue customAccessorGetter(JSGlobalObject*, PropertyName) const;
 
     union {
         EncodedJSValue value;
@@ -387,42 +392,46 @@ private:
         } customAccessor;
     } m_data;
 
-    unsigned m_attributes;
-    PropertyOffset m_offset;
+    unsigned m_attributes { 0 };
+    PropertyOffset m_offset { invalidOffset };
     JSValue m_thisValue;
-    JSObject* m_slotBase;
-    WatchpointSet* m_watchpointSet;
-    CacheabilityType m_cacheability;
-    PropertyType m_propertyType;
+    JSObject* m_slotBase { nullptr };
+    WatchpointSet* m_watchpointSet { nullptr };
+    CacheabilityType m_cacheability { CachingDisallowed };
+    PropertyType m_propertyType { TypeUnset };
     InternalMethodType m_internalMethodType;
-    AdditionalDataType m_additionalDataType;
-    bool m_isTaintedByOpaqueObject;
+    AdditionalDataType m_additionalDataType { AdditionalDataType::None };
+    bool m_isTaintedByOpaqueObject { false };
+public:
+    Optional<DisallowVMEntry> disallowVMEntry;
+private:
     union {
         DOMAttributeAnnotation domAttribute;
         ModuleNamespaceSlot moduleNamespaceSlot;
-    } m_additionalData;
+    } m_additionalData { { nullptr, nullptr } };
 };
 
-ALWAYS_INLINE JSValue PropertySlot::getValue(ExecState* exec, PropertyName propertyName) const
+ALWAYS_INLINE JSValue PropertySlot::getValue(JSGlobalObject* globalObject, PropertyName propertyName) const
 {
     if (m_propertyType == TypeValue)
         return JSValue::decode(m_data.value);
     if (m_propertyType == TypeGetter)
-        return functionGetter(exec);
+        return functionGetter(globalObject);
     if (m_propertyType == TypeCustomAccessor)
-        return customAccessorGetter(exec, propertyName);
-    return customGetter(exec, propertyName);
+        return customAccessorGetter(globalObject, propertyName);
+    return customGetter(globalObject, propertyName);
 }
 
-ALWAYS_INLINE JSValue PropertySlot::getValue(ExecState* exec, unsigned propertyName) const
+ALWAYS_INLINE JSValue PropertySlot::getValue(JSGlobalObject* globalObject, uint64_t propertyName) const
 {
+    VM& vm = getVM(globalObject);
     if (m_propertyType == TypeValue)
         return JSValue::decode(m_data.value);
     if (m_propertyType == TypeGetter)
-        return functionGetter(exec);
+        return functionGetter(globalObject);
     if (m_propertyType == TypeCustomAccessor)
-        return customAccessorGetter(exec, Identifier::from(exec, propertyName));
-    return customGetter(exec, Identifier::from(exec, propertyName));
+        return customAccessorGetter(globalObject, Identifier::from(vm, propertyName));
+    return customGetter(globalObject, Identifier::from(vm, propertyName));
 }
 
 } // namespace JSC
