@@ -2,6 +2,7 @@
  * Copyright (C) 2010 University of Szeged
  * Copyright (C) 2010 Renata Hodovan (hodovan@inf.u-szeged.hu)
  * All rights reserved.
+ * Copyright (C) 2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,7 +37,11 @@
 
 namespace JSC {
 
-class RegExpCache : private WeakHandleOwner {
+namespace Yarr {
+enum class Flags : uint8_t;
+}
+
+class RegExpCache final : private WeakHandleOwner {
     WTF_MAKE_FAST_ALLOCATED;
 
     friend class RegExp;
@@ -54,16 +59,15 @@ public:
     }
 
 private:
+    static constexpr unsigned maxStrongCacheablePatternLength = 256;
 
-    static const unsigned maxStrongCacheablePatternLength = 256;
+    static constexpr int maxStrongCacheableEntries = 32;
 
-    static const int maxStrongCacheableEntries = 32;
-
-    void finalize(Handle<Unknown>, void* context) override;
+    void finalize(Handle<Unknown>, void* context) final;
 
     RegExp* ensureEmptyRegExpSlow(VM&);
 
-    RegExp* lookupOrCreate(const WTF::String& patternString, RegExpFlags);
+    RegExp* lookupOrCreate(const WTF::String& patternString, OptionSet<Yarr::Flags>);
     void addToStrongCache(RegExp*);
     RegExpCacheMap m_weakCache; // Holds all regular expressions currently live.
     int m_nextEntryInStrongCache;
