@@ -25,6 +25,8 @@
 
 #pragma once
 
+#include "Heap.h"
+#include "JSObject.h"
 #include "RegExp.h"
 
 namespace JSC {
@@ -45,18 +47,18 @@ class RegExpCachedResult {
 public:
     ALWAYS_INLINE void record(VM& vm, JSObject* owner, RegExp* regExp, JSString* input, MatchResult result)
     {
-        vm.heap.writeBarrier(owner);
         m_lastRegExp.setWithoutWriteBarrier(regExp);
         m_lastInput.setWithoutWriteBarrier(input);
         m_result = result;
         m_reified = false;
+        vm.heap.writeBarrier(owner);
     }
 
-    JSArray* lastResult(ExecState*, JSObject* owner);
-    void setInput(ExecState*, JSObject* owner, JSString*);
+    JSArray* lastResult(JSGlobalObject*, JSObject* owner);
+    void setInput(JSGlobalObject*, JSObject* owner, JSString*);
 
-    JSString* leftContext(ExecState*, JSObject* owner);
-    JSString* rightContext(ExecState*, JSObject* owner);
+    JSString* leftContext(JSGlobalObject*, JSObject* owner);
+    JSString* rightContext(JSGlobalObject*, JSObject* owner);
 
     JSString* input()
     {
