@@ -34,50 +34,45 @@
 namespace JSC {
 
 ALWAYS_INLINE unsigned getRegExpObjectLastIndexAsUnsigned(
-    ExecState* exec, RegExpObject* regExpObject, const String& input)
+    JSGlobalObject* globalObject, RegExpObject* regExpObject, const String& input)
 {
-    VM& vm = exec->vm();
+    VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
     JSValue jsLastIndex = regExpObject->getLastIndex();
     unsigned lastIndex;
     if (LIKELY(jsLastIndex.isUInt32())) {
         lastIndex = jsLastIndex.asUInt32();
-        if (lastIndex > input.length()) {
-            scope.release();
-            regExpObject->setLastIndex(exec, 0);
+        if (lastIndex > input.length())
             return UINT_MAX;
-        }
     } else {
-        double doubleLastIndex = jsLastIndex.toInteger(exec);
+        double doubleLastIndex = jsLastIndex.toInteger(globalObject);
         RETURN_IF_EXCEPTION(scope, UINT_MAX);
-        if (doubleLastIndex < 0 || doubleLastIndex > input.length()) {
-            scope.release();
-            regExpObject->setLastIndex(exec, 0);
+        if (doubleLastIndex > input.length())
             return UINT_MAX;
-        }
-        lastIndex = static_cast<unsigned>(doubleLastIndex);
+        lastIndex = (doubleLastIndex < 0) ? 0 : static_cast<unsigned>(doubleLastIndex);
     }
     return lastIndex;
 }
 
-inline JSValue RegExpObject::execInline(ExecState* exec, JSGlobalObject* globalObject, JSString* string)
+inline JSValue RegExpObject::execInline(JSGlobalObject* globalObject, JSString* string)
 {
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     RegExp* regExp = this->regExp();
-    String input = string->value(exec);
+    String input = string->value(globalObject);
     RETURN_IF_EXCEPTION(scope, { });
 
     bool globalOrSticky = regExp->globalOrSticky();
+    unsigned lastIndex = getRegExpObjectLastIndexAsUnsigned(globalObject, this, input);
+    RETURN_IF_EXCEPTION(scope, { });
+    if (lastIndex == UINT_MAX && globalOrSticky) {
+        scope.release();
+        setLastIndex(globalObject, 0);
+        return jsNull();
+    }
 
-    unsigned lastIndex;
-    if (globalOrSticky) {
-        lastIndex = getRegExpObjectLastIndexAsUnsigned(exec, this, input);
-        EXCEPTION_ASSERT(!scope.exception() || lastIndex == UINT_MAX);
-        if (lastIndex == UINT_MAX)
-            return jsNull();
-    } else
+    if (!globalOrSticky)
         lastIndex = 0;
 
     MatchResult result;
@@ -87,12 +82,12 @@ inline JSValue RegExpObject::execInline(ExecState* exec, JSGlobalObject* globalO
         RETURN_IF_EXCEPTION(scope, { });
         scope.release();
         if (globalOrSticky)
-            setLastIndex(exec, 0);
+            setLastIndex(globalObject, 0);
         return jsNull();
     }
 
     if (globalOrSticky)
-        setLastIndex(exec, result.end);
+        setLastIndex(globalObject, result.end);
     RETURN_IF_EXCEPTION(scope, { });
     globalObject->regExpGlobalData().recordMatch(vm, globalObject, regExp, string, result);
     return array;
@@ -100,29 +95,32 @@ inline JSValue RegExpObject::execInline(ExecState* exec, JSGlobalObject* globalO
 
 // Shared implementation used by test and exec.
 inline MatchResult RegExpObject::matchInline(
-    ExecState* exec, JSGlobalObject* globalObject, JSString* string)
+    JSGlobalObject* globalObject, JSString* string)
 {
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     RegExp* regExp = this->regExp();
-    String input = string->value(exec);
+    String input = string->value(globalObject);
     RETURN_IF_EXCEPTION(scope, { });
 
     if (!regExp->global() && !regExp->sticky()) {
         scope.release();
-        return globalObject->regExpGlobalData().performMatch(vm, globalObject, regExp, string, input, 0);
+        return globalObject->regExpGlobalData().performMatch(globalObject, regExp, string, input, 0);
     }
 
-    unsigned lastIndex = getRegExpObjectLastIndexAsUnsigned(exec, this, input);
-    EXCEPTION_ASSERT(!scope.exception() || (lastIndex == UINT_MAX));
-    if (lastIndex == UINT_MAX)
+    unsigned lastIndex = getRegExpObjectLastIndexAsUnsigned(globalObject, this, input);
+    RETURN_IF_EXCEPTION(scope, { });
+    if (lastIndex == UINT_MAX) {
+        scope.release();
+        setLastIndex(globalObject, 0);
         return MatchResult::failed();
+    }
 
-    MatchResult result = globalObject->regExpGlobalData().performMatch(vm, globalObject, regExp, string, input, lastIndex);
+    MatchResult result = globalObject->regExpGlobalData().performMatch(globalObject, regExp, string, input, lastIndex);
     RETURN_IF_EXCEPTION(scope, { });
     scope.release();
-    setLastIndex(exec, result.end);
+    setLastIndex(globalObject, result.end);
     return result;
 }
 
@@ -143,18 +141,18 @@ inline unsigned advanceStringUnicode(String s, unsigned length, unsigned current
 }
 
 template<typename FixEndFunc>
-JSValue collectMatches(VM& vm, ExecState* exec, JSString* string, const String& s, JSGlobalObject* globalObject, RegExp* regExp, const FixEndFunc& fixEnd)
+JSValue collectMatches(VM& vm, JSGlobalObject* globalObject, JSString* string, const String& s, RegExp* regExp, const FixEndFunc& fixEnd)
 {
     auto scope = DECLARE_THROW_SCOPE(vm);
 
-    MatchResult result = globalObject->regExpGlobalData().performMatch(vm, globalObject, regExp, string, s, 0);
+    MatchResult result = globalObject->regExpGlobalData().performMatch(globalObject, regExp, string, s, 0);
     RETURN_IF_EXCEPTION(scope, { });
     if (!result)
         return jsNull();
 
     static unsigned maxSizeForDirectPath = 100000;
 
-    JSArray* array = constructEmptyArray(exec, nullptr);
+    JSArray* array = constructEmptyArray(globalObject, nullptr);
     RETURN_IF_EXCEPTION(scope, { });
 
     bool hasException = false;
@@ -162,14 +160,14 @@ JSValue collectMatches(VM& vm, ExecState* exec, JSString* string, const String&
     auto iterate = [&] () {
         size_t end = result.end;
         size_t length = end - result.start;
-        array->putDirectIndex(exec, arrayIndex++, jsSubstringOfResolved(vm, string, result.start, length));
+        array->putDirectIndex(globalObject, arrayIndex++, jsSubstringOfResolved(vm, string, result.start, length));
         if (UNLIKELY(scope.exception())) {
             hasException = true;
             return;
         }
         if (!length)
             end = fixEnd(end);
-        result = globalObject->regExpGlobalData().performMatch(vm, globalObject, regExp, string, s, end);
+        result = globalObject->regExpGlobalData().performMatch(globalObject, regExp, string, s, end);
         if (UNLIKELY(scope.exception())) {
             hasException = true;
             return;
@@ -183,7 +181,7 @@ JSValue collectMatches(VM& vm, ExecState* exec, JSString* string, const String&
             MatchResult savedResult = result;
             do {
                 if (array->length() + matchCount > MAX_STORAGE_VECTOR_LENGTH) {
-                    throwOutOfMemoryError(exec, scope);
+                    throwOutOfMemoryError(globalObject, scope);
                     return jsUndefined();
                 }
 
@@ -197,7 +195,7 @@ JSValue collectMatches(VM& vm, ExecState* exec, JSString* string, const String&
                 // will leave the cached result in the state it ought to have had just before the
                 // OOM! On the other hand, if this loop concludes that the result is small enough,
                 // then the iterate() loop below will overwrite the cached result anyway.
-                result = globalObject->regExpGlobalData().performMatch(vm, globalObject, regExp, string, s, end);
+                result = globalObject->regExpGlobalData().performMatch(globalObject, regExp, string, s, end);
                 RETURN_IF_EXCEPTION(scope, { });
             } while (result);
 
