@@ -28,7 +28,7 @@
 #include "config.h"
 #include "RuntimeType.h"
 
-#include "JSCInlines.h"
+#include "JSCJSValueInlines.h"
 
 namespace JSC {
 
@@ -51,10 +51,12 @@ RuntimeType runtimeTypeForValue(VM& vm, JSValue value)
         return TypeBoolean;
     if (value.isObject())
         return TypeObject;
-    if (value.isFunction(vm))
+    if (value.isCallable(vm))
         return TypeFunction;
     if (value.isSymbol())
         return TypeSymbol;
+    if (value.isBigInt())
+        return TypeBigInt;
 
     return TypeNothing;
 }
@@ -77,6 +79,10 @@ String runtimeTypeAsString(RuntimeType type)
         return "Boolean"_s;
     if (type == TypeFunction)
         return "Function"_s;
+    if (type == TypeSymbol)
+        return "Symbol"_s;
+    if (type == TypeBigInt)
+        return "BigInt"_s;
     if (type == TypeNothing)
         return "(Nothing)"_s;
 
