@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2015-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,6 +27,7 @@
 
 #include "JSCast.h"
 #include "ScopeOffset.h"
+#include "VM.h"
 #include <wtf/Assertions.h>
 #include <wtf/CagedUniquePtr.h>
 
@@ -42,8 +43,15 @@ class ScopedArgumentsTable final : public JSCell {
     friend class CachedScopedArgumentsTable;
 
 public:
-    typedef JSCell Base;
-    static const unsigned StructureFlags = Base::StructureFlags | StructureIsImmortal;
+    using Base = JSCell;
+    static constexpr unsigned StructureFlags = Base::StructureFlags | StructureIsImmortal;
+    static constexpr bool needsDestruction = true;
+
+    template<typename CellType, SubspaceAccess mode>
+    static IsoSubspace* subspaceFor(VM& vm)
+    {
+        return vm.scopedArgumentsTableSpace<mode>();
+    }
 
 private:
     ScopedArgumentsTable(VM&);
@@ -51,27 +59,21 @@ private:
 
 public:
     static ScopedArgumentsTable* create(VM&);
-    static ScopedArgumentsTable* create(VM&, uint32_t length);
+    static ScopedArgumentsTable* tryCreate(VM&, uint32_t length);
 
-    static const bool needsDestruction = true;
     static void destroy(JSCell*);
 
-    ScopedArgumentsTable* clone(VM&);
-
     uint32_t length() const { return m_length; }
-    ScopedArgumentsTable* setLength(VM&, uint32_t newLength);
+    ScopedArgumentsTable* trySetLength(VM&, uint32_t newLength);
 
-    ScopeOffset get(uint32_t i) const
-    {
-        return const_cast<ScopedArgumentsTable*>(this)->at(i);
-    }
+    ScopeOffset get(uint32_t i) const { return at(i); }
 
     void lock()
     {
         m_locked = true;
     }
 
-    ScopedArgumentsTable* set(VM&, uint32_t index, ScopeOffset);
+    ScopedArgumentsTable* trySet(VM&, uint32_t index, ScopeOffset);
 
     DECLARE_INFO;
 
@@ -80,13 +82,15 @@ public:
     static ptrdiff_t offsetOfLength() { return OBJECT_OFFSETOF(ScopedArgumentsTable, m_length); }
     static ptrdiff_t offsetOfArguments() { return OBJECT_OFFSETOF(ScopedArgumentsTable, m_arguments); }
 
-    typedef CagedUniquePtr<Gigacage::Primitive, ScopeOffset[]> ArgumentsPtr;
+    typedef CagedUniquePtr<Gigacage::Primitive, ScopeOffset> ArgumentsPtr;
 
 private:
-    ScopeOffset& at(uint32_t i)
+    ScopedArgumentsTable* tryClone(VM&);
+
+    ScopeOffset& at(uint32_t i) const
     {
         ASSERT_WITH_SECURITY_IMPLICATION(i < m_length);
-        return m_arguments[i];
+        return m_arguments.get(length())[i];
     }
 
     uint32_t m_length;
