@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -70,28 +70,26 @@ ThrowScope::~ThrowScope()
         simulateThrow();
 }
 
-void ThrowScope::throwException(ExecState* exec, Exception* exception)
+Exception* ThrowScope::throwException(JSGlobalObject* globalObject, Exception* exception)
 {
     if (m_vm.exception() && m_vm.exception() != exception)
         m_vm.verifyExceptionCheckNeedIsSatisfied(m_recursionDepth, m_location);
 
-    m_vm.throwException(exec, exception);
+    return m_vm.throwException(globalObject, exception);
 }
 
-JSValue ThrowScope::throwException(ExecState* exec, JSValue error)
+Exception* ThrowScope::throwException(JSGlobalObject* globalObject, JSValue error)
 {
     if (!error.isCell() || !jsDynamicCast<Exception*>(m_vm, error.asCell()))
         m_vm.verifyExceptionCheckNeedIsSatisfied(m_recursionDepth, m_location);
 
-    return m_vm.throwException(exec, error);
+    return m_vm.throwException(globalObject, error);
 }
 
-JSObject* ThrowScope::throwException(ExecState* exec, JSObject* obj)
+Exception* ThrowScope::throwException(JSGlobalObject* globalObject, JSObject* obj)
 {
-    if (!jsDynamicCast<Exception*>(m_vm, obj))
-        m_vm.verifyExceptionCheckNeedIsSatisfied(m_recursionDepth, m_location);
-
-    return m_vm.throwException(exec, obj);
+    m_vm.verifyExceptionCheckNeedIsSatisfied(m_recursionDepth, m_location);
+    return m_vm.throwException(globalObject, obj);
 }
 
 void ThrowScope::simulateThrow()
