@@ -38,13 +38,13 @@ typename Adaptor::Type toNativeFromValue(JSValue value)
 }
 
 template<typename Adaptor>
-typename Adaptor::Type toNativeFromValue(ExecState* exec, JSValue value)
+typename Adaptor::Type toNativeFromValue(JSGlobalObject* globalObject, JSValue value)
 {
     if (value.isInt32())
         return Adaptor::toNativeFromInt32(value.asInt32());
     if (value.isNumber())
         return Adaptor::toNativeFromDouble(value.asDouble());
-    return Adaptor::toNativeFromDouble(value.toNumber(exec));
+    return Adaptor::toNativeFromDouble(value.toNumber(globalObject));
 }
 
 template<typename Adaptor>
