@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,15 +29,13 @@
 #include "config.h"
 #include "TypeProfilerLog.h"
 
-#include "JSCInlines.h"
-#include "SlotVisitor.h"
+#include "JSCJSValueInlines.h"
 #include "TypeLocation.h"
 
-
 namespace JSC {
 
 namespace TypeProfilerLogInternal {
-static const bool verbose = false;
+static constexpr bool verbose = false;
 }
 
 TypeProfilerLog::TypeProfilerLog(VM& vm)
