@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014, 2015 Apple Inc. All Rights Reserved.
+ * Copyright (C) 2014-2019 Apple Inc. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -79,17 +79,19 @@ void TypeSet::addTypeInformation(RuntimeType type, RefPtr<StructureShape>&& pass
     }
 }
 
-void TypeSet::invalidateCache()
+void TypeSet::invalidateCache(VM& vm)
 {
     ConcurrentJSLocker locker(m_lock);
-    auto keepMarkedStructuresFilter = [] (Structure* structure) -> bool { return Heap::isMarked(structure); };
+    auto keepMarkedStructuresFilter = [&] (Structure* structure) -> bool {
+        return vm.heap.isMarked(structure);
+    };
     m_structureSet.genericFilter(keepMarkedStructuresFilter);
 }
 
 String TypeSet::dumpTypes() const
 {
     if (m_seenTypes == TypeNothing)
-        return ASCIILiteral("(Unreached Statement)");
+        return "(Unreached Statement)"_s;
 
     StringBuilder seen;
 
@@ -172,44 +174,48 @@ String TypeSet::displayName() const
     // Therefore, more specific types must be checked first.
 
     if (doesTypeConformTo(TypeFunction))
-        return ASCIILiteral("Function");
+        return "Function"_s;
     if (doesTypeConformTo(TypeUndefined))
-        return ASCIILiteral("Undefined");
+        return "Undefined"_s;
     if (doesTypeConformTo(TypeNull))
-        return ASCIILiteral("Null");
+        return "Null"_s;
     if (doesTypeConformTo(TypeBoolean))
-        return ASCIILiteral("Boolean");
+        return "Boolean"_s;
     if (doesTypeConformTo(TypeAnyInt))
-        return ASCIILiteral("Integer");
+        return "Integer"_s;
     if (doesTypeConformTo(TypeNumber | TypeAnyInt))
-        return ASCIILiteral("Number");
+        return "Number"_s;
     if (doesTypeConformTo(TypeString))
-        return ASCIILiteral("String");
+        return "String"_s;
     if (doesTypeConformTo(TypeSymbol))
-        return ASCIILiteral("Symbol");
+        return "Symbol"_s;
+    if (doesTypeConformTo(TypeBigInt))
+        return "BigInt"_s;
 
     if (doesTypeConformTo(TypeNull | TypeUndefined))
-        return ASCIILiteral("(?)");
+        return "(?)"_s;
 
     if (doesTypeConformTo(TypeFunction | TypeNull | TypeUndefined))
-        return ASCIILiteral("Function?");
+        return "Function?"_s;
     if (doesTypeConformTo(TypeBoolean | TypeNull | TypeUndefined))
-        return ASCIILiteral("Boolean?");
+        return "Boolean?"_s;
     if (doesTypeConformTo(TypeAnyInt | TypeNull | TypeUndefined))
-        return ASCIILiteral("Integer?");
+        return "Integer?"_s;
     if (doesTypeConformTo(TypeNumber | TypeAnyInt | TypeNull | TypeUndefined))
-        return ASCIILiteral("Number?");
+        return "Number?"_s;
     if (doesTypeConformTo(TypeString | TypeNull | TypeUndefined))
-        return ASCIILiteral("String?");
+        return "String?"_s;
     if (doesTypeConformTo(TypeSymbol | TypeNull | TypeUndefined))
-        return ASCIILiteral("Symbol?");
+        return "Symbol?"_s;
+    if (doesTypeConformTo(TypeBigInt | TypeNull | TypeUndefined))
+        return "BigInt?"_s;
 
     if (doesTypeConformTo(TypeObject | TypeFunction | TypeString))
-        return ASCIILiteral("Object");
+        return "Object"_s;
     if (doesTypeConformTo(TypeObject | TypeFunction | TypeString | TypeNull | TypeUndefined))
-        return ASCIILiteral("Object?");
+        return "Object?"_s;
 
-    return ASCIILiteral("(many)");
+    return "(many)"_s;
 }
 
 String TypeSet::leastCommonAncestor() const
@@ -239,6 +245,7 @@ Ref<Inspector::Protocol::Runtime::TypeSet> TypeSet::inspectorTypeSet() const
         .setIsString((m_seenTypes & TypeString) != TypeNothing)
         .setIsObject((m_seenTypes & TypeObject) != TypeNothing)
         .setIsSymbol((m_seenTypes & TypeSymbol) != TypeNothing)
+        .setIsBigInt((m_seenTypes & TypeBigInt) != TypeNothing)
         .release();
 }
 
@@ -253,9 +260,7 @@ String TypeSet::toJSONString() const
     json.append('{');
 
     json.appendLiteral("\"displayTypeName\":");
-    json.append('"');
-    json.append(displayName());
-    json.append('"');
+    json.appendQuotedJSONString(displayName());
     json.append(',');
 
     json.appendLiteral("\"primitiveTypeNames\":");
@@ -321,10 +326,10 @@ String TypeSet::toJSONString() const
 }
 
 StructureShape::StructureShape()
-    : m_proto(nullptr)
-    , m_propertyHash(nullptr)
-    , m_final(false)
+    : m_final(false)
     , m_isInDictionaryMode(false)
+    , m_proto(nullptr)
+    , m_propertyHash(nullptr)
 {
 }
 
@@ -362,7 +367,7 @@ String StructureShape::propertyHash()
         builder.append(m_proto->propertyHash());
     }
 
-    m_propertyHash = std::make_unique<String>(builder.toString());
+    m_propertyHash = makeUnique<String>(builder.toString());
     return *m_propertyHash;
 }
 
@@ -388,7 +393,7 @@ String StructureShape::leastCommonAncestor(const Vector<Ref<StructureShape>>& sh
                 // This is unlikely to happen, because we usually bottom out at "Object", but there are some sets of Objects
                 // that may cause this behavior. We fall back to "Object" because it's our version of Top.
                 if (!origin->m_proto)
-                    return ASCIILiteral("Object");
+                    return "Object"_s;
                 origin = origin->m_proto.get();
             }
         }
@@ -407,18 +412,10 @@ String StructureShape::stringRepresentation()
 
     representation.append('{');
     while (curShape) {
-        for (auto it = curShape->m_fields.begin(), end = curShape->m_fields.end(); it != end; ++it) {
-            String prop((*it).get());
-            representation.append(prop);
-            representation.appendLiteral(", ");
-        }
-
-        if (curShape->m_proto) {
-            representation.appendLiteral("__proto__ [");
-            representation.append(curShape->m_proto->m_constructorName);
-            representation.appendLiteral("], ");
-        }
-
+        for (auto& field : curShape->m_fields)
+            representation.append(StringView { field.get() }, ", ");
+        if (curShape->m_proto)
+            representation.append("__proto__ [", curShape->m_proto->m_constructorName, "], ");
         curShape = curShape->m_proto;
     }
 
@@ -442,9 +439,7 @@ String StructureShape::toJSONString() const
     json.append('{');
 
     json.appendLiteral("\"constructorName\":");
-    json.append('"');
-    json.append(m_constructorName);
-    json.append('"');
+    json.appendQuotedJSONString(m_constructorName);
     json.append(',');
 
     json.appendLiteral("\"isInDictionaryMode\":");
@@ -463,9 +458,7 @@ String StructureShape::toJSONString() const
         hasAnItem = true;
 
         String fieldName((*it).get());
-        json.append('"');
-        json.append(fieldName);
-        json.append('"');
+        json.appendQuotedJSONString(fieldName);
     }
     json.append(']');
     json.append(',');
@@ -479,9 +472,7 @@ String StructureShape::toJSONString() const
         hasAnItem = true;
 
         String fieldName((*it).get());
-        json.append('"');
-        json.append(fieldName);
-        json.append('"');
+        json.appendQuotedJSONString(fieldName);
     }
     json.append(']');
     json.append(',');
@@ -506,9 +497,9 @@ Ref<Inspector::Protocol::Runtime::StructureDescription> StructureShape::inspecto
     while (currentShape) {
         auto fields = JSON::ArrayOf<String>::create();
         auto optionalFields = JSON::ArrayOf<String>::create();
-        for (auto field : currentShape->m_fields)
+        for (const auto& field : currentShape->m_fields)
             fields->addItem(field.get());
-        for (auto field : currentShape->m_optionalFields)
+        for (const auto& field : currentShape->m_optionalFields)
             optionalFields->addItem(field.get());
 
         currentObject->setFields(&fields.get());
@@ -547,23 +538,23 @@ Ref<StructureShape> StructureShape::merge(Ref<StructureShape>&& a, Ref<Structure
     ASSERT(a->hasSamePrototypeChain(b.get()));
 
     auto merged = StructureShape::create();
-    for (auto field : a->m_fields) {
+    for (const auto& field : a->m_fields) {
         if (b->m_fields.contains(field))
             merged->m_fields.add(field);
         else
             merged->m_optionalFields.add(field);
     }
 
-    for (auto field : b->m_fields) {
+    for (const auto& field : b->m_fields) {
         if (!merged->m_fields.contains(field)) {
             auto addResult = merged->m_optionalFields.add(field);
             ASSERT_UNUSED(addResult, addResult.isNewEntry);
         }
     }
 
-    for (auto field : a->m_optionalFields)
+    for (const auto& field : a->m_optionalFields)
         merged->m_optionalFields.add(field);
-    for (auto field : b->m_optionalFields)
+    for (const auto& field : b->m_optionalFields)
         merged->m_optionalFields.add(field);
 
     ASSERT(a->m_constructorName == b->m_constructorName);
