@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2011, 2015 Apple Inc. All rights reserved.
+ *  Copyright (C) 2011-2019 Apple Inc. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -23,22 +23,13 @@
 
 #include "InitializeThreading.h"
 #include "JSCInlines.h"
-#include "JSGlobalObject.h"
-#include <errno.h>
+#include "YarrFlags.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <wtf/Vector.h>
 #include <wtf/text/StringBuilder.h>
 
-#if !OS(WINDOWS)
-#include <unistd.h>
-#endif
-
-#if HAVE(SYS_TIME_H)
-#include <sys/time.h>
-#endif
-
 #if COMPILER(MSVC)
 #include <crtdbg.h>
 #include <mmsystem.h>
@@ -101,12 +92,9 @@ struct RegExpTest {
     Vector<int, 32> expectVector;
 };
 
-class GlobalObject : public JSGlobalObject {
-private:
-    GlobalObject(VM&, Structure*, const Vector<String>& arguments);
-
+class GlobalObject final : public JSGlobalObject {
 public:
-    typedef JSGlobalObject Base;
+    using Base = JSGlobalObject;
 
     static GlobalObject* create(VM& vm, Structure* structure, const Vector<String>& arguments)
     {
@@ -116,20 +104,23 @@ public:
 
     DECLARE_INFO;
 
-    static const bool needsDestructor = false;
+    static constexpr bool needsDestructor = true;
 
     static Structure* createStructure(VM& vm, JSValue prototype)
     {
-        return Structure::create(vm, 0, prototype, TypeInfo(GlobalObjectType, StructureFlags), info());
+        return Structure::create(vm, nullptr, prototype, TypeInfo(GlobalObjectType, StructureFlags), info());
     }
 
-protected:
+private:
+    GlobalObject(VM&, Structure*, const Vector<String>& arguments);
+
     void finishCreation(VM& vm, const Vector<String>& arguments)
     {
         Base::finishCreation(vm);
         UNUSED_PARAM(arguments);
     }
 };
+STATIC_ASSERT_ISO_SUBSPACE_SHARABLE(GlobalObject, JSGlobalObject);
 
 const ClassInfo GlobalObject::s_info = { "global", &JSGlobalObject::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(GlobalObject) };
 
@@ -174,8 +165,7 @@ int main(int argc, char** argv)
     timeBeginPeriod(1);
 #endif
 
-    // Initialize JSC before getting VM.
-    JSC::initializeThreading();
+    JSC::initialize();
 
     // We can't use destructors in the following code because it uses Windows
     // Structured Exception Handling
@@ -186,12 +176,12 @@ int main(int argc, char** argv)
     return res;
 }
 
-static bool testOneRegExp(VM& vm, RegExp* regexp, RegExpTest* regExpTest, bool verbose, unsigned int lineNumber)
+static bool testOneRegExp(JSGlobalObject* globalObject, RegExp* regexp, RegExpTest* regExpTest, bool verbose, unsigned lineNumber)
 {
     bool result = true;
     Vector<int> outVector;
     outVector.resize(regExpTest->expectVector.size());
-    int matchResult = regexp->match(vm, regExpTest->subject, regExpTest->offset, outVector);
+    int matchResult = regexp->match(globalObject, regExpTest->subject, regExpTest->offset, outVector);
 
     if (matchResult != regExpTest->result) {
         result = false;
@@ -319,20 +309,27 @@ static RegExp* parseRegExpLine(VM& vm, char* line, int lineLength, const char**
     StringBuilder pattern;
 
     if (line[0] != '/')
-        return 0;
+        return nullptr;
 
     int i = scanString(line + 1, lineLength - 1, pattern, '/') + 1;
 
     if ((i >= lineLength) || (line[i] != '/'))
-        return 0;
+        return nullptr;
 
     ++i;
 
-    RegExp* r = RegExp::create(vm, pattern.toString(), regExpFlags(line + i));
+    auto flags = Yarr::parseFlags(line + i);
+    if (!flags) {
+        *regexpError = Yarr::errorMessage(Yarr::ErrorCode::InvalidRegularExpressionFlags);
+        return nullptr;
+    }
+
+    RegExp* r = RegExp::create(vm, pattern.toString(), flags.value());
     if (!r->isValid()) {
         *regexpError = r->errorMessage();
         return nullptr;
     }
+
     return r;
 }
 
@@ -341,19 +338,19 @@ static RegExpTest* parseTestLine(char* line, int lineLength)
     StringBuilder subjectString;
 
     if ((line[0] != ' ') || (line[1] != '"'))
-        return 0;
+        return nullptr;
 
     int i = scanString(line + 2, lineLength - 2, subjectString, '"') + 2;
 
     if ((i >= (lineLength - 2)) || (line[i] != '"') || (line[i+1] != ',') || (line[i+2] != ' '))
-        return 0;
+        return nullptr;
 
     i += 3;
 
     int offset;
 
     if (sscanf(line + i, "%d, ", &offset) != 1)
-        return 0;
+        return nullptr;
 
     while (line[i] && line[i] != ' ')
         ++i;
@@ -363,7 +360,7 @@ static RegExpTest* parseTestLine(char* line, int lineLength)
     int matchResult;
 
     if (sscanf(line + i, "%d, ", &matchResult) != 1)
-        return 0;
+        return nullptr;
 
     while (line[i] && line[i] != ' ')
         ++i;
@@ -371,7 +368,7 @@ static RegExpTest* parseTestLine(char* line, int lineLength)
     ++i;
 
     if (line[i++] != '(')
-        return 0;
+        return nullptr;
 
     int start, end;
 
@@ -384,7 +381,7 @@ static RegExpTest* parseTestLine(char* line, int lineLength)
     while (line[i] && line[i] != ')') {
         if (sscanf(line + i, "%d, %d", &start, &end) != 2) {
             delete result;
-            return 0;
+            return nullptr;
         }
 
         result->expectVector.append(start);
@@ -400,7 +397,7 @@ static RegExpTest* parseTestLine(char* line, int lineLength)
             break;
         if (!line[i] || (line[i] != ',')) {
             delete result;
-            return 0;
+            return nullptr;
         }
         i++;
     }
@@ -428,9 +425,9 @@ static bool runFromFiles(GlobalObject* globalObject, const Vector<String>& files
             continue;
         }
 
-        RegExp* regexp = 0;
+        RegExp* regexp = nullptr;
         size_t lineLength = 0;
-        char* linePtr = 0;
+        char* linePtr = nullptr;
         unsigned int lineNumber = 0;
         const char* regexpError = nullptr;
 
@@ -456,7 +453,7 @@ static bool runFromFiles(GlobalObject* globalObject, const Vector<String>& files
 
                 if (regexp && regExpTest) {
                     ++tests;
-                    if (!testOneRegExp(vm, regexp, regExpTest, verbose, lineNumber)) {
+                    if (!testOneRegExp(globalObject, regexp, regExpTest, verbose, lineNumber)) {
                         failures++;
                         printf("Failure on line %u\n", lineNumber);
                     }
@@ -466,7 +463,7 @@ static bool runFromFiles(GlobalObject* globalObject, const Vector<String>& files
                     delete regExpTest;
             } else if (linePtr[0] == '-') {
                 tests++;
-                regexp = 0; // Reset the live regexp to avoid confusing other subsequent tests
+                regexp = nullptr; // Reset the live regexp to avoid confusing other subsequent tests
                 bool successfullyParsed = parseRegExpLine(vm, linePtr + 1, lineLength - 1, &regexpError);
                 if (successfullyParsed) {
                     failures++;
