@@ -54,6 +54,13 @@ namespace Wasm {
 struct CompilationContext;
 struct ModuleInformation;
 
+using BlockSignature = const Signature*;
+
+enum class TableElementType : uint8_t {
+    Anyref,
+    Funcref
+};
+
 inline bool isValueType(Type type)
 {
     switch (type) {
@@ -62,12 +69,22 @@ inline bool isValueType(Type type)
     case F32:
     case F64:
         return true;
+    case Anyref:
+    case Funcref:
+        return Options::useWebAssemblyReferences();
     default:
         break;
     }
     return false;
 }
 
+inline bool isSubtype(Type sub, Type parent)
+{
+    if (sub == parent)
+        return true;
+    return sub == Funcref && parent == Anyref;
+}
+
 enum class ExternalKind : uint8_t {
     // FIXME auto-generate this. https://bugs.webkit.org/show_bug.cgi?id=165231
     Function = 0,
@@ -107,6 +124,7 @@ inline const char* makeString(ExternalKind kind)
 }
 
 struct Import {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     const Name module;
     const Name field;
     ExternalKind kind;
@@ -114,6 +132,7 @@ struct Import {
 };
 
 struct Export {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     const Name field;
     ExternalKind kind;
     unsigned kindIndex; // Index in the vector of the corresponding kind.
@@ -121,32 +140,42 @@ struct Export {
 
 String makeString(const Name& characters);
 
-struct Global {
+struct GlobalInformation {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     enum Mutability : uint8_t {
         // FIXME auto-generate this. https://bugs.webkit.org/show_bug.cgi?id=165231
         Mutable = 1,
         Immutable = 0
     };
 
-    enum InitializationType {
+    enum InitializationType : uint8_t {
         IsImport,
         FromGlobalImport,
+        FromRefFunc,
         FromExpression
     };
 
+    enum class BindingMode : uint8_t {
+        EmbeddedInInstance = 0,
+        Portable,
+    };
+
     Mutability mutability;
     Type type;
     InitializationType initializationType { IsImport };
+    BindingMode bindingMode { BindingMode::EmbeddedInInstance };
     uint64_t initialBitsOrImportNumber { 0 };
 };
 
 struct FunctionData {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     size_t start;
     size_t end;
     Vector<uint8_t> data;
 };
 
 class I32InitExpr {
+    WTF_MAKE_FAST_ALLOCATED;
     enum Type : uint8_t {
         Global,
         Const
@@ -182,6 +211,7 @@ private:
 };
 
 struct Segment {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     uint32_t sizeInBytes;
     I32InitExpr offset;
     // Bytes are allocated at the end.
@@ -197,26 +227,31 @@ struct Segment {
 };
 
 struct Element {
-    Element(I32InitExpr offset)
-        : offset(offset)
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
+    Element(uint32_t tableIndex, I32InitExpr offset)
+        : tableIndex(tableIndex)
+        , offset(offset)
     { }
 
+    uint32_t tableIndex;
     I32InitExpr offset;
     Vector<uint32_t> functionIndices;
 };
 
 class TableInformation {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     TableInformation()
     {
         ASSERT(!*this);
     }
 
-    TableInformation(uint32_t initial, Optional<uint32_t> maximum, bool isImport)
+    TableInformation(uint32_t initial, Optional<uint32_t> maximum, bool isImport, TableElementType type)
         : m_initial(initial)
         , m_maximum(maximum)
         , m_isImport(isImport)
         , m_isValid(true)
+        , m_type(type)
     {
         ASSERT(*this);
     }
@@ -225,15 +260,19 @@ public:
     bool isImport() const { return m_isImport; }
     uint32_t initial() const { return m_initial; }
     Optional<uint32_t> maximum() const { return m_maximum; }
+    TableElementType type() const { return m_type; }
+    Wasm::Type wasmType() const { return m_type == TableElementType::Funcref ? Type::Funcref : Type::Anyref; }
 
 private:
     uint32_t m_initial;
     Optional<uint32_t> m_maximum;
     bool m_isImport { false };
     bool m_isValid { false };
+    TableElementType m_type;
 };
 
 struct CustomSection {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     Name name;
     Vector<uint8_t> payload;
 };
@@ -257,16 +296,19 @@ inline bool isValidNameType(Int val)
 }
 
 struct UnlinkedWasmToWasmCall {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     CodeLocationNearCall<WasmEntryPtrTag> callLocation;
     size_t functionIndexSpace;
 };
 
 struct Entrypoint {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     std::unique_ptr<B3::Compilation> compilation;
     RegisterAtOffsetList calleeSaveRegisters;
 };
 
 struct InternalFunction {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     CodeLocationDataLabelPtr<WasmEntryPtrTag> calleeMoveLocation;
     Entrypoint entrypoint;
 };
@@ -275,6 +317,7 @@ struct InternalFunction {
 // with all imports, and then all internal functions. WasmToWasmImportableFunction and FunctionIndexSpace are only
 // meant as fast lookup tables for these opcodes and do not own code.
 struct WasmToWasmImportableFunction {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     using LoadLocation = MacroAssemblerCodePtr<WasmEntryPtrTag>*;
     static ptrdiff_t offsetOfSignatureIndex() { return OBJECT_OFFSETOF(WasmToWasmImportableFunction, signatureIndex); }
     static ptrdiff_t offsetOfEntrypointLoadLocation() { return OBJECT_OFFSETOF(WasmToWasmImportableFunction, entrypointLoadLocation); }
