@@ -29,7 +29,7 @@
 
 #include "CallLinkInfo.h"
 #include "JSCast.h"
-#include "PromiseDeferredTimer.h"
+#include "PromiseTimer.h"
 #include "Structure.h"
 #include "WasmCallee.h"
 #include "WasmFormat.h"
@@ -49,7 +49,7 @@ class Plan;
 class JSWebAssemblyCodeBlock final : public JSCell {
 public:
     typedef JSCell Base;
-    static const unsigned StructureFlags = Base::StructureFlags | StructureIsImmortal;
+    static constexpr unsigned StructureFlags = Base::StructureFlags | StructureIsImmortal;
 
     static JSWebAssemblyCodeBlock* create(VM&, Ref<Wasm::CodeBlock>, const Wasm::ModuleInformation&);
     static Structure* createStructure(VM& vm, JSGlobalObject* globalObject, JSValue prototype)
@@ -84,7 +84,7 @@ public:
 private:
     JSWebAssemblyCodeBlock(VM&, Ref<Wasm::CodeBlock>&&, const Wasm::ModuleInformation&);
     DECLARE_EXPORT_INFO;
-    static const bool needsDestruction = true;
+    static constexpr bool needsDestruction = true;
     static void destroy(JSCell*);
     static void visitChildren(JSCell*, SlotVisitor&);
 
