@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2009, 2014-2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2009-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2020 Alexey Shvayka <shvaikalesh@gmail.com>.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -10,17 +11,17 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
- * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
- * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+ * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #pragma once
@@ -41,7 +42,9 @@ template<class Delegate, typename CharType>
 class Parser {
 private:
     template<class FriendDelegate>
-    friend ErrorCode parse(FriendDelegate&, const String& pattern, bool isUnicode, unsigned backReferenceLimit);
+    friend ErrorCode parse(FriendDelegate&, const String& pattern, bool isUnicode, unsigned backReferenceLimit, bool isNamedForwardReferenceAllowed);
+
+    enum class UnicodeParseContext : uint8_t { PatternCodePoint, GroupName };
 
     /*
      * CharacterClassParserDelegate:
@@ -54,9 +57,10 @@ private:
      */
     class CharacterClassParserDelegate {
     public:
-        CharacterClassParserDelegate(Delegate& delegate, ErrorCode& err)
+        CharacterClassParserDelegate(Delegate& delegate, ErrorCode& err, bool isUnicode)
             : m_delegate(delegate)
             , m_errorCode(err)
+            , m_isUnicode(isUnicode)
             , m_state(Empty)
             , m_character(0)
         {
@@ -85,13 +89,14 @@ private:
         {
             switch (m_state) {
             case AfterCharacterClass:
-                // Following a builtin character class we need look out for a hyphen.
+                // Following a built-in character class we need look out for a hyphen.
                 // We're looking for invalid ranges, such as /[\d-x]/ or /[\d-\d]/.
-                // If we see a hyphen following a charater class then unlike usual
+                // If we see a hyphen following a character class then unlike usual
                 // we'll report it to the delegate immediately, and put ourself into
-                // a poisoned state. Any following calls to add another character or
-                // character class will result in an error. (A hypen following a
-                // character-class is itself valid, but only  at the end of a regex).
+                // a poisoned state. In a unicode pattern, any following calls to add
+                // another character or character class will result in syntax error.
+                // A hypen following a character class is itself valid, but only at
+                // the end of a regex.
                 if (hyphenIsRange && ch == '-') {
                     m_delegate.atomCharacterClassAtom('-');
                     m_state = AfterCharacterClassHyphen;
@@ -116,21 +121,20 @@ private:
 
             case CachedCharacterHyphen:
                 if (ch < m_character) {
-                    m_errorCode = ErrorCode::CharacterClassOutOfOrder;
+                    m_errorCode = ErrorCode::CharacterClassRangeOutOfOrder;
                     return;
                 }
                 m_delegate.atomCharacterClassRange(m_character, ch);
                 m_state = Empty;
                 return;
 
-                // See coment in atomBuiltInCharacterClass below.
-                // This too is technically an error, per ECMA-262, and again we
-                // we chose to allow this.  Note a subtlely here that while we
-                // diverge from the spec's definition of CharacterRange we do
-                // remain in compliance with the grammar.  For example, consider
-                // the expression /[\d-a-z]/.  We comply with the grammar in
-                // this case by not allowing a-z to be matched as a range.
+                // If we hit this case, we have an invalid range like /[\d-a]/.
+                // See coment in atomBuiltInCharacterClass() below.
             case AfterCharacterClassHyphen:
+                if (m_isUnicode) {
+                    m_errorCode = ErrorCode::CharacterClassRangeInvalid;
+                    return;
+                }
                 m_delegate.atomCharacterClassAtom(ch);
                 m_state = Empty;
                 return;
@@ -151,23 +155,27 @@ private:
                 FALLTHROUGH;
             case Empty:
             case AfterCharacterClass:
-                m_state = AfterCharacterClass;
                 m_delegate.atomCharacterClassBuiltIn(classID, invert);
+                m_state = AfterCharacterClass;
                 return;
 
                 // If we hit either of these cases, we have an invalid range that
-                // looks something like /[x-\d]/ or /[\d-\d]/.
-                // According to ECMA-262 this should be a syntax error, but
-                // empirical testing shows this to break teh webz.  Instead we
-                // comply with to the ECMA-262 grammar, and assume the grammar to
-                // have matched the range correctly, but tweak our interpretation
-                // of CharacterRange.  Effectively we implicitly handle the hyphen
-                // as if it were escaped, e.g. /[\w-_]/ is treated as /[\w\-_]/.
+                // looks something like /[a-\d]/ or /[\d-\d]/.
+                // Since ES2015, this should be syntax error in a unicode pattern,
+                // yet gracefully handled in a regular regex to avoid breaking the web.
+                // Effectively we handle the hyphen as if it was (implicitly) escaped,
+                // e.g. /[\d-a-z]/ is treated as /[\d\-a\-z]/.
+                // See usages of CharacterRangeOrUnion abstract op in
+                // https://tc39.es/ecma262/#sec-regular-expression-patterns-semantics
             case CachedCharacterHyphen:
                 m_delegate.atomCharacterClassAtom(m_character);
                 m_delegate.atomCharacterClassAtom('-');
                 FALLTHROUGH;
             case AfterCharacterClassHyphen:
+                if (m_isUnicode) {
+                    m_errorCode = ErrorCode::CharacterClassRangeInvalid;
+                    return;
+                }
                 m_delegate.atomCharacterClassBuiltIn(classID, invert);
                 m_state = Empty;
                 return;
@@ -195,12 +203,12 @@ private:
         NO_RETURN_DUE_TO_ASSERT void assertionWordBoundary(bool) { RELEASE_ASSERT_NOT_REACHED(); }
         NO_RETURN_DUE_TO_ASSERT void atomBackReference(unsigned) { RELEASE_ASSERT_NOT_REACHED(); }
         NO_RETURN_DUE_TO_ASSERT void atomNamedBackReference(const String&) { RELEASE_ASSERT_NOT_REACHED(); }
-        NO_RETURN_DUE_TO_ASSERT bool isValidNamedForwardReference(const String&) { RELEASE_ASSERT_NOT_REACHED(); }
         NO_RETURN_DUE_TO_ASSERT void atomNamedForwardReference(const String&) { RELEASE_ASSERT_NOT_REACHED(); }
 
     private:
         Delegate& m_delegate;
         ErrorCode& m_errorCode;
+        bool m_isUnicode;
         enum CharacterClassConstructionState {
             Empty,
             CachedCharacter,
@@ -211,12 +219,13 @@ private:
         UChar32 m_character;
     };
 
-    Parser(Delegate& delegate, const String& pattern, bool isUnicode, unsigned backReferenceLimit)
+    Parser(Delegate& delegate, const String& pattern, bool isUnicode, unsigned backReferenceLimit, bool isNamedForwardReferenceAllowed)
         : m_delegate(delegate)
-        , m_backReferenceLimit(backReferenceLimit)
         , m_data(pattern.characters<CharType>())
         , m_size(pattern.length())
         , m_isUnicode(isUnicode)
+        , m_backReferenceLimit(backReferenceLimit)
+        , m_isNamedForwardReferenceAllowed(isNamedForwardReferenceAllowed)
     {
     }
 
@@ -225,7 +234,7 @@ private:
     // For non-unicode patterns, most any character can be escaped.
     bool isIdentityEscapeAnError(int ch)
     {
-        if (m_isUnicode && !strchr("^$\\.*+?()[]{}|/", ch)) {
+        if (m_isUnicode && (!strchr("^$\\.*+?()[]{}|/", ch) || !ch)) {
             m_errorCode = ErrorCode::InvalidIdentityEscape;
             return true;
         }
@@ -269,12 +278,9 @@ private:
         // Assertions
         case 'b':
             consume();
-            if (inCharacterClass) {
-                if (isIdentityEscapeAnError('b'))
-                    break;
-
+            if (inCharacterClass)
                 delegate.atomPatternCharacter('\b');
-            } else {
+            else {
                 delegate.assertionWordBoundary(false);
                 return false;
             }
@@ -318,6 +324,23 @@ private:
             delegate.atomBuiltInCharacterClass(BuiltInCharacterClassID::WordClassID, true);
             break;
 
+        case '0': {
+            consume();
+
+            if (!peekIsDigit()) {
+                delegate.atomPatternCharacter(0);
+                break;
+            }
+
+            if (m_isUnicode) {
+                m_errorCode = ErrorCode::InvalidOctalEscape;
+                break;
+            }
+
+            delegate.atomPatternCharacter(consumeOctal(2));
+            break;
+        }
+
         // DecimalEscape
         case '1':
         case '2':
@@ -328,39 +351,33 @@ private:
         case '7':
         case '8':
         case '9': {
-            // To match Firefox, we parse an invalid backreference in the range [1-7] as an octal escape.
+            // For non-Unicode patterns, invalid backreferences are parsed as octal or decimal escapes.
             // First, try to parse this as backreference.
             if (!inCharacterClass) {
                 ParseState state = saveState();
 
                 unsigned backReference = consumeNumber();
                 if (backReference <= m_backReferenceLimit) {
+                    m_maxSeenBackReference = std::max(m_maxSeenBackReference, backReference);
                     delegate.atomBackReference(backReference);
                     break;
                 }
 
                 restoreState(state);
-
                 if (m_isUnicode) {
                     m_errorCode = ErrorCode::InvalidBackreference;
-                    return false;
+                    break;
                 }
             }
 
-            // Not a backreference, and not octal. Just a number.
-            if (peek() >= '8') {
-                delegate.atomPatternCharacter(consume());
+            if (m_isUnicode) {
+                m_errorCode = ErrorCode::InvalidOctalEscape;
                 break;
             }
 
-            // Fall-through to handle this as an octal escape.
-            FALLTHROUGH;
-        }
-
-        // Octal escape
-        case '0':
-            delegate.atomPatternCharacter(consumeOctal());
+            delegate.atomPatternCharacter(peek() < '8' ? consumeOctal(3) : consume());
             break;
+        }
 
         // ControlEscape
         case 'f':
@@ -391,12 +408,28 @@ private:
             if (!atEndOfPattern()) {
                 int control = consume();
 
-                // To match Firefox, inside a character class, we also accept numbers and '_' as control characters.
-                if (inCharacterClass ? WTF::isASCIIAlphanumeric(control) || (control == '_') : WTF::isASCIIAlpha(control)) {
+                if (WTF::isASCIIAlpha(control)) {
+                    delegate.atomPatternCharacter(control & 0x1f);
+                    break;
+                }
+
+                if (m_isUnicode) {
+                    m_errorCode = ErrorCode::InvalidControlLetterEscape;
+                    break;
+                }
+
+                // https://tc39.es/ecma262/#prod-annexB-ClassControlLetter
+                if (inCharacterClass && (WTF::isASCIIDigit(control) || control == '_')) {
                     delegate.atomPatternCharacter(control & 0x1f);
                     break;
                 }
             }
+
+            if (m_isUnicode) {
+                m_errorCode = ErrorCode::InvalidIdentityEscape;
+                break;
+            }
+
             restoreState(state);
             delegate.atomPatternCharacter('\\');
             break;
@@ -420,28 +453,30 @@ private:
         case 'k': {
             consume();
             ParseState state = saveState();
-            if (!atEndOfPattern() && !inCharacterClass) {
-                if (consume() == '<') {
-                    auto groupName = tryConsumeGroupName();
-                    if (groupName) {
-                        if (m_captureGroupNames.contains(groupName.value())) {
-                            delegate.atomNamedBackReference(groupName.value());
-                            break;
-                        }
+            if (!inCharacterClass && tryConsume('<')) {
+                auto groupName = tryConsumeGroupName();
+                if (hasError(m_errorCode))
+                    break;
 
-                        if (delegate.isValidNamedForwardReference(groupName.value())) {
-                            delegate.atomNamedForwardReference(groupName.value());
-                            break;
-                        }
+                if (groupName) {
+                    if (m_captureGroupNames.contains(groupName.value())) {
+                        delegate.atomNamedBackReference(groupName.value());
+                        break;
                     }
-                    if (m_isUnicode) {
-                        m_errorCode = ErrorCode::InvalidBackreference;
+
+                if (m_isNamedForwardReferenceAllowed) {
+                    m_forwardReferenceNames.add(groupName.value());
+                        delegate.atomNamedForwardReference(groupName.value());
                         break;
                     }
                 }
             }
+
             restoreState(state);
+            if (!isIdentityEscapeAnError('k')) {
             delegate.atomPatternCharacter('k');
+                m_kIdentityEscapeSeen = true;
+            }
             break;
         }
 
@@ -472,64 +507,11 @@ private:
 
         // UnicodeEscape
         case 'u': {
-            consume();
-            if (atEndOfPattern()) {
-                if (isIdentityEscapeAnError('u'))
-                    break;
-
-                delegate.atomPatternCharacter('u');
-                break;
-            }
-
-            if (m_isUnicode && peek() == '{') {
-                consume();
-                UChar32 codePoint = 0;
-                do {
-                    if (atEndOfPattern() || !isASCIIHexDigit(peek())) {
-                        m_errorCode = ErrorCode::InvalidUnicodeEscape;
-                        break;
-                    }
-
-                    codePoint = (codePoint << 4) | toASCIIHexValue(consume());
-
-                    if (codePoint > UCHAR_MAX_VALUE)
-                        m_errorCode = ErrorCode::InvalidUnicodeEscape;
-                } while (!atEndOfPattern() && peek() != '}');
-                if (!atEndOfPattern() && peek() == '}')
-                    consume();
-                else if (!hasError(m_errorCode))
-                    m_errorCode = ErrorCode::InvalidUnicodeEscape;
+            int codePoint = tryConsumeUnicodeEscape<UnicodeParseContext::PatternCodePoint>();
                 if (hasError(m_errorCode))
-                    return false;
-
-                delegate.atomPatternCharacter(codePoint);
                 break;
-            }
-            int u = tryConsumeHex(4);
-            if (u == -1) {
-                if (isIdentityEscapeAnError('u'))
-                    break;
 
-                delegate.atomPatternCharacter('u');
-            } else {
-                // If we have the first of a surrogate pair, look for the second.
-                if (U16_IS_LEAD(u) && m_isUnicode && (patternRemaining() >= 6) && peek() == '\\') {
-                    ParseState state = saveState();
-                    consume();
-
-                    if (tryConsume('u')) {
-                        int surrogate2 = tryConsumeHex(4);
-                        if (U16_IS_TRAIL(surrogate2)) {
-                            u = U16_GET_SUPPLEMENTARY(u, surrogate2);
-                            delegate.atomPatternCharacter(u);
-                            break;
-                        }
-                    }
-
-                    restoreState(state);
-                }
-                delegate.atomPatternCharacter(u);
-            }
+            delegate.atomPatternCharacter(codePoint == -1 ? 'u' : codePoint);
             break;
         }
 
@@ -552,10 +534,13 @@ private:
         return true;
     }
 
+    template<UnicodeParseContext context>
     UChar32 consumePossibleSurrogatePair()
     {
+        bool unicodePatternOrGroupName = m_isUnicode || context == UnicodeParseContext::GroupName;
+
         UChar32 ch = consume();
-        if (U16_IS_LEAD(ch) && m_isUnicode && (patternRemaining() > 0)) {
+        if (U16_IS_LEAD(ch) && unicodePatternOrGroupName && !atEndOfPattern()) {
             ParseState state = saveState();
 
             UChar32 surrogate2 = consume();
@@ -585,7 +570,7 @@ private:
     /*
      * parseCharacterClass():
      *
-     * Helper for parseTokens(); calls dirctly and indirectly (via parseCharacterClassEscape)
+     * Helper for parseTokens(); calls directly and indirectly (via parseCharacterClassEscape)
      * to an instance of CharacterClassParserDelegate, to describe the character class to the
      * delegate.
      */
@@ -595,7 +580,7 @@ private:
         ASSERT(peek() == '[');
         consume();
 
-        CharacterClassParserDelegate characterClassConstructor(m_delegate, m_errorCode);
+        CharacterClassParserDelegate characterClassConstructor(m_delegate, m_errorCode, m_isUnicode);
 
         characterClassConstructor.begin(tryConsume('^'));
 
@@ -611,7 +596,7 @@ private:
                 break;
 
             default:
-                characterClassConstructor.atomPatternCharacter(consumePossibleSurrogatePair(), true);
+                characterClassConstructor.atomPatternCharacter(consumePossibleSurrogatePair<UnicodeParseContext::PatternCodePoint>(), true);
             }
 
             if (hasError(m_errorCode))
@@ -632,6 +617,8 @@ private:
         ASSERT(peek() == '(');
         consume();
 
+        auto type = ParenthesesType::Subpattern;
+
         if (tryConsume('?')) {
             if (atEndOfPattern()) {
                 m_errorCode = ErrorCode::ParenthesesTypeInvalid;
@@ -645,15 +632,25 @@ private:
 
             case '=':
                 m_delegate.atomParentheticalAssertionBegin();
+                type = ParenthesesType::Assertion;
                 break;
 
             case '!':
                 m_delegate.atomParentheticalAssertionBegin(true);
+                type = ParenthesesType::Assertion;
                 break;
 
             case '<': {
                 auto groupName = tryConsumeGroupName();
+                if (hasError(m_errorCode))
+                    break;
+
                 if (groupName) {
+                    if (m_kIdentityEscapeSeen) {
+                        m_errorCode = ErrorCode::InvalidNamedBackReference;
+                        break;
+                    }
+
                     auto setAddResult = m_captureGroupNames.add(groupName.value());
                     if (setAddResult.isNewEntry)
                         m_delegate.atomParenthesesSubpatternBegin(true, groupName);
@@ -671,26 +668,35 @@ private:
         } else
             m_delegate.atomParenthesesSubpatternBegin();
 
-        ++m_parenthesesNestingDepth;
+        if (type == ParenthesesType::Subpattern)
+            ++m_numSubpatterns;
+
+        m_parenthesesStack.append(type);
     }
 
     /*
      * parseParenthesesEnd():
      *
      * Helper for parseTokens(); checks for parse errors (due to unmatched parentheses).
+     *
+     * The boolean value returned by this method indicates whether the token parsed
+     * was either an Atom or, for web compatibility reasons, QuantifiableAssertion
+     * in non-Unicode pattern.
      */
-    void parseParenthesesEnd()
+    bool parseParenthesesEnd()
     {
         ASSERT(!hasError(m_errorCode));
         ASSERT(peek() == ')');
         consume();
 
-        if (m_parenthesesNestingDepth > 0)
-            m_delegate.atomParenthesesEnd();
-        else
+        if (m_parenthesesStack.isEmpty()) {
             m_errorCode = ErrorCode::ParenthesesUnmatched;
+            return false;
+        }
 
-        --m_parenthesesNestingDepth;
+        m_delegate.atomParenthesesEnd();
+        auto type = m_parenthesesStack.takeLast();
+        return type == ParenthesesType::Subpattern || !m_isUnicode;
     }
 
     /*
@@ -741,8 +747,7 @@ private:
                 break;
 
             case ')':
-                parseParenthesesEnd();
-                lastTokenWasAnAtom = true;
+                lastTokenWasAnAtom = parseParenthesesEnd();
                 break;
 
             case '^':
@@ -768,6 +773,17 @@ private:
                 lastTokenWasAnAtom = true;
                 break;
 
+            case ']':
+            case '}':
+                if (m_isUnicode) {
+                    m_errorCode = ErrorCode::BracketUnmatched;
+                    break;
+                }
+
+                m_delegate.atomPatternCharacter(consume());
+                lastTokenWasAnAtom = true;
+                break;
+
             case '\\':
                 lastTokenWasAnAtom = parseAtomEscape();
                 break;
@@ -811,13 +827,18 @@ private:
                     }
                 }
 
+                if (m_isUnicode) {
+                    m_errorCode = ErrorCode::QuantifierIncomplete;
+                    break;
+                }
+
                 restoreState(state);
+                // if we did not find a complete quantifer, fall through to the default case.
+                FALLTHROUGH;
             }
-            // if we did not find a complete quantifer, fall through to the default case.
-            FALLTHROUGH;
 
             default:
-                m_delegate.atomPatternCharacter(consumePossibleSurrogatePair());
+                m_delegate.atomPatternCharacter(consumePossibleSurrogatePair<UnicodeParseContext::PatternCodePoint>());
                 lastTokenWasAnAtom = true;
             }
 
@@ -825,7 +846,7 @@ private:
                 return;
         }
 
-        if (m_parenthesesNestingDepth > 0)
+        if (!m_parenthesesStack.isEmpty())
             m_errorCode = ErrorCode::MissingParentheses;
     }
 
@@ -837,14 +858,87 @@ private:
     ErrorCode parse()
     {
         if (m_size > MAX_PATTERN_SIZE)
-            m_errorCode = ErrorCode::PatternTooLarge;
-        else
-            parseTokens();
-        ASSERT(atEndOfPattern() || hasError(m_errorCode));
+            return ErrorCode::PatternTooLarge;
+
+        parseTokens();
+
+        if (!hasError(m_errorCode)) {
+            ASSERT(atEndOfPattern());
+            handleIllegalReferences();
+            ASSERT(atEndOfPattern());
+        }
 
         return m_errorCode;
     }
 
+    void handleIllegalReferences()
+    {
+        bool shouldReparse = false;
+
+        if (m_maxSeenBackReference > m_numSubpatterns) {
+            // Contains illegal numeric backreference. See https://tc39.es/ecma262/#prod-annexB-AtomEscape
+            if (m_isUnicode) {
+                m_errorCode = ErrorCode::InvalidBackreference;
+                return;
+            }
+
+            m_backReferenceLimit = m_numSubpatterns;
+            shouldReparse = true;
+        }
+
+        if (m_kIdentityEscapeSeen && !m_captureGroupNames.isEmpty()) {
+            m_errorCode = ErrorCode::InvalidNamedBackReference;
+            return;
+        }
+
+        if (containsIllegalNamedForwardReference()) {
+            // \k<a> is parsed as named reference in Unicode patterns because of strict IdentityEscape grammar.
+            // See https://tc39.es/ecma262/#sec-patterns-static-semantics-early-errors
+            if (m_isUnicode || !m_captureGroupNames.isEmpty()) {
+                m_errorCode = ErrorCode::InvalidNamedBackReference;
+                return;
+            }
+
+            m_isNamedForwardReferenceAllowed = false;
+            shouldReparse = true;
+        }
+
+        if (shouldReparse) {
+            resetForReparsing();
+            parseTokens();
+        }
+    }
+
+    bool containsIllegalNamedForwardReference()
+    {
+        if (m_forwardReferenceNames.isEmpty())
+            return false;
+
+        if (m_captureGroupNames.isEmpty())
+            return true;
+
+        for (auto& entry : m_forwardReferenceNames) {
+            if (!m_captureGroupNames.contains(entry))
+                return true;
+        }
+
+        return false;
+    }
+
+    void resetForReparsing()
+    {
+        ASSERT(!hasError(m_errorCode));
+
+        m_delegate.resetForReparsing();
+        m_index = 0;
+        m_numSubpatterns = 0;
+        m_maxSeenBackReference = 0;
+        m_kIdentityEscapeSeen = false;
+        m_parenthesesStack.clear();
+        m_captureGroupNames.clear();
+        m_forwardReferenceNames.clear();
+    }
+
     // Misc helper functions:
 
     typedef unsigned ParseState;
@@ -888,70 +982,73 @@ private:
         return peek() - '0';
     }
 
+    template<UnicodeParseContext context>
     int tryConsumeUnicodeEscape()
     {
-        if (!tryConsume('u'))
-            return -1;
+        ASSERT(!hasError(m_errorCode));
 
-        if (m_isUnicode && tryConsume('{')) {
+        bool unicodePatternOrGroupName = m_isUnicode || context == UnicodeParseContext::GroupName;
+
+        if (!tryConsume('u') || atEndOfPattern()) {
+            if (unicodePatternOrGroupName)
+                m_errorCode = ErrorCode::InvalidUnicodeEscape;
+            return -1;
+        }
+
+        if (unicodePatternOrGroupName && tryConsume('{')) {
             int codePoint = 0;
             do {
                 if (atEndOfPattern() || !isASCIIHexDigit(peek())) {
-                    m_errorCode = ErrorCode::InvalidUnicodeEscape;
+                    m_errorCode = ErrorCode::InvalidUnicodeCodePointEscape;
                     return -1;
                 }
 
                 codePoint = (codePoint << 4) | toASCIIHexValue(consume());
 
                 if (codePoint > UCHAR_MAX_VALUE) {
-                    m_errorCode = ErrorCode::InvalidUnicodeEscape;
+                    m_errorCode = ErrorCode::InvalidUnicodeCodePointEscape;
                     return -1;
                 }
             } while (!atEndOfPattern() && peek() != '}');
-            if (!atEndOfPattern() && peek() == '}')
-                consume();
-            else if (!hasError(m_errorCode))
-                m_errorCode = ErrorCode::InvalidUnicodeEscape;
-            if (hasError(m_errorCode))
+
+            if (!tryConsume('}')) {
+                m_errorCode = ErrorCode::InvalidUnicodeCodePointEscape;
                 return -1;
+            }
 
             return codePoint;
         }
 
-        int u = tryConsumeHex(4);
-        if (u == -1)
+        int codeUnit = tryConsumeHex(4);
+        if (codeUnit == -1) {
+            if (unicodePatternOrGroupName)
+                m_errorCode = ErrorCode::InvalidUnicodeEscape;
             return -1;
+        }
 
         // If we have the first of a surrogate pair, look for the second.
-        if (U16_IS_LEAD(u) && m_isUnicode && (patternRemaining() >= 6) && peek() == '\\') {
+        if (U16_IS_LEAD(codeUnit) && unicodePatternOrGroupName && patternRemaining() >= 6 && peek() == '\\') {
             ParseState state = saveState();
             consume();
 
             if (tryConsume('u')) {
                 int surrogate2 = tryConsumeHex(4);
-                if (U16_IS_TRAIL(surrogate2)) {
-                    u = U16_GET_SUPPLEMENTARY(u, surrogate2);
-                    return u;
-                }
+                if (U16_IS_TRAIL(surrogate2))
+                    return U16_GET_SUPPLEMENTARY(codeUnit, surrogate2);
             }
 
             restoreState(state);
         }
 
-        return u;
+        return codeUnit;
     }
 
     int tryConsumeIdentifierCharacter()
     {
-        int ch = peek();
+        if (tryConsume('\\'))
+            return tryConsumeUnicodeEscape<UnicodeParseContext::GroupName>();
 
-        if (ch == '\\') {
-            consume();
-            ch = tryConsumeUnicodeEscape();
-        } else
-            consume();
-
-        return ch;
+        return consumePossibleSurrogatePair<UnicodeParseContext::GroupName>();
     }
 
     bool isIdentifierStart(int ch)
@@ -989,14 +1086,13 @@ private:
         return n.hasOverflowed() ? quantifyInfinite : n.unsafeGet();
     }
 
-    unsigned consumeOctal()
+    // https://tc39.es/ecma262/#prod-annexB-LegacyOctalEscapeSequence
+    unsigned consumeOctal(unsigned count)
     {
-        ASSERT(WTF::isASCIIOctalDigit(peek()));
-
-        unsigned n = consumeDigit();
-        while (n < 32 && !atEndOfPattern() && WTF::isASCIIOctalDigit(peek()))
-            n = n * 8 + consumeDigit();
-        return n;
+        unsigned octal = 0;
+        while (count-- && octal < 32 && !atEndOfPattern() && WTF::isASCIIOctalDigit(peek()))
+            octal = octal * 8 + consumeDigit();
+        return octal;
     }
 
     bool tryConsume(UChar ch)
@@ -1033,7 +1129,7 @@ private:
 
         if (isIdentifierStart(ch)) {
             StringBuilder identifierBuilder;
-            identifierBuilder.append(ch);
+            identifierBuilder.appendCharacter(ch);
 
             while (!atEndOfPattern()) {
                 ch = tryConsumeIdentifierCharacter();
@@ -1043,7 +1139,7 @@ private:
                 if (!isIdentifierPart(ch))
                     break;
 
-                identifierBuilder.append(ch);
+                identifierBuilder.appendCharacter(ch);
             }
         }
 
@@ -1064,7 +1160,7 @@ private:
         bool foundEquals = false;
         unsigned errors = 0;
 
-        expressionBuilder.append(consume());
+        expressionBuilder.appendCharacter(consume());
 
         while (!atEndOfPattern()) {
             int ch = peek();
@@ -1099,25 +1195,32 @@ private:
             } else if (!isUnicodePropertyValueExpressionChar(ch))
                 errors++;
             else
-                expressionBuilder.append(ch);
+                expressionBuilder.appendCharacter(ch);
         }
 
         m_errorCode = ErrorCode::InvalidUnicodePropertyExpression;
         return WTF::nullopt;
     }
 
+    enum class ParenthesesType : uint8_t { Subpattern, Assertion };
+
     Delegate& m_delegate;
-    unsigned m_backReferenceLimit;
     ErrorCode m_errorCode { ErrorCode::NoError };
     const CharType* m_data;
     unsigned m_size;
     unsigned m_index { 0 };
     bool m_isUnicode;
-    unsigned m_parenthesesNestingDepth { 0 };
+    unsigned m_backReferenceLimit;
+    unsigned m_numSubpatterns { 0 };
+    unsigned m_maxSeenBackReference { 0 };
+    bool m_isNamedForwardReferenceAllowed;
+    bool m_kIdentityEscapeSeen { false };
+    Vector<ParenthesesType, 16> m_parenthesesStack;
     HashSet<String> m_captureGroupNames;
+    HashSet<String> m_forwardReferenceNames;
 
     // Derived by empirical testing of compile time in PCRE and WREC.
-    static const unsigned MAX_PATTERN_SIZE = 1024 * 1024;
+    static constexpr unsigned MAX_PATTERN_SIZE = 1024 * 1024;
 };
 
 /*
@@ -1146,13 +1249,14 @@ private:
  *    void atomParenthesesEnd();
  *    void atomBackReference(unsigned subpatternId);
  *    void atomNamedBackReference(const String& subpatternName);
- *    bool isValidNamedForwardReference(const String& subpatternName);
  *    void atomNamedForwardReference(const String& subpatternName);
  *
  *    void quantifyAtom(unsigned min, unsigned max, bool greedy);
  *
  *    void disjunction();
  *
+ *    void resetForReparsing();
+ *
  * The regular expression is described by a sequence of assertion*() and atom*()
  * callbacks to the delegate, describing the terms in the regular expression.
  * Following an atom a quantifyAtom() call may occur to indicate that the previous
@@ -1183,11 +1287,11 @@ private:
  */
 
 template<class Delegate>
-ErrorCode parse(Delegate& delegate, const String& pattern, bool isUnicode, unsigned backReferenceLimit = quantifyInfinite)
+ErrorCode parse(Delegate& delegate, const String& pattern, bool isUnicode, unsigned backReferenceLimit = quantifyInfinite, bool isNamedForwardReferenceAllowed = true)
 {
     if (pattern.is8Bit())
-        return Parser<Delegate, LChar>(delegate, pattern, isUnicode, backReferenceLimit).parse();
-    return Parser<Delegate, UChar>(delegate, pattern, isUnicode, backReferenceLimit).parse();
+        return Parser<Delegate, LChar>(delegate, pattern, isUnicode, backReferenceLimit, isNamedForwardReferenceAllowed).parse();
+    return Parser<Delegate, UChar>(delegate, pattern, isUnicode, backReferenceLimit, isNamedForwardReferenceAllowed).parse();
 }
 
 } } // namespace JSC::Yarr
