@@ -283,11 +283,11 @@ struct _xmlParserCtxt {
     void *            *pushTab;       /* array of data for push */
     xmlHashTablePtr    attsDefault;   /* defaulted attributes if any */
     xmlHashTablePtr    attsSpecial;   /* non-CDATA attributes if any */
-    int                nsWellFormed;  /* is the document XML Nanespace okay */
+    int                nsWellFormed;  /* is the document XML Namespace okay */
     int                options;       /* Extra options */
 
     /*
-     * Those fields are needed only for treaming parsing so far
+     * Those fields are needed only for streaming parsing so far
      */
     int               dictNames;    /* Use dictionary names for the tree */
     int               freeElemsNr;  /* number of freed element nodes */
@@ -894,12 +894,12 @@ XMLPUBFUN xmlDocPtr XMLCALL
 XMLPUBFUN xmlDocPtr XMLCALL
         xmlSAXParseMemory   (xmlSAXHandlerPtr sax,
                      const char *buffer,
-                                     int size,
+                     int size,
                      int recovery);
 XMLPUBFUN xmlDocPtr XMLCALL
         xmlSAXParseMemoryWithData (xmlSAXHandlerPtr sax,
                      const char *buffer,
-                                     int size,
+                     int size,
                      int recovery,
                      void *data);
 XMLPUBFUN xmlDocPtr XMLCALL
@@ -1097,7 +1097,7 @@ typedef enum {
     XML_PARSE_PEDANTIC  = 1<<7, /* pedantic error reporting */
     XML_PARSE_NOBLANKS  = 1<<8, /* remove blank nodes */
     XML_PARSE_SAX1  = 1<<9, /* use the SAX1 interface internally */
-    XML_PARSE_XINCLUDE  = 1<<10,/* Implement XInclude substitition  */
+    XML_PARSE_XINCLUDE  = 1<<10,/* Implement XInclude substitution  */
     XML_PARSE_NONET = 1<<11,/* Forbid network access */
     XML_PARSE_NODICT    = 1<<12,/* Do not reuse the context dictionary */
     XML_PARSE_NSCLEAN   = 1<<13,/* remove redundant namespaces declarations */
@@ -1191,7 +1191,7 @@ XMLPUBFUN xmlDocPtr XMLCALL
 /**
  * xmlFeature:
  *
- * Used to examine the existance of features that can be enabled
+ * Used to examine the existence of features that can be enabled
  * or disabled at compile-time.
  * They used to be called XML_FEATURE_xxx but this clashed with Expat
  */
