@@ -26,12 +26,13 @@
 #pragma once
 
 #include <wtf/MonotonicTime.h>
-#include <wtf/Optional.h>
 #include <wtf/Seconds.h>
 
 namespace WTF {
 
 struct CPUTime {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
+
     MonotonicTime cpuTime;
     Seconds userTime;
     Seconds systemTime;
