@@ -181,7 +181,7 @@ template <typename Target, typename Source> struct BoundsChecker<Target, Source,
     {
         // When converting value to unsigned Source, value will become a big value if value is negative.
         // Casted value will become bigger than Target::max as Source is bigger than Target.
-        return static_cast<typename std::make_unsigned<Source>::type>(value) <= std::numeric_limits<Target>::max();
+        return static_cast<std::make_unsigned_t<Source>>(value) <= std::numeric_limits<Target>::max();
     }
 };
 
@@ -242,22 +242,22 @@ template <typename Target, typename Source> static inline bool convertSafely(Sou
 
 template <typename T> struct RemoveChecked {
     typedef T CleanType;
-    static const CleanType DefaultValue = 0;
+    static constexpr CleanType DefaultValue = 0;
 };
 
 template <typename T> struct RemoveChecked<Checked<T, ConditionalCrashOnOverflow>> {
     using CleanType = typename RemoveChecked<T>::CleanType;
-    static const CleanType DefaultValue = 0;
+    static constexpr CleanType DefaultValue = 0;
 };
 
 template <typename T> struct RemoveChecked<Checked<T, CrashOnOverflow>> {
     typedef typename RemoveChecked<T>::CleanType CleanType;
-    static const CleanType DefaultValue = 0;
+    static constexpr CleanType DefaultValue = 0;
 };
 
 template <typename T> struct RemoveChecked<Checked<T, RecordOverflow>> {
     typedef typename RemoveChecked<T>::CleanType CleanType;
-    static const CleanType DefaultValue = 0;
+    static constexpr CleanType DefaultValue = 0;
 };
 
 // The ResultBase and SignednessSelector are used to workaround typeof not being
@@ -545,7 +545,7 @@ template <class OverflowHandler, typename = std::enable_if_t<!std::is_scalar<Ove
 inline constexpr bool observesOverflow() { return true; }
 
 template <>
-inline constexpr bool observesOverflow<AssertNoOverflow>() { return !ASSERT_DISABLED; }
+inline constexpr bool observesOverflow<AssertNoOverflow>() { return ASSERT_ENABLED; }
 
 template <typename U, typename V, typename R> static inline bool safeAdd(U lhs, V rhs, R& result)
 {
@@ -611,6 +611,13 @@ public:
         this->overflowed();
     }
 
+    Checked(const Checked& value)
+    {
+        if (value.hasOverflowed())
+            this->overflowed();
+        m_value = static_cast<T>(value.m_value);
+    }
+
     template <typename U> Checked(U value)
     {
         if (!isInBounds<T>(value))
