@@ -45,8 +45,18 @@ private:
         HashFunctions, ValueTraits, ValueTraits> HashTableType;
 
 public:
+    /*
+     * Since figuring out the entries of an iterator is confusing, here is a cheat sheet:
+     * const KeyType& key = iterator->key;
+     */
     typedef HashTableConstIteratorAdapter<HashTableType, ValueType> iterator;
     typedef HashTableConstIteratorAdapter<HashTableType, ValueType> const_iterator;
+
+    /*
+     * Since figuring out the entries of an AddResult is confusing, here is a cheat sheet:
+     * iterator iter = addResult.iterator;
+     * bool isNewEntry = addResult.isNewEntry;
+     */
     typedef typename HashTableType::AddResult AddResult;
 
     HashSet()
@@ -127,6 +137,8 @@ public:
     template<typename OtherCollection>
     bool operator!=(const OtherCollection&) const;
 
+    void checkConsistency() const;
+
 private:
     HashTableType m_impl;
 };
@@ -382,6 +394,12 @@ void HashSet<T, U, V>::add(std::initializer_list<std::reference_wrapper<const Va
         add(value);
 }
 
+template<typename T, typename U, typename V>
+inline void HashSet<T, U, V>::checkConsistency() const
+{
+    m_impl.checkTableConsistency();
+}
+
 } // namespace WTF
 
 using WTF::HashSet;
