@@ -29,6 +29,9 @@
 
 namespace WTF {
 
+template<typename KeyTypeArg, typename ValueTypeArg>
+struct KeyValuePair;
+
 // IndexSparseSet is an efficient set of integers that can only be valued
 // between zero and size() - 1.
 //
@@ -73,6 +76,7 @@ struct DefaultIndexSparseSetTraits<KeyValuePair<KeyType, ValueType>> {
 
 template<typename EntryType = unsigned, typename EntryTypeTraits = DefaultIndexSparseSetTraits<EntryType>, typename OverflowHandler = CrashOnOverflow>
 class IndexSparseSet {
+    WTF_MAKE_FAST_ALLOCATED;
     typedef Vector<EntryType, 0, OverflowHandler> ValueList;
 public:
     explicit IndexSparseSet(unsigned size);
@@ -96,6 +100,8 @@ public:
 
     void sort();
 
+    void validate();
+
     const ValueList& values() const { return m_values; }
 
 private:
@@ -144,7 +150,7 @@ inline bool IndexSparseSet<EntryType, EntryTypeTraits, OverflowHandler>::remove(
     if (position >= m_values.size())
         return false;
 
-    if (m_values[position] == value) {
+    if (EntryTypeTraits::key(m_values[position]) == value) {
         EntryType lastValue = m_values.last();
         m_values[position] = WTFMove(lastValue);
         m_map[EntryTypeTraits::key(lastValue)] = position;
@@ -211,6 +217,24 @@ void IndexSparseSet<EntryType, EntryTypeTraits, OverflowHandler>::sort()
         [&] (const EntryType& a, const EntryType& b) {
             return EntryTypeTraits::key(a) < EntryTypeTraits::key(b);
         });
+
+    // Bring m_map back in sync with m_values
+    for (unsigned index = 0; index < m_values.size(); ++index) {
+        unsigned key = EntryTypeTraits::key(m_values[index]);
+        m_map[key] = index;
+    }
+
+#if ASSERT_ENABLED
+    validate();
+#endif
+}
+
+template<typename EntryType, typename EntryTypeTraits, typename OverflowHandler>
+void IndexSparseSet<EntryType, EntryTypeTraits, OverflowHandler>::validate()
+{
+    RELEASE_ASSERT(m_values.size() <= m_map.size());
+    for (const EntryType& entry : *this)
+        RELEASE_ASSERT(contains(EntryTypeTraits::key(entry)));
 }
 
 template<typename EntryType, typename EntryTypeTraits, typename OverflowHandler>
