@@ -59,9 +59,10 @@ void removeLanguageChangeObserver(void* context)
 
 void languageDidChange()
 {
-    ObserverMap::iterator end = observerMap().end();
-    for (ObserverMap::iterator iter = observerMap().begin(); iter != end; ++iter)
-        iter->value(iter->key);
+    for (auto& observer : copyToVector(observerMap())) {
+        if (observerMap().contains(observer.key))
+            observer.value(observer.key);
+    }
 }
 
 String defaultLanguage()
