@@ -28,12 +28,13 @@
 #include <wtf/DataLog.h>
 #include <wtf/HashMap.h>
 #include <wtf/LoggingHashID.h>
+#include <wtf/LoggingHashTraits.h>
 
 namespace WTF {
 
 template<
     const char* typeArguments,
-    typename KeyArg, typename MappedArg, typename HashArg = typename DefaultHash<KeyArg>::Hash,
+    typename KeyArg, typename MappedArg, typename HashArg = DefaultHash<KeyArg>,
     typename KeyTraitsArg = HashTraits<KeyArg>, typename MappedTraitsArg = HashTraits<MappedArg>,
     typename LoggingKeyTraits = LoggingHashKeyTraits<KeyArg>,
     typename LoggingValueTraits = LoggingHashValueTraits<MappedArg>>
@@ -51,6 +52,11 @@ public:
     typedef typename HashMap::const_iterator const_iterator;
     typedef typename HashMap::AddResult AddResult;
 
+private:
+    typedef typename HashMap::MappedTraits::PeekType MappedPeekType;
+
+public:
+
     LoggingHashMap()
     {
         dataLog("auto* ", m_id, " = new HashMap<", typeArguments, ">();\n");
@@ -105,9 +111,9 @@ public:
     const_iterator random() const { return m_map.random(); }
 
     auto keys() { return m_map.keys(); }
-    const auto keys() const { return m_map.keys(); }
+    auto keys() const { return m_map.keys(); }
     auto values() { return m_map.values(); }
-    const auto values() const { return m_map.values(); }
+    auto values() const { return m_map.values(); }
 
     iterator find(const KeyType& key)
     {
