@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,12 +29,13 @@
 #include "config.h"
 #include <wtf/MetaAllocator.h>
 
-#include <wtf/DataLog.h>
-#include <wtf/FastMalloc.h>
-#include <wtf/ProcessID.h>
-
 namespace WTF {
 
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(MetaAllocatorHandle);
+
+DECLARE_ALLOCATOR_WITH_HEAP_IDENTIFIER(MetaAllocatorFreeSpace);
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(MetaAllocatorFreeSpace);
+
 MetaAllocator::~MetaAllocator()
 {
     for (FreeSpaceNode* node = m_freeSpaceSizeMap.first(); node;) {
@@ -48,22 +49,21 @@ MetaAllocator::~MetaAllocator()
 #endif
 }
 
-void MetaAllocatorTracker::notify(MetaAllocatorHandle* handle)
+void MetaAllocatorTracker::notify(MetaAllocatorHandle& handle)
 {
-    m_allocations.insert(handle);
+    m_allocations.insert(&handle);
 }
 
-void MetaAllocatorTracker::release(MetaAllocatorHandle* handle)
+void MetaAllocatorTracker::release(MetaAllocatorHandle& handle)
 {
-    m_allocations.remove(handle);
+    m_allocations.remove(&handle);
 }
 
-ALWAYS_INLINE void MetaAllocator::release(MetaAllocatorHandle* handle)
+void MetaAllocator::release(const LockHolder&, MetaAllocatorHandle& handle)
 {
-    LockHolder locker(&m_lock);
-    if (handle->sizeInBytes()) {
-        void* start = handle->start().untaggedPtr();
-        size_t sizeInBytes = handle->sizeInBytes();
+    if (handle.sizeInBytes()) {
+        void* start = handle.start().untaggedPtr();
+        size_t sizeInBytes = handle.sizeInBytes();
         decrementPageOccupancy(start, sizeInBytes);
         addFreeSpaceFromReleasedHandle(FreeSpacePtr(start), sizeInBytes);
     }
@@ -72,21 +72,19 @@ ALWAYS_INLINE void MetaAllocator::release(MetaAllocatorHandle* handle)
         m_tracker->release(handle);
 }
 
-MetaAllocatorHandle::MetaAllocatorHandle(MetaAllocator* allocator, void* start, size_t sizeInBytes, void* ownerUID)
+MetaAllocatorHandle::MetaAllocatorHandle(MetaAllocator& allocator, void* start, size_t sizeInBytes)
     : m_allocator(allocator)
     , m_start(start)
     , m_end(reinterpret_cast<char*>(start) + sizeInBytes)
-    , m_ownerUID(ownerUID)
 {
-    ASSERT(allocator);
     ASSERT(start);
     ASSERT(sizeInBytes);
 }
 
 MetaAllocatorHandle::~MetaAllocatorHandle()
 {
-    ASSERT(m_allocator);
-    m_allocator->release(this);
+    auto locker = holdLock(allocator().m_lock);
+    allocator().release(locker, *this);
 }
 
 void MetaAllocatorHandle::shrink(size_t newSizeInBytes)
@@ -94,9 +92,10 @@ void MetaAllocatorHandle::shrink(size_t newSizeInBytes)
     size_t sizeInBytes = this->sizeInBytes();
     ASSERT(newSizeInBytes <= sizeInBytes);
 
-    LockHolder locker(&m_allocator->m_lock);
+    MetaAllocator& allocator = this->allocator();
+    LockHolder locker(&allocator.m_lock);
 
-    newSizeInBytes = m_allocator->roundUp(newSizeInBytes);
+    newSizeInBytes = allocator.roundUp(newSizeInBytes);
 
     ASSERT(newSizeInBytes <= sizeInBytes);
 
@@ -107,11 +106,11 @@ void MetaAllocatorHandle::shrink(size_t newSizeInBytes)
     size_t freeSize = sizeInBytes - newSizeInBytes;
     uintptr_t freeEnd = freeStart + freeSize;
 
-    uintptr_t firstCompletelyFreePage = (freeStart + m_allocator->m_pageSize - 1) & ~(m_allocator->m_pageSize - 1);
+    uintptr_t firstCompletelyFreePage = (freeStart + allocator.m_pageSize - 1) & ~(allocator.m_pageSize - 1);
     if (firstCompletelyFreePage < freeEnd)
-        m_allocator->decrementPageOccupancy(reinterpret_cast<void*>(firstCompletelyFreePage), freeSize - (firstCompletelyFreePage - freeStart));
+        allocator.decrementPageOccupancy(reinterpret_cast<void*>(firstCompletelyFreePage), freeSize - (firstCompletelyFreePage - freeStart));
 
-    m_allocator->addFreeSpaceFromReleasedHandle(MetaAllocator::FreeSpacePtr(freeStart), freeSize);
+    allocator.addFreeSpaceFromReleasedHandle(MetaAllocator::FreeSpacePtr(freeStart), freeSize);
 
     m_end = m_start + newSizeInBytes;
 }
@@ -121,13 +120,13 @@ void MetaAllocatorHandle::dump(PrintStream& out) const
     out.print(RawPointer(start().untaggedPtr()), "...", RawPointer(end().untaggedPtr()));
 }
 
-MetaAllocator::MetaAllocator(size_t allocationGranule, size_t pageSize)
+MetaAllocator::MetaAllocator(Lock& lock, size_t allocationGranule, size_t pageSize)
     : m_allocationGranule(allocationGranule)
     , m_pageSize(pageSize)
     , m_bytesAllocated(0)
     , m_bytesReserved(0)
     , m_bytesCommitted(0)
-    , m_tracker(0)
+    , m_lock(lock)
 #ifndef NDEBUG
     , m_mallocBalance(0)
 #endif
@@ -151,10 +150,8 @@ MetaAllocator::MetaAllocator(size_t allocationGranule, size_t pageSize)
     ASSERT(static_cast<size_t>(1) << m_logAllocationGranule == m_allocationGranule);
 }
 
-RefPtr<MetaAllocatorHandle> MetaAllocator::allocate(size_t sizeInBytes, void* ownerUID)
+RefPtr<MetaAllocatorHandle> MetaAllocator::allocate(const LockHolder&, size_t sizeInBytes)
 {
-    LockHolder locker(&m_lock);
-
     if (!sizeInBytes)
         return nullptr;
 
@@ -189,17 +186,16 @@ RefPtr<MetaAllocatorHandle> MetaAllocator::allocate(size_t sizeInBytes, void* ow
     m_numAllocations++;
 #endif
 
-    auto handle = adoptRef(*new MetaAllocatorHandle(this, start.untaggedPtr(), sizeInBytes, ownerUID));
+    auto handle = adoptRef(*new MetaAllocatorHandle(*this, start.untaggedPtr(), sizeInBytes));
 
     if (UNLIKELY(!!m_tracker))
-        m_tracker->notify(handle.ptr());
+        m_tracker->notify(*handle.ptr());
 
-    return WTFMove(handle);
+    return handle;
 }
 
-MetaAllocator::Statistics MetaAllocator::currentStatistics()
+MetaAllocator::Statistics MetaAllocator::currentStatistics(const LockHolder&)
 {
-    LockHolder locker(&m_lock);
     Statistics result;
     result.bytesAllocated = m_bytesAllocated;
     result.bytesReserved = m_bytesReserved;
@@ -212,7 +208,7 @@ MetaAllocator::FreeSpacePtr MetaAllocator::findAndRemoveFreeSpace(size_t sizeInB
     FreeSpaceNode* node = m_freeSpaceSizeMap.findLeastGreaterThanOrEqual(sizeInBytes);
 
     if (!node)
-        return 0;
+        return nullptr;
 
     size_t nodeSizeInBytes = node->sizeInBytes();
     ASSERT(nodeSizeInBytes >= sizeInBytes);
@@ -398,15 +394,29 @@ void MetaAllocator::incrementPageOccupancy(void* address, size_t sizeInBytes)
     uintptr_t firstPage = reinterpret_cast<uintptr_t>(address) >> m_logPageSize;
     uintptr_t lastPage = (reinterpret_cast<uintptr_t>(address) + sizeInBytes - 1) >> m_logPageSize;
 
+    uintptr_t currentPageStart = 0;
+    size_t count = 0;
+    auto flushNeedPages = [&] {
+        if (!currentPageStart)
+            return;
+        notifyNeedPage(reinterpret_cast<void*>(currentPageStart << m_logPageSize), count);
+        currentPageStart = 0;
+        count = 0;
+    };
+
     for (uintptr_t page = firstPage; page <= lastPage; ++page) {
-        HashMap<uintptr_t, size_t>::iterator iter = m_pageOccupancyMap.find(page);
-        if (iter == m_pageOccupancyMap.end()) {
-            m_pageOccupancyMap.add(page, 1);
+        auto result = m_pageOccupancyMap.add(page, 1);
+        if (result.isNewEntry) {
             m_bytesCommitted += m_pageSize;
-            notifyNeedPage(reinterpret_cast<void*>(page << m_logPageSize));
-        } else
-            iter->value++;
+            if (!currentPageStart)
+                currentPageStart = page;
+            ++count;
+        } else {
+            result.iterator->value++;
+            flushNeedPages();
+        }
     }
+    flushNeedPages();
 }
 
 void MetaAllocator::decrementPageOccupancy(void* address, size_t sizeInBytes)
@@ -414,15 +424,29 @@ void MetaAllocator::decrementPageOccupancy(void* address, size_t sizeInBytes)
     uintptr_t firstPage = reinterpret_cast<uintptr_t>(address) >> m_logPageSize;
     uintptr_t lastPage = (reinterpret_cast<uintptr_t>(address) + sizeInBytes - 1) >> m_logPageSize;
 
+    uintptr_t currentPageStart = 0;
+    size_t count = 0;
+    auto flushFreePages = [&] {
+        if (!currentPageStart)
+            return;
+        notifyPageIsFree(reinterpret_cast<void*>(currentPageStart << m_logPageSize), count);
+        currentPageStart = 0;
+        count = 0;
+    };
+
     for (uintptr_t page = firstPage; page <= lastPage; ++page) {
         HashMap<uintptr_t, size_t>::iterator iter = m_pageOccupancyMap.find(page);
         ASSERT(iter != m_pageOccupancyMap.end());
         if (!--(iter->value)) {
             m_pageOccupancyMap.remove(iter);
             m_bytesCommitted -= m_pageSize;
-            notifyPageIsFree(reinterpret_cast<void*>(page << m_logPageSize));
-        }
+            if (!currentPageStart)
+                currentPageStart = page;
+            ++count;
+        } else
+            flushFreePages();
     }
+    flushFreePages();
 }
 
 bool MetaAllocator::isInAllocatedMemory(const AbstractLocker&, void* address)
@@ -444,7 +468,7 @@ MetaAllocator::FreeSpaceNode* MetaAllocator::allocFreeSpaceNode()
 #ifndef NDEBUG
     m_mallocBalance++;
 #endif
-    return new (NotNull, fastMalloc(sizeof(FreeSpaceNode))) FreeSpaceNode();
+    return new (NotNull, MetaAllocatorFreeSpaceMalloc::malloc(sizeof(FreeSpaceNode))) FreeSpaceNode();
 }
 
 void MetaAllocator::freeFreeSpaceNode(FreeSpaceNode* node)
@@ -452,7 +476,7 @@ void MetaAllocator::freeFreeSpaceNode(FreeSpaceNode* node)
 #ifndef NDEBUG
     m_mallocBalance--;
 #endif
-    fastFree(node);
+    MetaAllocatorFreeSpaceMalloc::free(node);
 }
 
 #if ENABLE(META_ALLOCATOR_PROFILE)
