@@ -355,7 +355,8 @@ void ensureHashtableSize(unsigned numThreads)
     // Check again, since the hashtable could have rehashed while we were locking it. Also,
     // lockHashtable() creates an initial hashtable for us.
     oldHashtable = hashtable.load();
-    if (oldHashtable && static_cast<double>(oldHashtable->size) / static_cast<double>(numThreads) >= maxLoadFactor) {
+    RELEASE_ASSERT(oldHashtable);
+    if (static_cast<double>(oldHashtable->size) / static_cast<double>(numThreads) >= maxLoadFactor) {
         if (verbose)
             dataLog(toString(Thread::current(), ": after locking, no need to rehash because ", oldHashtable->size, " / ", numThreads, " >= ", maxLoadFactor, "\n"));
         unlockHashtable(bucketsToUnlock);
