@@ -29,11 +29,8 @@
 #include <wtf/RandomNumber.h>
 
 #include <limits>
-#include <limits.h>
 #include <stdint.h>
-#include <stdlib.h>
 #include <wtf/CryptographicallyRandomNumber.h>
-#include <wtf/RandomNumberSeed.h>
 #include <wtf/WeakRandom.h>
 
 namespace WTF {
