@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2018 Apple Inc. All rights reserved.
+ *  Copyright (C) 2005-2019 Apple Inc. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -48,6 +48,10 @@
 #define NS_RELEASES_ARGUMENT
 #endif
 
+#ifndef __OBJC__
+typedef struct objc_object *id;
+#endif
+
 namespace WTF {
 
 // Unlike most most of our smart pointers, RetainPtr can take either the pointer type or the pointed-to type,
@@ -336,15 +340,13 @@ template<typename T> inline RetainPtr<typename RetainPtr<T>::HelperPtrType> reta
 }
 
 template <typename T> struct IsSmartPtr<RetainPtr<T>> {
-    static const bool value = true;
+    static constexpr bool value = true;
 };
 
 template<typename P> struct HashTraits<RetainPtr<P>> : SimpleClassHashTraits<RetainPtr<P>> {
 };
 
-template<typename P> struct DefaultHash<RetainPtr<P>> {
-    typedef PtrHash<RetainPtr<P>> Hash;
-};
+template<typename P> struct DefaultHash<RetainPtr<P>> : PtrHash<RetainPtr<P>> { };
 
 template <typename P>
 struct RetainPtrObjectHashTraits : SimpleClassHashTraits<RetainPtr<P>> {
@@ -366,9 +368,19 @@ struct RetainPtrObjectHash {
     {
         return CFEqual(a.get(), b.get());
     }
-    static const bool safeToCompareToEmptyOrDeleted = false;
+    static constexpr bool safeToCompareToEmptyOrDeleted = false;
 };
 
+inline bool safeCFEqual(CFTypeRef a, CFTypeRef b)
+{
+    return (!a && !b) || (a && b && CFEqual(a, b));
+}
+
+inline CFHashCode safeCFHash(CFTypeRef a)
+{
+    return a ? CFHash(a) : 0;
+}
+
 #ifdef __OBJC__
 template<typename T> T* dynamic_objc_cast(id object)
 {
