@@ -32,6 +32,7 @@
 #include <wtf/PrintStream.h>
 #include <wtf/TimeWithDynamicClockType.h>
 #include <wtf/WallTime.h>
+#include <wtf/text/TextStream.h>
 
 namespace WTF {
 
@@ -70,6 +71,12 @@ void Seconds::dump(PrintStream& out) const
     out.print(m_value, " sec");
 }
 
+TextStream& operator<<(TextStream& ts, Seconds seconds)
+{
+    ts << seconds.value() << "s";
+    return ts;
+}
+
 void sleep(Seconds value)
 {
     // It's very challenging to find portable ways of sleeping for less than a second. On UNIX, you want to
