@@ -32,8 +32,11 @@
 
 namespace WTF {
 
+DECLARE_ALLOCATOR_WITH_HEAP_IDENTIFIER(SmallPtrSet);
+
 template<typename PtrType, unsigned SmallArraySize = 8>
 class SmallPtrSet {
+    WTF_MAKE_FAST_ALLOCATED;
     WTF_MAKE_NONCOPYABLE(SmallPtrSet);
     static_assert(std::is_trivially_destructible<PtrType>::value, "We currently don't support non-trivially destructible pointer types.");
     static_assert(sizeof(PtrType) == sizeof(void*), "Only support pointer sized things.");
@@ -71,7 +74,7 @@ public:
     ~SmallPtrSet()
     {
         if (!isSmall())
-            fastFree(m_buffer);
+            SmallPtrSetMalloc::free(m_buffer);
     }
 
     inline void add(PtrType ptr)
@@ -123,6 +126,7 @@ public:
     }
 
     class iterator {
+        WTF_MAKE_FAST_ALLOCATED;
     public:
         iterator& operator++()
         {
@@ -207,7 +211,7 @@ private:
         bool wasSmall = isSmall();
         void** oldBuffer = wasSmall ? m_smallStorage : m_buffer;
         unsigned oldCapacity = m_capacity;
-        m_buffer = static_cast<void**>(fastMalloc(allocationSize));
+        m_buffer = static_cast<void**>(SmallPtrSetMalloc::malloc(allocationSize));
         memset(m_buffer, -1, allocationSize);
         m_capacity = size;
 
@@ -219,7 +223,7 @@ private:
         }
 
         if (!wasSmall)
-            fastFree(oldBuffer);
+            SmallPtrSetMalloc::free(oldBuffer);
     }
 
 
