@@ -33,6 +33,7 @@ namespace WTF {
 
 template<typename T, typename CounterType = unsigned>
 class Spectrum {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     typedef typename HashMap<T, CounterType>::iterator iterator;
     typedef typename HashMap<T, CounterType>::const_iterator const_iterator;
@@ -41,6 +42,7 @@ public:
 
     void add(const T& key, CounterType count = 1)
     {
+        Locker locker(m_lock);
         if (!count)
             return;
         typename HashMap<T, CounterType>::AddResult result = m_map.add(key, count);
@@ -57,6 +59,7 @@ public:
 
     CounterType get(const T& key) const
     {
+        Locker locker(m_lock);
         const_iterator iter = m_map.find(key);
         if (iter == m_map.end())
             return 0;
@@ -65,12 +68,11 @@ public:
 
     size_t size() const { return m_map.size(); }
 
-    iterator begin() { return m_map.begin(); }
-    iterator end() { return m_map.end(); }
     const_iterator begin() const { return m_map.begin(); }
     const_iterator end() const { return m_map.end(); }
 
     struct KeyAndCount {
+        WTF_MAKE_STRUCT_FAST_ALLOCATED;
         KeyAndCount() { }
 
         KeyAndCount(const T& key, CounterType count)
@@ -96,6 +98,7 @@ public:
     // Returns a list ordered from lowest-count to highest-count.
     Vector<KeyAndCount> buildList() const
     {
+        Locker locker(m_lock);
         Vector<KeyAndCount> list;
         for (const_iterator iter = begin(); iter != end(); ++iter)
             list.append(KeyAndCount(iter->key, iter->value));
@@ -104,17 +107,23 @@ public:
         return list;
     }
 
-    void clear() { m_map.clear(); }
+    void clear()
+    {
+        Locker locker(m_lock);
+        m_map.clear();
+    }
 
     template<typename Functor>
     void removeIf(const Functor& functor)
     {
+        Locker locker(m_lock);
         m_map.removeIf([&functor] (typename HashMap<T, CounterType>::KeyValuePairType& pair) {
                 return functor(KeyAndCount(pair.key, pair.value));
             });
     }
 
 private:
+    mutable Lock m_lock;
     HashMap<T, CounterType> m_map;
 };
 
