@@ -47,6 +47,8 @@ enum TracePointCode {
     WebAssemblyCompileEnd,
     WebAssemblyExecuteStart,
     WebAssemblyExecuteEnd,
+    DumpJITMemoryStart,
+    DumpJITMemoryStop,
 
     WebCoreRange = 5000,
     MainResourceLoadDidStartProvisional,
@@ -74,6 +76,16 @@ enum TracePointCode {
     DisplayListRecordStart,
     DisplayListRecordEnd,
     DisplayRefreshDispatchingToMainThread,
+    ComputeEventRegionsStart,
+    ComputeEventRegionsEnd,
+    ScheduleRenderingUpdate,
+    TriggerRenderingUpdate,
+    RenderingUpdateStart,
+    RenderingUpdateEnd,
+    CompositingUpdateStart,
+    CompositingUpdateEnd,
+    DispatchTouchEventsStart,
+    DispatchTouchEventsEnd,
 
     WebKitRange = 10000,
     WebHTMLViewPaintStart,
@@ -118,6 +130,7 @@ inline void tracePoint(TracePointCode code, uint64_t data1 = 0, uint64_t data2 =
 }
 
 class TraceScope {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
 
     TraceScope(TracePointCode entryCode, TracePointCode exitCode, uint64_t data1 = 0, uint64_t data2 = 0, uint64_t data3 = 0, uint64_t data4 = 0)
