@@ -33,7 +33,8 @@ namespace WTF {
 
 class PrintStream;
 
-class TimeWithDynamicClockType {
+class TimeWithDynamicClockType final {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     TimeWithDynamicClockType() { }
 
