@@ -26,9 +26,16 @@
 #include "config.h"
 
 #include <wtf/Bag.h>
+#include <wtf/Platform.h>
 #include <wtf/RefCountedArray.h>
 #include <wtf/RefPtr.h>
 
+#if OS(DARWIN)
+#include <mach/vm_types.h>
+#include <mach/vm_param.h>
+#include <mach/vm_types.h>
+#endif
+
 namespace WTF {
 
 namespace {
@@ -43,5 +50,9 @@ static_assert(sizeof(RefPtr<DummyStruct>) == sizeof(DummyStruct*), "");
 
 static_assert(sizeof(RefCountedArray<DummyStruct>) == sizeof(void*), "");
 
+#if OS(DARWIN) && CPU(ADDRESS64)
+static_assert(MACH_VM_MAX_ADDRESS <= ((1ULL << OS_CONSTANT(EFFECTIVE_ADDRESS_WIDTH)) - 1));
+#endif
+
 } // namespace WTF
 
