@@ -30,8 +30,6 @@
 #include "config.h"
 #include <wtf/WorkQueue.h>
 
-#include <wtf/WallTime.h>
-#include <wtf/text/WTFString.h>
 #include <wtf/threads/BinarySemaphore.h>
 
 #if PLATFORM(JAVA)
@@ -74,6 +72,19 @@ void WorkQueue::dispatch(Function<void()>&& function)
 void WorkQueue::dispatchAfter(Seconds delay, Function<void()>&& function)
 {
     RefPtr<WorkQueue> protect(this);
+#if OS(WINDOWS) && PLATFORM(JAVA)
+    // From empirical testing, we've seen CreateTimerQueueTimer() sometimes fire up to 5+ ms early.
+    // This causes havoc for clients of this code that expect to not be called back until the
+    // specified duration has expired. Other folks online have also observed some slop in the
+    // firing times of CreateTimerQuqueTimer(). From the data posted at
+    // http://omeg.pl/blog/2011/11/on-winapi-timers-and-their-resolution, it appears that the slop
+    // can be up to about 10 ms. To ensure that we don't fire the timer early, we'll tack on a
+    // slop adjustment to the duration, and we'll use double the worst amount of slop observed
+    // so far.
+    const Seconds slopAdjustment { 20_ms };
+    if (delay)
+        delay += slopAdjustment;
+#endif
     m_runLoop->dispatchAfter(delay, [protect, function = WTFMove(function)] {
 #if PLATFORM(JAVA)
         AttachThreadAsDaemonToJavaEnv autoAttach;
