@@ -41,6 +41,18 @@
 #define DYLD_IOS_VERSION_12_0 0x000C0000
 #endif
 
+#ifndef DYLD_IOS_VERSION_13_0
+#define DYLD_IOS_VERSION_13_0 0x000D0000
+#endif
+
+#ifndef DYLD_IOS_VERSION_13_2
+#define DYLD_IOS_VERSION_13_2 0x000D0200
+#endif
+
+#ifndef DYLD_IOS_VERSION_13_4
+#define DYLD_IOS_VERSION_13_4 0x000D0400
+#endif
+
 #ifndef DYLD_MACOSX_VERSION_10_13
 #define DYLD_MACOSX_VERSION_10_13 0x000A0D00
 #endif
@@ -49,6 +61,18 @@
 #define DYLD_MACOSX_VERSION_10_14 0x000A0E00
 #endif
 
+#ifndef DYLD_MACOSX_VERSION_10_15
+#define DYLD_MACOSX_VERSION_10_15 0x000A0F00
+#endif
+
+#ifndef DYLD_MACOSX_VERSION_10_15_1
+#define DYLD_MACOSX_VERSION_10_15_1 0x000A0F01
+#endif
+
+#ifndef DYLD_MACOSX_VERSION_10_15_4
+#define DYLD_MACOSX_VERSION_10_15_4 0x000A0F04
+#endif
+
 #else
 
 #define DYLD_IOS_VERSION_3_0 0x00030000
@@ -61,11 +85,17 @@
 #define DYLD_IOS_VERSION_11_0 0x000B0000
 #define DYLD_IOS_VERSION_11_3 0x000B0300
 #define DYLD_IOS_VERSION_12_0 0x000C0000
+#define DYLD_IOS_VERSION_13_0 0x000D0000
+#define DYLD_IOS_VERSION_13_2 0x000D0200
+#define DYLD_IOS_VERSION_13_4 0x000D0400
 
 #define DYLD_MACOSX_VERSION_10_11 0x000A0B00
 #define DYLD_MACOSX_VERSION_10_12 0x000A0C00
 #define DYLD_MACOSX_VERSION_10_13 0x000A0D00
 #define DYLD_MACOSX_VERSION_10_14 0x000A0E00
+#define DYLD_MACOSX_VERSION_10_15 0x000A0F00
+#define DYLD_MACOSX_VERSION_10_15_1 0x000A0F01
+#define DYLD_MACOSX_VERSION_10_15_4 0x000A0F04
 
 #endif
 
