@@ -26,6 +26,10 @@
 #include <wtf/text/IntegerToStringConversion.h>
 #include <wtf/text/WTFString.h>
 
+#if OS(WINDOWS)
+#include <wtf/text/win/WCharStringExtras.h>
+#endif
+
 // Define 'NO_IMPLICIT_ATOMICSTRING' before including this header,
 // to disallow (expensive) implicit String-->AtomicString conversions.
 #ifdef NO_IMPLICIT_ATOMICSTRING
@@ -107,6 +111,7 @@ public:
     WTF_EXPORT_PRIVATE static AtomicString number(unsigned);
     WTF_EXPORT_PRIVATE static AtomicString number(unsigned long);
     WTF_EXPORT_PRIVATE static AtomicString number(unsigned long long);
+    WTF_EXPORT_PRIVATE static AtomicString number(float);
     WTF_EXPORT_PRIVATE static AtomicString number(double);
     // If we need more overloads of the number function, we can add all the others that String has, but these seem to do for now.
 
@@ -152,6 +157,14 @@ public:
     operator NSString*() const { return m_string; }
 #endif
 
+#if OS(WINDOWS) && U_ICU_VERSION_MAJOR_NUM >= 59 && !PLATFORM(JAVA)
+    AtomicString(const wchar_t* characters, unsigned length)
+        : AtomicString(ucharFrom(characters), length) { }
+
+    AtomicString(const wchar_t* characters)
+        : AtomicString(ucharFrom(characters)) { }
+#endif
+
     // AtomicString::fromUTF8 will return a null string if the input data contains invalid UTF-8 sequences.
     static AtomicString fromUTF8(const char*, size_t);
     static AtomicString fromUTF8(const char*);
