@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2003-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,9 +32,12 @@
 
 namespace WTF {
 
+DECLARE_ALLOCATOR_WITH_HEAP_IDENTIFIER(CStringBuffer);
+
 // CStringBuffer is the ref-counted storage class for the characters in a CString.
 // The data is implicitly allocated 1 character longer than length(), as it is zero-terminated.
-class CStringBuffer : public RefCounted<CStringBuffer> {
+class CStringBuffer final : public RefCounted<CStringBuffer> {
+    WTF_MAKE_FAST_ALLOCATED_WITH_HEAP_IDENTIFIER(CStringBuffer);
 public:
     const char* data() { return mutableData(); }
     size_t length() const { return m_length; }
@@ -52,7 +55,8 @@ private:
 
 // A container for a null-terminated char array supporting copy-on-write
 // assignment.  The contained char array may be null.
-class CString {
+class CString final {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     CString() { }
     WTF_EXPORT_PRIVATE CString(const char*);
@@ -63,7 +67,7 @@ public:
 
     const char* data() const
     {
-        return m_buffer ? m_buffer->data() : 0;
+        return m_buffer ? m_buffer->data() : nullptr;
     }
     WTF_EXPORT_PRIVATE char* mutableData();
     size_t length() const
@@ -80,6 +84,9 @@ public:
 
     WTF_EXPORT_PRIVATE unsigned hash() const;
 
+    // Useful if you want your CString to hold dynamic data.
+    WTF_EXPORT_PRIVATE void grow(size_t newLength);
+
 private:
     void copyBufferIfNeeded();
     void init(const char*, size_t length);
@@ -95,13 +102,11 @@ WTF_EXPORT_PRIVATE bool operator<(const CString& a, const CString& b);
 struct CStringHash {
     static unsigned hash(const CString& string) { return string.hash(); }
     WTF_EXPORT_PRIVATE static bool equal(const CString& a, const CString& b);
-    static const bool safeToCompareToEmptyOrDeleted = true;
+    static constexpr bool safeToCompareToEmptyOrDeleted = true;
 };
 
 template<typename T> struct DefaultHash;
-template<> struct DefaultHash<CString> {
-    typedef CStringHash Hash;
-};
+template<> struct DefaultHash<CString> : CStringHash { };
 
 template<typename T> struct HashTraits;
 template<> struct HashTraits<CString> : SimpleClassHashTraits<CString> { };
