@@ -31,6 +31,7 @@
 #include <wtf/Optional.h>
 #include <wtf/RetainPtr.h>
 #include <wtf/Vector.h>
+#include <wtf/text/ASCIILiteral.h>
 #include <wtf/text/CString.h>
 #include <wtf/text/ConversionMode.h>
 #include <wtf/text/LChar.h>
@@ -48,7 +49,8 @@ namespace WTF {
 
 // StringView is a non-owning reference to a string, similar to the proposed std::string_view.
 
-class StringView {
+class StringView final {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     StringView();
 #if CHECK_STRINGVIEW_LIFETIME
@@ -59,13 +61,15 @@ public:
     StringView& operator=(const StringView&);
 #endif
 
-    StringView(const AtomicString&);
+    StringView(const AtomString&);
     StringView(const String&);
     StringView(const StringImpl&);
     StringView(const StringImpl*);
     StringView(const LChar*, unsigned length);
     StringView(const UChar*, unsigned length);
     StringView(const char*);
+    StringView(const char*, unsigned length);
+    explicit StringView(ASCIILiteral);
 
     static StringView empty();
 
@@ -90,13 +94,15 @@ public:
     const LChar* characters8() const;
     const UChar* characters16() const;
 
+    bool isAllASCII() const;
+
     String toString() const;
     String toStringWithoutCopying() const;
-    AtomicString toAtomicString() const;
-    RefPtr<AtomicStringImpl> toExistingAtomicString() const;
+    AtomString toAtomString() const;
+    RefPtr<AtomStringImpl> toExistingAtomString() const;
 
 #if USE(CF)
-    // This function converts null strings to empty strings.
+    // These functions convert null strings to empty strings.
     WTF_EXPORT_PRIVATE RetainPtr<CFStringRef> createCFString() const;
     WTF_EXPORT_PRIVATE RetainPtr<CFStringRef> createCFStringWithoutCopying() const;
 #endif
@@ -116,9 +122,13 @@ public:
     void getCharactersWithUpconvert(LChar*) const;
     void getCharactersWithUpconvert(UChar*) const;
 
+    enum class CaseConvertType { Upper, Lower };
+    WTF_EXPORT_PRIVATE void getCharactersWithASCIICase(CaseConvertType, LChar*) const;
+    WTF_EXPORT_PRIVATE void getCharactersWithASCIICase(CaseConvertType, UChar*) const;
+
     StringView substring(unsigned start, unsigned length = std::numeric_limits<unsigned>::max()) const;
-    StringView left(unsigned len) const { return substring(0, len); }
-    StringView right(unsigned len) const { return substring(length() - len, len); }
+    StringView left(unsigned length) const { return substring(0, length); }
+    StringView right(unsigned length) const { return substring(this->length() - length, length); }
 
     template<typename MatchedCharacterPredicate>
     StringView stripLeadingAndTrailingMatchedCharacters(const MatchedCharacterPredicate&);
@@ -129,10 +139,9 @@ public:
 
     size_t find(UChar, unsigned start = 0) const;
     size_t find(CodeUnitMatchFunction, unsigned start = 0) const;
+    WTF_EXPORT_PRIVATE size_t find(StringView, unsigned start = 0) const;
 
-    WTF_EXPORT_PRIVATE size_t find(StringView, unsigned start) const;
-
-    size_t reverseFind(UChar, unsigned index = UINT_MAX) const;
+    size_t reverseFind(UChar, unsigned index = std::numeric_limits<unsigned>::max()) const;
 
     WTF_EXPORT_PRIVATE size_t findIgnoringASCIICase(const StringView&) const;
     WTF_EXPORT_PRIVATE size_t findIgnoringASCIICase(const StringView&, unsigned startOffset) const;
@@ -141,12 +150,20 @@ public:
     WTF_EXPORT_PRIVATE String convertToASCIIUppercase() const;
 
     bool contains(UChar) const;
+    bool contains(CodeUnitMatchFunction) const;
+    bool contains(StringView string) const { return find(string, 0) != notFound; }
+    WTF_EXPORT_PRIVATE bool contains(const char*) const;
+
     WTF_EXPORT_PRIVATE bool containsIgnoringASCIICase(const StringView&) const;
     WTF_EXPORT_PRIVATE bool containsIgnoringASCIICase(const StringView&, unsigned startOffset) const;
 
+    template<bool isSpecialCharacter(UChar)> bool isAllSpecialCharacters() const;
+
+    WTF_EXPORT_PRIVATE bool startsWith(UChar) const;
     WTF_EXPORT_PRIVATE bool startsWith(const StringView&) const;
     WTF_EXPORT_PRIVATE bool startsWithIgnoringASCIICase(const StringView&) const;
 
+    WTF_EXPORT_PRIVATE bool endsWith(UChar) const;
     WTF_EXPORT_PRIVATE bool endsWith(const StringView&) const;
     WTF_EXPORT_PRIVATE bool endsWithIgnoringASCIICase(const StringView&) const;
 
@@ -162,6 +179,7 @@ public:
 
 private:
     friend bool equal(StringView, StringView);
+    friend WTF_EXPORT_PRIVATE bool equalRespectingNullity(StringView, StringView);
 
     void initialize(const LChar*, unsigned length);
     void initialize(const UChar*, unsigned length);
@@ -173,11 +191,13 @@ private:
     WTF_EXPORT_PRIVATE bool underlyingStringIsValid() const;
     WTF_EXPORT_PRIVATE void setUnderlyingString(const StringImpl*);
     WTF_EXPORT_PRIVATE void setUnderlyingString(const StringView&);
+    void adoptUnderlyingString(UnderlyingString*);
 #else
     bool underlyingStringIsValid() const { return true; }
     void setUnderlyingString(const StringImpl*) { }
     void setUnderlyingString(const StringView&) { }
 #endif
+
     void clear();
 
     const void* m_characters { nullptr };
@@ -185,7 +205,6 @@ private:
     bool m_is8Bit { true };
 
 #if CHECK_STRINGVIEW_LIFETIME
-    void adoptUnderlyingString(UnderlyingString*);
     UnderlyingString* m_underlyingString { nullptr };
 #endif
 };
@@ -198,7 +217,11 @@ bool equal(StringView, const LChar* b);
 bool equalIgnoringASCIICase(StringView, StringView);
 bool equalIgnoringASCIICase(StringView, const char*);
 
+WTF_EXPORT_PRIVATE bool equalRespectingNullity(StringView, StringView);
+bool equalIgnoringNullity(StringView, StringView);
+
 template<unsigned length> bool equalLettersIgnoringASCIICase(StringView, const char (&lowercaseLetters)[length]);
+template<unsigned length> bool startsWithLettersIgnoringASCIICase(StringView, const char (&lowercaseLetters)[length]);
 
 inline bool operator==(StringView a, StringView b) { return equal(a, b); }
 inline bool operator==(StringView a, const LChar *b);
@@ -221,14 +244,17 @@ WTF_EXPORT_PRIVATE StringViewWithUnderlyingString normalizedNFC(StringView);
 
 WTF_EXPORT_PRIVATE String normalizedNFC(const String&);
 
+WTF_EXPORT_PRIVATE Optional<uint16_t> parseUInt16(StringView);
+
 }
 
-#include <wtf/text/AtomicString.h>
+#include <wtf/text/AtomString.h>
 #include <wtf/text/WTFString.h>
 
 namespace WTF {
 
 struct StringViewWithUnderlyingString {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
     StringView view;
     String underlyingString;
 };
@@ -327,6 +353,16 @@ inline StringView::StringView(const char* characters)
     initialize(reinterpret_cast<const LChar*>(characters), strlen(characters));
 }
 
+inline StringView::StringView(const char* characters, unsigned length)
+{
+    initialize(reinterpret_cast<const LChar*>(characters), length);
+}
+
+inline StringView::StringView(ASCIILiteral string)
+{
+    initialize(string.characters8(), string.length());
+}
+
 inline StringView::StringView(const StringImpl& string)
 {
     setUnderlyingString(&string);
@@ -362,8 +398,8 @@ inline StringView::StringView(const String& string)
     initialize(string.characters16(), string.length());
 }
 
-inline StringView::StringView(const AtomicString& atomicString)
-    : StringView(atomicString.string())
+inline StringView::StringView(const AtomString& atomString)
+    : StringView(atomString.string())
 {
 }
 
@@ -376,7 +412,7 @@ inline void StringView::clear()
 
 inline StringView StringView::empty()
 {
-    return StringView(reinterpret_cast<const LChar*>(""), 0);
+    return StringView("", 0);
 }
 
 inline const LChar* StringView::characters8() const
@@ -393,7 +429,15 @@ inline const UChar* StringView::characters16() const
     return static_cast<const UChar*>(m_characters);
 }
 
+inline bool StringView::isAllASCII() const
+{
+    if (is8Bit())
+        return charactersAreAllASCII(characters8(), length());
+    return charactersAreAllASCII(characters16(), length());
+}
+
 class StringView::UpconvertedCharacters {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     explicit UpconvertedCharacters(const StringView&);
     operator const UChar*() const { return m_characters; }
@@ -468,6 +512,18 @@ inline bool StringView::contains(UChar character) const
     return find(character) != notFound;
 }
 
+inline bool StringView::contains(CodeUnitMatchFunction function) const
+{
+    return find(function) != notFound;
+}
+
+template<bool isSpecialCharacter(UChar)> inline bool StringView::isAllSpecialCharacters() const
+{
+    if (is8Bit())
+        return WTF::isAllSpecialCharacters<isSpecialCharacter>(characters8(), length());
+    return WTF::isAllSpecialCharacters<isSpecialCharacter>(characters16(), length());
+}
+
 inline void StringView::getCharactersWithUpconvert(LChar* destination) const
 {
     ASSERT(is8Bit());
@@ -504,18 +560,18 @@ inline String StringView::toString() const
     return String(characters16(), m_length);
 }
 
-inline AtomicString StringView::toAtomicString() const
+inline AtomString StringView::toAtomString() const
 {
     if (is8Bit())
-        return AtomicString(characters8(), m_length);
-    return AtomicString(characters16(), m_length);
+        return AtomString(characters8(), m_length);
+    return AtomString(characters16(), m_length);
 }
 
-inline RefPtr<AtomicStringImpl> StringView::toExistingAtomicString() const
+inline RefPtr<AtomStringImpl> StringView::toExistingAtomString() const
 {
     if (is8Bit())
-        return AtomicStringImpl::lookUp(characters8(), m_length);
-    return AtomicStringImpl::lookUp(characters16(), m_length);
+        return AtomStringImpl::lookUp(characters8(), m_length);
+    return AtomStringImpl::lookUp(characters16(), m_length);
 }
 
 inline float StringView::toFloat(bool& isValid) const
@@ -647,6 +703,7 @@ inline bool equalIgnoringASCIICase(StringView a, const char* b)
 }
 
 class StringView::SplitResult {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     SplitResult(StringView, UChar separator, bool allowEmptyEntries);
 
@@ -661,6 +718,7 @@ private:
 };
 
 class StringView::GraphemeClusters {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     explicit GraphemeClusters(const StringView&);
 
@@ -673,6 +731,7 @@ private:
 };
 
 class StringView::CodePoints {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     explicit CodePoints(const StringView&);
 
@@ -685,6 +744,7 @@ private:
 };
 
 class StringView::CodeUnits {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     explicit CodeUnits(const StringView&);
 
@@ -697,7 +757,14 @@ private:
 };
 
 class StringView::SplitResult::Iterator {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
+    using iterator_category = std::forward_iterator_tag;
+    using value_type = StringView;
+    using difference_type = ptrdiff_t;
+    using pointer = value_type*;
+    using reference = value_type&;
+
     StringView operator*() const;
 
     WTF_EXPORT_PRIVATE Iterator& operator++();
@@ -721,6 +788,7 @@ private:
 };
 
 class StringView::GraphemeClusters::Iterator {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     Iterator() = delete;
     WTF_EXPORT_PRIVATE Iterator(const StringView&, unsigned index);
@@ -744,6 +812,7 @@ private:
 };
 
 class StringView::CodePoints::Iterator {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     Iterator(const StringView&, unsigned index);
 
@@ -752,7 +821,6 @@ public:
 
     bool operator==(const Iterator&) const;
     bool operator!=(const Iterator&) const;
-    Iterator& operator=(const Iterator&);
 
 private:
     std::reference_wrapper<const StringView> m_stringView;
@@ -761,6 +829,7 @@ private:
 };
 
 class StringView::CodeUnits::Iterator {
+    WTF_MAKE_FAST_ALLOCATED;
 public:
     Iterator(const StringView&, unsigned index);
 
@@ -832,14 +901,6 @@ inline auto StringView::CodePoints::Iterator::operator++() -> Iterator&
     return *this;
 }
 
-inline auto StringView::CodePoints::Iterator::operator=(const Iterator& other) -> Iterator&
-{
-    m_stringView = other.m_stringView;
-    m_nextCodePointOffset = other.m_nextCodePointOffset;
-    m_codePoint = other.m_codePoint;
-    return *this;
-}
-
 inline UChar32 StringView::CodePoints::Iterator::operator*() const
 {
     ASSERT(m_nextCodePointOffset);
@@ -1007,6 +1068,19 @@ template<unsigned length> inline bool equalLettersIgnoringASCIICase(StringView s
     return equalLettersIgnoringASCIICaseCommon(string, lowercaseLetters);
 }
 
+template<unsigned length> inline bool startsWithLettersIgnoringASCIICase(StringView string, const char (&lowercaseLetters)[length])
+{
+    return startsWithLettersIgnoringASCIICaseCommon(string, lowercaseLetters);
+}
+
+inline bool equalIgnoringNullity(StringView a, StringView b)
+{
+    // FIXME: equal(StringView, StringView) ignores nullity; consider changing to be like other string classes and respecting it.
+    return equal(a, b);
+}
+
+WTF_EXPORT_PRIVATE int codePointCompare(StringView, StringView);
+
 } // namespace WTF
 
 using WTF::append;
