@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2008, 2010 Apple Inc. All rights reserved.
+ * Copyright (C) 2004-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,11 +31,11 @@
 
 namespace WTF {
 
-static const size_t printBufferSize = 100; // large enough for any integer or floating point value in string format, including trailing null character
+static constexpr size_t printBufferSize = 100; // large enough for any integer or floating point value in string format, including trailing null character
 
 static inline bool hasFractions(double val)
 {
-    static const double s_epsilon = 0.0001;
+    static constexpr double s_epsilon = 0.0001;
     int ival = static_cast<int>(val);
     double dval = static_cast<double>(ival);
     return fabs(val - dval) > s_epsilon;
@@ -46,6 +46,12 @@ TextStream& TextStream::operator<<(bool b)
     return *this << (b ? "1" : "0");
 }
 
+TextStream& TextStream::operator<<(char c)
+{
+    m_text.append(c);
+    return *this;
+}
+
 TextStream& TextStream::operator<<(int i)
 {
     m_text.appendNumber(i);
@@ -87,7 +93,7 @@ TextStream& TextStream::operator<<(float f)
     if (m_formattingFlags & Formatting::NumberRespectingIntegers)
         return *this << FormatNumberRespectingIntegers(f);
 
-    m_text.appendFixedWidthNumber(f, 2);
+    m_text.append(FormattedNumber::fixedWidth(f, 2));
     return *this;
 }
 
@@ -96,7 +102,7 @@ TextStream& TextStream::operator<<(double d)
     if (m_formattingFlags & Formatting::NumberRespectingIntegers)
         return *this << FormatNumberRespectingIntegers(d);
 
-    m_text.appendFixedWidthNumber(d, 2);
+    m_text.append(FormattedNumber::fixedWidth(d, 2));
     return *this;
 }
 
@@ -122,7 +128,7 @@ TextStream& TextStream::operator<<(const String& string)
 TextStream& TextStream::operator<<(const FormatNumberRespectingIntegers& numberToFormat)
 {
     if (hasFractions(numberToFormat.value)) {
-        m_text.appendFixedWidthNumber(numberToFormat.value, 2);
+        m_text.append(FormattedNumber::fixedWidth(numberToFormat.value, 2));
         return *this;
     }
 
