@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,7 +26,7 @@
 #include "config.h"
 #include <wtf/threads/Signals.h>
 
-#if USE(PTHREADS) && HAVE(MACHINE_CONTEXT)
+#if OS(UNIX)
 
 #if HAVE(MACH_EXCEPTIONS)
 extern "C" {
@@ -46,19 +46,53 @@ extern "C" {
 
 #include <wtf/Atomics.h>
 #include <wtf/DataLog.h>
-#include <wtf/LocklessBag.h>
 #include <wtf/NeverDestroyed.h>
+#include <wtf/PlatformRegisters.h>
 #include <wtf/ThreadGroup.h>
-#include <wtf/ThreadMessage.h>
 #include <wtf/Threading.h>
-
+#include <wtf/WTFConfig.h>
 
 namespace WTF {
 
+#if HAVE(MACH_EXCEPTIONS)
+static exception_mask_t toMachMask(Signal);
+#endif
 
-static LazyNeverDestroyed<LocklessBag<SignalHandler>> handlers[static_cast<size_t>(Signal::NumberOfSignals)] = { };
-static std::once_flag initializeOnceFlags[static_cast<size_t>(Signal::NumberOfSignals)];
-static struct sigaction oldActions[static_cast<size_t>(Signal::NumberOfSignals)];
+void SignalHandlers::add(Signal signal, SignalHandler&& handler)
+{
+    Config::AssertNotFrozenScope assertScope;
+    static Lock lock;
+    auto locker = holdLock(lock);
+
+    size_t signalIndex = static_cast<size_t>(signal);
+    size_t nextFree = numberOfHandlers[signalIndex];
+#if HAVE(MACH_EXCEPTIONS)
+    if (signal != Signal::Usr)
+        addedExceptions |= toMachMask(signal);
+#endif
+    RELEASE_ASSERT(nextFree < maxNumberOfHandlers);
+    SignalHandlerMemory* memory = &handlers[signalIndex][nextFree];
+    new (memory) SignalHandler(WTFMove(handler));
+
+    // We deliberately do not want to increment the count until after we've
+    // fully initialized the memory. This way, forEachHandler() won't see a
+    // partially initialized handler.
+    storeStoreFence();
+    numberOfHandlers[signalIndex]++;
+    loadLoadFence();
+}
+
+template<typename Func>
+inline void SignalHandlers::forEachHandler(Signal signal, const Func& func) const
+{
+    size_t signalIndex = static_cast<size_t>(signal);
+    size_t handlerIndex = numberOfHandlers[signalIndex];
+    while (handlerIndex--) {
+        auto* memory = const_cast<SignalHandlerMemory*>(&handlers[signalIndex][handlerIndex]);
+        const SignalHandler& handler = *bitwise_cast<SignalHandler*>(memory);
+        func(handler);
+    }
+}
 
 #if HAVE(MACH_EXCEPTIONS)
 // You can read more about mach exceptions here:
@@ -66,27 +100,28 @@ static struct sigaction oldActions[static_cast<size_t>(Signal::NumberOfSignals)]
 // and the Mach interface Generator (MiG) here:
 // http://www.cs.cmu.edu/afs/cs/project/mach/public/doc/unpublished/mig.ps
 
-static mach_port_t exceptionPort;
 static constexpr size_t maxMessageSize = 1 * KB;
 
-static void startMachExceptionHandlerThread()
+void startMachExceptionHandlerThread()
 {
     static std::once_flag once;
     std::call_once(once, [] {
-        kern_return_t kr = mach_port_allocate(mach_task_self(), MACH_PORT_RIGHT_RECEIVE, &exceptionPort);
+        Config::AssertNotFrozenScope assertScope;
+        SignalHandlers& handlers = g_wtfConfig.signalHandlers;
+        kern_return_t kr = mach_port_allocate(mach_task_self(), MACH_PORT_RIGHT_RECEIVE, &handlers.exceptionPort);
         RELEASE_ASSERT(kr == KERN_SUCCESS);
-        kr = mach_port_insert_right(mach_task_self(), exceptionPort, exceptionPort, MACH_MSG_TYPE_MAKE_SEND);
+        kr = mach_port_insert_right(mach_task_self(), handlers.exceptionPort, handlers.exceptionPort, MACH_MSG_TYPE_MAKE_SEND);
         RELEASE_ASSERT(kr == KERN_SUCCESS);
 
         dispatch_source_t source = dispatch_source_create(
-            DISPATCH_SOURCE_TYPE_MACH_RECV, exceptionPort, 0, DISPATCH_TARGET_QUEUE_DEFAULT);
+            DISPATCH_SOURCE_TYPE_MACH_RECV, handlers.exceptionPort, 0, DISPATCH_TARGET_QUEUE_DEFAULT);
         RELEASE_ASSERT(source);
 
         dispatch_source_set_event_handler(source, ^{
             UNUSED_PARAM(source); // Capture a pointer to source in user space to silence the leaks tool.
 
             kern_return_t kr = mach_msg_server_once(
-                mach_exc_server, maxMessageSize, exceptionPort, MACH_MSG_TIMEOUT_NONE);
+                mach_exc_server, maxMessageSize, handlers.exceptionPort, MACH_MSG_TIMEOUT_NONE);
             RELEASE_ASSERT(kr == KERN_SUCCESS);
         });
 
@@ -99,8 +134,10 @@ static void startMachExceptionHandlerThread()
 static Signal fromMachException(exception_type_t type)
 {
     switch (type) {
-    case EXC_BAD_ACCESS: return Signal::BadAccess;
-    case EXC_BAD_INSTRUCTION: return Signal::Ill;
+    case EXC_BAD_ACCESS: return Signal::AccessFault;
+    case EXC_BAD_INSTRUCTION: return Signal::IllegalInstruction;
+    case EXC_ARITHMETIC: return Signal::FloatingPoint;
+    case EXC_BREAKPOINT: return Signal::Breakpoint;
     default: break;
     }
     return Signal::Unknown;
@@ -109,8 +146,10 @@ static Signal fromMachException(exception_type_t type)
 static exception_mask_t toMachMask(Signal signal)
 {
     switch (signal) {
-    case Signal::BadAccess: return EXC_MASK_BAD_ACCESS;
-    case Signal::Ill: return EXC_MASK_BAD_INSTRUCTION;
+    case Signal::AccessFault: return EXC_MASK_BAD_ACCESS;
+    case Signal::IllegalInstruction: return EXC_MASK_BAD_INSTRUCTION;
+    case Signal::FloatingPoint: return EXC_MASK_ARITHMETIC;
+    case Signal::Breakpoint: return EXC_MASK_BREAKPOINT;
     default: break;
     }
     RELEASE_ASSERT_NOT_REACHED();
@@ -145,7 +184,8 @@ kern_return_t catch_mach_exception_raise_state(
     thread_state_t outState,
     mach_msg_type_number_t* outStateCount)
 {
-    RELEASE_ASSERT(port == exceptionPort);
+    SignalHandlers& handlers = g_wtfConfig.signalHandlers;
+    RELEASE_ASSERT(port == handlers.exceptionPort);
     // If we wanted to distinguish between SIGBUS and SIGSEGV for EXC_BAD_ACCESS on Darwin we could do:
     // if (exceptionData[0] == KERN_INVALID_ADDRESS)
     //    signal = SIGSEGV;
@@ -172,13 +212,21 @@ kern_return_t catch_mach_exception_raise_state(
 #endif
 
     SigInfo info;
-    if (signal == Signal::BadAccess) {
+    if (signal == Signal::AccessFault) {
         ASSERT_UNUSED(dataCount, dataCount == 2);
         info.faultingAddress = reinterpret_cast<void*>(exceptionData[1]);
+#if CPU(ADDRESS64)
+        // If the faulting address is out of the range of any valid memory, we would
+        // not have any reason to handle it. Just let the default handler take care of it.
+        static constexpr unsigned validAddressBits = OS_CONSTANT(EFFECTIVE_ADDRESS_WIDTH);
+        static constexpr uintptr_t invalidAddressMask = ~((1ull << validAddressBits) - 1);
+        if (bitwise_cast<uintptr_t>(info.faultingAddress) & invalidAddressMask)
+            return KERN_FAILURE;
+#endif
     }
 
     bool didHandle = false;
-    handlers[static_cast<size_t>(signal)]->iterate([&] (const SignalHandler& handler) {
+    handlers.forEachHandler(signal, [&] (const SignalHandler& handler) {
         SignalAction handlerResult = handler(signal, info, registers);
         didHandle |= handlerResult == SignalAction::Handled;
     });
@@ -188,21 +236,20 @@ kern_return_t catch_mach_exception_raise_state(
     return KERN_FAILURE;
 }
 
-};
+}; // extern "C"
 
-static bool useMach { false };
 void handleSignalsWithMach()
 {
-    useMach = true;
+    Config::AssertNotFrozenScope assertScope;
+    g_wtfConfig.signalHandlers.useMach = true;
 }
 
-
-exception_mask_t activeExceptions { 0 };
-
+static exception_mask_t activeExceptions;
 inline void setExceptionPorts(const AbstractLocker& threadGroupLocker, Thread& thread)
 {
     UNUSED_PARAM(threadGroupLocker);
-    kern_return_t result = thread_set_exception_ports(thread.machThread(), activeExceptions, exceptionPort, EXCEPTION_STATE | MACH_EXCEPTION_CODES, MACHINE_THREAD_STATE);
+    SignalHandlers& handlers = g_wtfConfig.signalHandlers;
+    kern_return_t result = thread_set_exception_ports(thread.machThread(), handlers.addedExceptions &activeExceptions, handlers.exceptionPort, EXCEPTION_STATE | MACH_EXCEPTION_CODES, MACHINE_THREAD_STATE);
     if (result != KERN_SUCCESS) {
         dataLogLn("thread set port failed due to ", mach_error_string(result));
         CRASH();
@@ -214,6 +261,7 @@ static ThreadGroup& activeThreads()
     static std::once_flag initializeKey;
     static ThreadGroup* activeThreadsPtr = nullptr;
     std::call_once(initializeKey, [&] {
+        Config::AssertNotFrozenScope assertScope;
         static NeverDestroyed<std::shared_ptr<ThreadGroup>> activeThreads { ThreadGroup::create() };
         activeThreadsPtr = activeThreads.get().get();
     });
@@ -227,11 +275,8 @@ void registerThreadForMachExceptionHandling(Thread& thread)
         setExceptionPorts(locker, thread);
 }
 
-#else
-static constexpr bool useMach = false;
 #endif // HAVE(MACH_EXCEPTIONS)
 
-
 inline size_t offsetForSystemSignal(int sig)
 {
     Signal signal = fromSystemSignal(sig);
@@ -240,20 +285,22 @@ inline size_t offsetForSystemSignal(int sig)
 
 static void jscSignalHandler(int, siginfo_t*, void*);
 
-void installSignalHandler(Signal signal, SignalHandler&& handler)
+void addSignalHandler(Signal signal, SignalHandler&& handler)
 {
+    Config::AssertNotFrozenScope assertScope;
+    SignalHandlers& handlers = g_wtfConfig.signalHandlers;
     ASSERT(signal < Signal::Unknown);
-#if HAVE(MACH_EXCEPTIONS)
-    ASSERT(!useMach || signal != Signal::Usr);
+    ASSERT(!handlers.useMach || signal != Signal::Usr);
 
-    if (useMach)
+#if HAVE(MACH_EXCEPTIONS)
+    if (handlers.useMach)
         startMachExceptionHandlerThread();
 #endif
 
+    static std::once_flag initializeOnceFlags[static_cast<size_t>(Signal::NumberOfSignals)];
     std::call_once(initializeOnceFlags[static_cast<size_t>(signal)], [&] {
-        handlers[static_cast<size_t>(signal)].construct();
-
-        if (!useMach) {
+        Config::AssertNotFrozenScope assertScope;
+        if (!handlers.useMach) {
             struct sigaction action;
             action.sa_sigaction = jscSignalHandler;
             auto result = sigfillset(&action.sa_mask);
@@ -263,19 +310,26 @@ void installSignalHandler(Signal signal, SignalHandler&& handler)
             RELEASE_ASSERT(!result);
             action.sa_flags = SA_SIGINFO;
             auto systemSignals = toSystemSignal(signal);
-            result = sigaction(std::get<0>(systemSignals), &action, &oldActions[offsetForSystemSignal(std::get<0>(systemSignals))]);
+            result = sigaction(std::get<0>(systemSignals), &action, &handlers.oldActions[offsetForSystemSignal(std::get<0>(systemSignals))]);
             if (std::get<1>(systemSignals))
-                result |= sigaction(*std::get<1>(systemSignals), &action, &oldActions[offsetForSystemSignal(*std::get<1>(systemSignals))]);
+                result |= sigaction(*std::get<1>(systemSignals), &action, &handlers.oldActions[offsetForSystemSignal(*std::get<1>(systemSignals))]);
             RELEASE_ASSERT(!result);
         }
-
     });
 
-    handlers[static_cast<size_t>(signal)]->add(WTFMove(handler));
+    handlers.add(signal, WTFMove(handler));
+}
 
+void activateSignalHandlersFor(Signal signal)
+{
+    UNUSED_PARAM(signal);
 #if HAVE(MACH_EXCEPTIONS)
+    const SignalHandlers& handlers = g_wtfConfig.signalHandlers;
+    ASSERT(signal < Signal::Unknown);
+    ASSERT(!handlers.useMach || signal != Signal::Usr);
+
     auto locker = holdLock(activeThreads().getLock());
-    if (useMach) {
+    if (handlers.useMach) {
         activeExceptions |= toMachMask(signal);
 
         for (auto& thread : activeThreads().threads(locker))
@@ -287,6 +341,7 @@ void installSignalHandler(Signal signal, SignalHandler&& handler)
 void jscSignalHandler(int sig, siginfo_t* info, void* ucontext)
 {
     Signal signal = fromSystemSignal(sig);
+    SignalHandlers& handlers = g_wtfConfig.signalHandlers;
 
     auto restoreDefault = [&] {
         struct sigaction defaultAction;
@@ -299,20 +354,24 @@ void jscSignalHandler(int sig, siginfo_t* info, void* ucontext)
 
     // This shouldn't happen but we might as well be careful.
     if (signal == Signal::Unknown) {
-        dataLogLn("We somehow got called for an unknown signal ", sig, ", halp.");
+        dataLogLn("We somehow got called for an unknown signal ", sig, ", help.");
         restoreDefault();
         return;
     }
 
     SigInfo sigInfo;
-    if (signal == Signal::BadAccess)
+    if (signal == Signal::AccessFault)
         sigInfo.faultingAddress = info->si_addr;
 
+#if HAVE(MACHINE_CONTEXT)
     PlatformRegisters& registers = registersFromUContext(reinterpret_cast<ucontext_t*>(ucontext));
+#else
+    PlatformRegisters registers { };
+#endif
 
     bool didHandle = false;
     bool restoreDefaultHandler = false;
-    handlers[static_cast<size_t>(signal)]->iterate([&] (const SignalHandler& handler) {
+    handlers.forEachHandler(signal, [&] (const SignalHandler& handler) {
         switch (handler(signal, sigInfo, registers)) {
         case SignalAction::Handled:
             didHandle = true;
@@ -331,7 +390,7 @@ void jscSignalHandler(int sig, siginfo_t* info, void* ucontext)
     }
 
     unsigned oldActionIndex = static_cast<size_t>(signal) + (sig == SIGBUS);
-    struct sigaction& oldAction = oldActions[static_cast<size_t>(oldActionIndex)];
+    struct sigaction& oldAction = handlers.oldActions[static_cast<size_t>(oldActionIndex)];
     if (signal == Signal::Usr) {
         if (oldAction.sa_sigaction)
             oldAction.sa_sigaction(sig, info, ucontext);
@@ -349,6 +408,18 @@ void jscSignalHandler(int sig, siginfo_t* info, void* ucontext)
     }
 }
 
+void SignalHandlers::initialize()
+{
+#if HAVE(MACH_EXCEPTIONS)
+    // In production configurations, this does not matter because signal handler
+    // installations will always trigger this initialization. However, in debugging
+    // configurations, we may end up disabling the use of all signal handlers but
+    // we still need this to be initialized. Hence, we need to initialize it
+    // eagerly to ensure that it is done before we freeze the WTF::Config.
+    activeThreads();
+#endif
+}
+
 } // namespace WTF
 
-#endif // USE(PTHREADS) && HAVE(MACHINE_CONTEXT)
+#endif // OS(UNIX)
