@@ -33,15 +33,23 @@
 
 namespace WebCore {
 
-static ExceptionOr<Vector<String>> convertAndValidate(unsigned version, const Vector<String>& supportedNetworks, const PaymentCoordinator& paymentCoordinator)
+static bool requiresSupportedNetworks(unsigned version, const ApplePayRequestBase& request)
 {
-    if (supportedNetworks.isEmpty())
-        return Exception { TypeError, "At least one supported network must be provided." };
+#if ENABLE(APPLE_PAY_INSTALLMENTS)
+    return version < 8 || !request.installmentConfiguration;
+#else
+    UNUSED_PARAM(version);
+    UNUSED_PARAM(request);
+    return true;
+#endif
+}
 
+static ExceptionOr<Vector<String>> convertAndValidate(Document& document, unsigned version, const Vector<String>& supportedNetworks, const PaymentCoordinator& paymentCoordinator)
+{
     Vector<String> result;
     result.reserveInitialCapacity(supportedNetworks.size());
     for (auto& supportedNetwork : supportedNetworks) {
-        auto validatedNetwork = paymentCoordinator.validatedPaymentNetwork(version, supportedNetwork);
+        auto validatedNetwork = paymentCoordinator.validatedPaymentNetwork(document, version, supportedNetwork);
         if (!validatedNetwork)
             return Exception { TypeError, makeString("\"", supportedNetwork, "\" is not a valid payment network.") };
         result.uncheckedAppend(*validatedNetwork);
@@ -50,9 +58,9 @@ static ExceptionOr<Vector<String>> convertAndValidate(unsigned version, const Ve
     return WTFMove(result);
 }
 
-ExceptionOr<ApplePaySessionPaymentRequest> convertAndValidate(unsigned version, ApplePayRequestBase& request, const PaymentCoordinator& paymentCoordinator)
+ExceptionOr<ApplePaySessionPaymentRequest> convertAndValidate(Document& document, unsigned version, ApplePayRequestBase& request, const PaymentCoordinator& paymentCoordinator)
 {
-    if (!version || !paymentCoordinator.supportsVersion(version))
+    if (!version || !paymentCoordinator.supportsVersion(document, version))
         return Exception { InvalidAccessError, makeString('"', version, "\" is not a supported version.") };
 
     ApplePaySessionPaymentRequest result;
@@ -64,7 +72,10 @@ ExceptionOr<ApplePaySessionPaymentRequest> convertAndValidate(unsigned version,
         return merchantCapabilities.releaseException();
     result.setMerchantCapabilities(merchantCapabilities.releaseReturnValue());
 
-    auto supportedNetworks = convertAndValidate(version, request.supportedNetworks, paymentCoordinator);
+    if (requiresSupportedNetworks(version, request) && request.supportedNetworks.isEmpty())
+        return Exception { TypeError, "At least one supported network must be provided." };
+
+    auto supportedNetworks = convertAndValidate(document, version, request.supportedNetworks, paymentCoordinator);
     if (supportedNetworks.hasException())
         return supportedNetworks.releaseException();
     result.setSupportedNetworks(supportedNetworks.releaseReturnValue());
@@ -94,6 +105,15 @@ ExceptionOr<ApplePaySessionPaymentRequest> convertAndValidate(unsigned version,
     if (version >= 3)
         result.setSupportedCountries(WTFMove(request.supportedCountries));
 
+#if ENABLE(APPLE_PAY_INSTALLMENTS)
+    if (request.installmentConfiguration) {
+        auto installmentConfiguration = PaymentInstallmentConfiguration::create(*request.installmentConfiguration);
+        if (installmentConfiguration.hasException())
+            return installmentConfiguration.releaseException();
+        result.setInstallmentConfiguration(installmentConfiguration.releaseReturnValue());
+    }
+#endif
+
     return WTFMove(result);
 }
 
