@@ -28,10 +28,13 @@
 
 #if ENABLE(APPLE_PAY)
 
+#include <wtf/IsoMallocInlines.h>
 #include <wtf/text/StringBuilder.h>
 
 namespace WebCore {
 
+WTF_MAKE_ISO_ALLOCATED_IMPL(ApplePayShippingMethodSelectedEvent);
+
 static inline ApplePayShippingMethod convert(const ApplePaySessionPaymentRequest::ShippingMethod& shippingMethod)
 {
     ApplePayShippingMethod convertedMethod;
@@ -43,7 +46,7 @@ static inline ApplePayShippingMethod convert(const ApplePaySessionPaymentRequest
     return convertedMethod;
 }
 
-ApplePayShippingMethodSelectedEvent::ApplePayShippingMethodSelectedEvent(const AtomicString& type, const ApplePaySessionPaymentRequest::ShippingMethod& shippingMethod)
+ApplePayShippingMethodSelectedEvent::ApplePayShippingMethodSelectedEvent(const AtomString& type, const ApplePaySessionPaymentRequest::ShippingMethod& shippingMethod)
     : Event(type, CanBubble::No, IsCancelable::No)
     , m_shippingMethod(convert(shippingMethod))
 {
