@@ -33,6 +33,7 @@
 #include "HTTPParsers.h"
 #include "JSAbortSignal.h"
 #include "Logging.h"
+#include "Quirks.h"
 #include "ScriptExecutionContext.h"
 #include "SecurityOrigin.h"
 #include "Settings.h"
@@ -144,25 +145,12 @@ ExceptionOr<void> FetchRequest::initializeOptions(const Init& init)
     return { };
 }
 
-static inline bool needsSignalQuirk(ScriptExecutionContext& context)
-{
-    if (!is<Document>(context))
-        return false;
-
-    auto& document = downcast<Document>(context);
-    if (!document.settings().needsSiteSpecificQuirks())
-        return false;
-
-    auto host = document.topDocument().url().host();
-    return equalLettersIgnoringASCIICase(host, "leetcode.com") || equalLettersIgnoringASCIICase(host, "www.thrivepatientportal.com");
-}
-
 static inline Optional<Exception> processInvalidSignal(ScriptExecutionContext& context)
 {
-    ASCIILiteral message { "FetchRequestInit.signal should be undefined, null or an AbortSignal object."_s };
+    ASCIILiteral message { "FetchRequestInit.signal should be undefined, null or an AbortSignal object. This will throw in a future release."_s };
     context.addConsoleMessage(MessageSource::JS, MessageLevel::Warning, message);
 
-    if (needsSignalQuirk(context))
+    if (is<Document>(context) && downcast<Document>(context).quirks().shouldIgnoreInvalidSignal())
         return { };
 
     RELEASE_LOG_ERROR(ResourceLoading, "FetchRequestInit.signal should be undefined, null or an AbortSignal object.");
@@ -173,7 +161,7 @@ ExceptionOr<void> FetchRequest::initializeWith(const String& url, Init&& init)
 {
     ASSERT(scriptExecutionContext());
     // FIXME: Tighten the URL parsing algorithm according https://url.spec.whatwg.org/#concept-url-parser.
-    URL requestURL = scriptExecutionContext()->completeURL(url);
+    URL requestURL = scriptExecutionContext()->completeURL(url, ScriptExecutionContext::ForceUTF8::Yes);
     if (!requestURL.isValid() || !requestURL.user().isEmpty() || !requestURL.pass().isEmpty())
         return Exception { TypeError, "URL is not valid or contains user credentials."_s };
 
@@ -215,9 +203,6 @@ ExceptionOr<void> FetchRequest::initializeWith(const String& url, Init&& init)
 
 ExceptionOr<void> FetchRequest::initializeWith(FetchRequest& input, Init&& init)
 {
-    if (input.isDisturbedOrLocked())
-        return Exception {TypeError, "Request input is disturbed or locked."_s };
-
     m_request = input.m_request;
     m_options = input.m_options;
     m_referrer = input.m_referrer;
@@ -237,25 +222,16 @@ ExceptionOr<void> FetchRequest::initializeWith(FetchRequest& input, Init&& init)
     } else
         m_signal->follow(input.m_signal.get());
 
-    if (init.headers) {
-        auto fillResult = m_headers->fill(*init.headers);
+    if (init.hasMembers()) {
+        auto fillResult = init.headers ? m_headers->fill(*init.headers) : m_headers->fill(input.headers());
         if (fillResult.hasException())
-            return fillResult.releaseException();
-    } else {
-        auto fillResult = m_headers->fill(input.headers());
-        if (fillResult.hasException())
-            return fillResult.releaseException();
-    }
+            return fillResult;
+    } else
+        m_headers->setInternalHeaders(HTTPHeaderMap { input.headers().internalHeaders() });
 
-    if (init.body) {
-        auto setBodyResult = setBody(WTFMove(*init.body));
-        if (setBodyResult.hasException())
-            return setBodyResult.releaseException();
-    } else {
-        auto setBodyResult = setBody(input);
-        if (setBodyResult.hasException())
-            return setBodyResult.releaseException();
-    }
+    auto setBodyResult = init.body ? setBody(WTFMove(*init.body)) : setBody(input);
+    if (setBodyResult.hasException())
+        return setBodyResult;
 
     updateContentType();
     return { };
@@ -267,7 +243,9 @@ ExceptionOr<void> FetchRequest::setBody(FetchBody::Init&& body)
         return Exception { TypeError, makeString("Request has method '", m_request.httpMethod(), "' and cannot have a body") };
 
     ASSERT(scriptExecutionContext());
-    extractBody(*scriptExecutionContext(), WTFMove(body));
+    auto result = extractBody(WTFMove(body));
+    if (result.hasException())
+        return result;
 
     if (m_options.keepAlive && hasReadableStreamBody())
         return Exception { TypeError, "Request cannot have a ReadableStream body and keepalive set to true"_s };
@@ -276,6 +254,9 @@ ExceptionOr<void> FetchRequest::setBody(FetchBody::Init&& body)
 
 ExceptionOr<void> FetchRequest::setBody(FetchRequest& request)
 {
+    if (request.isDisturbedOrLocked())
+        return Exception { TypeError, "Request input is disturbed or locked."_s };
+
     if (!request.isBodyNull()) {
         if (!methodCanHaveBody(m_request))
             return Exception { TypeError, makeString("Request has method '", m_request.httpMethod(), "' and cannot have a body") };
@@ -303,7 +284,7 @@ ExceptionOr<Ref<FetchRequest>> FetchRequest::create(ScriptExecutionContext& cont
             return result.releaseException();
     }
 
-    return WTFMove(request);
+    return request;
 }
 
 String FetchRequest::referrer() const
@@ -330,7 +311,7 @@ ResourceRequest FetchRequest::resourceRequest() const
     request.setHTTPHeaderFields(m_headers->internalHeaders());
 
     if (!isBodyNull())
-        request.setHTTPBody(body().bodyAsFormData(*scriptExecutionContext()));
+        request.setHTTPBody(body().bodyAsFormData());
 
     return request;
 }
@@ -343,7 +324,7 @@ ExceptionOr<Ref<FetchRequest>> FetchRequest::clone(ScriptExecutionContext& conte
     auto clone = adoptRef(*new FetchRequest(context, WTF::nullopt, FetchHeaders::create(m_headers.get()), ResourceRequest { m_request }, FetchOptions { m_options}, String { m_referrer }));
     clone->cloneBody(*this);
     clone->m_signal->follow(m_signal);
-    return WTFMove(clone);
+    return clone;
 }
 
 const char* FetchRequest::activeDOMObjectName() const
@@ -351,11 +332,5 @@ const char* FetchRequest::activeDOMObjectName() const
     return "Request";
 }
 
-bool FetchRequest::canSuspendForDocumentSuspension() const
-{
-    // FIXME: We can probably do the same strategy as XHR.
-    return !isActive();
-}
-
 } // namespace WebCore
 
