@@ -30,6 +30,8 @@
 
 #include "IDBCursorInfo.h"
 #include "IDBDatabase.h"
+#include "IDBDatabaseNameAndVersion.h"
+#include "IDBDatabaseNameAndVersionRequest.h"
 #include "IDBGetRecordData.h"
 #include "IDBIterateCursorData.h"
 #include "IDBKeyRangeData.h"
@@ -115,6 +117,19 @@ void IDBConnectionProxy::completeOpenDBRequest(const IDBResultData& resultData)
     if (!request)
         return;
 
+    if (request->isContextSuspended()) {
+        switch (resultData.type()) {
+        case IDBResultType::OpenDatabaseUpgradeNeeded: {
+            abortOpenAndUpgradeNeeded(resultData.databaseConnectionIdentifier(), resultData.transactionInfo().identifier());
+            auto result = IDBResultData::error(resultData.requestIdentifier(), IDBError { UnknownError, "Version change transaction on cached page is aborted to unblock other connections"_s });
+            request->performCallbackOnOriginThread(*request, &IDBOpenDBRequest::requestCompleted, result);
+            return;
+        }
+        default:
+            break;
+        }
+    }
+
     request->performCallbackOnOriginThread(*request, &IDBOpenDBRequest::requestCompleted, resultData);
 }
 
@@ -225,7 +240,8 @@ void IDBConnectionProxy::openCursor(TransactionOperation& operation, const IDBCu
 void IDBConnectionProxy::iterateCursor(TransactionOperation& operation, const IDBIterateCursorData& data)
 {
     const IDBRequestData requestData { operation };
-    saveOperation(operation);
+    if (data.option != IndexedDB::CursorIterateOption::DoNotReply)
+        saveOperation(operation);
 
     callConnectionOnMainThread(&IDBConnectionToServer::iterateCursor, requestData, data);
 }
@@ -268,12 +284,18 @@ void IDBConnectionProxy::fireVersionChangeEvent(uint64_t databaseConnectionIdent
     if (!database)
         return;
 
+    if (database->isContextSuspended()) {
+        didFireVersionChangeEvent(databaseConnectionIdentifier, requestIdentifier, IndexedDB::ConnectionClosedOnBehalfOfServer::Yes);
+        database->performCallbackOnOriginThread(*database, &IDBDatabase::connectionToServerLost, IDBError { UnknownError, "Connection on cached page closed to unblock other connections"_s});
+        return;
+    }
+
     database->performCallbackOnOriginThread(*database, &IDBDatabase::fireVersionChangeEvent, requestIdentifier, requestedVersion);
 }
 
-void IDBConnectionProxy::didFireVersionChangeEvent(uint64_t databaseConnectionIdentifier, const IDBResourceIdentifier& requestIdentifier)
+void IDBConnectionProxy::didFireVersionChangeEvent(uint64_t databaseConnectionIdentifier, const IDBResourceIdentifier& requestIdentifier, IndexedDB::ConnectionClosedOnBehalfOfServer connectionClosed)
 {
-    callConnectionOnMainThread(&IDBConnectionToServer::didFireVersionChangeEvent, databaseConnectionIdentifier, requestIdentifier);
+    callConnectionOnMainThread(&IDBConnectionToServer::didFireVersionChangeEvent, databaseConnectionIdentifier, requestIdentifier, connectionClosed);
 }
 
 void IDBConnectionProxy::notifyOpenDBRequestBlocked(const IDBResourceIdentifier& requestIdentifier, uint64_t oldVersion, uint64_t newVersion)
@@ -403,21 +425,12 @@ void IDBConnectionProxy::didCloseFromServer(uint64_t databaseConnectionIdentifie
         database = m_databaseConnectionMap.get(databaseConnectionIdentifier);
     }
 
-    // If the IDBDatabase object is gone, message back to the server so it doesn't hang
-    // waiting for a reply that will never come.
-    if (!database) {
-        m_connectionToServer.confirmDidCloseFromServer(databaseConnectionIdentifier);
+    if (!database)
         return;
-    }
 
     database->performCallbackOnOriginThread(*database, &IDBDatabase::didCloseFromServer, error);
 }
 
-void IDBConnectionProxy::confirmDidCloseFromServer(IDBDatabase& database)
-{
-    callConnectionOnMainThread(&IDBConnectionToServer::confirmDidCloseFromServer, database.databaseConnectionIdentifier());
-}
-
 void IDBConnectionProxy::connectionToServerLost(const IDBError& error)
 {
     Vector<uint64_t> databaseConnectionIdentifiers;
@@ -458,6 +471,15 @@ void IDBConnectionProxy::connectionToServerLost(const IDBError& error)
         auto result = IDBResultData::error(requestIdentifier, error);
         request->performCallbackOnOriginThread(*request, &IDBOpenDBRequest::requestCompleted, result);
     }
+
+    Vector<IDBResourceIdentifier> infoCallbackIdentifiers;
+    {
+        Locker<Lock> lock(m_databaseInfoMapLock);
+        infoCallbackIdentifiers = copyToVector(m_databaseInfoCallbacks.keys());
+    }
+
+    for (auto& requestIdentifier : infoCallbackIdentifiers)
+        didGetAllDatabaseNamesAndVersions(requestIdentifier, { });
 }
 
 void IDBConnectionProxy::scheduleMainThreadTasks()
@@ -485,12 +507,36 @@ void IDBConnectionProxy::handleMainThreadTasks()
         task->performTask();
 }
 
-void IDBConnectionProxy::getAllDatabaseNames(const SecurityOrigin& mainFrameOrigin, const SecurityOrigin& openingOrigin, Function<void (const Vector<String>&)>&& callback)
+void IDBConnectionProxy::getAllDatabaseNamesAndVersions(ScriptExecutionContext& context, Function<void(Optional<Vector<IDBDatabaseNameAndVersion>>&&)>&& callback)
 {
-    // This method is only meant to be called by the web inspector on the main thread.
-    RELEASE_ASSERT(isMainThread());
+    ClientOrigin origin { context.securityOrigin()->data(), context.topOrigin().data() };
 
-    m_connectionToServer.getAllDatabaseNames(mainFrameOrigin, openingOrigin, WTFMove(callback));
+    IDBDatabaseNameAndVersionRequest* request;
+    {
+        Locker<Lock> locker(m_databaseInfoMapLock);
+        auto newRequest = IDBDatabaseNameAndVersionRequest::create(context, *this, WTFMove(callback));
+        ASSERT(!m_databaseInfoCallbacks.contains(newRequest->resourceIdentifier()));
+        request = newRequest.ptr();
+        m_databaseInfoCallbacks.add(newRequest->resourceIdentifier(), WTFMove(newRequest));
+    }
+
+    callConnectionOnMainThread(&IDBConnectionToServer::getAllDatabaseNamesAndVersions, request->resourceIdentifier(), origin);
+}
+
+void IDBConnectionProxy::didGetAllDatabaseNamesAndVersions(const IDBResourceIdentifier& requestIdentifier, Optional<Vector<IDBDatabaseNameAndVersion>>&& databases)
+{
+    RefPtr<IDBDatabaseNameAndVersionRequest> request;
+    {
+        Locker<Lock> locker(m_databaseInfoMapLock);
+        auto takenRequest = m_databaseInfoCallbacks.take(requestIdentifier);
+
+        if (!takenRequest)
+            return;
+
+        request = WTFMove(*takenRequest);
+    }
+
+    request->performCallbackOnOriginThread(*request, &IDBDatabaseNameAndVersionRequest::complete, WTFMove(databases));
 }
 
 void IDBConnectionProxy::registerDatabaseConnection(IDBDatabase& database)
@@ -529,6 +575,8 @@ void IDBConnectionProxy::forgetTransaction(IDBTransaction& transaction)
 template<typename KeyType, typename ValueType>
 void removeItemsMatchingCurrentThread(HashMap<KeyType, ValueType>& map)
 {
+    // FIXME: Revisit when introducing WebThread aware thread comparison.
+    // https://bugs.webkit.org/show_bug.cgi?id=204345
     auto& currentThread = Thread::current();
 
     Vector<KeyType> keys;
@@ -542,6 +590,25 @@ void removeItemsMatchingCurrentThread(HashMap<KeyType, ValueType>& map)
         map.remove(key);
 }
 
+template<typename KeyType, typename ValueType>
+void setMatchingItemsContextSuspended(ScriptExecutionContext& currentContext, HashMap<KeyType, ValueType>& map, bool isContextSuspended)
+{
+    // FIXME: Revisit when introducing WebThread aware thread comparison.
+    // https://bugs.webkit.org/show_bug.cgi?id=204345
+    auto& currentThread = Thread::current();
+    for (auto& iterator : map) {
+        if (&iterator.value->originThread() != &currentThread)
+            continue;
+
+        auto* context = iterator.value->scriptExecutionContext();
+        if (!context)
+            continue;
+
+        if (context == &currentContext)
+            iterator.value->setIsContextSuspended(isContextSuspended);
+    }
+}
+
 void IDBConnectionProxy::forgetActivityForCurrentThread()
 {
     ASSERT(!isMainThread());
@@ -564,6 +631,22 @@ void IDBConnectionProxy::forgetActivityForCurrentThread()
         Locker<Lock> lock(m_transactionOperationLock);
         removeItemsMatchingCurrentThread(m_activeOperations);
     }
+    {
+        Locker<Lock> lock(m_databaseInfoMapLock);
+        removeItemsMatchingCurrentThread(m_databaseInfoCallbacks);
+    }
+}
+
+void IDBConnectionProxy::setContextSuspended(ScriptExecutionContext& currentContext, bool isContextSuspended)
+{
+    {
+        Locker<Lock> lock(m_databaseConnectionMapLock);
+        setMatchingItemsContextSuspended(currentContext, m_databaseConnectionMap, isContextSuspended);
+    }
+    {
+        Locker<Lock> lock(m_openDBRequestMapLock);
+        setMatchingItemsContextSuspended(currentContext, m_openDBRequestMap, isContextSuspended);
+    }
 }
 
 } // namesapce IDBClient
