@@ -47,7 +47,7 @@ typedef HashMap<IDBKeyData, std::unique_ptr<IndexValueEntry>, IDBKeyDataHash, ID
 class IndexValueStore {
     WTF_MAKE_FAST_ALLOCATED;
 public:
-    IndexValueStore(bool unique);
+    explicit IndexValueStore(bool unique);
 
     const IDBKeyData* lowestValueForKey(const IDBKeyData&) const;
     Vector<IDBKeyData> allValuesForKey(const IDBKeyData&, uint32_t limit) const;
