@@ -126,7 +126,7 @@ IDBGetResult MemoryIndex::getResultForKeyRange(IndexedDB::IndexRecordType type,
     if (!keyValue)
         return { };
 
-    return type == IndexedDB::IndexRecordType::Key ? IDBGetResult(*keyValue) : IDBGetResult(m_objectStore.valueForKeyRange(*keyValue));
+    return type == IndexedDB::IndexRecordType::Key ? IDBGetResult(*keyValue) : IDBGetResult(*keyValue, m_objectStore.valueForKeyRange(*keyValue), m_objectStore.info().keyPath());
 }
 
 uint64_t MemoryIndex::countForKeyRange(const IDBKeyRangeData& inRange)
@@ -156,7 +156,7 @@ void MemoryIndex::getAllRecords(const IDBKeyRangeData& keyRangeData, Optional<ui
 {
     LOG(IndexedDB, "MemoryIndex::getAllRecords");
 
-    result = { type };
+    result = { type, m_objectStore.info().keyPath() };
 
     if (!m_records)
         return;
@@ -179,10 +179,8 @@ void MemoryIndex::getAllRecords(const IDBKeyRangeData& keyRangeData, Optional<ui
 
         auto allValues = m_records->allValuesForKey(key, targetCount - currentCount);
         for (auto& keyValue : allValues) {
-            if (type == IndexedDB::GetAllType::Keys) {
-                IDBKeyData keyCopy { keyValue };
-                result.addKey(WTFMove(keyCopy));
-            } else
+            result.addKey(IDBKeyData(keyValue));
+            if (type == IndexedDB::GetAllType::Values)
                 result.addValue(m_objectStore.valueForKeyRange(keyValue));
         }
 
@@ -196,7 +194,7 @@ IDBError MemoryIndex::putIndexKey(const IDBKeyData& valueKey, const IndexKey& in
     LOG(IndexedDB, "MemoryIndex::provisionalPutIndexKey");
 
     if (!m_records) {
-        m_records = std::make_unique<IndexValueStore>(m_info.unique());
+        m_records = makeUnique<IndexValueStore>(m_info.unique());
         notifyCursorsOfAllRecordsChanged();
     }
 
@@ -261,7 +259,7 @@ MemoryIndexCursor* MemoryIndex::maybeOpenCursor(const IDBCursorInfo& info)
     if (!result.isNewEntry)
         return nullptr;
 
-    result.iterator->value = std::make_unique<MemoryIndexCursor>(*this, info);
+    result.iterator->value = makeUnique<MemoryIndexCursor>(*this, info);
     return result.iterator->value.get();
 }
 
