@@ -55,7 +55,7 @@ IDBTransactionInfo IDBTransactionInfo::versionChange(const IDBServer::IDBConnect
     IDBTransactionInfo result((IDBResourceIdentifier(connection)));
     result.m_mode = IDBTransactionMode::Versionchange;
     result.m_newVersion = newVersion;
-    result.m_originalDatabaseInfo = std::make_unique<IDBDatabaseInfo>(originalDatabaseInfo);
+    result.m_originalDatabaseInfo = makeUnique<IDBDatabaseInfo>(originalDatabaseInfo);
 
     return result;
 }
@@ -67,7 +67,7 @@ IDBTransactionInfo::IDBTransactionInfo(const IDBTransactionInfo& info)
     , m_objectStores(info.m_objectStores)
 {
     if (info.m_originalDatabaseInfo)
-        m_originalDatabaseInfo = std::make_unique<IDBDatabaseInfo>(*info.m_originalDatabaseInfo);
+        m_originalDatabaseInfo = makeUnique<IDBDatabaseInfo>(*info.m_originalDatabaseInfo);
 }
 
 IDBTransactionInfo::IDBTransactionInfo(const IDBTransactionInfo& that, IsolatedCopyTag)
@@ -91,7 +91,7 @@ void IDBTransactionInfo::isolatedCopy(const IDBTransactionInfo& source, IDBTrans
         destination.m_objectStores.uncheckedAppend(objectStore.isolatedCopy());
 
     if (source.m_originalDatabaseInfo)
-        destination.m_originalDatabaseInfo = std::make_unique<IDBDatabaseInfo>(*source.m_originalDatabaseInfo, IDBDatabaseInfo::IsolatedCopy);
+        destination.m_originalDatabaseInfo = makeUnique<IDBDatabaseInfo>(*source.m_originalDatabaseInfo, IDBDatabaseInfo::IsolatedCopy);
 }
 
 #if !LOG_DISABLED
