@@ -34,10 +34,13 @@
 
 #if ENABLE(WEB_RTC)
 
+#include <wtf/IsoMallocInlines.h>
 #include <wtf/NeverDestroyed.h>
 
 namespace WebCore {
 
+WTF_MAKE_ISO_ALLOCATED_IMPL(RTCRtpTransceiver);
+
 RTCRtpTransceiver::RTCRtpTransceiver(Ref<RTCRtpSender>&& sender, Ref<RTCRtpReceiver>&& receiver, std::unique_ptr<RTCRtpTransceiverBackend>&& backend)
     : m_direction(RTCRtpTransceiverDirection::Sendrecv)
     , m_sender(WTFMove(sender))
@@ -106,6 +109,13 @@ void RTCRtpTransceiver::stop()
         m_backend->stop();
 }
 
+ExceptionOr<void> RTCRtpTransceiver::setCodecPreferences(const Vector<RTCRtpCodecCapability>& codecs)
+{
+    if (!m_backend)
+        return { };
+    return m_backend->setCodecPreferences(codecs);
+}
+
 bool RTCRtpTransceiver::stopped() const
 {
     if (m_backend)
