@@ -31,7 +31,7 @@
 
 ALLOW_UNUSED_PARAMETERS_BEGIN
 
-#include <webrtc/api/datachannelinterface.h>
+#include <webrtc/api/data_channel_interface.h>
 
 ALLOW_UNUSED_PARAMETERS_END
 
@@ -41,6 +41,7 @@ struct DataChannelInit;
 
 namespace WebCore {
 
+class Document;
 class RTCDataChannelEvent;
 class RTCDataChannelHandlerClient;
 struct RTCDataChannelInit;
@@ -53,7 +54,7 @@ public:
     ~LibWebRTCDataChannelHandler();
 
     static webrtc::DataChannelInit fromRTCDataChannelInit(const RTCDataChannelInit&);
-    static Ref<RTCDataChannelEvent> channelEvent(ScriptExecutionContext&, rtc::scoped_refptr<webrtc::DataChannelInterface>&&);
+    static Ref<RTCDataChannelEvent> channelEvent(Document&, rtc::scoped_refptr<webrtc::DataChannelInterface>&&);
 
 private:
     // RTCDataChannelHandler API
