@@ -27,34 +27,19 @@
 // @conditional=ENABLE(STREAMS_API)
 // @internal
 
-function privateInitializeReadableStreamDefaultReader(stream)
-{
-    "use strict";
-
-    if (!@isReadableStream(stream))
-       @throwTypeError("ReadableStreamDefaultReader needs a ReadableStream");
-    if (@isReadableStreamLocked(stream))
-       @throwTypeError("ReadableStream is locked");
-
-    @readableStreamReaderGenericInitialize(this, stream);
-    this.@readRequests = [];
-
-    return this;
-}
-
 function readableStreamReaderGenericInitialize(reader, stream)
 {
     "use strict";
 
-    reader.@ownerReadableStream = stream;
-    stream.@reader = reader;
-    if (stream.@state === @streamReadable)
-        reader.@closedPromiseCapability = @newPromiseCapability(@Promise);
-    else if (stream.@state === @streamClosed)
-        reader.@closedPromiseCapability = { @promise: @Promise.@resolve() };
+    @putByIdDirectPrivate(reader, "ownerReadableStream", stream);
+    @putByIdDirectPrivate(stream, "reader", reader);
+    if (@getByIdDirectPrivate(stream, "state") === @streamReadable)
+        @putByIdDirectPrivate(reader, "closedPromiseCapability", @newPromiseCapability(@Promise));
+    else if (@getByIdDirectPrivate(stream, "state") === @streamClosed)
+        @putByIdDirectPrivate(reader, "closedPromiseCapability", { @promise: @Promise.@resolve() });
     else {
-        @assert(stream.@state === @streamErrored);
-        reader.@closedPromiseCapability = { @promise: @newHandledRejectedPromise(stream.@storedError) };
+        @assert(@getByIdDirectPrivate(stream, "state") === @streamErrored);
+        @putByIdDirectPrivate(reader, "closedPromiseCapability", { @promise: @newHandledRejectedPromise(@getByIdDirectPrivate(stream, "storedError")) });
     }
 }
 
@@ -66,43 +51,55 @@ function privateInitializeReadableStreamDefaultController(stream, underlyingSour
         @throwTypeError("ReadableStreamDefaultController needs a ReadableStream");
 
     // readableStreamController is initialized with null value.
-    if (stream.@readableStreamController !== null)
+    if (@getByIdDirectPrivate(stream, "readableStreamController") !== null)
         @throwTypeError("ReadableStream already has a controller");
 
-    this.@controlledReadableStream = stream;
-    this.@underlyingSource = underlyingSource;
-    this.@queue = @newQueue();
-    this.@started = false;
-    this.@closeRequested = false;
-    this.@pullAgain = false;
-    this.@pulling = false;
-    this.@strategy = @validateAndNormalizeQueuingStrategy(size, highWaterMark);
-
-    const controller = this;
-    @promiseInvokeOrNoopNoCatch(underlyingSource, "start", [this]).@then(() => {
-        controller.@started = true;
-        @assert(!controller.@pulling);
-        @assert(!controller.@pullAgain);
-        @readableStreamDefaultControllerCallPullIfNeeded(controller);
-    }, (error) => {
-        if (stream.@state === @streamReadable)
-            @readableStreamDefaultControllerError(controller, error);
-    });
-
-    this.@cancel = @readableStreamDefaultControllerCancel;
-
-    this.@pull = @readableStreamDefaultControllerPull;
+    @putByIdDirectPrivate(this, "controlledReadableStream", stream);
+    @putByIdDirectPrivate(this, "underlyingSource", underlyingSource);
+    @putByIdDirectPrivate(this, "queue", @newQueue());
+    @putByIdDirectPrivate(this, "started", false);
+    @putByIdDirectPrivate(this, "closeRequested", false);
+    @putByIdDirectPrivate(this, "pullAgain", false);
+    @putByIdDirectPrivate(this, "pulling", false);
+    @putByIdDirectPrivate(this, "strategy", @validateAndNormalizeQueuingStrategy(size, highWaterMark));
 
     return this;
 }
 
+// https://streams.spec.whatwg.org/#set-up-readable-stream-default-controller, starting from step 6.
+// The other part is implemented in privateInitializeReadableStreamDefaultController.
+function setupReadableStreamDefaultController(stream, underlyingSource, size, highWaterMark, startMethod, pullMethod, cancelMethod)
+{
+    "use strict";
+    const controller = new @ReadableStreamDefaultController(stream, underlyingSource, size, highWaterMark, @isReadableStream);
+    const startAlgorithm = () => @promiseInvokeOrNoopMethodNoCatch(underlyingSource, startMethod, [controller]);
+    const pullAlgorithm = () => @promiseInvokeOrNoopMethod(underlyingSource, pullMethod, [controller]);
+    const cancelAlgorithm = (reason) => @promiseInvokeOrNoopMethod(underlyingSource, cancelMethod, [reason]);
+
+    @putByIdDirectPrivate(controller, "pullAlgorithm", pullAlgorithm);
+    @putByIdDirectPrivate(controller, "cancelAlgorithm", cancelAlgorithm);
+    @putByIdDirectPrivate(controller, "pull", @readableStreamDefaultControllerPull);
+    @putByIdDirectPrivate(controller, "cancel", @readableStreamDefaultControllerCancel);
+    @putByIdDirectPrivate(stream, "readableStreamController", controller);
+
+    startAlgorithm().@then(() => {
+        @putByIdDirectPrivate(controller, "started", true);
+        @assert(!@getByIdDirectPrivate(controller, "pulling"));
+        @assert(!@getByIdDirectPrivate(controller, "pullAgain"));
+        @readableStreamDefaultControllerCallPullIfNeeded(controller);
+    }, (error) => {
+        @readableStreamDefaultControllerError(controller, error);
+    });
+}
+
 function readableStreamDefaultControllerError(controller, error)
 {
     "use strict";
 
-    const stream = controller.@controlledReadableStream;
-    @assert(stream.@state === @streamReadable);
-    controller.@queue = @newQueue();
+    const stream = @getByIdDirectPrivate(controller, "controlledReadableStream");
+    if (@getByIdDirectPrivate(stream, "state") !== @streamReadable)
+        return;
+    @putByIdDirectPrivate(controller, "queue", @newQueue());
     @readableStreamError(stream, error);
 }
 
@@ -113,7 +110,7 @@ function readableStreamPipeTo(stream, sink)
 
     const reader = new @ReadableStreamDefaultReader(stream);
 
-    reader.@closedPromiseCapability.@promise.@then(() => { }, (e) => { sink.error(e); });
+    @getByIdDirectPrivate(reader, "closedPromiseCapability").@promise.@then(() => { }, (e) => { sink.error(e); });
 
     function doPipe() {
         @readableStreamDefaultReaderRead(reader).@then(function(result) {
@@ -128,6 +125,8 @@ function readableStreamPipeTo(stream, sink)
                 return;
             }
             doPipe();
+        }, function(e) {
+            sink.error(e);
         });
     }
     doPipe();
@@ -150,20 +149,22 @@ function readableStreamTee(stream, shouldClone)
         reason2: @undefined,
     };
 
-    teeState.cancelPromiseCapability = @newPromiseCapability(@InternalPromise);
+    teeState.cancelPromiseCapability = @newPromiseCapability(@Promise);
 
     const pullFunction = @readableStreamTeePullFunction(teeState, reader, shouldClone);
 
-    const branch1 = new @ReadableStream({
-        "pull": pullFunction,
-        "cancel": @readableStreamTeeBranch1CancelFunction(teeState, stream)
-    });
-    const branch2 = new @ReadableStream({
-        "pull": pullFunction,
-        "cancel": @readableStreamTeeBranch2CancelFunction(teeState, stream)
-    });
+    const branch1Source = { };
+    @putByIdDirectPrivate(branch1Source, "pull", pullFunction);
+    @putByIdDirectPrivate(branch1Source, "cancel", @readableStreamTeeBranch1CancelFunction(teeState, stream));
 
-    reader.@closedPromiseCapability.@promise.@then(@undefined, function(e) {
+    const branch2Source = { };
+    @putByIdDirectPrivate(branch2Source, "pull", pullFunction);
+    @putByIdDirectPrivate(branch2Source, "cancel", @readableStreamTeeBranch2CancelFunction(teeState, stream));
+
+    const branch1 = new @ReadableStream(branch1Source);
+    const branch2 = new @ReadableStream(branch2Source);
+
+    @getByIdDirectPrivate(reader, "closedPromiseCapability").@promise.@then(@undefined, function(e) {
         if (teeState.closedOrErrored)
             return;
         @readableStreamDefaultControllerError(branch1.@readableStreamController, e);
@@ -258,7 +259,7 @@ function isReadableStream(stream)
     // Spec tells to return true only if stream has a readableStreamController internal slot.
     // However, since it is a private slot, it cannot be checked using hasOwnProperty().
     // Therefore, readableStreamController is initialized with null value.
-    return @isObject(stream) && stream.@readableStreamController !== @undefined;
+    return @isObject(stream) && @getByIdDirectPrivate(stream, "readableStreamController") !== @undefined;
 }
 
 function isReadableStreamDefaultReader(reader)
@@ -268,7 +269,7 @@ function isReadableStreamDefaultReader(reader)
     // Spec tells to return true only if reader has a readRequests internal slot.
     // However, since it is a private slot, it cannot be checked using hasOwnProperty().
     // Since readRequests is initialized with an empty array, the following test is ok.
-    return @isObject(reader) && !!reader.@readRequests;
+    return @isObject(reader) && !!@getByIdDirectPrivate(reader, "readRequests");
 }
 
 function isReadableStreamDefaultController(controller)
@@ -279,7 +280,7 @@ function isReadableStreamDefaultController(controller)
     // However, since it is a private slot, it cannot be checked using hasOwnProperty().
     // underlyingSource is obtained in ReadableStream constructor: if undefined, it is set
     // to an empty object. Therefore, following test is ok.
-    return @isObject(controller) && !!controller.@underlyingSource;
+    return @isObject(controller) && !!@getByIdDirectPrivate(controller, "underlyingSource");
 }
 
 function readableStreamError(stream, error)
@@ -287,62 +288,62 @@ function readableStreamError(stream, error)
     "use strict";
 
     @assert(@isReadableStream(stream));
-    @assert(stream.@state === @streamReadable);
-    stream.@state = @streamErrored;
-    stream.@storedError = error;
+    @assert(@getByIdDirectPrivate(stream, "state") === @streamReadable);
+    @putByIdDirectPrivate(stream, "state", @streamErrored);
+    @putByIdDirectPrivate(stream, "storedError", error);
 
-    if (!stream.@reader)
+    if (!@getByIdDirectPrivate(stream, "reader"))
         return;
 
-    const reader = stream.@reader;
+    const reader = @getByIdDirectPrivate(stream, "reader");
 
     if (@isReadableStreamDefaultReader(reader)) {
-        const requests = reader.@readRequests;
+        const requests = @getByIdDirectPrivate(reader, "readRequests");
         for (let index = 0, length = requests.length; index < length; ++index)
             requests[index].@reject.@call(@undefined, error);
-        reader.@readRequests = [];
+        @putByIdDirectPrivate(reader, "readRequests", []);
     } else {
         @assert(@isReadableStreamBYOBReader(reader));
-        const requests = reader.@readIntoRequests;
+        const requests = @getByIdDirectPrivate(reader, "readIntoRequests");
         for (let index = 0, length = requests.length; index < length; ++index)
             requests[index].@reject.@call(@undefined, error);
-        reader.@readIntoRequests = [];
+        @putByIdDirectPrivate(reader, "readIntoRequests", []);
     }
 
-    reader.@closedPromiseCapability.@reject.@call(@undefined, error);
-    reader.@closedPromiseCapability.@promise.@promiseIsHandled = true;
+    @getByIdDirectPrivate(reader, "closedPromiseCapability").@reject.@call(@undefined, error);
+    const promise = @getByIdDirectPrivate(reader, "closedPromiseCapability").@promise;
+    @putPromiseInternalField(promise, @promiseFieldFlags, @getPromiseInternalField(promise, @promiseFieldFlags) | @promiseFlagsIsHandled);
 }
 
 function readableStreamDefaultControllerCallPullIfNeeded(controller)
 {
     "use strict";
 
-    const stream = controller.@controlledReadableStream;
+    const stream = @getByIdDirectPrivate(controller, "controlledReadableStream");
 
     if (!@readableStreamDefaultControllerCanCloseOrEnqueue(controller))
         return;
-    if (!controller.@started)
+    if (!@getByIdDirectPrivate(controller, "started"))
         return;
-    if ((!@isReadableStreamLocked(stream) || !stream.@reader.@readRequests.length) && @readableStreamDefaultControllerGetDesiredSize(controller) <= 0)
+    if ((!@isReadableStreamLocked(stream) || !@getByIdDirectPrivate(@getByIdDirectPrivate(stream, "reader"), "readRequests").length) && @readableStreamDefaultControllerGetDesiredSize(controller) <= 0)
         return;
 
-    if (controller.@pulling) {
-        controller.@pullAgain = true;
+    if (@getByIdDirectPrivate(controller, "pulling")) {
+        @putByIdDirectPrivate(controller, "pullAgain", true);
         return;
     }
 
-    @assert(!controller.@pullAgain);
-    controller.@pulling = true;
+    @assert(!@getByIdDirectPrivate(controller, "pullAgain"));
+    @putByIdDirectPrivate(controller, "pulling", true);
 
-    @promiseInvokeOrNoop(controller.@underlyingSource, "pull", [controller]).@then(function() {
-        controller.@pulling = false;
-        if (controller.@pullAgain) {
-            controller.@pullAgain = false;
+    @getByIdDirectPrivate(controller, "pullAlgorithm").@call(@undefined).@then(function() {
+        @putByIdDirectPrivate(controller, "pulling", false);
+        if (@getByIdDirectPrivate(controller, "pullAgain")) {
+            @putByIdDirectPrivate(controller, "pullAgain", false);
             @readableStreamDefaultControllerCallPullIfNeeded(controller);
         }
     }, function(error) {
-        if (stream.@state === @streamReadable)
-            @readableStreamDefaultControllerError(controller, error);
+        @readableStreamDefaultControllerError(controller, error);
     });
 }
 
@@ -351,21 +352,22 @@ function isReadableStreamLocked(stream)
    "use strict";
 
     @assert(@isReadableStream(stream));
-    return !!stream.@reader;
+    return !!@getByIdDirectPrivate(stream, "reader");
 }
 
 function readableStreamDefaultControllerGetDesiredSize(controller)
 {
    "use strict";
 
-   const stream = controller.@controlledReadableStream;
+    const stream = @getByIdDirectPrivate(controller, "controlledReadableStream");
+    const state = @getByIdDirectPrivate(stream, "state");
 
-   if (stream.@state === @streamErrored)
-       return null;
-   if (stream.@state === @streamClosed)
-       return 0;
+    if (state === @streamErrored)
+        return null;
+    if (state === @streamClosed)
+        return 0;
 
-   return controller.@strategy.highWaterMark - controller.@queue.size;
+    return @getByIdDirectPrivate(controller, "strategy").highWaterMark - @getByIdDirectPrivate(controller, "queue").size;
 }
 
 
@@ -373,7 +375,7 @@ function readableStreamReaderGenericCancel(reader, reason)
 {
     "use strict";
 
-    const stream = reader.@ownerReadableStream;
+    const stream = @getByIdDirectPrivate(reader, "ownerReadableStream");
     @assert(!!stream);
     return @readableStreamCancel(stream, reason);
 }
@@ -382,31 +384,32 @@ function readableStreamCancel(stream, reason)
 {
     "use strict";
 
-    stream.@disturbed = true;
-    if (stream.@state === @streamClosed)
+    @putByIdDirectPrivate(stream, "disturbed", true);
+    const state = @getByIdDirectPrivate(stream, "state");
+    if (state === @streamClosed)
         return @Promise.@resolve();
-    if (stream.@state === @streamErrored)
-        return @Promise.@reject(stream.@storedError);
+    if (state === @streamErrored)
+        return @Promise.@reject(@getByIdDirectPrivate(stream, "storedError"));
     @readableStreamClose(stream);
-    return stream.@readableStreamController.@cancel(stream.@readableStreamController, reason).@then(function() {  });
+    return @getByIdDirectPrivate(stream, "readableStreamController").@cancel(@getByIdDirectPrivate(stream, "readableStreamController"), reason).@then(function() {  });
 }
 
 function readableStreamDefaultControllerCancel(controller, reason)
 {
     "use strict";
 
-    controller.@queue = @newQueue();
-    return @promiseInvokeOrNoop(controller.@underlyingSource, "cancel", [reason]);
+    @putByIdDirectPrivate(controller, "queue", @newQueue());
+    return @getByIdDirectPrivate(controller, "cancelAlgorithm").@call(@undefined, reason);
 }
 
 function readableStreamDefaultControllerPull(controller)
 {
     "use strict";
 
-    const stream = controller.@controlledReadableStream;
-    if (controller.@queue.content.length) {
-        const chunk = @dequeueValue(controller.@queue);
-        if (controller.@closeRequested && controller.@queue.content.length === 0)
+    const stream = @getByIdDirectPrivate(controller, "controlledReadableStream");
+    if (@getByIdDirectPrivate(controller, "queue").content.length) {
+        const chunk = @dequeueValue(@getByIdDirectPrivate(controller, "queue"));
+        if (@getByIdDirectPrivate(controller, "closeRequested") && @getByIdDirectPrivate(controller, "queue").content.length === 0)
             @readableStreamClose(stream);
         else
             @readableStreamDefaultControllerCallPullIfNeeded(controller);
@@ -422,47 +425,47 @@ function readableStreamDefaultControllerClose(controller)
     "use strict";
 
     @assert(@readableStreamDefaultControllerCanCloseOrEnqueue(controller));
-    controller.@closeRequested = true;
-    if (controller.@queue.content.length === 0)
-        @readableStreamClose(controller.@controlledReadableStream);
+    @putByIdDirectPrivate(controller, "closeRequested", true);
+    if (@getByIdDirectPrivate(controller, "queue").content.length === 0)
+        @readableStreamClose(@getByIdDirectPrivate(controller, "controlledReadableStream"));
 }
 
 function readableStreamClose(stream)
 {
     "use strict";
 
-    @assert(stream.@state === @streamReadable);
-    stream.@state = @streamClosed;
-    const reader = stream.@reader;
+    @assert(@getByIdDirectPrivate(stream, "state") === @streamReadable);
+    @putByIdDirectPrivate(stream, "state", @streamClosed);
+    const reader = @getByIdDirectPrivate(stream, "reader");
 
     if (!reader)
         return;
 
     if (@isReadableStreamDefaultReader(reader)) {
-        const requests = reader.@readRequests;
+        const requests = @getByIdDirectPrivate(reader, "readRequests");
         for (let index = 0, length = requests.length; index < length; ++index)
             requests[index].@resolve.@call(@undefined, {value:@undefined, done: true});
-        reader.@readRequests = [];
+        @putByIdDirectPrivate(reader, "readRequests", []);
     }
 
-    reader.@closedPromiseCapability.@resolve.@call();
+    @getByIdDirectPrivate(reader, "closedPromiseCapability").@resolve.@call();
 }
 
 function readableStreamFulfillReadRequest(stream, chunk, done)
 {
     "use strict";
 
-    stream.@reader.@readRequests.@shift().@resolve.@call(@undefined, {value: chunk, done: done});
+    @getByIdDirectPrivate(@getByIdDirectPrivate(stream, "reader"), "readRequests").@shift().@resolve.@call(@undefined, {value: chunk, done: done});
 }
 
 function readableStreamDefaultControllerEnqueue(controller, chunk)
 {
     "use strict";
 
-    const stream = controller.@controlledReadableStream;
+    const stream = @getByIdDirectPrivate(controller, "controlledReadableStream");
     @assert(@readableStreamDefaultControllerCanCloseOrEnqueue(controller));
 
-    if (@isReadableStreamLocked(stream) && stream.@reader.@readRequests.length) {
+    if (@isReadableStreamLocked(stream) && @getByIdDirectPrivate(@getByIdDirectPrivate(stream, "reader"), "readRequests").length) {
         @readableStreamFulfillReadRequest(stream, chunk, false);
         @readableStreamDefaultControllerCallPullIfNeeded(controller);
         return;
@@ -470,13 +473,12 @@ function readableStreamDefaultControllerEnqueue(controller, chunk)
 
     try {
         let chunkSize = 1;
-        if (controller.@strategy.size !== @undefined)
-            chunkSize = controller.@strategy.size(chunk);
-        @enqueueValueWithSize(controller.@queue, chunk, chunkSize);
+        if (@getByIdDirectPrivate(controller, "strategy").size !== @undefined)
+            chunkSize = @getByIdDirectPrivate(controller, "strategy").size(chunk);
+        @enqueueValueWithSize(@getByIdDirectPrivate(controller, "queue"), chunk, chunkSize);
     }
     catch(error) {
-        if (stream.@state === @streamReadable)
-            @readableStreamDefaultControllerError(controller, error);
+        @readableStreamDefaultControllerError(controller, error);
         throw error;
     }
     @readableStreamDefaultControllerCallPullIfNeeded(controller);
@@ -486,28 +488,29 @@ function readableStreamDefaultReaderRead(reader)
 {
     "use strict";
 
-    const stream = reader.@ownerReadableStream;
+    const stream = @getByIdDirectPrivate(reader, "ownerReadableStream");
     @assert(!!stream);
+    const state = @getByIdDirectPrivate(stream, "state");
 
-    stream.@disturbed = true;
-    if (stream.@state === @streamClosed)
+    @putByIdDirectPrivate(stream, "disturbed", true);
+    if (state === @streamClosed)
         return @Promise.@resolve({value: @undefined, done: true});
-    if (stream.@state === @streamErrored)
-        return @Promise.@reject(stream.@storedError);
-    @assert(stream.@state === @streamReadable);
+    if (state === @streamErrored)
+        return @Promise.@reject(@getByIdDirectPrivate(stream, "storedError"));
+    @assert(state === @streamReadable);
 
-    return stream.@readableStreamController.@pull(stream.@readableStreamController);
+    return @getByIdDirectPrivate(stream, "readableStreamController").@pull(@getByIdDirectPrivate(stream, "readableStreamController"));
 }
 
 function readableStreamAddReadRequest(stream)
 {
     "use strict";
 
-    @assert(@isReadableStreamDefaultReader(stream.@reader));
-    @assert(stream.@state == @streamReadable);
+    @assert(@isReadableStreamDefaultReader(@getByIdDirectPrivate(stream, "reader")));
+    @assert(@getByIdDirectPrivate(stream, "state") == @streamReadable);
 
     const readRequest = @newPromiseCapability(@Promise);
-    stream.@reader.@readRequests.@push(readRequest);
+    @getByIdDirectPrivate(@getByIdDirectPrivate(stream, "reader"), "readRequests").@push(readRequest);
 
     return readRequest.@promise;
 }
@@ -517,29 +520,30 @@ function isReadableStreamDisturbed(stream)
     "use strict";
 
     @assert(@isReadableStream(stream));
-    return stream.@disturbed;
+    return @getByIdDirectPrivate(stream, "disturbed");
 }
 
 function readableStreamReaderGenericRelease(reader)
 {
     "use strict";
 
-    @assert(!!reader.@ownerReadableStream);
-    @assert(reader.@ownerReadableStream.@reader === reader);
+    @assert(!!@getByIdDirectPrivate(reader, "ownerReadableStream"));
+    @assert(@getByIdDirectPrivate(@getByIdDirectPrivate(reader, "ownerReadableStream"), "reader") === reader);
 
-    if (reader.@ownerReadableStream.@state === @streamReadable)
-        reader.@closedPromiseCapability.@reject.@call(@undefined, new @TypeError("releasing lock of reader whose stream is still in readable state"));
+    if (@getByIdDirectPrivate(@getByIdDirectPrivate(reader, "ownerReadableStream"), "state") === @streamReadable)
+        @getByIdDirectPrivate(reader, "closedPromiseCapability").@reject.@call(@undefined, @makeTypeError("releasing lock of reader whose stream is still in readable state"));
     else
-        reader.@closedPromiseCapability = { @promise: @newHandledRejectedPromise(new @TypeError("reader released lock")) };
+        @putByIdDirectPrivate(reader, "closedPromiseCapability", { @promise: @newHandledRejectedPromise(@makeTypeError("reader released lock")) });
 
-    reader.@closedPromiseCapability.@promise.@promiseIsHandled = true;
-    reader.@ownerReadableStream.@reader = @undefined;
-    reader.@ownerReadableStream = @undefined;
+    const promise = @getByIdDirectPrivate(reader, "closedPromiseCapability").@promise;
+    @putPromiseInternalField(promise, @promiseFieldFlags, @getPromiseInternalField(promise, @promiseFieldFlags) | @promiseFlagsIsHandled);
+    @putByIdDirectPrivate(@getByIdDirectPrivate(reader, "ownerReadableStream"), "reader", @undefined);
+    @putByIdDirectPrivate(reader, "ownerReadableStream", @undefined);
 }
 
 function readableStreamDefaultControllerCanCloseOrEnqueue(controller)
 {
     "use strict";
 
-    return !controller.@closeRequested && controller.@controlledReadableStream.@state === @streamReadable;
+    return !@getByIdDirectPrivate(controller, "closeRequested") && @getByIdDirectPrivate(@getByIdDirectPrivate(controller, "controlledReadableStream"), "state") === @streamReadable;
 }
