@@ -28,6 +28,7 @@
 #include "EventTarget.h"
 #include "ExceptionOr.h"
 #include <wtf/Forward.h>
+#include <wtf/LoggerHelper.h>
 
 #define DEBUG_AUDIONODE_REFERENCES 0
 
@@ -44,9 +45,14 @@ class AudioParam;
 // An AudioDestinationNode has one input and no outputs and represents the final destination to the audio hardware.
 // Most processing nodes such as filters will have one input and one output, although multiple inputs and outputs are possible.
 
-class AudioNode : public EventTargetWithInlineData {
+class AudioNode
+    : public EventTargetWithInlineData
+#if !RELEASE_LOG_DISABLED
+    , private LoggerHelper
+#endif
+{
     WTF_MAKE_NONCOPYABLE(AudioNode);
-    WTF_MAKE_FAST_ALLOCATED;
+    WTF_MAKE_ISO_ALLOCATED(AudioNode);
 public:
     enum { ProcessingSizeInFrames = 128 };
 
@@ -75,6 +81,7 @@ public:
         NodeTypeAnalyser,
         NodeTypeDynamicsCompressor,
         NodeTypeWaveShaper,
+        NodeTypeBasicInspector,
         NodeTypeEnd
     };
 
@@ -98,6 +105,7 @@ public:
 
     // Can be called from main thread or context's audio thread.  It must be called while the context's graph lock is held.
     void finishDeref(RefType refType);
+    virtual void didBecomeMarkedForDeletion() { }
 
     // The AudioNodeInput(s) (if any) will already have their input data available when process() is called.
     // Subclasses will take this input data and put the results in the AudioBus(s) of its AudioNodeOutput(s) (if any).
@@ -174,10 +182,6 @@ public:
     ChannelCountMode internalChannelCountMode() const { return m_channelCountMode; }
     AudioBus::ChannelInterpretation internalChannelInterpretation() const { return m_channelInterpretation; }
 
-    // EventTarget
-    EventTargetInterface eventTargetInterface() const override;
-    ScriptExecutionContext* scriptExecutionContext() const final;
-
 protected:
     // Inputs and outputs must be created before the AudioNode is initialized.
     void addInput(std::unique_ptr<AudioNodeInput>);
@@ -191,7 +195,18 @@ protected:
     // Force all inputs to take any channel interpretation changes into account.
     void updateChannelsForInputs();
 
+#if !RELEASE_LOG_DISABLED
+    const Logger& logger() const final { return m_logger.get(); }
+    const void* logIdentifier() const final { return m_logIdentifier; }
+    const char* logClassName() const final { return "AudioNode"; }
+    WTFLogChannel& logChannel() const final;
+#endif
+
 private:
+    // EventTarget
+    EventTargetInterface eventTargetInterface() const override;
+    ScriptExecutionContext* scriptExecutionContext() const final;
+
     volatile bool m_isInitialized;
     NodeType m_nodeType;
     Ref<AudioContext> m_context;
@@ -217,10 +232,25 @@ private:
     void refEventTarget() override { ref(); }
     void derefEventTarget() override { deref(); }
 
+#if !RELEASE_LOG_DISABLED
+    mutable Ref<const Logger> m_logger;
+    const void* m_logIdentifier;
+#endif
+
 protected:
     unsigned m_channelCount;
     ChannelCountMode m_channelCountMode;
     AudioBus::ChannelInterpretation m_channelInterpretation;
 };
 
+String convertEnumerationToString(AudioNode::NodeType);
+
 } // namespace WebCore
+
+namespace WTF {
+
+template<> struct LogArgument<WebCore::AudioNode::NodeType> {
+    static String toString(WebCore::AudioNode::NodeType type) { return convertEnumerationToString(type); }
+};
+
+} // namespace WTF
