@@ -25,24 +25,30 @@
 #pragma once
 
 #include "AudioNode.h"
+#include "ChannelSplitterOptions.h"
 
 namespace WebCore {
 
 class AudioContext;
 
 class ChannelSplitterNode final : public AudioNode {
+    WTF_MAKE_ISO_ALLOCATED(ChannelSplitterNode);
 public:
-    static RefPtr<ChannelSplitterNode> create(AudioContext&, float sampleRate, unsigned numberOfOutputs);
+    static ExceptionOr<Ref<ChannelSplitterNode>> create(BaseAudioContext&, const ChannelSplitterOptions& = { });
 
     // AudioNode
     void process(size_t framesToProcess) override;
     void reset() override;
 
+    ExceptionOr<void> setChannelCount(unsigned) final;
+    ExceptionOr<void> setChannelCountMode(ChannelCountMode) final;
+    ExceptionOr<void> setChannelInterpretation(ChannelInterpretation) final;
+
 private:
     double tailTime() const override { return 0; }
     double latencyTime() const override { return 0; }
 
-    ChannelSplitterNode(AudioContext&, float sampleRate, unsigned numberOfOutputs);
+    ChannelSplitterNode(BaseAudioContext&, unsigned numberOfOutputs);
 };
 
 } // namespace WebCore
