@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2011, Google Inc. All rights reserved.
+ * Copyright (C) 2020, Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,7 +32,9 @@
 #include "AudioContext.h"
 #include "AudioNodeOutput.h"
 #include "Logging.h"
+#include "MediaElementAudioSourceOptions.h"
 #include "MediaPlayer.h"
+#include <wtf/IsoMallocInlines.h>
 #include <wtf/Locker.h>
 
 // These are somewhat arbitrary limits, but we need to do some kind of sanity-checking.
@@ -40,22 +43,37 @@ const unsigned maxSampleRate = 192000;
 
 namespace WebCore {
 
-Ref<MediaElementAudioSourceNode> MediaElementAudioSourceNode::create(AudioContext& context, HTMLMediaElement& mediaElement)
+WTF_MAKE_ISO_ALLOCATED_IMPL(MediaElementAudioSourceNode);
+
+ExceptionOr<Ref<MediaElementAudioSourceNode>> MediaElementAudioSourceNode::create(BaseAudioContext& context, MediaElementAudioSourceOptions&& options)
 {
-    return adoptRef(*new MediaElementAudioSourceNode(context, mediaElement));
+    RELEASE_ASSERT(options.mediaElement);
+
+    if (context.isStopped())
+        return Exception { InvalidStateError };
+
+    if (options.mediaElement->audioSourceNode())
+        return Exception { InvalidStateError, "Media element is already associated with an audio source node"_s };
+
+    context.lazyInitialize();
+
+    auto node = adoptRef(*new MediaElementAudioSourceNode(context, *options.mediaElement));
+
+    options.mediaElement->setAudioSourceNode(node.ptr());
+    context.refNode(node.get()); // context keeps reference until node is disconnected
+
+    return node;
 }
 
-MediaElementAudioSourceNode::MediaElementAudioSourceNode(AudioContext& context, HTMLMediaElement& mediaElement)
-    : AudioNode(context, context.sampleRate())
-    , m_mediaElement(mediaElement)
-    , m_sourceNumberOfChannels(0)
-    , m_sourceSampleRate(0)
+MediaElementAudioSourceNode::MediaElementAudioSourceNode(BaseAudioContext& context, Ref<HTMLMediaElement>&& mediaElement)
+    : AudioNode(context)
+    , m_mediaElement(WTFMove(mediaElement))
 {
-    // Default to stereo. This could change depending on what the media element .src is set to.
-    addOutput(std::make_unique<AudioNodeOutput>(this, 2));
-
     setNodeType(NodeTypeMediaElementAudioSource);
 
+    // Default to stereo. This could change depending on what the media element .src is set to.
+    addOutput(makeUnique<AudioNodeOutput>(this, 2));
+
     initialize();
 }
 
@@ -82,11 +100,11 @@ void MediaElementAudioSourceNode::setFormat(size_t numberOfChannels, float sourc
         m_sourceSampleRate = sourceSampleRate;
 
         // Synchronize with process().
-        std::lock_guard<MediaElementAudioSourceNode> lock(*this);
+        auto locker = holdLock(*this);
 
         if (sourceSampleRate != sampleRate()) {
             double scaleFactor = sourceSampleRate / sampleRate();
-            m_multiChannelResampler = std::make_unique<MultiChannelResampler>(scaleFactor, numberOfChannels);
+            m_multiChannelResampler = makeUnique<MultiChannelResampler>(scaleFactor, numberOfChannels);
         } else {
             // Bypass resampling.
             m_multiChannelResampler = nullptr;
@@ -94,7 +112,7 @@ void MediaElementAudioSourceNode::setFormat(size_t numberOfChannels, float sourc
 
         {
             // The context must be locked when changing the number of output channels.
-            AudioContext::AutoLocker contextLocker(context());
+            BaseAudioContext::AutoLocker contextLocker(context());
 
             // Do any necesssary re-configuration to the output's number of channels.
             output(0)->setNumberOfChannels(numberOfChannels);
@@ -110,10 +128,8 @@ bool MediaElementAudioSourceNode::wouldTaintOrigin()
     if (m_mediaElement->didPassCORSAccessCheck())
         return false;
 
-    if (auto* scriptExecutionContext = context().scriptExecutionContext()) {
-        if (auto* origin = scriptExecutionContext->securityOrigin())
-            return m_mediaElement->wouldTaintOrigin(*origin);
-    }
+    if (auto* origin = context().origin())
+        return m_mediaElement->wouldTaintOrigin(*origin);
 
     return true;
 }
@@ -136,6 +152,10 @@ void MediaElementAudioSourceNode::process(size_t numberOfFrames)
         outputBus->zero();
         return;
     }
+    if (m_sourceNumberOfChannels != outputBus->numberOfChannels()) {
+        outputBus->zero();
+        return;
+    }
 
     if (AudioSourceProvider* provider = mediaElement().audioSourceProvider()) {
         if (m_multiChannelResampler.get()) {
