@@ -35,8 +35,9 @@ class AudioBus;
 class AudioContext;
 
 class OfflineAudioDestinationNode final : public AudioDestinationNode {
+    WTF_MAKE_ISO_ALLOCATED(OfflineAudioDestinationNode);
 public:
-    static Ref<OfflineAudioDestinationNode> create(AudioContext& context, AudioBuffer* renderTarget)
+    static Ref<OfflineAudioDestinationNode> create(BaseAudioContext& context, AudioBuffer* renderTarget)
     {
         return adoptRef(*new OfflineAudioDestinationNode(context, renderTarget));
     }
@@ -49,12 +50,12 @@ public:
 
     // AudioDestinationNode
     void enableInput(const String&) override { }
-    void startRendering() override;
+    ExceptionOr<void> startRendering() override;
 
-    float sampleRate() const override { return m_renderTarget->sampleRate(); }
+    float sampleRate() const final { return m_renderTarget->sampleRate(); }
 
 private:
-    OfflineAudioDestinationNode(AudioContext&, AudioBuffer* renderTarget);
+    OfflineAudioDestinationNode(BaseAudioContext&, AudioBuffer* renderTarget);
 
     // This AudioNode renders into this AudioBuffer.
     RefPtr<AudioBuffer> m_renderTarget;
@@ -65,10 +66,7 @@ private:
     // Rendering thread.
     RefPtr<Thread> m_renderThread;
     bool m_startedRendering;
-    void offlineRender();
-
-    // For completion callback on main thread.
-    void notifyComplete();
+    bool offlineRender();
 };
 
 } // namespace WebCore
