@@ -80,6 +80,11 @@ void DatabaseTracker::initializeTracker(const String& databasePath)
     staticTracker = new DatabaseTracker(databasePath);
 }
 
+bool DatabaseTracker::isInitialized()
+{
+    return !!staticTracker;
+}
+
 DatabaseTracker& DatabaseTracker::singleton()
 {
     if (!staticTracker)
@@ -189,7 +194,7 @@ ExceptionOr<void> DatabaseTracker::canEstablishDatabase(DatabaseContext& context
     if (exception.code() != QuotaExceededError)
         doneCreatingDatabase(origin, name);
 
-    return WTFMove(exception);
+    return exception;
 }
 
 // Note: a thought about performance: hasAdequateQuotaForOrigin() was also
@@ -220,7 +225,7 @@ ExceptionOr<void> DatabaseTracker::retryCanEstablishDatabase(DatabaseContext& co
     ASSERT(exception.code() == QuotaExceededError);
     doneCreatingDatabase(origin, name);
 
-    return WTFMove(exception);
+    return exception;
 }
 
 bool DatabaseTracker::hasEntryForOriginNoLock(const SecurityOriginData& origin)
@@ -271,7 +276,7 @@ unsigned long long DatabaseTracker::maximumSize(Database& database)
 
     unsigned long long quota = quotaNoLock(origin);
     unsigned long long diskUsage = usage(origin);
-    unsigned long long databaseFileSize = SQLiteFileSystem::getDatabaseFileSize(database.fileName());
+    unsigned long long databaseFileSize = SQLiteFileSystem::getDatabaseFileSize(database.fileNameIsolatedCopy());
     ASSERT(databaseFileSize <= diskUsage);
 
     if (diskUsage > quota)
@@ -289,19 +294,7 @@ unsigned long long DatabaseTracker::maximumSize(Database& database)
 
 void DatabaseTracker::closeAllDatabases(CurrentQueryBehavior currentQueryBehavior)
 {
-    Vector<Ref<Database>> openDatabases;
-    {
-        LockHolder openDatabaseMapLock(m_openDatabaseMapGuard);
-        if (!m_openDatabaseMap)
-            return;
-        for (auto& nameMap : m_openDatabaseMap->values()) {
-            for (auto& set : nameMap->values()) {
-                for (auto& database : *set)
-                    openDatabases.append(*database);
-            }
-        }
-    }
-    for (auto& database : openDatabases) {
+    for (auto& database : openDatabases()) {
         if (currentQueryBehavior == CurrentQueryBehavior::Interrupt)
             database->interrupt();
         database->close();
@@ -315,12 +308,7 @@ String DatabaseTracker::originPath(const SecurityOriginData& origin) const
 
 static String generateDatabaseFileName()
 {
-    StringBuilder stringBuilder;
-
-    stringBuilder.append(createCanonicalUUIDString());
-    stringBuilder.appendLiteral(".db");
-
-    return stringBuilder.toString();
+    return makeString(createCanonicalUUIDString(), ".db");
 }
 
 String DatabaseTracker::fullPathForDatabaseNoLock(const SecurityOriginData& origin, const String& name, bool createIfNotExists)
@@ -530,7 +518,25 @@ void DatabaseTracker::setDatabaseDetails(const SecurityOriginData& origin, const
 void DatabaseTracker::doneCreatingDatabase(Database& database)
 {
     LockHolder lockDatabase(m_databaseGuard);
-    doneCreatingDatabase(database.securityOrigin(), database.stringIdentifier());
+    doneCreatingDatabase(database.securityOrigin(), database.stringIdentifierIsolatedCopy());
+}
+
+Vector<Ref<Database>> DatabaseTracker::openDatabases()
+{
+    Vector<Ref<Database>> openDatabases;
+    {
+        LockHolder openDatabaseMapLock(m_openDatabaseMapGuard);
+
+        if (m_openDatabaseMap) {
+            for (auto& nameMap : m_openDatabaseMap->values()) {
+                for (auto& set : nameMap->values()) {
+                    for (auto& database : *set)
+                        openDatabases.append(*database);
+                }
+            }
+        }
+    }
+    return openDatabases;
 }
 
 void DatabaseTracker::addOpenDatabase(Database& database)
@@ -538,7 +544,7 @@ void DatabaseTracker::addOpenDatabase(Database& database)
     LockHolder openDatabaseMapLock(m_openDatabaseMapGuard);
 
     if (!m_openDatabaseMap)
-        m_openDatabaseMap = std::make_unique<DatabaseOriginMap>();
+        m_openDatabaseMap = makeUnique<DatabaseOriginMap>();
 
     auto origin = database.securityOrigin();
 
@@ -548,7 +554,7 @@ void DatabaseTracker::addOpenDatabase(Database& database)
         m_openDatabaseMap->add(origin.isolatedCopy(), nameMap);
     }
 
-    String name = database.stringIdentifier();
+    String name = database.stringIdentifierIsolatedCopy();
     auto* databaseSet = nameMap->get(name);
     if (!databaseSet) {
         databaseSet = new DatabaseSet;
@@ -557,7 +563,7 @@ void DatabaseTracker::addOpenDatabase(Database& database)
 
     databaseSet->add(&database);
 
-    LOG(StorageAPI, "Added open Database %s (%p)\n", database.stringIdentifier().utf8().data(), &database);
+    LOG(StorageAPI, "Added open Database %s (%p)\n", database.stringIdentifierIsolatedCopy().utf8().data(), &database);
 }
 
 void DatabaseTracker::removeOpenDatabase(Database& database)
@@ -575,7 +581,7 @@ void DatabaseTracker::removeOpenDatabase(Database& database)
         return;
     }
 
-    String name = database.stringIdentifier();
+    String name = database.stringIdentifierIsolatedCopy();
     auto* databaseSet = nameMap->get(name);
     if (!databaseSet) {
         ASSERT_NOT_REACHED();
@@ -584,7 +590,7 @@ void DatabaseTracker::removeOpenDatabase(Database& database)
 
     databaseSet->remove(&database);
 
-    LOG(StorageAPI, "Removed open Database %s (%p)\n", database.stringIdentifier().utf8().data(), &database);
+    LOG(StorageAPI, "Removed open Database %s (%p)\n", database.stringIdentifierIsolatedCopy().utf8().data(), &database);
 
     if (!databaseSet->isEmpty())
         return;
@@ -649,11 +655,8 @@ unsigned long long DatabaseTracker::usage(const SecurityOriginData& origin)
 {
     String originPath = this->originPath(origin);
     unsigned long long diskUsage = 0;
-    for (auto& fileName : FileSystem::listDirectory(originPath, "*.db"_s)) {
-        long long size;
-        FileSystem::getFileSize(fileName, size);
-        diskUsage += size;
-    }
+    for (auto& fileName : FileSystem::listDirectory(originPath, "*.db"_s))
+        diskUsage += SQLiteFileSystem::getDatabaseFileSize(fileName);
     return diskUsage;
 }
 
@@ -948,7 +951,7 @@ void DatabaseTracker::recordCreatingDatabase(const SecurityOriginData& origin, c
     // We don't use HashMap::ensure here to avoid making an isolated copy of the origin every time.
     auto* nameSet = m_beingCreated.get(origin);
     if (!nameSet) {
-        auto ownedSet = std::make_unique<HashCountedSet<String>>();
+        auto ownedSet = makeUnique<HashCountedSet<String>>();
         nameSet = ownedSet.get();
         m_beingCreated.add(origin.isolatedCopy(), WTFMove(ownedSet));
     }
@@ -994,7 +997,7 @@ void DatabaseTracker::recordDeletingDatabase(const SecurityOriginData& origin, c
     // We don't use HashMap::ensure here to avoid making an isolated copy of the origin every time.
     auto* nameSet = m_beingDeleted.get(origin);
     if (!nameSet) {
-        auto ownedSet = std::make_unique<HashSet<String>>();
+        auto ownedSet = makeUnique<HashSet<String>>();
         nameSet = ownedSet.get();
         m_beingDeleted.add(origin.isolatedCopy(), WTFMove(ownedSet));
     }
@@ -1207,7 +1210,7 @@ void DatabaseTracker::removeDeletedOpenedDatabases()
                             continue;
 
                         // If this database has been deleted or if its database file no longer matches the current version, this database is no longer valid and it should be marked as deleted.
-                        if (databaseFileName.isNull() || databaseFileName != FileSystem::pathGetFileName(db->fileName())) {
+                        if (databaseFileName.isNull() || databaseFileName != FileSystem::pathGetFileName(db->fileNameIsolatedCopy())) {
                             deletedDatabases.append(db);
                             foundDeletedDatabase = true;
                         }
