@@ -41,17 +41,16 @@
 #endif
 #endif
 
-#if OS(WINDOWS)
-#undef WEBCORE_EXPORT
-#if PLATFORM(JAVA)
-#define WEBCORE_EXPORT
-#else
-#define WEBCORE_EXPORT WTF_EXPORT_DECLARATION
-#endif
-
-#else
+#if !OS(WINDOWS) && !PLATFORM(JAVA)
 #include <pthread.h>
-#endif // OS(WINDOWS)
+#endif // !OS(WINDOWS)
+
+#if PLATFORM(JAVA)
+#undef WEBCORE_EXPORT
+#define WEBCORE_EXPORT
+#undef WEBCORE_TESTSUPPORT_EXPORT
+#define WEBCORE_TESTSUPPORT_EXPORT WTF_EXPORT_DECLARATION
+#endif
 
 #include <sys/types.h>
 #include <fcntl.h>
@@ -154,6 +153,11 @@
 #endif
 
 #if PLATFORM(MAC)
+#if !USE(APPLE_INTERNAL_SDK)
+/* SecTrustedApplication.h declares SecTrustedApplicationCreateFromPath(...) to
+ * be unavailable on macOS, so do not include that header. */
+#define _SECURITY_SECTRUSTEDAPPLICATION_H_
+#endif
 #include <CoreServices/CoreServices.h>
 #endif
 
@@ -165,7 +169,6 @@
 #else
 #if USE(APPKIT)
 #import <Cocoa/Cocoa.h>
-#import <wtf/mac/AppKitCompatibilityDeclarations.h>
 #endif
 #endif // PLATFORM(IOS_FAMILY)
 #endif
@@ -175,9 +178,10 @@
 #ifdef __OBJC__
 #if !PLATFORM(WIN) && (!PLATFORM(MAC) || __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300)
 #import <wtf/FastMalloc.h>
+#import <wtf/HashMap.h>
 #import <wtf/Optional.h>
 #import <wtf/StdLibExtras.h>
-#import <wtf/text/AtomicString.h>
+#import <wtf/text/AtomString.h>
 #import <wtf/text/WTFString.h>
 #endif
 #endif
