@@ -107,15 +107,11 @@ void AccessibilityMediaObject::toggleMute()
 
 void AccessibilityMediaObject::increment()
 {
-    if (dispatchAccessibilityEventWithType(AccessibilityEventType::Increment))
-        return;
     mediaSeek(AXSeekForward);
 }
 
 void AccessibilityMediaObject::decrement()
 {
-    if (dispatchAccessibilityEventWithType(AccessibilityEventType::Decrement))
-        return;
     mediaSeek(AXSeekBackward);
 }
 
