@@ -25,11 +25,15 @@
 
 #pragma once
 
+#include <wtf/Forward.h>
 #include <wtf/Markable.h>
 #include <wtf/Seconds.h>
 
 namespace WebCore {
 
+class Element;
+class WebAnimation;
+
 inline double secondsToWebAnimationsAPITime(const Seconds time)
 {
     // Precision of time values
@@ -46,19 +50,7 @@ inline double secondsToWebAnimationsAPITime(const Seconds time)
 
 const auto timeEpsilon = Seconds::fromMilliseconds(0.001);
 
-struct WebAnimationsMarkableDoubleTraits {
-    static bool isEmptyValue(double value)
-    {
-        return std::isnan(value);
-    }
-
-    static constexpr double emptyValue()
-    {
-        return std::numeric_limits<double>::quiet_NaN();
-    }
-};
-
-using MarkableDouble = Markable<double, WebAnimationsMarkableDoubleTraits>;
+bool compareAnimationsByCompositeOrder(const WebAnimation&, const WebAnimation&);
 
 } // namespace WebCore
 
