@@ -31,7 +31,7 @@
 #include <wtf/Brigand.h>
 #include <wtf/HashMap.h>
 #include <wtf/StdLibExtras.h>
-#include <wtf/text/WTFString.h>
+#include <wtf/URL.h>
 
 #if ENABLE(WEBGL)
 #include "WebGLAny.h"
@@ -43,7 +43,6 @@ class ArrayBufferView;
 class DataView;
 class JSValue;
 class JSObject;
-template<typename> class Strong;
 }
 
 namespace WebCore {
@@ -63,6 +62,7 @@ template<typename T>
 struct IDLType {
     using ImplementationType = T;
     using StorageType = T;
+    using SequenceStorageType = T;
 
     using ParameterType = T;
     using NullableParameterType = Optional<ImplementationType>;
@@ -83,6 +83,7 @@ struct IDLUnsupportedType : IDLType<void> { };
 struct IDLNull : IDLType<std::nullptr_t> { };
 
 struct IDLAny : IDLType<JSC::Strong<JSC::Unknown>> {
+    using SequenceStorageType = JSC::JSValue;
     using ParameterType = JSC::JSValue;
     using NullableParameterType = JSC::JSValue;
 
@@ -131,6 +132,7 @@ template<typename StringType> struct IDLString : IDLType<StringType> {
     static bool isNullValue(const StringType& value) { return value.isNull(); }
     static bool isNullValue(const UncachedString& value) { return value.string.isNull(); }
     static bool isNullValue(const OwnedString& value) { return value.string.isNull(); }
+    static bool isNullValue(const URL& value) { return value.isNull(); }
     template <typename U> static U&& extractValueFromNullable(U&& value) { return std::forward<U>(value); }
 };
 struct IDLDOMString : IDLString<String> { };
@@ -141,11 +143,11 @@ template<typename T> struct IDLTreatNullAsEmptyAdaptor : IDLString<String> {
     using InnerType = T;
 };
 
-template<typename T> struct IDLAtomicStringAdaptor : IDLString<AtomicString> {
+template<typename T> struct IDLAtomStringAdaptor : IDLString<AtomString> {
     using InnerType = T;
 };
 
-template<typename T> struct IDLRequiresExistingAtomicStringAdaptor : IDLString<AtomicString> {
+template<typename T> struct IDLRequiresExistingAtomStringAdaptor : IDLString<AtomString> {
     using InnerType = T;
 };
 
@@ -273,7 +275,6 @@ struct IDLJSON : IDLType<String> {
 struct IDLScheduledAction : IDLType<std::unique_ptr<ScheduledAction>> { };
 template<typename T> struct IDLSerializedScriptValue : IDLWrapper<T> { };
 template<typename T> struct IDLEventListener : IDLWrapper<T> { };
-template<typename T> struct IDLXPathNSResolver : IDLWrapper<T> { };
 
 struct IDLIDBKey : IDLWrapper<IDBKey> { };
 struct IDLIDBKeyData : IDLWrapper<IDBKeyData> { };
