@@ -30,16 +30,12 @@ public:
 
     bool equals(const CSSInheritedValue&) const { return true; }
 
-#if COMPILER(MSVC)
-    // FIXME: This should be private, but for some reason MSVC then fails to invoke it from LazyNeverDestroyed::construct.
-public:
-#else
 private:
-    friend class LazyNeverDestroyed<CSSInheritedValue>;
-#endif
-    CSSInheritedValue()
+    friend LazyNeverDestroyed<CSSInheritedValue>;
+    CSSInheritedValue(StaticCSSValueTag)
         : CSSValue(InheritedClass)
     {
+        makeStatic();
     }
 };
 
