@@ -56,7 +56,7 @@ private:
     CSSStyleSheet* m_styleSheet;
 };
 
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
 static bool isAcceptableCSSStyleSheetParent(Node* parentNode)
 {
     // Only these nodes can be parents of StyleSheets, and they need to call clearOwnerNode() when moved out of document.
@@ -67,7 +67,7 @@ static bool isAcceptableCSSStyleSheetParent(Node* parentNode)
         || is<SVGStyleElement>(*parentNode)
         || parentNode->nodeType() == Node::PROCESSING_INSTRUCTION_NODE;
 }
-#endif
+#endif // ASSERT_ENABLED
 
 Ref<CSSStyleSheet> CSSStyleSheet::create(Ref<StyleSheetContents>&& sheet, CSSImportRule* ownerRule)
 {
@@ -263,7 +263,7 @@ RefPtr<CSSRuleList> CSSStyleSheet::rules()
     unsigned ruleCount = length();
     for (unsigned i = 0; i < ruleCount; ++i)
         ruleList->rules().append(item(i));
-    return WTFMove(ruleList);
+    return ruleList;
 }
 
 ExceptionOr<unsigned> CSSStyleSheet::insertRule(const String& ruleString, unsigned index)
@@ -329,7 +329,7 @@ RefPtr<CSSRuleList> CSSStyleSheet::cssRules()
     if (!canAccessRules())
         return nullptr;
     if (!m_ruleListCSSOMWrapper)
-        m_ruleListCSSOMWrapper = std::make_unique<StyleSheetCSSRuleList>(this);
+        m_ruleListCSSOMWrapper = makeUnique<StyleSheetCSSRuleList>(this);
     return m_ruleListCSSOMWrapper.get();
 }
 
