@@ -40,7 +40,7 @@ public:
     virtual bool isTemplateContent() const { return false; }
 
     // From the NonElementParentNode interface - https://dom.spec.whatwg.org/#interface-nonelementparentnode
-    WEBCORE_EXPORT Element* getElementById(const AtomicString&) const;
+    WEBCORE_EXPORT Element* getElementById(const AtomString&) const;
 
 protected:
     DocumentFragment(Document&, ConstructionType = CreateContainer);
