@@ -31,6 +31,7 @@ namespace WebCore {
 
 class ScriptExecutionContext;
 class Event;
+class EventTarget;
 
 class EventListener : public RefCounted<EventListener> {
 public:
@@ -55,6 +56,10 @@ public:
     virtual bool isAttribute() const { return false; }
     Type type() const { return m_type; }
 
+#if ASSERT_ENABLED
+    virtual void checkValidityForEventTarget(EventTarget&) { }
+#endif
+
 #if PLATFORM(JAVA)
     virtual bool isJavaEventListener() const { return false; }
 #endif
