@@ -34,8 +34,6 @@ class RangeBoundaryPoint {
 public:
     explicit RangeBoundaryPoint(Node* container);
 
-    explicit RangeBoundaryPoint(const RangeBoundaryPoint&);
-
     const Position toPosition() const;
 
     Node* container() const;
@@ -68,13 +66,6 @@ inline RangeBoundaryPoint::RangeBoundaryPoint(Node* container)
     ASSERT(m_containerNode);
 }
 
-inline RangeBoundaryPoint::RangeBoundaryPoint(const RangeBoundaryPoint& other)
-    : m_containerNode(other.container())
-    , m_offsetInContainer(other.offset())
-    , m_childBeforeBoundary(other.childBefore())
-{
-}
-
 inline Node* RangeBoundaryPoint::container() const
 {
     return m_containerNode.get();
