@@ -36,7 +36,7 @@
 
 namespace WebCore {
 
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
 static bool isSingleTagNameSelector(const CSSSelector& selector)
 {
     return selector.isLastInTagHistory() && selector.match() == CSSSelector::Tag;
@@ -46,7 +46,7 @@ static bool isSingleClassNameSelector(const CSSSelector& selector)
 {
     return selector.isLastInTagHistory() && selector.match() == CSSSelector::Class;
 }
-#endif
+#endif // ASSERT_ENABLED
 
 enum class IdMatchingType : uint8_t {
     None,
@@ -83,7 +83,7 @@ SelectorDataList::SelectorDataList(const CSSSelectorList& selectorList)
 
     m_selectors.reserveInitialCapacity(selectorCount);
     for (const CSSSelector* selector = selectorList.first(); selector; selector = CSSSelectorList::next(selector))
-        m_selectors.uncheckedAppend(SelectorData(selector));
+        m_selectors.uncheckedAppend({ selector });
 
     if (selectorCount == 1) {
         const CSSSelector& selector = *m_selectors.first().selector;
@@ -221,7 +221,7 @@ ALWAYS_INLINE void SelectorDataList::executeFastPathForIdSelector(const Containe
     ASSERT(m_selectors.size() == 1);
     ASSERT(idSelector);
 
-    const AtomicString& idToMatch = idSelector->value();
+    const AtomString& idToMatch = idSelector->value();
     if (UNLIKELY(rootNode.treeScope().containsMultipleElementsWithId(idToMatch))) {
         const Vector<Element*>* elements = rootNode.treeScope().getAllElementsById(idToMatch);
         ASSERT(elements);
@@ -263,7 +263,7 @@ static ContainerNode& filterRootById(ContainerNode& rootNode, const CSSSelector&
     bool inAdjacentChain = false;
     for (; selector; selector = selector->tagHistory()) {
         if (canBeUsedForIdFastPath(*selector)) {
-            const AtomicString& idToMatch = selector->value();
+            const AtomString& idToMatch = selector->value();
             if (ContainerNode* searchRoot = rootNode.treeScope().getElementById(idToMatch)) {
                 if (LIKELY(!rootNode.treeScope().containsMultipleElementsWithId(idToMatch))) {
                     if (inAdjacentChain)
@@ -283,7 +283,7 @@ static ContainerNode& filterRootById(ContainerNode& rootNode, const CSSSelector&
     return rootNode;
 }
 
-static ALWAYS_INLINE bool localNameMatches(const Element& element, const AtomicString& localName, const AtomicString& lowercaseLocalName)
+static ALWAYS_INLINE bool localNameMatches(const Element& element, const AtomString& localName, const AtomString& lowercaseLocalName)
 {
     if (element.isHTMLElement() && element.document().isHTMLDocument())
         return element.localName() == lowercaseLocalName;
@@ -292,7 +292,7 @@ static ALWAYS_INLINE bool localNameMatches(const Element& element, const AtomicS
 }
 
 template <typename SelectorQueryTrait>
-static inline void elementsForLocalName(const ContainerNode& rootNode, const AtomicString& localName, const AtomicString& lowercaseLocalName, typename SelectorQueryTrait::OutputType& output)
+static inline void elementsForLocalName(const ContainerNode& rootNode, const AtomString& localName, const AtomString& lowercaseLocalName, typename SelectorQueryTrait::OutputType& output)
 {
     if (localName == lowercaseLocalName) {
         for (auto& element : elementDescendants(const_cast<ContainerNode&>(rootNode))) {
@@ -331,9 +331,9 @@ ALWAYS_INLINE void SelectorDataList::executeSingleTagNameSelectorData(const Cont
     ASSERT(isSingleTagNameSelector(*selectorData.selector));
 
     const QualifiedName& tagQualifiedName = selectorData.selector->tagQName();
-    const AtomicString& selectorLocalName = tagQualifiedName.localName();
-    const AtomicString& selectorLowercaseLocalName = selectorData.selector->tagLowercaseLocalName();
-    const AtomicString& selectorNamespaceURI = tagQualifiedName.namespaceURI();
+    const AtomString& selectorLocalName = tagQualifiedName.localName();
+    const AtomString& selectorLowercaseLocalName = selectorData.selector->tagLowercaseLocalName();
+    const AtomString& selectorNamespaceURI = tagQualifiedName.namespaceURI();
 
     if (selectorNamespaceURI == starAtom()) {
         if (selectorLocalName != starAtom()) {
@@ -361,7 +361,7 @@ ALWAYS_INLINE void SelectorDataList::executeSingleClassNameSelectorData(const Co
     ASSERT(m_selectors.size() == 1);
     ASSERT(isSingleClassNameSelector(*selectorData.selector));
 
-    const AtomicString& className = selectorData.selector->value();
+    const AtomString& className = selectorData.selector->value();
     for (auto& element : elementDescendants(const_cast<ContainerNode&>(rootNode))) {
         if (element.hasClass() && element.classNames().contains(className)) {
             SelectorQueryTrait::appendOutputForElement(output, &element);
@@ -405,11 +405,8 @@ template <typename SelectorQueryTrait>
 ALWAYS_INLINE void SelectorDataList::executeCompiledSimpleSelectorChecker(const ContainerNode& searchRootNode, SelectorCompiler::QuerySelectorSimpleSelectorChecker selectorChecker, typename SelectorQueryTrait::OutputType& output, const SelectorData& selectorData) const
 {
     for (auto& element : elementDescendants(const_cast<ContainerNode&>(searchRootNode))) {
-#if CSS_SELECTOR_JIT_PROFILING
-        selectorData.compiledSelectorUsed();
-#else
-        UNUSED_PARAM(selectorData);
-#endif
+        selectorData.compiledSelector.wasUsed();
+
         if (selectorChecker(&element)) {
             SelectorQueryTrait::appendOutputForElement(output, &element);
             if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
@@ -425,11 +422,8 @@ ALWAYS_INLINE void SelectorDataList::executeCompiledSelectorCheckerWithCheckingC
     checkingContext.scope = rootNode.isDocumentNode() ? nullptr : &rootNode;
 
     for (auto& element : elementDescendants(const_cast<ContainerNode&>(searchRootNode))) {
-#if CSS_SELECTOR_JIT_PROFILING
-        selectorData.compiledSelectorUsed();
-#else
-        UNUSED_PARAM(selectorData);
-#endif
+        selectorData.compiledSelector.wasUsed();
+
         if (selectorChecker(&element, &checkingContext)) {
             SelectorQueryTrait::appendOutputForElement(output, &element);
             if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
@@ -445,17 +439,15 @@ ALWAYS_INLINE void SelectorDataList::executeCompiledSingleMultiSelectorData(cons
     checkingContext.scope = rootNode.isDocumentNode() ? nullptr : &rootNode;
     for (auto& element : elementDescendants(const_cast<ContainerNode&>(rootNode))) {
         for (auto& selector : m_selectors) {
-#if CSS_SELECTOR_JIT_PROFILING
-            selector.compiledSelectorUsed();
-#endif
+            selector.compiledSelector.wasUsed();
+
             bool matched = false;
-            void* compiledSelectorChecker = selector.compiledSelectorCodeRef.code().executableAddress();
-            if (selector.compilationStatus == SelectorCompilationStatus::SimpleSelectorChecker) {
-                auto selectorChecker = SelectorCompiler::querySelectorSimpleSelectorCheckerFunction(compiledSelectorChecker, selector.compilationStatus);
+            if (selector.compiledSelector.status == SelectorCompilationStatus::SimpleSelectorChecker) {
+                auto selectorChecker = SelectorCompiler::querySelectorSimpleSelectorCheckerFunction(selector.compiledSelector);
                 matched = selectorChecker(&element);
             } else {
-                ASSERT(selector.compilationStatus == SelectorCompilationStatus::SelectorCheckerWithCheckingContext);
-                auto selectorChecker = SelectorCompiler::querySelectorSelectorCheckerFunctionWithCheckingContext(compiledSelectorChecker, selector.compilationStatus);
+                ASSERT(selector.compiledSelector.status == SelectorCompilationStatus::SelectorCheckerWithCheckingContext);
+                auto selectorChecker = SelectorCompiler::querySelectorSelectorCheckerFunctionWithCheckingContext(selector.compiledSelector);
                 matched = selectorChecker(&element, &checkingContext);
             }
             if (matched) {
@@ -468,21 +460,16 @@ ALWAYS_INLINE void SelectorDataList::executeCompiledSingleMultiSelectorData(cons
     }
 }
 
-static bool isCompiledSelector(SelectorCompilationStatus compilationStatus)
-{
-    return compilationStatus == SelectorCompilationStatus::SimpleSelectorChecker || compilationStatus == SelectorCompilationStatus::SelectorCheckerWithCheckingContext;
-}
-
 bool SelectorDataList::compileSelector(const SelectorData& selectorData)
 {
-    if (selectorData.compilationStatus != SelectorCompilationStatus::NotCompiled)
-        return isCompiledSelector(selectorData.compilationStatus);
+    auto& compiledSelector = selectorData.compiledSelector;
 
-    selectorData.compilationStatus = SelectorCompiler::compileSelector(selectorData.selector, SelectorCompiler::SelectorContext::QuerySelector, selectorData.compiledSelectorCodeRef);
-    return isCompiledSelector(selectorData.compilationStatus);
+    if (compiledSelector.status == SelectorCompilationStatus::NotCompiled)
+        SelectorCompiler::compileSelector(compiledSelector, selectorData.selector, SelectorCompiler::SelectorContext::QuerySelector);
+
+    return compiledSelector.status != SelectorCompilationStatus::CannotCompile;
 }
 
-
 #endif // ENABLE(CSS_SELECTOR_JIT)
 
 template <typename SelectorQueryTrait>
@@ -510,7 +497,7 @@ ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename S
         {
 #if ENABLE(CSS_SELECTOR_JIT)
         const SelectorData& selectorData = m_selectors.first();
-        ASSERT(selectorData.compilationStatus == SelectorCompilationStatus::NotCompiled);
+        ASSERT(selectorData.compiledSelector.status == SelectorCompilationStatus::NotCompiled);
         ASSERT(m_matchType == CompilableSingle || m_matchType == CompilableSingleWithRootFilter);
         if (compileSelector(selectorData)) {
             if (m_matchType == CompilableSingle) {
@@ -541,13 +528,12 @@ ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename S
         {
         CompiledSingleCase:
         const SelectorData& selectorData = m_selectors.first();
-        void* compiledSelectorChecker = selectorData.compiledSelectorCodeRef.code().executableAddress();
-        if (selectorData.compilationStatus == SelectorCompilationStatus::SimpleSelectorChecker) {
-            SelectorCompiler::QuerySelectorSimpleSelectorChecker selectorChecker = SelectorCompiler::querySelectorSimpleSelectorCheckerFunction(compiledSelectorChecker, selectorData.compilationStatus);
+        if (selectorData.compiledSelector.status == SelectorCompilationStatus::SimpleSelectorChecker) {
+            auto selectorChecker = SelectorCompiler::querySelectorSimpleSelectorCheckerFunction(selectorData.compiledSelector);
             executeCompiledSimpleSelectorChecker<SelectorQueryTrait>(*searchRootNode, selectorChecker, output, selectorData);
         } else {
-            ASSERT(selectorData.compilationStatus == SelectorCompilationStatus::SelectorCheckerWithCheckingContext);
-            SelectorCompiler::QuerySelectorSelectorCheckerWithCheckingContext selectorChecker = SelectorCompiler::querySelectorSelectorCheckerFunctionWithCheckingContext(compiledSelectorChecker, selectorData.compilationStatus);
+            ASSERT(selectorData.compiledSelector.status == SelectorCompilationStatus::SelectorCheckerWithCheckingContext);
+            auto selectorChecker = SelectorCompiler::querySelectorSelectorCheckerFunctionWithCheckingContext(selectorData.compiledSelector);
             executeCompiledSelectorCheckerWithCheckingContext<SelectorQueryTrait>(rootNode, *searchRootNode, selectorChecker, output, selectorData);
         }
         break;
@@ -556,7 +542,7 @@ ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename S
     case CompiledSingleWithRootFilter:
     case CompiledSingle:
         ASSERT_NOT_REACHED();
-#if ASSERT_DISABLED
+#if !ASSERT_ENABLED
         FALLTHROUGH;
 #endif
 #endif // ENABLE(CSS_SELECTOR_JIT)
@@ -633,7 +619,7 @@ ExceptionOr<SelectorQuery&> SelectorQueryCache::add(const String& selectors, Doc
     if (m_entries.size() == maximumSelectorQueryCacheSize)
         m_entries.remove(m_entries.random());
 
-    return *m_entries.add(selectors, std::make_unique<SelectorQuery>(WTFMove(selectorList))).iterator->value;
+    return *m_entries.add(selectors, makeUnique<SelectorQuery>(WTFMove(selectorList))).iterator->value;
 }
 
 }
