@@ -35,7 +35,7 @@
 #include "HTMLElement.h"
 #include "HTMLNames.h"
 #include "HTMLTemplateElement.h"
-#include "URL.h"
+#include <wtf/URL.h>
 #include "ProcessingInstruction.h"
 #include "XLinkNames.h"
 #include "XMLNSNames.h"
@@ -85,6 +85,61 @@ static const uint8_t entityMap[maximumEscapedentityCharacter + 1] = {
     EntitySubstitutionNbspIndex // noBreakSpace.
 };
 
+static bool elementCannotHaveEndTag(const Node& node)
+{
+    if (!is<HTMLElement>(node))
+        return false;
+
+    // From https://html.spec.whatwg.org/#serialising-html-fragments:
+    // If current node is an area, base, basefont, bgsound, br, col, embed, frame, hr, img,
+    // input, keygen, link, meta, param, source, track or wbr element, then continue on to
+    // the next child node at this point.
+    static const AtomStringImpl* const localNames[] = {
+        areaTag->localName().impl(),
+        baseTag->localName().impl(),
+        basefontTag->localName().impl(),
+        bgsoundTag->localName().impl(),
+        brTag->localName().impl(),
+        colTag->localName().impl(),
+        embedTag->localName().impl(),
+        frameTag->localName().impl(),
+        hrTag->localName().impl(),
+        imgTag->localName().impl(),
+        inputTag->localName().impl(),
+        keygenTag->localName().impl(),
+        linkTag->localName().impl(),
+        metaTag->localName().impl(),
+        paramTag->localName().impl(),
+        sourceTag->localName().impl(),
+        trackTag->localName().impl(),
+        wbrTag->localName().impl()
+    };
+
+    auto* const elementName = downcast<HTMLElement>(node).localName().impl();
+    for (auto* name : localNames) {
+        if (name == elementName)
+            return true;
+    }
+
+    return false;
+}
+
+// Rules of self-closure
+// 1. No elements in HTML documents use the self-closing syntax.
+// 2. Elements w/ children never self-close because they use a separate end tag.
+// 3. HTML elements which do not have a "forbidden" end tag will close with a separate end tag.
+// 4. Other elements self-close.
+static bool shouldSelfClose(const Element& element, SerializationSyntax syntax)
+{
+    if (syntax != SerializationSyntax::XML && element.document().isHTMLDocument())
+        return false;
+    if (element.hasChildNodes())
+        return false;
+    if (element.isHTMLElement() && !elementCannotHaveEndTag(element))
+        return false;
+    return true;
+}
+
 template<typename CharacterType>
 static inline void appendCharactersReplacingEntitiesInternal(StringBuilder& result, const String& source, unsigned offset, unsigned length, EntityMask entityMask)
 {
@@ -95,12 +150,12 @@ static inline void appendCharactersReplacingEntitiesInternal(StringBuilder& resu
         CharacterType character = text[i];
         uint8_t substitution = character < WTF_ARRAY_LENGTH(entityMap) ? entityMap[character] : static_cast<uint8_t>(EntitySubstitutionNullIndex);
         if (UNLIKELY(substitution != EntitySubstitutionNullIndex) && entitySubstitutionList[substitution].mask & entityMask) {
-            result.append(text + positionAfterLastEntity, i - positionAfterLastEntity);
-            result.append(entitySubstitutionList[substitution].characters, entitySubstitutionList[substitution].length);
+            result.appendSubstring(source, offset + positionAfterLastEntity, i - positionAfterLastEntity);
+            result.appendCharacters(entitySubstitutionList[substitution].characters, entitySubstitutionList[substitution].length);
             positionAfterLastEntity = i + 1;
         }
     }
-    result.append(text + positionAfterLastEntity, length - positionAfterLastEntity);
+    result.appendSubstring(source, offset + positionAfterLastEntity, length - positionAfterLastEntity);
 }
 
 void MarkupAccumulator::appendCharactersReplacingEntities(StringBuilder& result, const String& source, unsigned offset, unsigned length, EntityMask entityMask)
@@ -116,24 +171,22 @@ void MarkupAccumulator::appendCharactersReplacingEntities(StringBuilder& result,
         appendCharactersReplacingEntitiesInternal<UChar>(result, source, offset, length, entityMask);
 }
 
-MarkupAccumulator::MarkupAccumulator(Vector<Node*>* nodes, EAbsoluteURLs resolveUrlsMethod, const Range* range, EFragmentSerialization fragmentSerialization)
+MarkupAccumulator::MarkupAccumulator(Vector<Node*>* nodes, ResolveURLs resolveURLs, SerializationSyntax serializationSyntax)
     : m_nodes(nodes)
-    , m_range(range)
-    , m_resolveURLsMethod(resolveUrlsMethod)
-    , m_fragmentSerialization(fragmentSerialization)
-    , m_prefixLevel(0)
+    , m_resolveURLs(resolveURLs)
+    , m_serializationSyntax(serializationSyntax)
 {
 }
 
 MarkupAccumulator::~MarkupAccumulator() = default;
 
-String MarkupAccumulator::serializeNodes(Node& targetNode, EChildrenOnly childrenOnly, Vector<QualifiedName>* tagNamesToSkip)
+String MarkupAccumulator::serializeNodes(Node& targetNode, SerializedNodes root, Vector<QualifiedName>* tagNamesToSkip)
 {
-    serializeNodesWithNamespaces(targetNode, childrenOnly, 0, tagNamesToSkip);
+    serializeNodesWithNamespaces(targetNode, root, 0, tagNamesToSkip);
     return m_markup.toString();
 }
 
-void MarkupAccumulator::serializeNodesWithNamespaces(Node& targetNode, EChildrenOnly childrenOnly, const Namespaces* namespaces, Vector<QualifiedName>* tagNamesToSkip)
+void MarkupAccumulator::serializeNodesWithNamespaces(Node& targetNode, SerializedNodes root, const Namespaces* namespaces, Vector<QualifiedName>* tagNamesToSkip)
 {
     if (tagNamesToSkip && is<Element>(targetNode)) {
         for (auto& name : *tagNamesToSkip) {
@@ -151,32 +204,32 @@ void MarkupAccumulator::serializeNodesWithNamespaces(Node& targetNode, EChildren
         namespaceHash.set(XMLNames::xmlNamespaceURI->impl(), xmlAtom().impl());
     }
 
-    if (!childrenOnly)
-        appendStartTag(targetNode, &namespaceHash);
+    if (root == SerializedNodes::SubtreeIncludingNode)
+        startAppendingNode(targetNode, &namespaceHash);
 
     if (targetNode.document().isHTMLDocument() && elementCannotHaveEndTag(targetNode))
         return;
 
     Node* current = targetNode.hasTagName(templateTag) ? downcast<HTMLTemplateElement>(targetNode).content().firstChild() : targetNode.firstChild();
     for ( ; current; current = current->nextSibling())
-        serializeNodesWithNamespaces(*current, IncludeNode, &namespaceHash, tagNamesToSkip);
+        serializeNodesWithNamespaces(*current, SerializedNodes::SubtreeIncludingNode, &namespaceHash, tagNamesToSkip);
 
-    if (!childrenOnly)
-        appendEndTag(targetNode);
+    if (root == SerializedNodes::SubtreeIncludingNode)
+        endAppendingNode(targetNode);
 }
 
 String MarkupAccumulator::resolveURLIfNeeded(const Element& element, const String& urlString) const
 {
-    switch (m_resolveURLsMethod) {
-    case ResolveAllURLs:
+    switch (m_resolveURLs) {
+    case ResolveURLs::Yes:
         return element.document().completeURL(urlString).string();
 
-    case ResolveNonLocalURLs:
+    case ResolveURLs::YesExcludingLocalFileURLsForPrivacy:
         if (!element.document().url().isLocalFile())
             return element.document().completeURL(urlString).string();
         break;
 
-    case DoNotResolveURLs:
+    case ResolveURLs::No:
         break;
     }
     return urlString;
@@ -187,16 +240,30 @@ void MarkupAccumulator::appendString(const String& string)
     m_markup.append(string);
 }
 
-void MarkupAccumulator::appendStartTag(const Node& node, Namespaces* namespaces)
+void MarkupAccumulator::appendStringView(StringView view)
 {
-    appendStartMarkup(m_markup, node, namespaces);
+    m_markup.append(view);
+}
+
+void MarkupAccumulator::startAppendingNode(const Node& node, Namespaces* namespaces)
+{
+    if (is<Element>(node))
+        appendStartTag(m_markup, downcast<Element>(node), namespaces);
+    else
+        appendNonElementNode(m_markup, node, namespaces);
+
     if (m_nodes)
         m_nodes->append(const_cast<Node*>(&node));
 }
 
-void MarkupAccumulator::appendEndTag(const Element& element)
+void MarkupAccumulator::appendEndTag(StringBuilder& result, const Element& element)
 {
-    appendEndMarkup(m_markup, element);
+    if (shouldSelfClose(element, m_serializationSyntax) || (!element.hasChildNodes() && elementCannotHaveEndTag(element)))
+        return;
+    result.append('<');
+    result.append('/');
+    result.append(element.nodeNamePreservingCase());
+    result.append('>');
 }
 
 size_t MarkupAccumulator::totalLength(const Vector<String>& strings)
@@ -225,19 +292,18 @@ void MarkupAccumulator::appendCustomAttributes(StringBuilder&, const Element&, N
 void MarkupAccumulator::appendQuotedURLAttributeValue(StringBuilder& result, const Element& element, const Attribute& attribute)
 {
     ASSERT(element.isURLAttribute(attribute));
-    const String resolvedURLString = resolveURLIfNeeded(element, attribute.value());
+    String resolvedURLString = resolveURLIfNeeded(element, attribute.value());
     UChar quoteChar = '"';
-    String strippedURLString = resolvedURLString.stripWhiteSpace();
-    if (protocolIsJavaScript(strippedURLString)) {
+    if (WTF::protocolIsJavaScript(resolvedURLString)) {
         // minimal escaping for javascript urls
-        if (strippedURLString.contains('"')) {
-            if (strippedURLString.contains('\''))
-                strippedURLString.replaceWithLiteral('"', "&quot;");
+        if (resolvedURLString.contains('"')) {
+            if (resolvedURLString.contains('\''))
+                resolvedURLString.replaceWithLiteral('"', "&quot;");
             else
                 quoteChar = '\'';
         }
         result.append(quoteChar);
-        result.append(strippedURLString);
+        result.append(resolvedURLString);
         result.append(quoteChar);
         return;
     }
@@ -251,7 +317,7 @@ void MarkupAccumulator::appendQuotedURLAttributeValue(StringBuilder& result, con
 bool MarkupAccumulator::shouldAddNamespaceElement(const Element& element)
 {
     // Don't add namespace attribute if it is already defined for this elem.
-    const AtomicString& prefix = element.prefix();
+    const AtomString& prefix = element.prefix();
     if (prefix.isEmpty())
         return !element.hasAttribute(xmlnsAtom());
 
@@ -280,22 +346,19 @@ bool MarkupAccumulator::shouldAddNamespaceAttribute(const Attribute& attribute,
     return true;
 }
 
-void MarkupAccumulator::appendNamespace(StringBuilder& result, const AtomicString& prefix, const AtomicString& namespaceURI, Namespaces& namespaces, bool allowEmptyDefaultNS)
+void MarkupAccumulator::appendNamespace(StringBuilder& result, const AtomString& prefix, const AtomString& namespaceURI, Namespaces& namespaces, bool allowEmptyDefaultNS)
 {
     namespaces.checkConsistency();
     if (namespaceURI.isEmpty()) {
         // http://www.whatwg.org/specs/web-apps/current-work/multipage/the-xhtml-syntax.html#xml-fragment-serialization-algorithm
-        if (allowEmptyDefaultNS && namespaces.get(emptyAtom().impl())) {
-            result.append(' ');
-            result.append(xmlnsAtom().string());
-            result.appendLiteral("=\"\"");
-        }
+        if (allowEmptyDefaultNS && namespaces.get(emptyAtom().impl()))
+            result.append(' ', xmlnsAtom(), "=\"\"");
         return;
     }
 
     // Use emptyAtom()s's impl() for both null and empty strings since the HashMap can't handle 0 as a key
-    AtomicStringImpl* pre = prefix.isEmpty() ? emptyAtom().impl() : prefix.impl();
-    AtomicStringImpl* foundNS = namespaces.get(pre);
+    AtomStringImpl* pre = prefix.isEmpty() ? emptyAtom().impl() : prefix.impl();
+    AtomStringImpl* foundNS = namespaces.get(pre);
     if (foundNS != namespaceURI.impl()) {
         namespaces.set(pre, namespaceURI.impl());
         // Add namespace to prefix pair so we can do constraint checking later.
@@ -304,12 +367,9 @@ void MarkupAccumulator::appendNamespace(StringBuilder& result, const AtomicStrin
         // Make sure xml prefix and namespace are always known to uphold the constraints listed at http://www.w3.org/TR/xml-names11/#xmlReserved.
         if (namespaceURI.impl() == XMLNames::xmlNamespaceURI->impl())
             return;
-        result.append(' ');
-        result.append(xmlnsAtom().string());
-        if (!prefix.isEmpty()) {
-            result.append(':');
-            result.append(prefix);
-        }
+        result.append(' ', xmlnsAtom());
+        if (!prefix.isEmpty())
+            result.append(':', prefix);
 
         result.append('=');
         result.append('"');
@@ -320,7 +380,7 @@ void MarkupAccumulator::appendNamespace(StringBuilder& result, const AtomicStrin
 
 EntityMask MarkupAccumulator::entityMaskForText(const Text& text) const
 {
-    if (!text.document().isHTMLDocument())
+    if (!text.document().isHTMLDocument() || inXMLFragmentSerialization())
         return EntityMaskInPCDATA;
 
     const QualifiedName* parentName = nullptr;
@@ -335,19 +395,7 @@ EntityMask MarkupAccumulator::entityMaskForText(const Text& text) const
 void MarkupAccumulator::appendText(StringBuilder& result, const Text& text)
 {
     const String& textData = text.data();
-    unsigned start = 0;
-    unsigned length = textData.length();
-
-    if (m_range) {
-        if (&text == &m_range->endContainer())
-            length = m_range->endOffset();
-        if (&text == &m_range->startContainer()) {
-            start = m_range->startOffset();
-            length -= start;
-        }
-    }
-
-    appendCharactersReplacingEntities(result, textData, start, length, entityMaskForText(text));
+    appendCharactersReplacingEntities(result, textData, 0, textData.length(), entityMaskForText(text));
 }
 
 static void appendComment(StringBuilder& result, const String& comment)
@@ -370,7 +418,7 @@ void MarkupAccumulator::appendXMLDeclaration(StringBuilder& result, const Docume
         result.appendLiteral("\" encoding=\"");
         result.append(encoding);
     }
-    if (document.xmlStandaloneStatus() != Document::StandaloneUnspecified) {
+    if (document.xmlStandaloneStatus() != Document::StandaloneStatus::Unspecified) {
         result.appendLiteral("\" standalone=\"");
         if (document.xmlStandalone())
             result.appendLiteral("yes");
@@ -418,7 +466,7 @@ void MarkupAccumulator::appendProcessingInstruction(StringBuilder& result, const
     result.append('>');
 }
 
-void MarkupAccumulator::appendElement(StringBuilder& result, const Element& element, Namespaces* namespaces)
+void MarkupAccumulator::appendStartTag(StringBuilder& result, const Element& element, Namespaces* namespaces)
 {
     appendOpenTag(result, element, namespaces);
 
@@ -452,7 +500,7 @@ void MarkupAccumulator::appendOpenTag(StringBuilder& result, const Element& elem
 
 void MarkupAccumulator::appendCloseTag(StringBuilder& result, const Element& element)
 {
-    if (shouldSelfClose(element)) {
+    if (shouldSelfClose(element, m_serializationSyntax)) {
         if (element.isHTMLElement())
             result.append(' '); // XHTML 1.0 <-> HTML compatibility.
         result.append('/');
@@ -460,13 +508,6 @@ void MarkupAccumulator::appendCloseTag(StringBuilder& result, const Element& ele
     result.append('>');
 }
 
-static inline bool attributeIsInSerializedNamespace(const Attribute& attribute)
-{
-    return attribute.namespaceURI() == XMLNames::xmlNamespaceURI
-        || attribute.namespaceURI() == XLinkNames::xlinkNamespaceURI
-        || attribute.namespaceURI() == XMLNSNames::xmlnsNamespaceURI;
-}
-
 void MarkupAccumulator::generateUniquePrefix(QualifiedName& prefixedName, const Namespaces& namespaces)
 {
     // http://www.w3.org/TR/DOM-Level-3-Core/namespaces-algorithms.html#normalizeDocumentAlgo
@@ -477,7 +518,7 @@ void MarkupAccumulator::generateUniquePrefix(QualifiedName& prefixedName, const
         builder.clear();
         builder.appendLiteral("NS");
         builder.appendNumber(++m_prefixLevel);
-        const AtomicString& name = builder.toAtomicString();
+        const AtomString& name = builder.toAtomString();
         if (!namespaces.get(name.impl())) {
             prefixedName.setPrefix(name);
             return;
@@ -485,35 +526,61 @@ void MarkupAccumulator::generateUniquePrefix(QualifiedName& prefixedName, const
     } while (true);
 }
 
+// https://html.spec.whatwg.org/#attribute's-serialised-name
+static String htmlAttributeSerialization(const Attribute& attribute)
+{
+    if (attribute.namespaceURI().isEmpty())
+        return attribute.name().localName();
+
+    QualifiedName prefixedName = attribute.name();
+    if (attribute.namespaceURI() == XMLNames::xmlNamespaceURI)
+        prefixedName.setPrefix(xmlAtom());
+    else if (attribute.namespaceURI() == XMLNSNames::xmlnsNamespaceURI) {
+        if (prefixedName.localName() == xmlnsAtom())
+            return xmlnsAtom();
+        prefixedName.setPrefix(xmlnsAtom());
+    } else if (attribute.namespaceURI() == XLinkNames::xlinkNamespaceURI)
+        prefixedName.setPrefix(AtomString("xlink", AtomString::ConstructFromLiteral));
+    return prefixedName.toString();
+}
+
+// https://w3c.github.io/DOM-Parsing/#dfn-xml-serialization-of-the-attributes
+QualifiedName MarkupAccumulator::xmlAttributeSerialization(const Attribute& attribute, Namespaces* namespaces)
+{
+    QualifiedName prefixedName = attribute.name();
+    if (!attribute.namespaceURI().isEmpty()) {
+        if (attribute.namespaceURI() == XMLNames::xmlNamespaceURI) {
+            // Always use xml as prefix if the namespace is the XML namespace.
+            prefixedName.setPrefix(xmlAtom());
+        } else {
+            AtomStringImpl* foundNS = namespaces && attribute.prefix().impl() ? namespaces->get(attribute.prefix().impl()) : nullptr;
+            bool prefixIsAlreadyMappedToOtherNS = foundNS && foundNS != attribute.namespaceURI().impl();
+            if (attribute.prefix().isEmpty() || !foundNS || prefixIsAlreadyMappedToOtherNS) {
+                if (AtomStringImpl* prefix = namespaces ? namespaces->get(attribute.namespaceURI().impl()) : nullptr)
+                    prefixedName.setPrefix(AtomString(prefix));
+                else {
+                    bool shouldBeDeclaredUsingAppendNamespace = !attribute.prefix().isEmpty() && !foundNS;
+                    if (!shouldBeDeclaredUsingAppendNamespace && attribute.localName() != xmlnsAtom() && namespaces)
+                        generateUniquePrefix(prefixedName, *namespaces);
+                }
+            }
+        }
+    }
+    return prefixedName;
+}
+
 void MarkupAccumulator::appendAttribute(StringBuilder& result, const Element& element, const Attribute& attribute, Namespaces* namespaces)
 {
     bool isSerializingHTML = element.document().isHTMLDocument() && !inXMLFragmentSerialization();
 
     result.append(' ');
 
-    QualifiedName prefixedName = attribute.name();
-    if (isSerializingHTML && !attributeIsInSerializedNamespace(attribute))
-        result.append(attribute.name().localName());
+    Optional<QualifiedName> effectiveXMLPrefixedName;
+    if (isSerializingHTML)
+        result.append(htmlAttributeSerialization(attribute));
     else {
-        if (!attribute.namespaceURI().isEmpty()) {
-            if (attribute.namespaceURI() == XMLNames::xmlNamespaceURI) {
-                // Always use xml as prefix if the namespace is the XML namespace.
-                prefixedName.setPrefix(xmlAtom());
-            } else {
-                AtomicStringImpl* foundNS = namespaces && attribute.prefix().impl() ? namespaces->get(attribute.prefix().impl()) : 0;
-                bool prefixIsAlreadyMappedToOtherNS = foundNS && foundNS != attribute.namespaceURI().impl();
-                if (attribute.prefix().isEmpty() || !foundNS || prefixIsAlreadyMappedToOtherNS) {
-                    if (AtomicStringImpl* prefix = namespaces ? namespaces->get(attribute.namespaceURI().impl()) : 0)
-                        prefixedName.setPrefix(AtomicString(prefix));
-                    else {
-                        bool shouldBeDeclaredUsingAppendNamespace = !attribute.prefix().isEmpty() && !foundNS;
-                        if (!shouldBeDeclaredUsingAppendNamespace && attribute.localName() != xmlnsAtom() && namespaces)
-                            generateUniquePrefix(prefixedName, *namespaces);
-                    }
-                }
-            }
-        }
-        result.append(prefixedName.toString());
+        effectiveXMLPrefixedName = xmlAttributeSerialization(attribute, namespaces);
+        result.append(effectiveXMLPrefixedName->toString());
     }
 
     result.append('=');
@@ -527,7 +594,7 @@ void MarkupAccumulator::appendAttribute(StringBuilder& result, const Element& el
     }
 
     if (!isSerializingHTML && namespaces && shouldAddNamespaceAttribute(attribute, *namespaces))
-        appendNamespace(result, prefixedName.prefix(), prefixedName.namespaceURI(), *namespaces);
+        appendNamespace(result, effectiveXMLPrefixedName->prefix(), effectiveXMLPrefixedName->namespaceURI(), *namespaces);
 }
 
 void MarkupAccumulator::appendCDATASection(StringBuilder& result, const String& section)
@@ -538,7 +605,7 @@ void MarkupAccumulator::appendCDATASection(StringBuilder& result, const String&
     result.appendLiteral("]]>");
 }
 
-void MarkupAccumulator::appendStartMarkup(StringBuilder& result, const Node& node, Namespaces* namespaces)
+void MarkupAccumulator::appendNonElementNode(StringBuilder& result, const Node& node, Namespaces* namespaces)
 {
     if (namespaces)
         namespaces->checkConsistency();
@@ -562,7 +629,7 @@ void MarkupAccumulator::appendStartMarkup(StringBuilder& result, const Node& nod
         appendProcessingInstruction(result, downcast<ProcessingInstruction>(node).target(), downcast<ProcessingInstruction>(node).data());
         break;
     case Node::ELEMENT_NODE:
-        appendElement(result, downcast<Element>(node), namespaces);
+        ASSERT_NOT_REACHED();
         break;
     case Node::CDATA_SECTION_NODE:
         appendCDATASection(result, downcast<CDATASection>(node).data());
@@ -573,51 +640,4 @@ void MarkupAccumulator::appendStartMarkup(StringBuilder& result, const Node& nod
     }
 }
 
-// Rules of self-closure
-// 1. No elements in HTML documents use the self-closing syntax.
-// 2. Elements w/ children never self-close because they use a separate end tag.
-// 3. HTML elements which do not have a "forbidden" end tag will close with a separate end tag.
-// 4. Other elements self-close.
-bool MarkupAccumulator::shouldSelfClose(const Element& element)
-{
-    if (!inXMLFragmentSerialization() && element.document().isHTMLDocument())
-        return false;
-    if (element.hasChildNodes())
-        return false;
-    if (element.isHTMLElement() && !elementCannotHaveEndTag(element))
-        return false;
-    return true;
-}
-
-bool MarkupAccumulator::elementCannotHaveEndTag(const Node& node)
-{
-    if (!is<HTMLElement>(node))
-        return false;
-
-    // From https://html.spec.whatwg.org/#serialising-html-fragments:
-    // If current node is an area, base, basefont, bgsound, br, col, embed, frame, hr, img,
-    // input, keygen, link, meta, param, source, track or wbr element, then continue on to
-    // the next child node at this point.
-    static const HTMLQualifiedName* tags[] = { &areaTag.get(), &baseTag.get(), &basefontTag.get(), &bgsoundTag.get(),
-        &brTag.get(), &colTag.get(), &embedTag.get(), &frameTag.get(), &hrTag.get(), &imgTag.get(), &inputTag.get(),
-        &keygenTag.get(), &linkTag.get(), &metaTag.get(), &paramTag.get(), &sourceTag.get(), &trackTag.get(), &wbrTag.get() };
-    auto& element = downcast<HTMLElement>(node);
-    for (auto* tag : tags) {
-        if (element.hasTagName(*tag))
-            return true;
-    }
-    return false;
-}
-
-void MarkupAccumulator::appendEndMarkup(StringBuilder& result, const Element& element)
-{
-    if (shouldSelfClose(element) || (!element.hasChildNodes() && elementCannotHaveEndTag(element)))
-        return;
-
-    result.append('<');
-    result.append('/');
-    result.append(element.nodeNamePreservingCase());
-    result.append('>');
-}
-
 }
