@@ -29,9 +29,14 @@
 #include "Element.h"
 #include "NodeRareData.h"
 #include <JavaScriptCore/Identifier.h>
+#include <wtf/IsoMallocInlines.h>
+#include <wtf/Optional.h>
+#include <wtf/Variant.h>
 
 namespace WebCore {
 
+WTF_MAKE_ISO_ALLOCATED_IMPL(HTMLAllNamedSubCollection);
+
 Ref<HTMLAllCollection> HTMLAllCollection::create(Document& document, CollectionType type)
 {
     return adoptRef(*new HTMLAllCollection(document, type));
@@ -43,7 +48,7 @@ inline HTMLAllCollection::HTMLAllCollection(Document& document, CollectionType t
 }
 
 // https://html.spec.whatwg.org/multipage/infrastructure.html#dom-htmlallcollection-item
-Optional<Variant<RefPtr<HTMLCollection>, RefPtr<Element>>> HTMLAllCollection::namedOrIndexedItemOrItems(const AtomicString& nameOrIndex) const
+Optional<Variant<RefPtr<HTMLCollection>, RefPtr<Element>>> HTMLAllCollection::namedOrIndexedItemOrItems(const AtomString& nameOrIndex) const
 {
     if (nameOrIndex.isNull())
         return WTF::nullopt;
@@ -55,7 +60,7 @@ Optional<Variant<RefPtr<HTMLCollection>, RefPtr<Element>>> HTMLAllCollection::na
 }
 
 // https://html.spec.whatwg.org/multipage/infrastructure.html#concept-get-all-named
-Optional<Variant<RefPtr<HTMLCollection>, RefPtr<Element>>> HTMLAllCollection::namedItemOrItems(const AtomicString& name) const
+Optional<Variant<RefPtr<HTMLCollection>, RefPtr<Element>>> HTMLAllCollection::namedItemOrItems(const AtomString& name) const
 {
     auto namedItems = this->namedItems(name);
 
