@@ -34,7 +34,7 @@ public:
 private:
     HTMLAppletElement(const QualifiedName&, Document&);
 
-    void parseAttribute(const QualifiedName&, const AtomicString&) final;
+    void parseAttribute(const QualifiedName&, const AtomString&) final;
     bool isURLAttribute(const Attribute&) const final;
 
     bool rendererIsNeeded(const RenderStyle&) final;
@@ -44,6 +44,8 @@ private:
     void updateWidget(CreatePlugins) final;
 
     bool canEmbedJava() const;
+
+    bool isInteractiveContent() const final { return true; }
 };
 
 } // namespace WebCore
