@@ -34,6 +34,7 @@
 #include "DOMTokenList.h"
 #include "DocumentFragment.h"
 #include "ElementAncestorIterator.h"
+#include "EnterKeyHint.h"
 #include "Event.h"
 #include "EventListener.h"
 #include "EventNames.h"
@@ -84,7 +85,7 @@ Ref<HTMLElement> HTMLElement::create(const QualifiedName& tagName, Document& doc
 
 String HTMLElement::nodeName() const
 {
-    // FIXME: Would be nice to have an AtomicString lookup based off uppercase
+    // FIXME: Would be nice to have an AtomString lookup based off uppercase
     // ASCII characters that does not have to copy the string on a hit in the hash.
     if (document().isHTMLDocument()) {
         if (LIKELY(!tagQName().hasPrefix()))
@@ -103,7 +104,7 @@ static inline CSSValueID unicodeBidiAttributeForDirAuto(HTMLElement& element)
     return CSSValueIsolate;
 }
 
-unsigned HTMLElement::parseBorderWidthAttribute(const AtomicString& value) const
+unsigned HTMLElement::parseBorderWidthAttribute(const AtomString& value) const
 {
     if (auto optionalBorderWidth = parseHTMLNonNegativeInteger(value))
         return optionalBorderWidth.value();
@@ -111,13 +112,13 @@ unsigned HTMLElement::parseBorderWidthAttribute(const AtomicString& value) const
     return hasTagName(tableTag) ? 1 : 0;
 }
 
-void HTMLElement::applyBorderAttributeToStyle(const AtomicString& value, MutableStyleProperties& style)
+void HTMLElement::applyBorderAttributeToStyle(const AtomString& value, MutableStyleProperties& style)
 {
-    addPropertyToPresentationAttributeStyle(style, CSSPropertyBorderWidth, parseBorderWidthAttribute(value), CSSPrimitiveValue::CSS_PX);
+    addPropertyToPresentationAttributeStyle(style, CSSPropertyBorderWidth, parseBorderWidthAttribute(value), CSSUnitType::CSS_PX);
     addPropertyToPresentationAttributeStyle(style, CSSPropertyBorderStyle, CSSValueSolid);
 }
 
-void HTMLElement::mapLanguageAttributeToLocale(const AtomicString& value, MutableStyleProperties& style)
+void HTMLElement::mapLanguageAttributeToLocale(const AtomString& value, MutableStyleProperties& style)
 {
     if (!value.isEmpty()) {
         // Have to quote so the locale id is treated as a string instead of as a CSS keyword.
@@ -135,7 +136,7 @@ bool HTMLElement::isPresentationAttribute(const QualifiedName& name) const
     return StyledElement::isPresentationAttribute(name);
 }
 
-static bool isLTROrRTLIgnoringCase(const AtomicString& dirAttributeValue)
+static bool isLTROrRTLIgnoringCase(const AtomString& dirAttributeValue)
 {
     return equalLettersIgnoringASCIICase(dirAttributeValue, "rtl") || equalLettersIgnoringASCIICase(dirAttributeValue, "ltr");
 }
@@ -147,7 +148,7 @@ enum class ContentEditableType {
     PlaintextOnly
 };
 
-static inline ContentEditableType contentEditableType(const AtomicString& value)
+static inline ContentEditableType contentEditableType(const AtomString& value)
 {
     if (value.isNull())
         return ContentEditableType::Inherit;
@@ -166,7 +167,7 @@ static ContentEditableType contentEditableType(const HTMLElement& element)
     return contentEditableType(element.attributeWithoutSynchronization(contenteditableAttr));
 }
 
-void HTMLElement::collectStyleForPresentationAttribute(const QualifiedName& name, const AtomicString& value, MutableStyleProperties& style)
+void HTMLElement::collectStyleForPresentationAttribute(const QualifiedName& name, const AtomString& value, MutableStyleProperties& style)
 {
     if (name == alignAttr) {
         if (equalLettersIgnoringASCIICase(value, "middle"))
@@ -227,15 +228,6 @@ HTMLElement::EventHandlerNameMap HTMLElement::createEventHandlerNameMap()
 
     static const QualifiedName* const table[] = {
         &onabortAttr.get(),
-        &onaccessiblecontextmenuAttr.get(),
-        &onaccessibleclickAttr.get(),
-        &onaccessibledecrementAttr.get(),
-        &onaccessibledismissAttr.get(),
-        &onaccessiblefocusAttr.get(),
-        &onaccessibleincrementAttr.get(),
-        &onaccessiblescrollintoviewAttr.get(),
-        &onaccessiblesetvalueAttr.get(),
-        &onaccessibleselectAttr.get(),
         &onanimationendAttr.get(),
         &onanimationiterationAttr.get(),
         &onanimationstartAttr.get(),
@@ -273,6 +265,7 @@ HTMLElement::EventHandlerNameMap HTMLElement::createEventHandlerNameMap()
         &ongesturechangeAttr.get(),
         &ongestureendAttr.get(),
         &ongesturestartAttr.get(),
+        &ongotpointercaptureAttr.get(),
         &oninputAttr.get(),
         &oninvalidAttr.get(),
         &onkeydownAttr.get(),
@@ -282,6 +275,7 @@ HTMLElement::EventHandlerNameMap HTMLElement::createEventHandlerNameMap()
         &onloadeddataAttr.get(),
         &onloadedmetadataAttr.get(),
         &onloadstartAttr.get(),
+        &onlostpointercaptureAttr.get(),
         &onmousedownAttr.get(),
         &onmouseenterAttr.get(),
         &onmouseleaveAttr.get(),
@@ -294,6 +288,14 @@ HTMLElement::EventHandlerNameMap HTMLElement::createEventHandlerNameMap()
         &onpauseAttr.get(),
         &onplayAttr.get(),
         &onplayingAttr.get(),
+        &onpointerdownAttr.get(),
+        &onpointermoveAttr.get(),
+        &onpointerupAttr.get(),
+        &onpointercancelAttr.get(),
+        &onpointeroverAttr.get(),
+        &onpointeroutAttr.get(),
+        &onpointerenterAttr.get(),
+        &onpointerleaveAttr.get(),
         &onprogressAttr.get(),
         &onratechangeAttr.get(),
         &onresetAttr.get(),
@@ -346,7 +348,7 @@ HTMLElement::EventHandlerNameMap HTMLElement::createEventHandlerNameMap()
 
     struct UnusualMapping {
         const QualifiedName& attributeName;
-        const AtomicString& eventName;
+        const AtomString& eventName;
     };
 
     const UnusualMapping unusualPairsTable[] = {
@@ -373,13 +375,13 @@ void HTMLElement::populateEventHandlerNameMap(EventHandlerNameMap& map, const Qu
 
         // Remove the "on" prefix. Requires some memory allocation and computing a hash, but by not
         // using pointers from eventNames(), the passed-in table can be initialized at compile time.
-        AtomicString eventName = attributeName.string().substring(2);
+        AtomString eventName = attributeName.string().substring(2);
 
         map.add(attributeName.impl(), WTFMove(eventName));
     }
 }
 
-const AtomicString& HTMLElement::eventNameForEventHandlerAttribute(const QualifiedName& attributeName, const EventHandlerNameMap& map)
+const AtomString& HTMLElement::eventNameForEventHandlerAttribute(const QualifiedName& attributeName, const EventHandlerNameMap& map)
 {
     ASSERT(!attributeName.localName().isNull());
 
@@ -388,7 +390,7 @@ const AtomicString& HTMLElement::eventNameForEventHandlerAttribute(const Qualifi
         return nullAtom();
 
     // Fast early return for names that don't start with "on".
-    AtomicStringImpl& localName = *attributeName.localName().impl();
+    AtomStringImpl& localName = *attributeName.localName().impl();
     if (localName.length() < 3 || localName[0] != 'o' || localName[1] != 'n')
         return nullAtom();
 
@@ -396,7 +398,7 @@ const AtomicString& HTMLElement::eventNameForEventHandlerAttribute(const Qualifi
     return it == map.end() ? nullAtom() : it->value;
 }
 
-const AtomicString& HTMLElement::eventNameForEventHandlerAttribute(const QualifiedName& attributeName)
+const AtomString& HTMLElement::eventNameForEventHandlerAttribute(const QualifiedName& attributeName)
 {
     static NeverDestroyed<EventHandlerNameMap> map = createEventHandlerNameMap();
     return eventNameForEventHandlerAttribute(attributeName, map.get());
@@ -435,7 +437,7 @@ bool HTMLElement::matchesReadWritePseudoClass() const
     return editabilityFromContentEditableAttr(*this) != Editability::ReadOnly;
 }
 
-void HTMLElement::parseAttribute(const QualifiedName& name, const AtomicString& value)
+void HTMLElement::parseAttribute(const QualifiedName& name, const AtomString& value)
 {
     if (name == dirAttr) {
         dirAttributeChanged(value);
@@ -502,11 +504,11 @@ static Ref<DocumentFragment> textToFragment(Document& document, const String& te
 // or the empty string if the attribute is in a state that has no associated keyword value or if the attribute is
 // not in a defined state (e.g. the attribute is missing and there is no missing value default).
 // http://www.whatwg.org/specs/web-apps/current-work/multipage/common-dom-interfaces.html#limited-to-only-known-values
-static inline const AtomicString& toValidDirValue(const AtomicString& value)
+static inline const AtomString& toValidDirValue(const AtomString& value)
 {
-    static NeverDestroyed<AtomicString> ltrValue("ltr", AtomicString::ConstructFromLiteral);
-    static NeverDestroyed<AtomicString> rtlValue("rtl", AtomicString::ConstructFromLiteral);
-    static NeverDestroyed<AtomicString> autoValue("auto", AtomicString::ConstructFromLiteral);
+    static NeverDestroyed<AtomString> ltrValue("ltr", AtomString::ConstructFromLiteral);
+    static NeverDestroyed<AtomString> rtlValue("rtl", AtomString::ConstructFromLiteral);
+    static NeverDestroyed<AtomString> autoValue("auto", AtomString::ConstructFromLiteral);
     if (equalLettersIgnoringASCIICase(value, "ltr"))
         return ltrValue;
     if (equalLettersIgnoringASCIICase(value, "rtl"))
@@ -516,12 +518,12 @@ static inline const AtomicString& toValidDirValue(const AtomicString& value)
     return nullAtom();
 }
 
-const AtomicString& HTMLElement::dir() const
+const AtomString& HTMLElement::dir() const
 {
     return toValidDirValue(attributeWithoutSynchronization(dirAttr));
 }
 
-void HTMLElement::setDir(const AtomicString& value)
+void HTMLElement::setDir(const AtomString& value)
 {
     setAttributeWithoutSynchronization(dirAttr, value);
 }
@@ -599,7 +601,7 @@ ExceptionOr<void> HTMLElement::setOuterText(const String& text)
     return { };
 }
 
-void HTMLElement::applyAlignmentAttributeToStyle(const AtomicString& alignment, MutableStyleProperties& style)
+void HTMLElement::applyAlignmentAttributeToStyle(const AtomString& alignment, MutableStyleProperties& style)
 {
     // Vertical alignment with respect to the current baseline of the text
     // right or left means floating images.
@@ -662,11 +664,11 @@ String HTMLElement::contentEditable() const
 ExceptionOr<void> HTMLElement::setContentEditable(const String& enabled)
 {
     if (equalLettersIgnoringASCIICase(enabled, "true"))
-        setAttributeWithoutSynchronization(contenteditableAttr, AtomicString("true", AtomicString::ConstructFromLiteral));
+        setAttributeWithoutSynchronization(contenteditableAttr, AtomString("true", AtomString::ConstructFromLiteral));
     else if (equalLettersIgnoringASCIICase(enabled, "false"))
-        setAttributeWithoutSynchronization(contenteditableAttr, AtomicString("false", AtomicString::ConstructFromLiteral));
+        setAttributeWithoutSynchronization(contenteditableAttr, AtomString("false", AtomString::ConstructFromLiteral));
     else if (equalLettersIgnoringASCIICase(enabled, "plaintext-only"))
-        setAttributeWithoutSynchronization(contenteditableAttr, AtomicString("plaintext-only", AtomicString::ConstructFromLiteral));
+        setAttributeWithoutSynchronization(contenteditableAttr, AtomString("plaintext-only", AtomString::ConstructFromLiteral));
     else if (equalLettersIgnoringASCIICase(enabled, "inherit"))
         removeAttribute(contenteditableAttr);
     else
@@ -682,8 +684,8 @@ bool HTMLElement::draggable() const
 void HTMLElement::setDraggable(bool value)
 {
     setAttributeWithoutSynchronization(draggableAttr, value
-        ? AtomicString("true", AtomicString::ConstructFromLiteral)
-        : AtomicString("false", AtomicString::ConstructFromLiteral));
+        ? AtomString("true", AtomString::ConstructFromLiteral)
+        : AtomString("false", AtomString::ConstructFromLiteral));
 }
 
 bool HTMLElement::spellcheck() const
@@ -694,8 +696,8 @@ bool HTMLElement::spellcheck() const
 void HTMLElement::setSpellcheck(bool enable)
 {
     setAttributeWithoutSynchronization(spellcheckAttr, enable
-        ? AtomicString("true", AtomicString::ConstructFromLiteral)
-        : AtomicString("false", AtomicString::ConstructFromLiteral));
+        ? AtomString("true", AtomString::ConstructFromLiteral)
+        : AtomString("false", AtomString::ConstructFromLiteral));
 }
 
 void HTMLElement::click()
@@ -713,17 +715,10 @@ String HTMLElement::title() const
     return attributeWithoutSynchronization(titleAttr);
 }
 
-int HTMLElement::tabIndex() const
-{
-    if (supportsFocus())
-        return Element::tabIndex();
-    return -1;
-}
-
 bool HTMLElement::translate() const
 {
     for (auto& element : lineageOfType<HTMLElement>(*this)) {
-        const AtomicString& value = element.attributeWithoutSynchronization(translateAttr);
+        const AtomString& value = element.attributeWithoutSynchronization(translateAttr);
         if (equalLettersIgnoringASCIICase(value, "yes") || (value.isEmpty() && !value.isNull()))
             return true;
         if (equalLettersIgnoringASCIICase(value, "no"))
@@ -738,7 +733,7 @@ void HTMLElement::setTranslate(bool enable)
     setAttributeWithoutSynchronization(translateAttr, enable ? "yes" : "no");
 }
 
-bool HTMLElement::rendererIsNeeded(const RenderStyle& style)
+bool HTMLElement::rendererIsEverNeeded()
 {
     if (hasTagName(noscriptTag)) {
         RefPtr<Frame> frame = document().frame();
@@ -749,7 +744,7 @@ bool HTMLElement::rendererIsNeeded(const RenderStyle& style)
         if (frame && frame->loader().subframeLoader().allowPlugins())
             return false;
     }
-    return StyledElement::rendererIsNeeded(style);
+    return StyledElement::rendererIsEverNeeded();
 }
 
 RenderPtr<RenderElement> HTMLElement::createElementRenderer(RenderStyle&& style, const RenderTreePosition&)
@@ -762,6 +757,16 @@ HTMLFormElement* HTMLElement::form() const
     return HTMLFormElement::findClosestFormAncestor(*this);
 }
 
+FormNamedItem* HTMLElement::asFormNamedItem()
+{
+    return nullptr;
+}
+
+FormAssociatedElement* HTMLElement::asFormAssociatedElement()
+{
+    return nullptr;
+}
+
 static inline bool elementAffectsDirectionality(const Node& node)
 {
     if (!is<HTMLElement>(node))
@@ -801,7 +806,7 @@ void HTMLElement::childrenChanged(const ChildChange& change)
 
 bool HTMLElement::hasDirectionAuto() const
 {
-    const AtomicString& direction = attributeWithoutSynchronization(dirAttr);
+    const AtomString& direction = attributeWithoutSynchronization(dirAttr);
     return (hasTagName(bdiTag) && direction.isNull()) || equalLettersIgnoringASCIICase(direction, "auto");
 }
 
@@ -861,7 +866,7 @@ TextDirection HTMLElement::directionality(Node** strongDirectionalityTextNode) c
     return TextDirection::LTR;
 }
 
-void HTMLElement::dirAttributeChanged(const AtomicString& value)
+void HTMLElement::dirAttributeChanged(const AtomString& value)
 {
     RefPtr<Element> parent = parentElement();
 
@@ -1064,9 +1069,9 @@ bool HTMLElement::isActuallyDisabled() const
     return canBeActuallyDisabled() && isDisabledFormControl();
 }
 
-#if ENABLE(IOS_AUTOCORRECT_AND_AUTOCAPITALIZE)
+#if ENABLE(AUTOCAPITALIZE)
 
-const AtomicString& HTMLElement::autocapitalize() const
+const AtomString& HTMLElement::autocapitalize() const
 {
     return stringForAutocapitalizeType(autocapitalizeType());
 }
@@ -1076,11 +1081,15 @@ AutocapitalizeType HTMLElement::autocapitalizeType() const
     return autocapitalizeTypeForAttributeValue(attributeWithoutSynchronization(HTMLNames::autocapitalizeAttr));
 }
 
-void HTMLElement::setAutocapitalize(const AtomicString& value)
+void HTMLElement::setAutocapitalize(const AtomString& value)
 {
     setAttributeWithoutSynchronization(autocapitalizeAttr, value);
 }
 
+#endif
+
+#if ENABLE(AUTOCORRECT)
+
 bool HTMLElement::shouldAutocorrect() const
 {
     auto& autocorrectValue = attributeWithoutSynchronization(HTMLNames::autocorrectAttr);
@@ -1090,26 +1099,46 @@ bool HTMLElement::shouldAutocorrect() const
 
 void HTMLElement::setAutocorrect(bool autocorrect)
 {
-    setAttributeWithoutSynchronization(autocorrectAttr, autocorrect ? AtomicString("on", AtomicString::ConstructFromLiteral) : AtomicString("off", AtomicString::ConstructFromLiteral));
+    setAttributeWithoutSynchronization(autocorrectAttr, autocorrect ? AtomString("on", AtomString::ConstructFromLiteral) : AtomString("off", AtomString::ConstructFromLiteral));
 }
 
 #endif
 
 InputMode HTMLElement::canonicalInputMode() const
 {
-    return inputModeForAttributeValue(attributeWithoutSynchronization(inputmodeAttr));
+    auto mode = inputModeForAttributeValue(attributeWithoutSynchronization(inputmodeAttr));
+    if (mode == InputMode::Unspecified) {
+        if (document().quirks().needsInputModeNoneImplicitly(*this))
+            return InputMode::None;
+    }
+    return mode;
 }
 
-const AtomicString& HTMLElement::inputMode() const
+const AtomString& HTMLElement::inputMode() const
 {
     return stringForInputMode(canonicalInputMode());
 }
 
-void HTMLElement::setInputMode(const AtomicString& value)
+void HTMLElement::setInputMode(const AtomString& value)
 {
     setAttributeWithoutSynchronization(inputmodeAttr, value);
 }
 
+EnterKeyHint HTMLElement::canonicalEnterKeyHint() const
+{
+    return enterKeyHintForAttributeValue(attributeWithoutSynchronization(enterkeyhintAttr));
+}
+
+String HTMLElement::enterKeyHint() const
+{
+    return attributeValueForEnterKeyHint(canonicalEnterKeyHint());
+}
+
+void HTMLElement::setEnterKeyHint(const String& value)
+{
+    setAttributeWithoutSynchronization(enterkeyhintAttr, value);
+}
+
 } // namespace WebCore
 
 #ifndef NDEBUG
