@@ -47,15 +47,15 @@ public:
 
     static RefPtr<HTMLFrameSetElement> findContaining(Element* descendant);
 
-    Vector<AtomicString> supportedPropertyNames() const;
-    WindowProxy* namedItem(const AtomicString&);
+    Vector<AtomString> supportedPropertyNames() const;
+    WindowProxy* namedItem(const AtomString&);
 
 private:
     HTMLFrameSetElement(const QualifiedName&, Document&);
 
-    void parseAttribute(const QualifiedName&, const AtomicString&) final;
+    void parseAttribute(const QualifiedName&, const AtomString&) final;
     bool isPresentationAttribute(const QualifiedName&) const final;
-    void collectStyleForPresentationAttribute(const QualifiedName&, const AtomicString&, MutableStyleProperties&) final;
+    void collectStyleForPresentationAttribute(const QualifiedName&, const AtomString&, MutableStyleProperties&) final;
 
     void willAttachRenderers() final;
     bool rendererIsNeeded(const RenderStyle&) final;
