@@ -44,14 +44,10 @@ HTMLPictureElement::HTMLPictureElement(const QualifiedName& tagName, Document& d
 
 HTMLPictureElement::~HTMLPictureElement()
 {
-    document().removeViewportDependentPicture(*this);
-    document().removeAppearanceDependentPicture(*this);
 }
 
 void HTMLPictureElement::didMoveToNewDocument(Document& oldDocument, Document& newDocument)
 {
-    oldDocument.removeViewportDependentPicture(*this);
-    oldDocument.removeAppearanceDependentPicture(*this);
     HTMLElement::didMoveToNewDocument(oldDocument, newDocument);
     sourcesChanged();
 }
@@ -67,30 +63,6 @@ void HTMLPictureElement::sourcesChanged()
         element.selectImageSource();
 }
 
-bool HTMLPictureElement::viewportChangeAffectedPicture() const
-{
-    auto documentElement = makeRefPtr(document().documentElement());
-    MediaQueryEvaluator evaluator { document().printing() ? "print" : "screen", document(), documentElement ? documentElement->computedStyle() : nullptr };
-    for (auto& result : m_viewportDependentMediaQueryResults) {
-        LOG(MediaQueries, "HTMLPictureElement %p viewportChangeAffectedPicture evaluating media queries", this);
-        if (evaluator.evaluate(result.expression) != result.result)
-            return true;
-    }
-    return false;
-}
-
-bool HTMLPictureElement::appearanceChangeAffectedPicture() const
-{
-    auto documentElement = makeRefPtr(document().documentElement());
-    MediaQueryEvaluator evaluator { document().printing() ? "print" : "screen", document(), documentElement ? documentElement->computedStyle() : nullptr };
-    for (auto& result : m_appearanceDependentMediaQueryResults) {
-        LOG(MediaQueries, "HTMLPictureElement %p appearanceChangeAffectedPicture evaluating media queries", this);
-        if (evaluator.evaluate(result.expression) != result.result)
-            return true;
-    }
-    return false;
-}
-
 #if USE(SYSTEM_PREVIEW)
 bool HTMLPictureElement::isSystemPreviewImage() const
 {
