@@ -45,11 +45,13 @@
 #include "InspectorPageAgent.h"
 #include "Page.h"
 #include "ScriptController.h"
+#include "ScriptSourceCode.h"
 #include "ScriptState.h"
 #include "Settings.h"
 #include "Timer.h"
 #include "UserGestureIndicator.h"
 #include "WindowFeatures.h"
+#include <JavaScriptCore/FrameTracers.h>
 #include <JavaScriptCore/InspectorBackendDispatchers.h>
 #include <wtf/Deque.h>
 #include <wtf/text/CString.h>
@@ -79,43 +81,54 @@ public:
         ASSERT_ARG(message, !message.isEmpty());
 
         m_messages.append(message);
-        if (!m_timer.isActive())
-            m_timer.startOneShot(0_s);
+        scheduleOneShot();
     }
 
     void reset()
     {
         m_messages.clear();
-        m_timer.stop();
         m_inspectedPageController = nullptr;
     }
 
-    void timerFired()
-    {
-        ASSERT(m_inspectedPageController);
-
-        // Dispatching a message can possibly close the frontend and destroy
-        // the owning frontend client, so keep a protector reference here.
-        Ref<InspectorBackendDispatchTask> protectedThis(*this);
-
-        if (!m_messages.isEmpty())
-            m_inspectedPageController->dispatchMessageFromFrontend(m_messages.takeFirst());
-
-        if (!m_messages.isEmpty() && m_inspectedPageController)
-            m_timer.startOneShot(0_s);
-    }
-
 private:
     InspectorBackendDispatchTask(InspectorController* inspectedPageController)
         : m_inspectedPageController(inspectedPageController)
-        , m_timer(*this, &InspectorBackendDispatchTask::timerFired)
     {
         ASSERT_ARG(inspectedPageController, inspectedPageController);
     }
 
+    void scheduleOneShot()
+    {
+        if (m_hasScheduledTask)
+            return;
+        m_hasScheduledTask = true;
+
+        // The frontend can be closed and destroy the owning frontend client before or in the
+        // process of dispatching the task, so keep a protector reference here.
+        RunLoop::current().dispatch([this, protectedThis = makeRef(*this)] {
+            m_hasScheduledTask = false;
+            dispatchOneMessage();
+        });
+    }
+
+    void dispatchOneMessage()
+    {
+        // Owning frontend client may have been destroyed after the task was scheduled.
+        if (!m_inspectedPageController) {
+            ASSERT(m_messages.isEmpty());
+            return;
+        }
+
+        if (!m_messages.isEmpty())
+            m_inspectedPageController->dispatchMessageFromFrontend(m_messages.takeFirst());
+
+        if (!m_messages.isEmpty() && m_inspectedPageController)
+            scheduleOneShot();
+    }
+
     InspectorController* m_inspectedPageController { nullptr };
-    Timer m_timer;
     Deque<String> m_messages;
+    bool m_hasScheduledTask { false };
 };
 
 String InspectorFrontendClientLocal::Settings::getProperty(const String&)
@@ -127,6 +140,10 @@ void InspectorFrontendClientLocal::Settings::setProperty(const String&, const St
 {
 }
 
+void InspectorFrontendClientLocal::Settings::deleteProperty(const String&)
+{
+}
+
 InspectorFrontendClientLocal::InspectorFrontendClientLocal(InspectorController* inspectedPageController, Page* frontendPage, std::unique_ptr<Settings> settings)
     : m_inspectedPageController(inspectedPageController)
     , m_frontendPage(frontendPage)
@@ -146,6 +163,11 @@ InspectorFrontendClientLocal::~InspectorFrontendClientLocal()
     m_dispatchTask->reset();
 }
 
+void InspectorFrontendClientLocal::resetState()
+{
+    m_settings->deleteProperty(inspectorAttachedHeightSetting);
+}
+
 void InspectorFrontendClientLocal::windowObjectCleared()
 {
     if (m_frontendHost)
@@ -205,7 +227,7 @@ bool InspectorFrontendClientLocal::canAttachWindow()
 
 void InspectorFrontendClientLocal::setDockingUnavailable(bool unavailable)
 {
-    evaluateOnLoad(makeString("[\"setDockingUnavailable\", ", unavailable ? "true" : "false", ']'));
+    dispatch(makeString("[\"setDockingUnavailable\", ", unavailable ? "true" : "false", ']'));
 }
 
 void InspectorFrontendClientLocal::changeAttachedWindowHeight(unsigned height)
@@ -223,6 +245,11 @@ void InspectorFrontendClientLocal::changeAttachedWindowWidth(unsigned width)
     setAttachedWindowWidth(attachedWidth);
 }
 
+void InspectorFrontendClientLocal::changeSheetRect(const FloatRect& rect)
+{
+    setSheetRect(rect);
+}
+
 void InspectorFrontendClientLocal::openInNewTab(const String& url)
 {
     UserGestureIndicator indicator { ProcessingUserGesture };
@@ -270,7 +297,7 @@ void InspectorFrontendClientLocal::setAttachedWindow(DockSide dockSide)
 
     m_dockSide = dockSide;
 
-    evaluateOnLoad(makeString("[\"setDockSide\", \"", side, "\"]"));
+    dispatch(makeString("[\"setDockSide\", \"", side, "\"]"));
 }
 
 void InspectorFrontendClientLocal::restoreAttachedWindowHeight()
@@ -294,7 +321,7 @@ bool InspectorFrontendClientLocal::isDebuggingEnabled()
 
 void InspectorFrontendClientLocal::setDebuggingEnabled(bool enabled)
 {
-    evaluateOnLoad(makeString("[\"setDebuggingEnabled\", ", enabled ? "true" : "false", ']'));
+    dispatch(makeString("[\"setDebuggingEnabled\", ", enabled ? "true" : "false", ']'));
 }
 
 bool InspectorFrontendClientLocal::isTimelineProfilingEnabled()
@@ -306,7 +333,7 @@ bool InspectorFrontendClientLocal::isTimelineProfilingEnabled()
 
 void InspectorFrontendClientLocal::setTimelineProfilingEnabled(bool enabled)
 {
-    evaluateOnLoad(makeString("[\"setTimelineProfilingEnabled\", ", enabled ? "true" : "false", ']'));
+    dispatch(makeString("[\"setTimelineProfilingEnabled\", ", enabled ? "true" : "false", ']'));
 }
 
 bool InspectorFrontendClientLocal::isProfilingJavaScript()
@@ -318,28 +345,28 @@ bool InspectorFrontendClientLocal::isProfilingJavaScript()
 
 void InspectorFrontendClientLocal::startProfilingJavaScript()
 {
-    evaluateOnLoad("[\"startProfilingJavaScript\"]");
+    dispatch("[\"startProfilingJavaScript\"]");
 }
 
 void InspectorFrontendClientLocal::stopProfilingJavaScript()
 {
-    evaluateOnLoad("[\"stopProfilingJavaScript\"]");
+    dispatch("[\"stopProfilingJavaScript\"]");
 }
 
 void InspectorFrontendClientLocal::showConsole()
 {
-    evaluateOnLoad("[\"showConsole\"]");
+    dispatch("[\"showConsole\"]");
 }
 
 void InspectorFrontendClientLocal::showResources()
 {
-    evaluateOnLoad("[\"showResources\"]");
+    dispatch("[\"showResources\"]");
 }
 
 void InspectorFrontendClientLocal::showMainResourceForFrame(Frame* frame)
 {
-    String frameId = m_inspectedPageController->pageAgent()->frameId(frame);
-    evaluateOnLoad(makeString("[\"showMainResourceForFrame\", \"", frameId, "\"]"));
+    String frameId = m_inspectedPageController->ensurePageAgent().frameId(frame);
+    dispatch(makeString("[\"showMainResourceForFrame\", \"", frameId, "\"]"));
 }
 
 unsigned InspectorFrontendClientLocal::constrainedAttachedWindowHeight(unsigned preferredHeight, unsigned totalWindowHeight)
@@ -367,18 +394,44 @@ unsigned InspectorFrontendClientLocal::inspectionLevel() const
     return m_inspectedPageController->inspectionLevel() + 1;
 }
 
+void InspectorFrontendClientLocal::dispatch(const String& signature)
+{
+    ASSERT(!signature.isEmpty());
+    ASSERT(signature.startsWith('['));
+    ASSERT(signature.endsWith(']'));
+
+    evaluateOnLoad("InspectorFrontendAPI.dispatch(" + signature + ")");
+}
+
+void InspectorFrontendClientLocal::dispatchMessage(const String& messageObject)
+{
+    ASSERT(!messageObject.isEmpty());
+
+    evaluateOnLoad("InspectorFrontendAPI.dispatchMessage(" + messageObject + ")");
+}
+
+void InspectorFrontendClientLocal::dispatchMessageAsync(const String& messageObject)
+{
+    ASSERT(!messageObject.isEmpty());
+
+    evaluateOnLoad("InspectorFrontendAPI.dispatchMessageAsync(" + messageObject + ")");
+}
+
 bool InspectorFrontendClientLocal::evaluateAsBoolean(const String& expression)
 {
     auto& state = *mainWorldExecState(&m_frontendPage->mainFrame());
-    return m_frontendPage->mainFrame().script().executeScript(expression).toWTFString(&state) == "true";
+    return m_frontendPage->mainFrame().script().executeScriptIgnoringException(expression).toWTFString(&state) == "true";
 }
 
 void InspectorFrontendClientLocal::evaluateOnLoad(const String& expression)
 {
-    if (m_frontendLoaded)
-        m_frontendPage->mainFrame().script().executeScript("if (InspectorFrontendAPI) InspectorFrontendAPI.dispatch(" + expression + ")");
-    else
+    if (!m_frontendLoaded) {
         m_evaluateOnLoad.append(expression);
+        return;
+    }
+
+    JSC::SuspendExceptionScope scope(&m_frontendPage->inspectorController().vm());
+    m_frontendPage->mainFrame().script().evaluateIgnoringException(ScriptSourceCode(expression));
 }
 
 Page* InspectorFrontendClientLocal::inspectedPage() const
