@@ -37,17 +37,17 @@ class CachedSVGFont final : public CachedFont {
 public:
     CachedSVGFont(CachedResourceRequest&&, const PAL::SessionID&, const CookieJar*);
 
-    bool ensureCustomFontData(const AtomicString& remoteURI) override;
+    bool ensureCustomFontData(const AtomString& remoteURI) override;
 
-    RefPtr<Font> createFont(const FontDescription&, const AtomicString& remoteURI, bool syntheticBold, bool syntheticItalic, const FontFeatureSettings&, const FontVariantSettings&, FontSelectionSpecifiedCapabilities) override;
+    RefPtr<Font> createFont(const FontDescription&, const AtomString& remoteURI, bool syntheticBold, bool syntheticItalic, const FontFeatureSettings&, FontSelectionSpecifiedCapabilities) override;
 
 private:
-    FontPlatformData platformDataFromCustomData(const FontDescription&, bool bold, bool italic, const FontFeatureSettings&, const FontVariantSettings&, FontSelectionSpecifiedCapabilities);
+    FontPlatformData platformDataFromCustomData(const FontDescription&, bool bold, bool italic, const FontFeatureSettings&, FontSelectionSpecifiedCapabilities);
 
     SVGFontElement* getSVGFontById(const String&) const;
 
-    SVGFontElement* maybeInitializeExternalSVGFontElement(const AtomicString& remoteURI);
-    SVGFontFaceElement* firstFontFace(const AtomicString& remoteURI);
+    SVGFontElement* maybeInitializeExternalSVGFontElement(const AtomString& remoteURI);
+    SVGFontFaceElement* firstFontFace(const AtomString& remoteURI);
 
     RefPtr<SharedBuffer> m_convertedFont;
     RefPtr<SVGDocument> m_externalSVGDocument;
