@@ -69,14 +69,6 @@ RenderElement* AnimationBase::renderer() const
     return m_element ? m_element->renderer() : nullptr;
 }
 
-RenderBoxModelObject* AnimationBase::compositedRenderer() const
-{
-    auto* renderer = this->renderer();
-    if (!renderer || !renderer->isComposited())
-        return nullptr;
-    return downcast<RenderBoxModelObject>(renderer);
-}
-
 void AnimationBase::clear()
 {
     endAnimation();
@@ -89,7 +81,7 @@ void AnimationBase::setNeedsStyleRecalc(Element* element)
     if (!element || element->document().renderTreeBeingDestroyed())
         return;
 
-    ASSERT(element->document().pageCacheState() == Document::NotInPageCache);
+    ASSERT(element->document().backForwardCacheState() == Document::NotInBackForwardCache);
     element->invalidateStyle();
 }
 
@@ -679,7 +671,7 @@ void AnimationBase::freezeAtTime(double t)
     else
         m_pauseTime = m_startTime.valueOr(0) + t - m_animation->delay();
 
-    if (auto* renderer = compositedRenderer())
+    if (auto* renderer = this->renderer())
         renderer->suspendAnimations(MonotonicTime::fromRawSeconds(m_pauseTime.value()));
 }
 
