@@ -28,8 +28,17 @@
 #include <cstdint>
 #include <wtf/Assertions.h>
 
+namespace WTF {
+class TextStream;
+}
+
 namespace WebCore {
 
+enum class ScrollType : uint8_t {
+    User,
+    Programmatic
+};
+
 enum ScrollDirection : uint8_t {
     ScrollUp,
     ScrollDown,
@@ -44,7 +53,6 @@ enum ScrollLogicalDirection : uint8_t {
     ScrollInlineDirectionForward
 };
 
-
 inline ScrollDirection logicalToPhysical(ScrollLogicalDirection direction, bool isVertical, bool isFlipped)
 {
     switch (direction) {
@@ -210,8 +218,17 @@ enum class SelectionRevealMode : uint8_t  {
     DoNotReveal
 };
 
+enum class ScrollPositioningBehavior : uint8_t {
+    None,
+    Moves,
+    Stationary
+};
+
 using ScrollbarControlState = unsigned;
 using ScrollbarControlPartMask = unsigned;
 using ScrollingNodeID = uint64_t;
 
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollType);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollClamping);
+
 } // namespace WebCore
