@@ -31,13 +31,12 @@
 #include "SharedTimer.h"
 #include "ThreadGlobalData.h"
 #include "ThreadTimers.h"
-#include <limits.h>
 #include <limits>
 #include <math.h>
 #include <wtf/MainThread.h>
 #include <wtf/Vector.h>
 
-#if PLATFORM(IOS_FAMILY) || PLATFORM(MAC)
+#if PLATFORM(COCOA)
 #include <wtf/spi/darwin/dyldSPI.h>
 #endif
 
@@ -50,7 +49,7 @@ class TimerHeapReference;
 // Then we set a single shared system timer to fire at that time.
 //
 // When a timer's "next fire time" changes, we need to move it around in the priority queue.
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
 static ThreadTimerHeap& threadGlobalTimerHeap()
 {
     return threadGlobalData().threadTimers().timerHeap();
@@ -254,26 +253,23 @@ static bool shouldSuppressThreadSafetyCheck()
 }
 
 TimerBase::TimerBase()
-    : m_wasDeleted(false)
 {
 }
 
 TimerBase::~TimerBase()
 {
-    ASSERT(canAccessThreadLocalDataForThread(m_thread.get()));
-    RELEASE_ASSERT(canAccessThreadLocalDataForThread(m_thread.get()) || shouldSuppressThreadSafetyCheck());
+    ASSERT(canCurrentThreadAccessThreadLocalData(m_thread));
+    RELEASE_ASSERT(canCurrentThreadAccessThreadLocalData(m_thread) || shouldSuppressThreadSafetyCheck());
     stop();
     ASSERT(!inHeap());
-    if (m_heapItem) {
+    if (m_heapItem)
         m_heapItem->clearTimer();
-        m_heapItem = nullptr;
-    }
-    m_wasDeleted = true;
+    m_unalignedNextFireTime = MonotonicTime::nan();
 }
 
 void TimerBase::start(Seconds nextFireInterval, Seconds repeatInterval)
 {
-    ASSERT(canAccessThreadLocalDataForThread(m_thread.get()));
+    ASSERT(canCurrentThreadAccessThreadLocalData(m_thread));
 
     m_repeatInterval = repeatInterval;
     setNextFireTime(MonotonicTime::now() + nextFireInterval);
@@ -281,7 +277,7 @@ void TimerBase::start(Seconds nextFireInterval, Seconds repeatInterval)
 
 void TimerBase::stop()
 {
-    ASSERT(canAccessThreadLocalDataForThread(m_thread.get()));
+    ASSERT(canCurrentThreadAccessThreadLocalData(m_thread));
 
     m_repeatInterval = 0_s;
     setNextFireTime(MonotonicTime { });
@@ -304,7 +300,7 @@ Seconds TimerBase::nextFireInterval() const
 
 inline void TimerBase::checkHeapIndex() const
 {
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
     ASSERT(m_heapItem);
     auto& heap = m_heapItem->timerHeap();
     ASSERT(&heap == &threadGlobalTimerHeap());
@@ -438,7 +434,7 @@ void TimerBase::updateHeapIfNeeded(MonotonicTime oldTime)
     if (fireTime && hasValidHeapPosition())
         return;
 
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
     Optional<unsigned> oldHeapIndex;
     if (m_heapItem->isInHeap())
         oldHeapIndex = m_heapItem->heapIndex();
@@ -453,7 +449,7 @@ void TimerBase::updateHeapIfNeeded(MonotonicTime oldTime)
     else
         heapIncreaseKey();
 
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
     Optional<unsigned> newHeapIndex;
     if (m_heapItem->isInHeap())
         newHeapIndex = m_heapItem->heapIndex();
@@ -465,12 +461,15 @@ void TimerBase::updateHeapIfNeeded(MonotonicTime oldTime)
 
 void TimerBase::setNextFireTime(MonotonicTime newTime)
 {
-    ASSERT(canAccessThreadLocalDataForThread(m_thread.get()));
-    RELEASE_ASSERT(canAccessThreadLocalDataForThread(m_thread.get()) || shouldSuppressThreadSafetyCheck());
-    RELEASE_ASSERT_WITH_SECURITY_IMPLICATION(!m_wasDeleted);
+    ASSERT(canCurrentThreadAccessThreadLocalData(m_thread));
+    RELEASE_ASSERT(canCurrentThreadAccessThreadLocalData(m_thread) || shouldSuppressThreadSafetyCheck());
+    bool timerHasBeenDeleted = std::isnan(m_unalignedNextFireTime);
+    RELEASE_ASSERT_WITH_SECURITY_IMPLICATION(!timerHasBeenDeleted);
 
-    if (m_unalignedNextFireTime != newTime)
+    if (m_unalignedNextFireTime != newTime) {
+        RELEASE_ASSERT(!std::isnan(newTime));
         m_unalignedNextFireTime = newTime;
+    }
 
     // Keep heap valid while changing the next-fire time.
     MonotonicTime oldTime = nextFireTime();
@@ -481,9 +480,7 @@ void TimerBase::setNextFireTime(MonotonicTime newTime)
     }
 
     if (oldTime != newTime) {
-        // FIXME: This should be part of ThreadTimers, or another per-thread structure.
-        static std::atomic<unsigned> currentHeapInsertionOrder;
-        auto newOrder = currentHeapInsertionOrder++;
+        auto newOrder = threadGlobalData().threadTimers().nextHeapInsertionCount();
 
         if (!m_heapItem)
             m_heapItem = ThreadTimerHeapItem::create(*this, newTime, 0);
@@ -517,7 +514,9 @@ void TimerBase::didChangeAlignmentInterval()
 Seconds TimerBase::nextUnalignedFireInterval() const
 {
     ASSERT(isActive());
-    return std::max(m_unalignedNextFireTime - MonotonicTime::now(), 0_s);
+    auto result = std::max(m_unalignedNextFireTime - MonotonicTime::now(), 0_s);
+    RELEASE_ASSERT(std::isfinite(result));
+    return result;
 }
 
 } // namespace WebCore
