@@ -84,7 +84,7 @@ void FontCascadeFonts::GlyphPageCacheEntry::setGlyphDataForCharacter(UChar32 cha
 {
     ASSERT(!glyphDataForCharacter(character).glyph);
     if (!m_mixedFont) {
-        m_mixedFont = std::make_unique<MixedFontGlyphPage>(m_singleFont.get());
+        m_mixedFont = makeUnique<MixedFontGlyphPage>(m_singleFont.get());
         m_singleFont = nullptr;
     }
     m_mixedFont->setGlyphDataForCharacter(character, glyphData);
@@ -96,6 +96,8 @@ void FontCascadeFonts::GlyphPageCacheEntry::setSingleFontPage(RefPtr<GlyphPage>&
     m_singleFont = page;
 }
 
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(FontCascadeFonts);
+
 FontCascadeFonts::FontCascadeFonts(RefPtr<FontSelector>&& fontSelector)
     : m_cachedPrimaryFont(nullptr)
     , m_fontSelector(fontSelector)
@@ -140,7 +142,7 @@ static FontRanges realizeNextFallback(const FontCascadeDescription& description,
 
     auto& fontCache = FontCache::singleton();
     while (index < description.effectiveFamilyCount()) {
-        auto visitor = WTF::makeVisitor([&](const AtomicString& family) -> FontRanges {
+        auto visitor = WTF::makeVisitor([&](const AtomString& family) -> FontRanges {
             if (family.isEmpty())
                 return FontRanges();
             if (fontSelector) {
@@ -349,6 +351,10 @@ GlyphData FontCascadeFonts::glyphDataForSystemFallback(UChar32 character, const
     if (!systemFallbackFont)
         return GlyphData();
 
+#if HAVE(DISALLOWABLE_USER_INSTALLED_FONTS)
+    ASSERT(!systemFallbackFont->isUserInstalledFont() || description.shouldAllowUserInstalledFonts() == AllowUserInstalledFonts::Yes);
+#endif
+
     if (systemFallbackShouldBeInvisible)
         systemFallbackFont = const_cast<Font*>(&systemFallbackFont->invisibleFont());
 
