@@ -20,8 +20,13 @@
 #ifndef OpenGLShims_h
 #define OpenGLShims_h
 
+#if OS(WINDOWS)
+#include <GL/gl.h>
+#include <GLES2/gl2.h>
+#else
 #include <GL/gl.h>
 #include <GL/glext.h>
+#endif
 
 #if defined(GL_ES_VERSION_2_0)
 // Some openGL ES systems miss this typedef.
@@ -37,6 +42,7 @@ OpenGLFunctionTable* openGLFunctionTable();
 
 #if OS(WINDOWS)
 #define GLAPIENTRY __stdcall
+typedef char GLchar;
 #else
 #define GLAPIENTRY
 #endif
@@ -92,6 +98,7 @@ typedef void (GLAPIENTRY *glGetRenderbufferParameterivType) (GLenum, GLenum, GLi
 typedef void (GLAPIENTRY *glGetShaderInfoLogType) (GLuint, GLsizei, GLsizei*, char*);
 typedef void (GLAPIENTRY *glGetShaderivType) (GLuint, GLenum, GLint*);
 typedef void (GLAPIENTRY *glGetShaderSourceType) (GLuint, GLsizei, GLsizei*, char*);
+typedef const GLubyte* (GLAPIENTRY *glGetStringType) (GLenum);
 typedef const GLubyte* (GLAPIENTRY *glGetStringiType) (GLenum, GLuint);
 typedef GLint (GLAPIENTRY *glGetUniformLocationType) (GLuint, const char*);
 typedef void (GLAPIENTRY *glGetUniformfvType) (GLuint, GLint, GLfloat*);
@@ -199,6 +206,7 @@ typedef struct _OpenGLFunctionTable {
     FUNCTION_TABLE_ENTRY(glGetShaderInfoLog);
     FUNCTION_TABLE_ENTRY(glGetShaderiv);
     FUNCTION_TABLE_ENTRY(glGetShaderSource);
+    FUNCTION_TABLE_ENTRY(glGetString);
     FUNCTION_TABLE_ENTRY(glGetStringi);
     FUNCTION_TABLE_ENTRY(glGetUniformfv);
     FUNCTION_TABLE_ENTRY(glGetUniformiv);
@@ -327,6 +335,7 @@ typedef struct _OpenGLFunctionTable {
 #define glGetShaderInfoLog                     LOOKUP_GL_FUNCTION(glGetShaderInfoLog)
 #define glGetShaderiv                          LOOKUP_GL_FUNCTION(glGetShaderiv)
 #define glGetShaderSource                      LOOKUP_GL_FUNCTION(glGetShaderSource)
+#define glGetString                            LOOKUP_GL_FUNCTION(glGetString)
 #define glGetStringi                           LOOKUP_GL_FUNCTION(glGetStringi)
 #define glGetUniformfv                         LOOKUP_GL_FUNCTION(glGetUniformfv)
 #define glGetUniformiv                         LOOKUP_GL_FUNCTION(glGetUniformiv)
