@@ -28,17 +28,22 @@
 #if ENABLE(WEBGPU)
 
 #include "GPUExtent3D.h"
-#include "GPUTextureDimension.h"
 #include "GPUTextureFormat.h"
 #include "GPUTextureUsage.h"
 
 namespace WebCore {
 
+enum class GPUTextureDimension {
+    _1d,
+    _2d,
+    _3d
+};
+
 struct GPUTextureDescriptor {
     GPUExtent3D size;
-    unsigned long arrayLayerCount;
-    unsigned long mipLevelCount;
-    unsigned long sampleCount;
+    unsigned arrayLayerCount;
+    unsigned mipLevelCount;
+    unsigned sampleCount;
     GPUTextureDimension dimension;
     GPUTextureFormat format;
     GPUTextureUsageFlags usage;
