@@ -57,12 +57,12 @@ void BitmapTextureJava::updateContents(TextureMapper& mapper, GraphicsLayer* sou
 void BitmapTextureJava::didReset()
 {
     float devicePixelRatio = 1.0;
-    m_image = ImageBuffer::create(contentSize(), Accelerated, devicePixelRatio);
+    m_image = ImageBuffer::create(contentSize(), RenderingMode::Accelerated, devicePixelRatio);
 }
 
 void BitmapTextureJava::updateContents(Image* image, const IntRect& targetRect, const IntPoint& offset)
 {
-    m_image->context().drawImage(*image, targetRect, IntRect(offset, targetRect.size()), CompositeCopy);
+    m_image->context().drawImage(*image, targetRect, IntRect(offset, targetRect.size()), CompositeOperator::Copy);
 }
 
 RefPtr<BitmapTexture> BitmapTextureJava::applyFilters(TextureMapper&, const FilterOperations&)
