@@ -45,7 +45,7 @@ void GIFImageDecoder::setData(SharedBuffer& data, bool allDataReceived)
 
     ScalableImageDecoder::setData(data, allDataReceived);
     if (m_reader)
-        m_reader->setData(&data);
+        m_reader->setData(*m_data);
 }
 
 bool GIFImageDecoder::setSize(const IntSize& size)
@@ -53,11 +53,7 @@ bool GIFImageDecoder::setSize(const IntSize& size)
     if (ScalableImageDecoder::encodedDataStatus() >= EncodedDataStatus::SizeAvailable && this->size() == size)
         return true;
 
-    if (!ScalableImageDecoder::setSize(size))
-        return false;
-
-    prepareScaleDataIfNecessary();
-    return true;
+    return ScalableImageDecoder::setSize(size);
 }
 
 size_t GIFImageDecoder::frameCount() const
@@ -126,9 +122,6 @@ size_t GIFImageDecoder::findFirstRequiredFrameToDecode(size_t frameIndex)
             const auto* frameContext = m_reader->frameContext(i - 1);
             ASSERT(frameContext);
             IntRect frameRect(frameContext->xOffset, frameContext->yOffset, frameContext->width, frameContext->height);
-            // We would need to scale frameRect and check whether it fills the whole scaledSize(). But
-            // can check whether the original frameRect fills size() instead. If the frame fills the
-            // whole area then it can be decoded without dependencies.
             if (frameRect.contains({ { }, size() }))
                 return i;
         }
@@ -216,11 +209,11 @@ bool GIFImageDecoder::haveDecodedRow(unsigned frameIndex, const Vector<unsigned
     // that width == (size().width() - frameContext->xOffset), so
     // we must ensure we don't run off the end of either the source data or the
     // row's X-coordinates.
-    int xBegin = upperBoundScaledX(frameContext->xOffset);
-    int yBegin = upperBoundScaledY(frameContext->yOffset + rowNumber);
-    int xEnd = lowerBoundScaledX(std::min(static_cast<int>(frameContext->xOffset + width), size().width()) - 1, xBegin + 1) + 1;
-    int yEnd = lowerBoundScaledY(std::min(static_cast<int>(frameContext->yOffset + rowNumber + repeatCount), size().height()) - 1, yBegin + 1) + 1;
-    if (rowBuffer.isEmpty() || (xBegin < 0) || (yBegin < 0) || (xEnd <= xBegin) || (yEnd <= yBegin))
+    int xBegin = frameContext->xOffset;
+    int yBegin = frameContext->yOffset + rowNumber;
+    int xEnd = std::min(static_cast<int>(frameContext->xOffset + width), size().width());
+    int yEnd = std::min(static_cast<int>(frameContext->yOffset + rowNumber + repeatCount), size().height());
+    if (rowBuffer.isEmpty() || xEnd <= xBegin || yEnd <= yBegin)
         return true;
 
     // Get the colormap.
@@ -244,7 +237,7 @@ bool GIFImageDecoder::haveDecodedRow(unsigned frameIndex, const Vector<unsigned
     auto* currentAddress = buffer.backingStore()->pixelAt(xBegin, yBegin);
     // Write one row's worth of data into the frame.
     for (int x = xBegin; x < xEnd; ++x) {
-        const unsigned char sourceValue = rowBuffer[(m_scaled ? m_scaledColumns[x] : x) - frameContext->xOffset];
+        const unsigned char sourceValue = rowBuffer[x - frameContext->xOffset];
         if ((!frameContext->isTransparent || (sourceValue != frameContext->tpixel)) && (sourceValue < colorMapSize)) {
             const size_t colorIndex = static_cast<size_t>(sourceValue) * 3;
             buffer.backingStore()->setPixel(currentAddress, colorMap[colorIndex], colorMap[colorIndex + 1], colorMap[colorIndex + 2], 255);
@@ -287,7 +280,7 @@ bool GIFImageDecoder::frameComplete(unsigned frameIndex, unsigned frameDuration,
 
         // The whole frame was non-transparent, so it's possible that the entire
         // resulting buffer was non-transparent, and we can setHasAlpha(false).
-        if (rect.contains(IntRect(IntPoint(), scaledSize())))
+        if (rect.contains(IntRect(IntPoint(), size())))
             buffer.setHasAlpha(false);
         else if (frameIndex) {
             // Tricky case.  This frame does not have alpha only if everywhere
@@ -334,8 +327,8 @@ void GIFImageDecoder::decode(unsigned haltAtFrame, GIFQuery query, bool allDataR
         return;
 
     if (!m_reader) {
-        m_reader = std::make_unique<GIFImageReader>(this);
-        m_reader->setData(m_data.get());
+        m_reader = makeUnique<GIFImageReader>(this);
+        m_reader->setData(*m_data);
     }
 
     if (query == GIFSizeQuery) {
@@ -374,7 +367,7 @@ bool GIFImageDecoder::initFrameBuffer(unsigned frameIndex)
 
     if (!frameIndex) {
         // This is the first frame, so we're not relying on any previous data.
-        if (!buffer->initialize(scaledSize(), m_premultiplyAlpha))
+        if (!buffer->initialize(size(), m_premultiplyAlpha))
             return setFailed();
     } else {
         // The starting state for this frame depends on the previous frame's
@@ -402,8 +395,8 @@ bool GIFImageDecoder::initFrameBuffer(unsigned frameIndex)
             // We want to clear the previous frame to transparent, without
             // affecting pixels in the image outside of the frame.
             IntRect prevRect = prevBuffer->backingStore()->frameRect();
-            const IntSize& bufferSize = scaledSize();
-            if (!frameIndex || prevRect.contains(IntRect(IntPoint(), scaledSize()))) {
+            const IntSize& bufferSize = size();
+            if (!frameIndex || prevRect.contains(IntRect(IntPoint(), size()))) {
                 // Clearing the first frame, or a frame the size of the whole
                 // image, results in a completely empty image.
                 if (!buffer->initialize(bufferSize, m_premultiplyAlpha))
@@ -424,11 +417,7 @@ bool GIFImageDecoder::initFrameBuffer(unsigned frameIndex)
     if (frameRect.maxY() > size().height())
         frameRect.setHeight(size().height() - frameContext->yOffset);
 
-    int left = upperBoundScaledX(frameRect.x());
-    int right = lowerBoundScaledX(frameRect.maxX(), left);
-    int top = upperBoundScaledY(frameRect.y());
-    int bottom = lowerBoundScaledY(frameRect.maxY(), top);
-    buffer->backingStore()->setFrameRect(IntRect(left, top, right - left, bottom - top));
+    buffer->backingStore()->setFrameRect(frameRect);
 
     // Update our status to be partially complete.
     buffer->setDecodingStatus(DecodingStatus::Partial);
