@@ -522,6 +522,16 @@ String AXDefinitionListDefinitionText()
     return getLocalizedProperty("AXDefinitionListDefinitionText");
 }
 
+String AXFigureText()
+{
+    return getLocalizedProperty("AXFigureText");
+}
+
+String AXARIAContentGroupText(const String& ariaType)
+{
+    return getLocalizedProperty(String("AXARIAContentGroupText") + ariaType);
+}
+
 String AXButtonActionVerb()
 {
     return getLocalizedProperty("AXButtonActionVerb");
@@ -712,6 +722,11 @@ String unsupportedPluginText()
     return String::fromUTF8("Unsupported Plug-in");
 }
 
+String pluginTooSmallText()
+{
+    return String::fromUTF8("Plug-In too small");
+}
+
 String localizedString(const char* key)
 {
     return String::fromUTF8(key, strlen(key));
