@@ -432,7 +432,7 @@ String PlatformKeyboardEvent::keyValueForWPEKeyCode(unsigned keyCode)
     UChar32 unicodeCharacter = wpe_key_code_to_unicode(keyCode);
     if (unicodeCharacter && U_IS_UNICODE_CHAR(unicodeCharacter)) {
         StringBuilder builder;
-        builder.append(unicodeCharacter);
+        builder.appendCharacter(unicodeCharacter);
         return builder.toString();
     }
 
@@ -1309,7 +1309,7 @@ String PlatformKeyboardEvent::singleCharacterString(unsigned val)
     UChar32 unicodeCharacter = wpe_key_code_to_unicode(val);
     if (unicodeCharacter && U_IS_UNICODE_CHAR(unicodeCharacter)) {
         StringBuilder builder;
-        builder.append(unicodeCharacter);
+        builder.appendCharacter(unicodeCharacter);
         return builder.toString();
     }
 
@@ -1320,7 +1320,7 @@ void PlatformKeyboardEvent::disambiguateKeyDownEvent(Type type, bool backwardsCo
 {
     ASSERT(m_type == KeyDown);
     m_type = type;
-    if (backwardsCompatibility)
+    if (backwardsCompatibility || m_handledByInputMethod)
         return;
 
     if (type == PlatformEvent::RawKeyDown) {
