@@ -33,6 +33,7 @@
 
 #include "SQLiteDatabase.h"
 #include "SQLiteStatement.h"
+#include <pal/crypto/CryptoDigest.h>
 #include <sqlite3.h>
 #include <wtf/FileSystem.h>
 
@@ -46,11 +47,6 @@ SQLiteFileSystem::SQLiteFileSystem()
 {
 }
 
-int SQLiteFileSystem::openDatabase(const String& filename, sqlite3** database, bool)
-{
-    return sqlite3_open_v2(FileSystem::fileSystemRepresentation(filename).data(), database, SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE | SQLITE_OPEN_AUTOPROXY, nullptr);
-}
-
 String SQLiteFileSystem::appendDatabaseFileNameToPath(const String& path, const String& fileName)
 {
     return FileSystem::pathByAppendingComponent(path, fileName);
@@ -104,8 +100,19 @@ bool SQLiteFileSystem::truncateDatabaseFile(sqlite3* database)
 
 long long SQLiteFileSystem::getDatabaseFileSize(const String& fileName)
 {
-    long long size;
-    return FileSystem::getFileSize(fileName, size) ? size : 0;
+    long long fileSize = 0;
+    long long totalSize = 0;
+
+    if (FileSystem::getFileSize(fileName, fileSize))
+        totalSize += fileSize;
+
+    if (FileSystem::getFileSize(makeString(fileName, "-wal"_s), fileSize))
+        totalSize += fileSize;
+
+    if (FileSystem::getFileSize(makeString(fileName, "-shm"_s), fileSize))
+        totalSize += fileSize;
+
+    return totalSize;
 }
 
 Optional<WallTime> SQLiteFileSystem::databaseCreationTime(const String& fileName)
@@ -118,4 +125,22 @@ Optional<WallTime> SQLiteFileSystem::databaseModificationTime(const String& file
     return FileSystem::getFileModificationTime(fileName);
 }
 
+String SQLiteFileSystem::computeHashForFileName(const String& fileName)
+{
+    auto cryptoDigest = PAL::CryptoDigest::create(PAL::CryptoDigest::Algorithm::SHA_256);
+    cryptoDigest->addBytes(fileName.utf8().data(), fileName.utf8().length());
+    auto digest = cryptoDigest->computeHash();
+
+    // Convert digest to hex.
+    char* start = 0;
+    unsigned digestLength = digest.size();
+    CString result = CString::newUninitialized(digestLength * 2, start);
+    char* buffer = start;
+    for (size_t i = 0; i < digestLength; ++i) {
+        snprintf(buffer, 3, "%02X", digest.at(i));
+        buffer += 2;
+    }
+    return String::fromUTF8(result);
+}
+
 } // namespace WebCore
