@@ -45,9 +45,6 @@ SQLiteStatement::SQLiteStatement(SQLiteDatabase& db, const String& sql)
     : m_database(db)
     , m_query(sql)
     , m_statement(0)
-#ifndef NDEBUG
-    , m_isPrepared(false)
-#endif
 {
 }
 
@@ -79,7 +76,7 @@ int SQLiteStatement::prepare()
     if (tail && *tail)
         error = SQLITE_ERROR;
 
-#ifndef NDEBUG
+#if ASSERT_ENABLED
     m_isPrepared = error == SQLITE_OK;
 #endif
     return error;
@@ -108,7 +105,7 @@ int SQLiteStatement::step()
 
 int SQLiteStatement::finalize()
 {
-#ifndef NDEBUG
+#if ASSERT_ENABLED
     m_isPrepared = false;
 #endif
     if (!m_statement)
@@ -160,7 +157,7 @@ int SQLiteStatement::bindBlob(int index, const void* blob, int size)
     ASSERT(m_isPrepared);
     ASSERT(index > 0);
     ASSERT(static_cast<unsigned>(index) <= bindParameterCount());
-    ASSERT(blob);
+    ASSERT(blob || !size);
     ASSERT(size >= 0);
 
     if (!m_statement)
