@@ -321,7 +321,7 @@ void BidiResolverBase<Iterator, Run, DerivedClass>::appendRunInternal()
         }
 
         if (endOffset >= startOffset)
-            m_runs.appendRun(std::make_unique<Run>(startOffset, endOffset + 1, context(), m_direction));
+            m_runs.appendRun(makeUnique<Run>(startOffset, endOffset + 1, context(), m_direction));
 
         m_eor.increment();
         m_sor = m_eor;
@@ -896,7 +896,8 @@ void BidiResolverBase<Iterator, Run, DerivedClass>::createBidiRunsForLine(const
                         m_direction = m_status.lastStrong == U_LEFT_TO_RIGHT ? U_LEFT_TO_RIGHT : U_EUROPEAN_NUMBER;
                         break;
                     default:
-                        ASSERT_NOT_REACHED();
+                        // FIXME: handle neutrals in this case. See https://bugs.webkit.org/show_bug.cgi?id=204817
+                        break;
                 }
                 appendRun();
             }
