@@ -96,7 +96,7 @@ HitTestResult::HitTestResult(const HitTestResult& other)
     , m_isOverWidget(other.isOverWidget())
 {
     // Only copy the NodeSet in case of list hit test.
-    m_listBasedTestResult = other.m_listBasedTestResult ? std::make_unique<NodeSet>(*other.m_listBasedTestResult) : nullptr;
+    m_listBasedTestResult = other.m_listBasedTestResult ? makeUnique<NodeSet>(*other.m_listBasedTestResult) : nullptr;
 }
 
 HitTestResult::~HitTestResult() = default;
@@ -113,7 +113,7 @@ HitTestResult& HitTestResult::operator=(const HitTestResult& other)
     m_isOverWidget = other.isOverWidget();
 
     // Only copy the NodeSet in case of list hit test.
-    m_listBasedTestResult = other.m_listBasedTestResult ? std::make_unique<NodeSet>(*other.m_listBasedTestResult) : nullptr;
+    m_listBasedTestResult = other.m_listBasedTestResult ? makeUnique<NodeSet>(*other.m_listBasedTestResult) : nullptr;
 
     return *this;
 }
@@ -349,7 +349,7 @@ URL HitTestResult::absoluteImageURL() const
     if (!(m_innerNonSharedNode->renderer() && m_innerNonSharedNode->renderer()->isImage()))
         return URL();
 
-    AtomicString urlString;
+    AtomString urlString;
     if (is<HTMLEmbedElement>(*m_innerNonSharedNode)
         || is<HTMLImageElement>(*m_innerNonSharedNode)
         || is<HTMLInputElement>(*m_innerNonSharedNode)
@@ -690,14 +690,14 @@ void HitTestResult::append(const HitTestResult& other, const HitTestRequest& req
 const HitTestResult::NodeSet& HitTestResult::listBasedTestResult() const
 {
     if (!m_listBasedTestResult)
-        m_listBasedTestResult = std::make_unique<NodeSet>();
+        m_listBasedTestResult = makeUnique<NodeSet>();
     return *m_listBasedTestResult;
 }
 
 HitTestResult::NodeSet& HitTestResult::mutableListBasedTestResult()
 {
     if (!m_listBasedTestResult)
-        m_listBasedTestResult = std::make_unique<NodeSet>();
+        m_listBasedTestResult = makeUnique<NodeSet>();
     return *m_listBasedTestResult;
 }
 
