@@ -28,7 +28,6 @@
  */
 
 #include "config.h"
-
 #include "BasicShapes.h"
 
 #include "BasicShapeFunctions.h"
@@ -40,9 +39,10 @@
 #include "RenderBox.h"
 #include "SVGPathByteStream.h"
 #include "SVGPathUtilities.h"
-
+#include <wtf/MathExtras.h>
 #include <wtf/NeverDestroyed.h>
 #include <wtf/TinyLRUCache.h>
+#include <wtf/text/TextStream.h>
 
 namespace WebCore {
 
@@ -158,7 +158,7 @@ bool BasicShapeCircle::operator==(const BasicShape& other) const
 float BasicShapeCircle::floatValueForRadiusInBox(float boxWidth, float boxHeight) const
 {
     if (m_radius.type() == BasicShapeRadius::Value)
-        return floatValueForLength(m_radius.value(), sqrtf((boxWidth * boxWidth + boxHeight * boxHeight) / 2));
+        return floatValueForLength(m_radius.value(), std::hypot(boxWidth, boxHeight) / sqrtOfTwoFloat);
 
     float centerX = floatValueForCenterCoordinate(m_centerX, boxWidth);
     float centerY = floatValueForCenterCoordinate(m_centerY, boxHeight);
@@ -198,7 +198,14 @@ Ref<BasicShape> BasicShapeCircle::blend(const BasicShape& other, double progress
     result->setCenterX(m_centerX.blend(otherCircle.centerX(), progress));
     result->setCenterY(m_centerY.blend(otherCircle.centerY(), progress));
     result->setRadius(m_radius.blend(otherCircle.radius(), progress));
-    return WTFMove(result);
+    return result;
+}
+
+void BasicShapeCircle::dump(TextStream& ts) const
+{
+    ts.dumpProperty("center-x", centerX());
+    ts.dumpProperty("center-y", centerY());
+    ts.dumpProperty("radius", radius());
 }
 
 bool BasicShapeEllipse::operator==(const BasicShape& other) const
@@ -257,14 +264,22 @@ Ref<BasicShape> BasicShapeEllipse::blend(const BasicShape& other, double progres
         result->setCenterY(otherEllipse.centerY());
         result->setRadiusX(otherEllipse.radiusX());
         result->setRadiusY(otherEllipse.radiusY());
-        return WTFMove(result);
+        return result;
     }
 
     result->setCenterX(m_centerX.blend(otherEllipse.centerX(), progress));
     result->setCenterY(m_centerY.blend(otherEllipse.centerY(), progress));
     result->setRadiusX(m_radiusX.blend(otherEllipse.radiusX(), progress));
     result->setRadiusY(m_radiusY.blend(otherEllipse.radiusY(), progress));
-    return WTFMove(result);
+    return result;
+}
+
+void BasicShapeEllipse::dump(TextStream& ts) const
+{
+    ts.dumpProperty("center-x", centerX());
+    ts.dumpProperty("center-y", centerY());
+    ts.dumpProperty("radius-x", radiusX());
+    ts.dumpProperty("radius-y", radiusY());
 }
 
 bool BasicShapePolygon::operator==(const BasicShape& other) const
@@ -311,7 +326,7 @@ Ref<BasicShape> BasicShapePolygon::blend(const BasicShape& other, double progres
     size_t length = m_values.size();
     auto result = BasicShapePolygon::create();
     if (!length)
-        return WTFMove(result);
+        return result;
 
     result->setWindRule(otherPolygon.windRule());
 
@@ -321,7 +336,13 @@ Ref<BasicShape> BasicShapePolygon::blend(const BasicShape& other, double progres
             WebCore::blend(otherPolygon.values().at(i + 1), m_values.at(i + 1), progress));
     }
 
-    return WTFMove(result);
+    return result;
+}
+
+void BasicShapePolygon::dump(TextStream& ts) const
+{
+    ts.dumpProperty("wind-rule", windRule());
+    ts.dumpProperty("path", values());
 }
 
 BasicShapePath::BasicShapePath(std::unique_ptr<SVGPathByteStream>&& byteStream)
@@ -358,12 +379,18 @@ Ref<BasicShape> BasicShapePath::blend(const BasicShape& from, double progress) c
 
     auto& fromPath = downcast<BasicShapePath>(from);
 
-    auto resultingPathBytes = std::make_unique<SVGPathByteStream>();
+    auto resultingPathBytes = makeUnique<SVGPathByteStream>();
     buildAnimatedSVGPathByteStream(*fromPath.m_byteStream, *m_byteStream, *resultingPathBytes, progress);
 
     auto result = BasicShapePath::create(WTFMove(resultingPathBytes));
     result->setWindRule(windRule());
-    return WTFMove(result);
+    return result;
+}
+
+void BasicShapePath::dump(TextStream& ts) const
+{
+    ts.dumpProperty("wind-rule", windRule());
+    // FIXME: print the byte stream?
 }
 
 bool BasicShapeInset::operator==(const BasicShape& other) const
@@ -425,6 +452,50 @@ Ref<BasicShape> BasicShapeInset::blend(const BasicShape& from, double progress)
     result->setBottomRightRadius(WebCore::blend(fromInset.bottomRightRadius(), bottomRightRadius(), progress));
     result->setBottomLeftRadius(WebCore::blend(fromInset.bottomLeftRadius(), bottomLeftRadius(), progress));
 
-    return WTFMove(result);
+    return result;
 }
+
+void BasicShapeInset::dump(TextStream& ts) const
+{
+    ts.dumpProperty("top", top());
+    ts.dumpProperty("right", right());
+    ts.dumpProperty("bottom", bottom());
+    ts.dumpProperty("left", left());
+
+    ts.dumpProperty("top-left-radius", topLeftRadius());
+    ts.dumpProperty("top-right-radius", topRightRadius());
+    ts.dumpProperty("bottom-right-radius", bottomRightRadius());
+    ts.dumpProperty("bottom-left-radius", bottomLeftRadius());
 }
+
+static TextStream& operator<<(TextStream& ts, BasicShapeRadius::Type radiusType)
+{
+    switch (radiusType) {
+    case BasicShapeRadius::Value: ts << "value"; break;
+    case BasicShapeRadius::ClosestSide: ts << "closest-side"; break;
+    case BasicShapeRadius::FarthestSide: ts << "farthest-side"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, const BasicShapeRadius& radius)
+{
+    ts.dumpProperty("value", radius.value());
+    ts.dumpProperty("type", radius.type());
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, const BasicShapeCenterCoordinate& coordinate)
+{
+    ts.dumpProperty("direction", coordinate.direction() == BasicShapeCenterCoordinate::TopLeft ? "top left" : "bottom right");
+    ts.dumpProperty("length", coordinate.length());
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, const BasicShape& shape)
+{
+    shape.dump(ts);
+    return ts;
+}
+
+} // namespace WebCore
