@@ -52,7 +52,7 @@ FillLayer::FillLayer(FillLayerType type)
     , m_origin(static_cast<unsigned>(FillLayer::initialFillOrigin(type)))
     , m_repeatX(static_cast<unsigned>(FillLayer::initialFillRepeatX(type)))
     , m_repeatY(static_cast<unsigned>(FillLayer::initialFillRepeatY(type)))
-    , m_composite(FillLayer::initialFillComposite(type))
+    , m_composite(static_cast<unsigned>(FillLayer::initialFillComposite(type)))
     , m_sizeType(static_cast<unsigned>(FillSizeType::None))
     , m_blendMode(static_cast<unsigned>(FillLayer::initialFillBlendMode(type)))
     , m_maskSourceType(static_cast<unsigned>(FillLayer::initialFillMaskSourceType(type)))
@@ -76,7 +76,7 @@ FillLayer::FillLayer(FillLayerType type)
 }
 
 FillLayer::FillLayer(const FillLayer& o)
-    : m_next(o.m_next ? std::make_unique<FillLayer>(*o.m_next) : nullptr)
+    : m_next(o.m_next ? makeUnique<FillLayer>(*o.m_next) : nullptr)
     , m_image(o.m_image)
     , m_xPosition(o.m_xPosition)
     , m_yPosition(o.m_yPosition)
@@ -117,7 +117,7 @@ FillLayer::~FillLayer()
 
 FillLayer& FillLayer::operator=(const FillLayer& o)
 {
-    m_next = o.m_next ? std::make_unique<FillLayer>(*o.m_next) : nullptr;
+    m_next = o.m_next ? makeUnique<FillLayer>(*o.m_next) : nullptr;
 
     m_image = o.m_image;
     m_xPosition = o.m_xPosition;
@@ -354,10 +354,10 @@ bool FillLayer::hasOpaqueImage(const RenderElement& renderer) const
     if (!m_image)
         return false;
 
-    if (m_composite == CompositeClear || m_composite == CompositeCopy)
+    if (static_cast<CompositeOperator>(m_composite) == CompositeOperator::Clear || static_cast<CompositeOperator>(m_composite) == CompositeOperator::Copy)
         return true;
 
-    return static_cast<BlendMode>(m_blendMode) == BlendMode::Normal && m_composite == CompositeSourceOver && m_image->knownToBeOpaque(&renderer);
+    return static_cast<BlendMode>(m_blendMode) == BlendMode::Normal && static_cast<CompositeOperator>(m_composite) == CompositeOperator::SourceOver && m_image->knownToBeOpaque(&renderer);
 }
 
 bool FillLayer::hasRepeatXY() const
@@ -365,7 +365,7 @@ bool FillLayer::hasRepeatXY() const
     return repeatX() == FillRepeat::Repeat && repeatY() == FillRepeat::Repeat;
 }
 
-bool FillLayer::hasImage() const
+bool FillLayer::hasImageInAnyLayer() const
 {
     for (auto* layer = this; layer; layer = layer->m_next.get()) {
         if (layer->image())
