@@ -202,6 +202,11 @@ static void writeStyle(TextStream& ts, const RenderElement& renderer)
             if (!dashArray.isEmpty())
                 writeNameValuePair(ts, "dash array", dashArray);
 
+            if (is<SVGGeometryElement>(shape.graphicsElement())) {
+                double pathLength = downcast<SVGGeometryElement>(shape.graphicsElement()).pathLength();
+                writeIfNotDefault(ts, "path length", pathLength, 0.0);
+            }
+
             ts << "}]";
         }
 
@@ -222,9 +227,9 @@ static void writeStyle(TextStream& ts, const RenderElement& renderer)
     writeIfNotEmpty(ts, "end marker", svgStyle.markerEndResource());
 }
 
-static TextStream& writePositionAndStyle(TextStream& ts, const RenderElement& renderer, RenderAsTextBehavior behavior = RenderAsTextBehaviorNormal)
+static TextStream& writePositionAndStyle(TextStream& ts, const RenderElement& renderer, OptionSet<RenderAsTextFlag> behavior = { })
 {
-    if (behavior & RenderAsTextShowSVGGeometry) {
+    if (behavior.contains(RenderAsTextFlag::ShowSVGGeometry)) {
         if (is<RenderBox>(renderer)) {
             LayoutRect r = downcast<RenderBox>(renderer).frameRect();
             ts << " " << enclosingIntRect(r);
@@ -271,7 +276,7 @@ static TextStream& operator<<(TextStream& ts, const RenderSVGShape& shape)
         writeNameValuePair(ts, "r", element.r().value(lengthContext));
     } else if (is<SVGPolyElement>(svgElement)) {
         const SVGPolyElement& element = downcast<SVGPolyElement>(svgElement);
-        writeNameAndQuotedValue(ts, "points", element.pointList().valueAsString());
+        writeNameAndQuotedValue(ts, "points", element.points().valueAsString());
     } else if (is<SVGPathElement>(svgElement)) {
         const SVGPathElement& element = downcast<SVGPathElement>(svgElement);
         String pathString;
@@ -369,14 +374,14 @@ enum class WriteIndentOrNot {
     Yes
 };
 
-static void writeStandardPrefix(TextStream& ts, const RenderObject& object, RenderAsTextBehavior behavior, WriteIndentOrNot writeIndent = WriteIndentOrNot::Yes)
+static void writeStandardPrefix(TextStream& ts, const RenderObject& object, OptionSet<RenderAsTextFlag> behavior, WriteIndentOrNot writeIndent = WriteIndentOrNot::Yes)
 {
     if (writeIndent == WriteIndentOrNot::Yes)
         ts << indent;
 
     ts << object.renderName();
 
-    if (behavior & RenderAsTextShowAddresses)
+    if (behavior.contains(RenderAsTextFlag::ShowAddresses))
         ts << " " << static_cast<const void*>(&object);
 
     if (object.node())
@@ -385,7 +390,7 @@ static void writeStandardPrefix(TextStream& ts, const RenderObject& object, Rend
     writeDebugInfo(ts, object, behavior);
 }
 
-static void writeChildren(TextStream& ts, const RenderElement& parent, RenderAsTextBehavior behavior)
+static void writeChildren(TextStream& ts, const RenderElement& parent, OptionSet<RenderAsTextFlag> behavior)
 {
     TextStream::IndentScope indentScope(ts);
 
@@ -404,11 +409,11 @@ static inline void writeCommonGradientProperties(TextStream& ts, SVGSpreadMethod
         ts << " [gradientTransform=" << gradientTransform << "]";
 }
 
-void writeSVGResourceContainer(TextStream& ts, const RenderSVGResourceContainer& resource, RenderAsTextBehavior behavior)
+void writeSVGResourceContainer(TextStream& ts, const RenderSVGResourceContainer& resource, OptionSet<RenderAsTextFlag> behavior)
 {
     writeStandardPrefix(ts, resource, behavior);
 
-    const AtomicString& id = resource.element().getIdAttribute();
+    const AtomString& id = resource.element().getIdAttribute();
     writeNameAndQuotedValue(ts, "id", id);
 
     if (resource.resourceType() == MaskerResourceType) {
@@ -488,7 +493,7 @@ void writeSVGResourceContainer(TextStream& ts, const RenderSVGResourceContainer&
     writeChildren(ts, resource, behavior);
 }
 
-void writeSVGContainer(TextStream& ts, const RenderSVGContainer& container, RenderAsTextBehavior behavior)
+void writeSVGContainer(TextStream& ts, const RenderSVGContainer& container, OptionSet<RenderAsTextFlag> behavior)
 {
     // Currently RenderSVGResourceFilterPrimitive has no meaningful output.
     if (container.isSVGResourceFilterPrimitive())
@@ -500,7 +505,7 @@ void writeSVGContainer(TextStream& ts, const RenderSVGContainer& container, Rend
     writeChildren(ts, container, behavior);
 }
 
-void write(TextStream& ts, const RenderSVGRoot& root, RenderAsTextBehavior behavior)
+void write(TextStream& ts, const RenderSVGRoot& root, OptionSet<RenderAsTextFlag> behavior)
 {
     writeStandardPrefix(ts, root, behavior);
     writePositionAndStyle(ts, root, behavior);
@@ -508,7 +513,7 @@ void write(TextStream& ts, const RenderSVGRoot& root, RenderAsTextBehavior behav
     writeChildren(ts, root, behavior);
 }
 
-void writeSVGText(TextStream& ts, const RenderSVGText& text, RenderAsTextBehavior behavior)
+void writeSVGText(TextStream& ts, const RenderSVGText& text, OptionSet<RenderAsTextFlag> behavior)
 {
     writeStandardPrefix(ts, text, behavior);
     writeRenderSVGTextBox(ts, text);
@@ -517,7 +522,7 @@ void writeSVGText(TextStream& ts, const RenderSVGText& text, RenderAsTextBehavio
     writeChildren(ts, text, behavior);
 }
 
-void writeSVGInlineText(TextStream& ts, const RenderSVGInlineText& text, RenderAsTextBehavior behavior)
+void writeSVGInlineText(TextStream& ts, const RenderSVGInlineText& text, OptionSet<RenderAsTextFlag> behavior)
 {
     writeStandardPrefix(ts, text, behavior);
     ts << " " << enclosingIntRect(FloatRect(text.firstRunLocation(), text.floatLinesBoundingBox().size())) << "\n";
@@ -525,7 +530,7 @@ void writeSVGInlineText(TextStream& ts, const RenderSVGInlineText& text, RenderA
     writeSVGInlineTextBoxes(ts, text);
 }
 
-void writeSVGImage(TextStream& ts, const RenderSVGImage& image, RenderAsTextBehavior behavior)
+void writeSVGImage(TextStream& ts, const RenderSVGImage& image, OptionSet<RenderAsTextFlag> behavior)
 {
     writeStandardPrefix(ts, image, behavior);
     writePositionAndStyle(ts, image, behavior);
@@ -533,21 +538,21 @@ void writeSVGImage(TextStream& ts, const RenderSVGImage& image, RenderAsTextBeha
     writeResources(ts, image, behavior);
 }
 
-void write(TextStream& ts, const RenderSVGShape& shape, RenderAsTextBehavior behavior)
+void write(TextStream& ts, const RenderSVGShape& shape, OptionSet<RenderAsTextFlag> behavior)
 {
     writeStandardPrefix(ts, shape, behavior);
     ts << shape << "\n";
     writeResources(ts, shape, behavior);
 }
 
-void writeSVGGradientStop(TextStream& ts, const RenderSVGGradientStop& stop, RenderAsTextBehavior behavior)
+void writeSVGGradientStop(TextStream& ts, const RenderSVGGradientStop& stop, OptionSet<RenderAsTextFlag> behavior)
 {
     writeStandardPrefix(ts, stop, behavior);
 
     ts << " [offset=" << stop.element().offset() << "] [color=" << stop.element().stopColorIncludingOpacity() << "]\n";
 }
 
-void writeResources(TextStream& ts, const RenderObject& renderer, RenderAsTextBehavior behavior)
+void writeResources(TextStream& ts, const RenderObject& renderer, OptionSet<RenderAsTextFlag> behavior)
 {
     const RenderStyle& style = renderer.style();
     const SVGRenderStyle& svgStyle = style.svgStyle();
@@ -563,10 +568,12 @@ void writeResources(TextStream& ts, const RenderObject& renderer, RenderAsTextBe
             ts << " " << masker->resourceBoundingBox(renderer) << "\n";
         }
     }
-    if (!svgStyle.clipperResource().isEmpty()) {
-        if (RenderSVGResourceClipper* clipper = getRenderSVGResourceById<RenderSVGResourceClipper>(renderer.document(), svgStyle.clipperResource())) {
+    if (style.clipPath() && is<ReferenceClipPathOperation>(style.clipPath())) {
+        auto resourceClipPath = downcast<ReferenceClipPathOperation>(style.clipPath());
+        AtomString id = resourceClipPath->fragment();
+        if (RenderSVGResourceClipper* clipper = getRenderSVGResourceById<RenderSVGResourceClipper>(renderer.document(), id)) {
             ts << indent << " ";
-            writeNameAndQuotedValue(ts, "clipPath", svgStyle.clipperResource());
+            writeNameAndQuotedValue(ts, "clipPath", resourceClipPath->fragment());
             ts << " ";
             writeStandardPrefix(ts, *clipper, behavior, WriteIndentOrNot::No);
             ts << " " << clipper->resourceBoundingBox(renderer) << "\n";
@@ -578,7 +585,7 @@ void writeResources(TextStream& ts, const RenderObject& renderer, RenderAsTextBe
             const FilterOperation& filterOperation = *filterOperations.at(0);
             if (filterOperation.type() == FilterOperation::REFERENCE) {
                 const auto& referenceFilterOperation = downcast<ReferenceFilterOperation>(filterOperation);
-                AtomicString id = SVGURIReference::fragmentIdentifierFromIRIString(referenceFilterOperation.url(), renderer.document());
+                AtomString id = SVGURIReference::fragmentIdentifierFromIRIString(referenceFilterOperation.url(), renderer.document());
                 if (RenderSVGResourceFilter* filter = getRenderSVGResourceById<RenderSVGResourceFilter>(renderer.document(), id)) {
                     ts << indent << " ";
                     writeNameAndQuotedValue(ts, "filter", id);
