@@ -23,6 +23,7 @@
 #include "PathTraversalState.h"
 #include "RenderSVGTextPath.h"
 #include "SVGElement.h"
+#include "SVGGeometryElement.h"
 #include "SVGInlineTextBox.h"
 #include "SVGLengthContext.h"
 #include "SVGTextContentElement.h"
@@ -170,10 +171,19 @@ void SVGTextLayoutEngine::beginTextPathLayout(RenderSVGTextPath& textPath, SVGTe
     if (m_textPath.isEmpty())
         return;
 
-    m_textPathStartOffset = textPath.startOffset();
+    const auto& startOffset = textPath.startOffset();
     m_textPathLength = m_textPath.length();
-    if (m_textPathStartOffset > 0 && m_textPathStartOffset <= 1)
-        m_textPathStartOffset *= m_textPathLength;
+
+    if (textPath.startOffset().lengthType() == SVGLengthType::Percentage)
+        m_textPathStartOffset = startOffset.valueAsPercentage() * m_textPathLength;
+    else {
+        m_textPathStartOffset = startOffset.valueInSpecifiedUnits();
+        if (auto* tragetElement = textPath.targetElement()) {
+            // FIXME: A value of zero is valid. Need to differentiate this case from being unspecified.
+            if (float pathLength = tragetElement->pathLength())
+                m_textPathStartOffset *= m_textPathLength / pathLength;
+        }
+    }
 
     lineLayout.m_chunkLayoutBuilder.buildTextChunks(lineLayout.m_lineLayoutBoxes);
 
@@ -531,9 +541,8 @@ void SVGTextLayoutEngine::layoutTextOnLineOrPath(SVGInlineTextBox& textBox, Rend
             if (textPathOffset > m_textPathLength)
                 break;
 
-            bool success = false;
-            auto traversalState(m_textPath.traversalStateAtLength(textPathOffset, success));
-            ASSERT(success);
+            auto traversalState(m_textPath.traversalStateAtLength(textPathOffset));
+            ASSERT(traversalState.success());
 
             FloatPoint point = traversalState.current();
             x = point.x();
