@@ -26,10 +26,14 @@
 #include "config.h"
 #include "RenderTreeBuilderInline.h"
 
+#include "FullscreenManager.h"
+#include "RenderBlockFlow.h"
 #include "RenderChildIterator.h"
 #include "RenderFullScreen.h"
 #include "RenderInline.h"
 #include "RenderTable.h"
+#include "RenderTreeBuilderMultiColumn.h"
+#include "RenderTreeBuilderTable.h"
 
 namespace WebCore {
 
@@ -168,7 +172,7 @@ void RenderTreeBuilder::Inline::attachIgnoringContinuation(RenderInline& parent,
         // inline into continuations. This involves creating an anonymous block box to hold
         // |newChild|. We then make that block box a continuation of this inline. We take all of
         // the children after |beforeChild| and put them in a clone of this object.
-        auto newStyle = RenderStyle::createAnonymousStyleWithDisplay(parent.style(), BLOCK);
+        auto newStyle = RenderStyle::createAnonymousStyleWithDisplay(parent.style(), DisplayType::Block);
 
         // If inside an inline affected by in-flow positioning the block needs to be affected by it too.
         // Giving the block a layer like this allows it to collect the x/y offsets from inline parents later.
@@ -270,9 +274,9 @@ void RenderTreeBuilder::Inline::splitInlines(RenderInline& parent, RenderBlock*
     // that renderer is wrapped in a RenderFullScreen, so |this| is not its
     // parent. Since the splitting logic expects |this| to be the parent, set
     // |beforeChild| to be the RenderFullScreen.
-    const Element* fullScreenElement = parent.document().webkitCurrentFullScreenElement();
+    const Element* fullScreenElement = parent.document().fullscreenManager().currentFullscreenElement();
     if (fullScreenElement && beforeChild && beforeChild->node() == fullScreenElement)
-        beforeChild = parent.document().fullScreenRenderer();
+        beforeChild = parent.document().fullscreenManager().fullscreenRenderer();
 #endif
     // Now take all of the children from beforeChild to the end and remove
     // them from |this| and place them in the clone.
@@ -376,7 +380,7 @@ void RenderTreeBuilder::Inline::splitInlines(RenderInline& parent, RenderBlock*
 bool RenderTreeBuilder::Inline::newChildIsInline(const RenderInline& parent, const RenderObject& child)
 {
     // inline parent generates inline-table.
-    return child.isInline() || (m_builder.tableBuilder().childRequiresTable(parent, child) && parent.style().display() == INLINE);
+    return child.isInline() || (m_builder.tableBuilder().childRequiresTable(parent, child) && parent.style().display() == DisplayType::Inline);
 }
 
 void RenderTreeBuilder::Inline::childBecameNonInline(RenderInline& parent, RenderElement& child)
