@@ -33,6 +33,8 @@
 #include "MIMETypeRegistry.h"
 #include "ResourceError.h"
 #include "ResourceResponse.h"
+#include "ScriptExecutionContext.h"
+#include "SecurityOrigin.h"
 #include "ServiceWorkerJobData.h"
 #include "ServiceWorkerRegistration.h"
 
@@ -51,6 +53,11 @@ ServiceWorkerJob::~ServiceWorkerJob()
     ASSERT(m_creationThread.ptr() == &Thread::current());
 }
 
+RefPtr<DeferredPromise> ServiceWorkerJob::takePromise()
+{
+    return WTFMove(m_promise);
+}
+
 void ServiceWorkerJob::failedWithException(const Exception& exception)
 {
     ASSERT(m_creationThread.ptr() == &Thread::current());
@@ -106,45 +113,45 @@ void ServiceWorkerJob::fetchScriptWithContext(ScriptExecutionContext& context, F
     m_scriptLoader->loadAsynchronously(context, WTFMove(request), WTFMove(options), ContentSecurityPolicyEnforcement::DoNotEnforce, ServiceWorkersMode::None, *this);
 }
 
+ResourceError ServiceWorkerJob::validateServiceWorkerResponse(const ServiceWorkerJobData& jobData, const ResourceResponse& response)
+{
+    // Extract a MIME type from the response's header list. If this MIME type (ignoring parameters) is not a JavaScript MIME type, then:
+    if (!MIMETypeRegistry::isSupportedJavaScriptMIMEType(response.mimeType()))
+        return { errorDomainWebKitInternal, 0, response.url(), "MIME Type is not a JavaScript MIME type"_s };
+
+    String serviceWorkerAllowed = response.httpHeaderField(HTTPHeaderName::ServiceWorkerAllowed);
+    String maxScopeString;
+    if (serviceWorkerAllowed.isNull()) {
+        String path = jobData.scriptURL.path();
+        // Last part of the path is the script's filename.
+        maxScopeString = path.substring(0, path.reverseFind('/') + 1);
+    } else {
+        auto maxScope = URL(jobData.scriptURL, serviceWorkerAllowed);
+        if (SecurityOrigin::create(maxScope)->isSameOriginAs(SecurityOrigin::create(jobData.scriptURL)))
+            maxScopeString = maxScope.path();
+    }
+
+    String scopeString = jobData.scopeURL.path();
+    if (!scopeString.startsWith(maxScopeString))
+        return { errorDomainWebKitInternal, 0, response.url(), "Scope URL should start with the given script URL"_s };
+
+    return { };
+}
+
 void ServiceWorkerJob::didReceiveResponse(unsigned long, const ResourceResponse& response)
 {
     ASSERT(m_creationThread.ptr() == &Thread::current());
     ASSERT(!m_completed);
     ASSERT(m_scriptLoader);
 
-    // Extract a MIME type from the response's header list. If this MIME type (ignoring parameters) is not a JavaScript MIME type, then:
-    if (!MIMETypeRegistry::isSupportedJavaScriptMIMEType(response.mimeType())) {
-        m_scriptLoader->cancel();
-        m_scriptLoader = nullptr;
-
-        // Invoke Reject Job Promise with job and "SecurityError" DOMException.
-        Exception exception { SecurityError, "MIME Type is not a JavaScript MIME type"_s };
-        // Asynchronously complete these steps with a network error.
-        ResourceError error { errorDomainWebKitInternal, 0, response.url(), "Unexpected MIME type"_s };
-        m_client.jobFailedLoadingScript(*this, WTFMove(error), WTFMove(exception));
-        return;
-    }
-
-    String serviceWorkerAllowed = response.httpHeaderField(HTTPHeaderName::ServiceWorkerAllowed);
-    String maxScopeString;
-    if (serviceWorkerAllowed.isNull()) {
-        String path = m_jobData.scriptURL.path();
-        // Last part of the path is the script's filename.
-        maxScopeString = path.substring(0, path.reverseFind('/') + 1);
-    } else {
-        auto maxScope = URL(m_jobData.scriptURL, serviceWorkerAllowed);
-        maxScopeString = maxScope.path();
-    }
-
-    String scopeString = m_jobData.scopeURL.path();
-    if (scopeString.startsWith(maxScopeString))
+    auto error = validateServiceWorkerResponse(m_jobData, response);
+    if (error.isNull())
         return;
 
     m_scriptLoader->cancel();
     m_scriptLoader = nullptr;
 
-    Exception exception { SecurityError, "Scope URL should start with the given script URL"_s };
-    ResourceError error { errorDomainWebKitInternal, 0, response.url(), "Scope URL should start with the given script URL"_s };
+    Exception exception { SecurityError, error.localizedDescription() };
     m_client.jobFailedLoadingScript(*this, WTFMove(error), WTFMove(exception));
 }
 
@@ -163,15 +170,17 @@ void ServiceWorkerJob::notifyFinished()
     auto& error = scriptLoader->error();
     ASSERT(!error.isNull());
 
-    m_client.jobFailedLoadingScript(*this, error, Exception { error.isAccessControl() ? SecurityError : TypeError, "Script load failed"_s });
+    m_client.jobFailedLoadingScript(*this, error, Exception { error.isAccessControl() ? SecurityError : TypeError, makeString("Script ", scriptLoader->url().string(), " load failed") });
 }
 
-void ServiceWorkerJob::cancelPendingLoad()
+bool ServiceWorkerJob::cancelPendingLoad()
 {
     if (!m_scriptLoader)
-        return;
+        return false;
+
     m_scriptLoader->cancel();
     m_scriptLoader = nullptr;
+    return true;
 }
 
 } // namespace WebCore
