@@ -358,10 +358,10 @@ bool EditorClientJava::handleEditingKeyboardEvent(KeyboardEvent* evt)
     return frame->editor().insertText(evt->underlyingPlatformEvent()->text(), evt);
 }
 
-void EditorClientJava::handleKeyboardEvent(KeyboardEvent* evt)
+void EditorClientJava::handleKeyboardEvent(KeyboardEvent& evt)
 {
-    if (handleEditingKeyboardEvent(evt)) {
-        evt->setDefaultHandled();
+    if (handleEditingKeyboardEvent(&evt)) {
+        evt.setDefaultHandled();
     }
 }
 
@@ -459,7 +459,7 @@ void EditorClientJava::respondToChangedSelection(Frame *frame)
         // committed text which will be ignored in
         // JWebPane.processInputMethodEvent().
         frame->editor().cancelComposition();
-        setInputMethodState(false);
+        setInputMethodState(nullptr);
     }
 }
 
@@ -596,7 +596,7 @@ bool EditorClientJava::shouldMoveRangeAfterDelete(Range*, Range*)
     return true;
 }
 
-void EditorClientJava::setInputMethodState(bool enabled)
+void EditorClientJava::setInputMethodState(Element* element)
 {
     JNIEnv* env = WTF::GetJavaEnv();
 
@@ -609,11 +609,11 @@ void EditorClientJava::setInputMethodState(bool enabled)
     env->CallVoidMethod(
         m_webPage,
         midSetInputMethodState,
-        bool_to_jbool(enabled));
+        bool_to_jbool(element && element->shouldUseInputMethod()));
     WTF::CheckAndClearException(env);
 }
 
-void EditorClientJava::handleInputMethodKeydown(KeyboardEvent*)
+void EditorClientJava::handleInputMethodKeydown(KeyboardEvent&)
 {
     notImplemented();
 }
@@ -724,10 +724,4 @@ void EditorClientJava::requestCheckingOfString(TextCheckingRequest&, const Visib
     notImplemented();
 }
 
-String EditorClientJava::replacementURLForResource(Ref<WebCore::SharedBuffer>&&, const String&)
-{
-    notImplemented();
-    return { };
-}
-
 } // namespace WebCore
