@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,17 +26,18 @@
 #pragma once
 
 #include "Mutex.h"
+#include "StaticPerProcess.h"
 #include <mutex>
 
 namespace bmalloc {
 
 class IsoTLSEntry;
 
-class IsoTLSLayout {
+class IsoTLSLayout : public StaticPerProcess<IsoTLSLayout> {
 public:
-    IsoTLSLayout(const std::lock_guard<Mutex>&);
+    BEXPORT IsoTLSLayout(const LockHolder&);
 
-    void add(IsoTLSEntry*);
+    BEXPORT void add(IsoTLSEntry*);
 
     IsoTLSEntry* head() const { return m_head; }
 
@@ -44,6 +45,7 @@ private:
     IsoTLSEntry* m_head { nullptr };
     IsoTLSEntry* m_tail { nullptr };
 };
+DECLARE_STATIC_PER_PROCESS_STORAGE(IsoTLSLayout);
 
 } // namespace bmalloc
 
