@@ -29,6 +29,7 @@
 #include "FixedVector.h"
 #include "Mutex.h"
 #include "Range.h"
+#include "StaticPerProcess.h"
 #include <malloc/malloc.h>
 #include <mutex>
 
@@ -39,9 +40,9 @@ class Chunk;
 class Zone : public malloc_zone_t {
 public:
     // Enough capacity to track a 64GB heap, so probably enough for anything.
-    static const size_t capacity = 2048;
+    static constexpr size_t capacity = 2048;
 
-    Zone(std::lock_guard<Mutex>&);
+    Zone(const LockHolder&);
     Zone(task_t, memory_reader_t, vm_address_t);
 
     void addRange(Range);
