@@ -35,6 +35,7 @@
 
 #include <wtf/RefPtr.h>
 #include <JavaScriptCore/JavaScript.h>
+#include <JavaScriptCore/JSCConfig.h>
 #include <JavaScriptCore/TestRunnerUtils.h>
 
 RefPtr<TestRunner> gTestRunner;
@@ -45,7 +46,13 @@ JSGlobalContextRef gContext;
 extern "C" {
 #endif
 
-JNIEXPORT void JNICALL Java_com_sun_javafx_webkit_drt_DumpRenderTree_init
+JNIEXPORT void JNICALL Java_com_sun_javafx_webkit_drt_DumpRenderTree_initDRT
+    (JNIEnv* env, jclass cls)
+{
+    JSC::Config::configureForTesting();
+}
+
+JNIEXPORT void JNICALL Java_com_sun_javafx_webkit_drt_DumpRenderTree_initTest
     (JNIEnv* env, jclass cls, jstring testPath, jstring pixelsHash)
 {
     const char* testPathChars = env->GetStringUTFChars(testPath, NULL);
