@@ -225,11 +225,6 @@ void TestRunner::setTabKeyCyclesThroughElements(bool cycles)
     // FIXME: implement
 }
 
-void TestRunner::setUseDashboardCompatibilityMode(bool flag)
-{
-    // FIXME: implement
-}
-
 void TestRunner::setUserStyleSheetEnabled(bool flag)
 {
     // FIXME: implement
@@ -576,7 +571,7 @@ unsigned TestRunner::imageCountInGeneralPasteboard() const
 
 void TestRunner::forceImmediateCompletion()
 {
-    fprintf(testResult, "ERROR: TestRunner::forceImmediateCompletion() not implemented\n");
+    notifyDone();
 }
 
 void TestRunner::setSpellCheckerResults(JSContextRef, JSObjectRef)
@@ -584,11 +579,6 @@ void TestRunner::setSpellCheckerResults(JSContextRef, JSObjectRef)
     fprintf(testResult, "ERROR: TestRunner::setSpellCheckerResults() not implemented\n");
 }
 
-void TestRunner::setIDBPerOriginQuota(uint64_t)
-{
-    fprintf(testResult, "ERROR: TestRunner::setIDBPerOriginQuota() not implemented\n");
-}
-
 void TestRunner::setOnlyAcceptFirstPartyCookies(bool)
 {
     fprintf(testResult, "ERROR: TestRunner::setOnlyAcceptFirstPartyCookies() not implemented\n");
